package org.ow2.choreos.abstractionBase;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessagesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.OperationsMapping;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.list.listRemoval.ListRemoval;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.XMLBuiltInTypes;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 */
public class FunctionalAbstraction {


	//Variables.
	protected ServiceInterface serviceInterface;

	protected Integer functionalAbstractionID;//The id with which the abstraction has been stored in the hierarchy.

	private String targetSchemaRepresentationModel;//It is the schema representation model from which the mappings have calculated. It is not null if also the 'componentsMapping' is not null.

	protected List<ServiceInterface> representedInterfaces;

	protected InterfacesMapping interfacesMapping;

	protected FunctionalAbstraction ancestor;

	protected List<FunctionalAbstraction> descendants;

	private int dbKey;

	private String constraintViolation;
	private boolean hasConstraintViolation;

	private String namesDistanceCalculatorAlgorithm;
	
	private int ancestorId;


	//Constructor 1.
	public FunctionalAbstraction( String cNamesDistanceCalculatorAlgorithm, Integer cFunctionalAbstractionID, List<FunctionalAbstraction> cDescendants, InterfacesMapping cInterfacesMapping ){

		functionalAbstractionID = cFunctionalAbstractionID;

		ancestor = null;

		descendants = cDescendants;

		interfacesMapping = cInterfacesMapping;
		
		namesDistanceCalculatorAlgorithm = cNamesDistanceCalculatorAlgorithm;

		representedInterfaces = new ArrayList<ServiceInterface>();

		for( int i = 0; i < cInterfacesMapping.getServiceInterfaces().size(); ++i ) if( cInterfacesMapping.getServiceInterfaces().get( i ).getServiceInstances() != null ) representedInterfaces.add( cInterfacesMapping.getServiceInterfaces().get( i ) );

		for( int i = 0; cDescendants != null && i < cDescendants.size(); ++i ) representedInterfaces.addAll( cDescendants.get(i).getRepresentedInterfaces() );


		set( extractInterface( cNamesDistanceCalculatorAlgorithm ) );
		
		if( serviceInterface != null ){

			serviceInterface.transform( targetSchemaRepresentationModel );

			serviceInterface.setDBAbstractionKey( cFunctionalAbstractionID );
		}


		PrintMessages.print( PrintMessages.abstractionPrint, this );
	}


	//Constructor 2: It is used for retrieval from the service base.
	public FunctionalAbstraction( ServiceInterface representativeInterface, String namesDistanceCalculatorAlgorithm, Integer functionalAbstractionID, List<FunctionalAbstraction> descendants, InterfacesMapping interfacesMapping ){

		this.functionalAbstractionID = functionalAbstractionID;
		this.ancestor = null;
		this.descendants = descendants;
		this.interfacesMapping = interfacesMapping;
				
		this.namesDistanceCalculatorAlgorithm = namesDistanceCalculatorAlgorithm;

		representedInterfaces = new ArrayList<ServiceInterface>();

		for( int i = 0; i < interfacesMapping.getServiceInterfaces().size(); ++i ) if( interfacesMapping.getServiceInterfaces().get(i).getServiceInstances() != null ) this.representedInterfaces.add( interfacesMapping.getServiceInterfaces().get(i) );

		for( int i = 0; descendants != null && i < descendants.size(); ++i ) this.representedInterfaces.addAll( descendants.get(i).getRepresentedInterfaces() );

		serviceInterface = representativeInterface;

		serviceInterface.setConcrete(false);

		serviceInterface.setDBAbstractionKey(functionalAbstractionID);


		PrintMessages.print( PrintMessages.abstractionPrint, this );
	}
	
	
	//Empty constructor used for xml transformation
	public FunctionalAbstraction(){
		
	}

	//Methods used by the constructor.
	private ServiceInterface extractInterface( String namesDistanceCalculatorAlgorithm ){

		ServiceInterface serviceInterface = null;


		if( interfacesMapping != null ){

			String name1 = interfacesMapping.getServiceInterfaces().get( 0 ).getName();
			String name2 = interfacesMapping.getServiceInterfaces().get( 1 ).getName();

			String[] interfaceNames = { name1, name2 }; 

			serviceInterface = new ServiceInterface( NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, interfaceNames ) );

			serviceInterface.setConcrete( false );


			List<OperationsMapping> operationsMapping = null;

			for( int op = 0; ( operationsMapping = interfacesMapping.getOperationsMappings() ) != null && op < operationsMapping.size(); ++op ){
	
				Operation operation = extractOperation( namesDistanceCalculatorAlgorithm, operationsMapping.get( op ) );
	
				serviceInterface.add( operation );
			}
		}


		return serviceInterface;
	}

	private Operation extractOperation( String namesDistanceCalculatorAlgorithm, OperationsMapping operationsMapping ){

		Message inMessage = null;
		Message outMessage = null;

if(operationsMapping.getInputMappings() != null)
		inMessage = extractMessage( namesDistanceCalculatorAlgorithm, operationsMapping.getInputMappings(), true );
if(operationsMapping.getOutputMappings() != null)
		outMessage = extractMessage( namesDistanceCalculatorAlgorithm, operationsMapping.getOutputMappings(), false );

		String name1 = operationsMapping.getOperations().get( 0 ).getName();
		String name2 = operationsMapping.getOperations().get( 1 ).getName();

		String[] operationNames = { name1, name2 }; 


		Operation operation = new Operation( NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, operationNames ), inMessage, outMessage );

		operation.setConcrete( false );


		return operation;
	}

	private Message extractMessage( String namesDistanceCalculatorAlgorithm, MessagesMapping messagesMapping, boolean belongsToInputMessage ){

		Message message = null;

		if( ! messagesMapping.isTrivialWithoutMessages() ){//Trivial mapping in which the messages are null, not only they do not have types but also they do not have messages!

			String name1 = messagesMapping.getMessages().get( 0 ).getName();
			String name2 = messagesMapping.getMessages().get( 1 ).getName();

			String[] messageNames = { name1, name2 }; 

			message = new Message( NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, messageNames ) );

			message.setConcrete( false );

			message.setInOrOut( messagesMapping.getMessages().get( 0 ).getInOrOut() );


			if ( ! messagesMapping.isTrivial() ){//Trivial mapping in which the messages are not null but they have null types mapping!

				List<MessageTypesMapping> messageTypesMappings = null;

				for( int t = 0; ( messageTypesMappings = messagesMapping.getMessageTypesMapping() ) != null && t < messageTypesMappings.size(); ++t ){

					MessageType messageType = extractMessageType( namesDistanceCalculatorAlgorithm, messageTypesMappings.get( t ), belongsToInputMessage );

					message.add( messageType );
				}
			}
		}


		return message;
	}

	private MessageType extractMessageType( String namesDistanceCalculatorAlgorithm, MessageTypesMapping messageTypesMapping, boolean belongsToInputMessage ){

		targetSchemaRepresentationModel = messageTypesMapping.getSchemaRepresentationModel();


		MessageType messageType1 = messageTypesMapping.getMessageTypes().get( 0 );
		MessageType messageType2 = messageTypesMapping.getMessageTypes().get( 1 );

		String name1 = messageType1.getName();
		String type1 = messageType1.getReferencedComponent().getType();

		String name2 = messageType2.getName();
		String type2 = messageType2.getReferencedComponent().getType();


		MessageType messageType = null;

		String[] componentNames = { name1, name2 }; 

		String name = NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, componentNames );
		String type = null;

		String[] componentTypes = { type1, type2 };

		if( messageType1.getReferencedComponent().isBuiltInType() && messageType2.getReferencedComponent().isBuiltInType() ){

			type = XMLBuiltInTypes.getSpecificInputType( componentTypes );

			messageType = new MessageType( name, type, "type" );
		}

		else{

			type = NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, componentTypes );


			Component materializedComponent = extractComponent( namesDistanceCalculatorAlgorithm, messageTypesMapping.getComponentsAndStructuresMapping(), null, belongsToInputMessage, true );


			messageType = new MessageType( name, type, "type", materializedComponent );
		}

		messageType.setConcrete( false );


		return messageType;
	}


	private Component extractComponent( String namesDistanceCalculatorAlgorithm, ComponentsAndStructuresMapping componentsAndStructuresMapping, List<Map<String, String>> path, boolean belongsToInputMessage, boolean isGlobal ){

		Object componentOrStructure1 = componentsAndStructuresMapping.getComponentsAndStructureMapping().get( 0 );
		Object componentOrStructure2 = componentsAndStructuresMapping.getComponentsAndStructureMapping().get( 1 );


		String primarySchemaRepresentationModel = XMLSchemaRepresentationController.getSchemaRepresentationModel( componentOrStructure1 );

		Object[] sourceObjects = { componentOrStructure1, componentOrStructure2 }; 

		Map<String, String> tagAttributes = new Hashtable<String, String>();

		path = ListInsertion.addMap( tagAttributes, path );


		Object object = XMLSchemaRepresentationController.extractObject( sourceObjects, path, belongsToInputMessage, isGlobal, primarySchemaRepresentationModel, namesDistanceCalculatorAlgorithm );


		Component component = (Component) object;

		component.setConcrete( false );


		List<ComponentsAndStructuresMapping> componentsAndStructuresMappings = null;

		for( int i = 0; ( componentsAndStructuresMappings = componentsAndStructuresMapping.getComponentsAndStructuresMappings() ) != null && i < componentsAndStructuresMappings.size(); ++i ){

			Component leafComponent = extractComponent( namesDistanceCalculatorAlgorithm, componentsAndStructuresMappings.get( i ), path, belongsToInputMessage, false );

			component.add( leafComponent );

			path = ListRemoval.removeLastListElement( path );
		}


		return component;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//It did not find any checkConstraint method.


		if( serviceInterface.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, serviceInterface );


		return constraintViolation;
	}

	public String getTag(){

		return "Abstraction " + functionalAbstractionID; 
	}

	public int getDBKey(){

		return dbKey;
	}
	
	public String getNamesDistanceCalculatorAlgorithm(){
		
		return namesDistanceCalculatorAlgorithm;
	}
	
	public FunctionalAbstraction getAncestor(){

		return ancestor;
	}
	
	public int getAncestorId(){

		return ancestorId;
	}
	
	public Integer getID(){

		return functionalAbstractionID;
	}

	public ServiceInterface getInterface(){

		return serviceInterface;
	}

	public List<ServiceInterface> getRepresentedInterfaces(){

		return representedInterfaces;
	}

	public InterfacesMapping getInterfacesMapping(){

		return interfacesMapping;
	}
	
	public List<FunctionalAbstraction> getDescendants(){

		return descendants;
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void set( ServiceInterface cServiceInterface ){

		if( ObjectConstraintsManipulation.setContainmentError( this, cServiceInterface ) == null ) serviceInterface = cServiceInterface;
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}
	
	void setAncestor( FunctionalAbstraction cAncestor ){

		ancestor = cAncestor;
	}
	
	public void setAncestorId( int ancestor ){
		
		ancestorId = ancestor;
	}
	
	public void setInterface( ServiceInterface cServiceInterface ){

        serviceInterface = cServiceInterface;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize( Integer numOfRepresentedInterfaces, DefaultMutableTreeNode node ){//The variable 'numOfRepresentedInterfaces' holds the number of the represented interfaces in the whole hierarchy.

		Integer label = functionalAbstractionID - numOfRepresentedInterfaces + 1;

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Functional abstraction (" + label + ")" );


		child1 = serviceInterface.visualize( child1 );


		for( int k = 0; descendants != null && k < descendants.size(); ++k ) child1 = descendants.get( k ).visualize( numOfRepresentedInterfaces, child1 );


		node.add( child1 );


    	return node;
	}

	DefaultMutableTreeNode visualize( Integer numOfRepresentedInterfaces ){

		Integer label = functionalAbstractionID - numOfRepresentedInterfaces + 1;

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Functional abstraction (" + label + ")" );

		child1 = serviceInterface.visualize( child1 );


		if( interfacesMapping != null ){

			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "Mappings" );

			child1.add( child11 );

			child11 = interfacesMapping.visualize( child11 );
		}


		for( int k = 0; descendants != null && k < descendants.size(); ++k ) child1.add( descendants.get( k ).visualize( numOfRepresentedInterfaces ) );


		DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "Represented service interfaces" );

		child1.add( child11 );


		for( int k = 0; representedInterfaces != null && k < representedInterfaces.size(); ++k ){

			label = k + 1;

			child11 = representedInterfaces.get( k ).visualize( label, child11 );
		}


    	return child1;
	}

	DefaultMutableTreeNode visualizeServiceInterface( Integer interfaceID, String schemaRepresentationModel ){

		Integer k = interfaceID - 1;

		ServiceInterface serviceInterface = representedInterfaces.get( k );

		if( serviceInterface == null ) return null;


		return serviceInterface.visualize( interfaceID, schemaRepresentationModel, null );
	}


	//--------------------------------------------------------------------------
	public DefaultMutableTreeNode visualizeForQueryResults( Integer numOfRepresentedInterfaces ){

		Integer label = functionalAbstractionID - numOfRepresentedInterfaces + 1;

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Functional abstraction (" + label + ")");

		child1 = serviceInterface.visualizeForQueryResults2( child1 );

		if( interfacesMapping != null ){

			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "Mappings" );

			child1.add( child11 );

			child11 = interfacesMapping.visualize( child11 );
		}

		for( int k = 0; descendants != null && k < descendants.size(); ++k ) child1.add( descendants.get( k ).visualize( numOfRepresentedInterfaces ) );

		DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "Represented service interfaces" );

		child1.add( child11 );

		for( int k = 0; representedInterfaces != null && k < representedInterfaces.size(); ++k ){
					
			label = k + 1;

			//child11 = representedInterfaces.get( k ).visualize( label, child11 );
					
			child11.add(representedInterfaces.get( k ).visualizeForQueryResults( label, "" ));
		}

		return child1;
	}
	
	
	public DefaultMutableTreeNode visualizeForQueryResults2( Integer numOfRepresentedInterfaces, DefaultMutableTreeNode node ){//The variable 'numOfRepresentedInterfaces' holds the number of the represented interfaces in the whole hierarchy.

		Integer label = functionalAbstractionID - numOfRepresentedInterfaces + 1;

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Functional abstraction (" + label + ")" );


		child1 = serviceInterface.visualizeForQueryResults2( child1 );


		for( int k = 0; descendants != null && k < descendants.size(); ++k ) child1 = descendants.get( k ).visualize( numOfRepresentedInterfaces, child1 );


		node.add( child1 );


    	return node;
	}
	//--------------------------------------------------------------------------
	
	
	//Setters used for xml transformation
	
	public void setxServiceInterface( ServiceInterface cServiceInterface ){
		serviceInterface = cServiceInterface;
	}
	
	public void setxFunctionalAbstractionID( Integer cFunctionalAbstractionID ){
		functionalAbstractionID = cFunctionalAbstractionID;
	}
	
	public void setxTargetSchemaRepresentationModel( String cTargetSchemaRepresentationModel ){
		targetSchemaRepresentationModel = cTargetSchemaRepresentationModel;
	}
	
	public void setxRepresentedInterfaces( List<ServiceInterface> cRepresentedInterfaces ){
		representedInterfaces = cRepresentedInterfaces;
	}
	
	public void setxInterfacesMapping( InterfacesMapping cInterfacesMapping ){
		interfacesMapping = cInterfacesMapping;
	}
	
	public void setxAncestor( FunctionalAbstraction cAncestor ){
		ancestor = cAncestor;
	}
	
	public void setxDescendants( List<FunctionalAbstraction> cDescendants ){
		descendants = cDescendants;
	}
	
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
	
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
	
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
	
	public void setxNamesDistanceCalculatorAlgorithm( String cNamesDistanceCalculatorAlgorithm ){
		namesDistanceCalculatorAlgorithm = cNamesDistanceCalculatorAlgorithm;
	}
	
	public void setxAncestorId( int cAncestorId ){
		ancestorId = cAncestorId;
	}
	
	
	//Getters used for xml transformation
	
	public ServiceInterface getxServiceInterface(){
		return serviceInterface;
	}
	
	public Integer getxFunctionalAbstractionID(){
		return functionalAbstractionID;
	}
	
	public String getxTargetSchemaRepresentationModel(){
		return targetSchemaRepresentationModel;
	}
	
	public List<ServiceInterface> getxRepresentedInterfaces(){
		return representedInterfaces;
	}
	
	public InterfacesMapping getxInterfacesMapping(){
		return interfacesMapping;
	}
	
	public FunctionalAbstraction getxAncestor(){
		return ancestor;
	}
	
	public List<FunctionalAbstraction> getxDescendants(){
		return descendants;
	}
	
	public int getxDBKey(){
		return dbKey;
	}
	
	public String getxConstraintViolation(){
		return constraintViolation;
	}
	
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
	
	public String getxNamesDistanceCalculatorAlgorithm(){
		return namesDistanceCalculatorAlgorithm;
	}
	
	public int getxAncestorId(){
		return ancestorId;
	}
}
