package org.ow2.choreos.abstractionBase;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 */
public class FunctionalHierarchy extends Hierarchy {//This class is exclusively for the mode = 3!


	//Variables.
	protected List<ServiceInterface> unRepresentedInterfaces;

	protected Integer totalNumOfInterfaces;//This is the total number!

	protected String namesDistanceCalculatorAlgorithm;

	protected String schemaRepresentationModel;

	protected Hashtable<Integer, FunctionalAbstraction> functionalAbstractions;


	//Constructor.
	public FunctionalHierarchy( UserSettings options, List<ServiceInterface> cUnrepresentedInterfaces ) {

		super( options.getCollectionID(), options.getMode() );

		namesDistanceCalculatorAlgorithm = options.getNamesDistanceCalculatorAlgorithm();
		schemaRepresentationModel = options.getSchemaRepresentationModel();

		unRepresentedInterfaces = cUnrepresentedInterfaces;

		if( unRepresentedInterfaces != null ) totalNumOfInterfaces = unRepresentedInterfaces.size();
	}


	//Setters.
	//Remove all the elements of the existing hierarchy because the options have been changed.
	void update( UserSettings options, List<ServiceInterface> cUnRepresentedInterfaces ){//This function is only to re-produce the hierarchy of the same versions of interfaces.

		mode = options.getMode();
		namesDistanceCalculatorAlgorithm = options.getNamesDistanceCalculatorAlgorithm();
		schemaRepresentationModel = options.getSchemaRepresentationModel();

		unRepresentedInterfaces = cUnRepresentedInterfaces;
	}


	//Getters.
	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		//It did not find any checkConstraint method.


		if( hasConstraintViolation() ) return getConstraintViolation();


		if( functionalAbstractions != null )

			for (Enumeration<Integer> e = functionalAbstractions.keys(); e.hasMoreElements();){

				Integer k = e.nextElement(); 

				if( functionalAbstractions.get( k ).ancestor == null && functionalAbstractions.get( k ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, functionalAbstractions.get( k ) );
			}


		return getConstraintViolation();
	}

	public String getTag(){

		return "Functional hierarchy of the collection " + collectionID; 
	}

	FunctionalAbstraction lookUp( Integer functionalAbstractionID ){

		return functionalAbstractions != null ? functionalAbstractions.get( functionalAbstractionID ) : null;
	}

	public String getNamesDistanceCalculatorAlgorithm(){

		return namesDistanceCalculatorAlgorithm;
	}

	String getSchemaRepresentationModel(){

		return schemaRepresentationModel;
	}

	List<ServiceInterface> getUnrepresentedInterfaces(){

		return unRepresentedInterfaces;
	}

	Integer getTotalNumOfInterfaces(){

		return totalNumOfInterfaces;
	}


	List<ServiceInterface> getRepresentedInterfaces ( Integer functionalAbstractionID ){

		FunctionalAbstraction abstraction = lookUp( functionalAbstractionID );

		if( abstraction == null ) return null;

		return ( (FunctionalAbstraction) abstraction).getRepresentedInterfaces();
	}

	public Hashtable<Integer, FunctionalAbstraction> get(){

		return functionalAbstractions;
	}

	int getNumOfUnrepresentedInterfaces(){

		int numOfUnrepresentedInterfaces = 0;

		if( unRepresentedInterfaces != null ) numOfUnrepresentedInterfaces = unRepresentedInterfaces.size();


		return numOfUnrepresentedInterfaces;
	}

	int getTotalNumOfRepresentedInterfaces(){

		int numOfRepresentedInterfaces = 0;

		if( functionalAbstractions != null ){

			for( Enumeration<Integer> e = functionalAbstractions.keys(); e.hasMoreElements(); ){

				Integer k = e.nextElement(); 

				if( functionalAbstractions.get( k ).ancestor == null && functionalAbstractions.get( k ).getRepresentedInterfaces() != null ) numOfRepresentedInterfaces += functionalAbstractions.get( k ).getRepresentedInterfaces().size(); 
			}
		}


    	return numOfRepresentedInterfaces;
	}


	//Setters.
	ServiceInterface addAbstraction( Integer functionalAbstractionID, Integer descendant1, Integer descendant2, InterfacesMapping interfacesMapping, List<ServiceInterface> cUnrepresentedInterfaces ){//Operation for adding a functional abstraction.

		unRepresentedInterfaces = cUnrepresentedInterfaces;//Update the 'unRepresentedInterfaces' list at each step!

		ArrayList<FunctionalAbstraction> descenadants = null;
		FunctionalAbstraction functionalAbstractionDescendant1 = null;
		FunctionalAbstraction functionalAbstractionDescendant2 = null;

		if( functionalAbstractions == null ) functionalAbstractions = new Hashtable<Integer, FunctionalAbstraction>();

		else{

			functionalAbstractionDescendant1 = lookUp( descendant1 );//The descendants may refer to concrete interface, not to interface of abstraction.
			functionalAbstractionDescendant2 = lookUp( descendant2 );

			if( functionalAbstractionDescendant1 != null ){

				descenadants = new ArrayList<FunctionalAbstraction>();
				descenadants.add( functionalAbstractionDescendant1 );
			}

			if( functionalAbstractionDescendant2 != null ){

				if( descenadants == null ) descenadants = new ArrayList<FunctionalAbstraction>();
				descenadants.add( functionalAbstractionDescendant2 );
			}
		}

		FunctionalAbstraction addedFunctionalAbstraction = new FunctionalAbstraction( namesDistanceCalculatorAlgorithm, functionalAbstractionID, descenadants, interfacesMapping );

		functionalAbstractions.put( functionalAbstractionID, addedFunctionalAbstraction );

		if( functionalAbstractionDescendant1 != null ) functionalAbstractionDescendant1.setAncestor( addedFunctionalAbstraction );
		if( functionalAbstractionDescendant2 != null ) functionalAbstractionDescendant2.setAncestor( addedFunctionalAbstraction );

		return functionalAbstractions.get( functionalAbstractionID ).getInterface();
	}


	//Visualization.
	public DefaultMutableTreeNode visualize(){

		DefaultMutableTreeNode root = new DefaultMutableTreeNode( "Hierarchy" );

		if( unRepresentedInterfaces != null && ! unRepresentedInterfaces.isEmpty() ){

			DefaultMutableTreeNode child1 = new DefaultMutableTreeNode("Unrepresented service interfaces");
			root.add(child1);

			for( int k = 0; k < unRepresentedInterfaces.size(); ++k ){

				Integer label = k + 1;

				child1 = unRepresentedInterfaces.get( k ).visualize( label, child1 );
			}
		}

		if( functionalAbstractions != null ){

			DefaultMutableTreeNode child2 = new DefaultMutableTreeNode( "Functional abstractions" );

			root.add(child2);

			for (Enumeration<Integer> e = functionalAbstractions.keys(); e.hasMoreElements();){

				Integer k = e.nextElement(); 

				if( functionalAbstractions.get( k ).ancestor == null ) child2 = functionalAbstractions.get( k ).visualize( totalNumOfInterfaces, child2 );
			}
		}

    	return root;
	}

	DefaultMutableTreeNode visualize( Integer functionalAbstractionID ){

		functionalAbstractionID = functionalAbstractionID + totalNumOfInterfaces - 1;

		FunctionalAbstraction functionalAbstraction = lookUp( functionalAbstractionID );

		if( functionalAbstraction == null ) return null;
		
		return functionalAbstraction.visualize( totalNumOfInterfaces );
	}

	DefaultMutableTreeNode visualizeServiceInterface( Integer interfaceID, Integer ancestorID ){

		ancestorID = ancestorID + totalNumOfInterfaces - 1;

		FunctionalAbstraction functionalAbstraction = lookUp( ancestorID );

		if( functionalAbstraction == null ) return null;
		
		return functionalAbstraction.visualizeServiceInterface( interfaceID, schemaRepresentationModel );
	}
}
