package org.ow2.choreos.abstractionBase;

import java.util.Hashtable;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 */
public class IndependentHierarchies extends Hierarchy {


	//Variables.
	private FunctionalHierarchy functionalHierarchy;
	private NonFunctionalHierarchy nonFunctionalHierarchy;


	//Constructor.
	IndependentHierarchies( UserSettings options, List<ServiceInterface> allInterfaces ) {

		super( options.getCollectionID(), options.getMode() );


		functionalHierarchy = new FunctionalHierarchy( options, allInterfaces );

		nonFunctionalHierarchy = new NonFunctionalHierarchy( collectionID, mode, options.getThreshold() );
	}


	//Setters.
	//Remove all the elements of the existing hierarchy because the options have been changed.
	void update( UserSettings options, List<ServiceInterface> unRepresentedInterfaces ){//This function is only to re-produce the hierarchy of the same versions of interfaces.

		if( functionalHierarchy != null ) functionalHierarchy.update( options, unRepresentedInterfaces );

		if( nonFunctionalHierarchy != null ) nonFunctionalHierarchy.update( options );
	}


	//Getters.
	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation() ) return getConstraintViolation();


		//I did not find any checkConstraint method.


		if( functionalHierarchy.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, functionalHierarchy );

		//TODO:if( nonFunctionalHierarchy.hasFinalConstraintViolation() ) ConstraintsManipulation.setContainmentError( this, nonFunctionalHierarchy );


		return getConstraintViolation();
	}

	public String getTag(){

		return "Independent hierarchies of the collection " + collectionID; 
	}

	public FunctionalHierarchy getFunctionalHierarchy(){

		return functionalHierarchy;
	}

	Hashtable<Integer, FunctionalAbstraction> getFunctionaLAbstractions(){

		return functionalHierarchy.get();
	}

	public int getTotalNumOfUnrepresentedInterfaces(){

		int numOfRepresentedInterfaces = 0;

		if( functionalHierarchy != null ) numOfRepresentedInterfaces = functionalHierarchy.getNumOfUnrepresentedInterfaces();


		return numOfRepresentedInterfaces;
	}

	public int getTotalNumOfRepresentedInterfaces(){

		int numOfRepresentedInterfaces = 0;

		if( functionalHierarchy != null ) numOfRepresentedInterfaces = functionalHierarchy.getTotalNumOfRepresentedInterfaces();


		return numOfRepresentedInterfaces;
	}

	NonFunctionalHierarchy getNonFunctionaLHierarchy(){

		return nonFunctionalHierarchy;
	}

	Hashtable<Integer, NonFunctionalAbstraction> getNonFunctionalAbstractions(){

		return nonFunctionalHierarchy.get();
	}

	public int getTotalNumOfUnrepresentedInstances(){

		int numOfRepresentedInstances = 0;

		if( nonFunctionalHierarchy != null ) numOfRepresentedInstances = nonFunctionalHierarchy.getNumOfUnrepresentedInstances();


		return numOfRepresentedInstances;
	}

	public int getTotalNumOfRepresentedInstances(){

		int numOfRepresentedInstances = 0;

		if( nonFunctionalHierarchy != null ) numOfRepresentedInstances = nonFunctionalHierarchy.getTotalNumOfRepresentedInstances();


		return numOfRepresentedInstances;
	}


	//Visualization.
	DefaultMutableTreeNode visualizeFH(){

		if( functionalHierarchy == null) return null;

		return functionalHierarchy.visualize();
	}

	DefaultMutableTreeNode visualizeNFH(){

		if( nonFunctionalHierarchy == null) return null;

		return nonFunctionalHierarchy.visualize();
	}

	DefaultMutableTreeNode visualizeFA( Integer functionalAbstractionID ){

		if( functionalHierarchy == null) return null;

		return functionalHierarchy.visualize( functionalAbstractionID );
	}

	DefaultMutableTreeNode visualizeNFA( ServiceRegistration serviceRegistration, Integer nonFunctionalAbstractionID ){

		if( nonFunctionalHierarchy == null) return null;

		return nonFunctionalHierarchy.visualize( serviceRegistration, nonFunctionalAbstractionID );
	}

	DefaultMutableTreeNode visualizeServiceInterface( Integer interfaceID, Integer ancestorID ){

		if( functionalHierarchy == null) return null;

		return functionalHierarchy.visualizeServiceInterface( interfaceID, ancestorID );
	}

	DefaultMutableTreeNode visualizeNFH( ServiceRegistration serviceRegistration ){

		if( nonFunctionalHierarchy == null) return null;

		return nonFunctionalHierarchy.visualize();
	}
}
