package org.ow2.choreos.abstractionBase;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.dataStructures.interval.Interval;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;


public class NonFunctionalAbstraction{

	//Variables.
	private Integer nonFunctionalAbstractionID;

	protected Hashtable<String, String> nonFunctionalDescription;//The qualifiers are indexed by the property names!

	@SuppressWarnings("unchecked")
	protected Map<String, Interval> mappingsToRanges;//The ranges are indexed by the property names!

	protected Map<Integer, ServiceInstance> representedServiceInstances;

	protected NonFunctionalAbstraction ancestor;
	protected ArrayList<NonFunctionalAbstraction> descendants;

	private boolean empty = false;//If it is empty, the 'mappingsToRanges' attribute is null! It is the only attribute!
	
	private int dbKey;


	//Constructor.
	public NonFunctionalAbstraction( Integer nonFunctionalAbstractionID, JointGroup jointGroup, NonFunctionalAbstraction nonFunctionalAbstractionAncestor ){
		this.nonFunctionalAbstractionID = nonFunctionalAbstractionID;
		this.nonFunctionalDescription = jointGroup.getQualifiers();
		this.mappingsToRanges = jointGroup.getMappingsToRanges();
		this.representedServiceInstances = jointGroup.getRepresentedServiceInstances();
		this.ancestor = nonFunctionalAbstractionAncestor;
		this.descendants = null;

		if( jointGroup.isEmpty() ) empty = true;
	}
	
	
	//Empty constructor used for xml transformation
	public NonFunctionalAbstraction(){
		
	}


	//Methods.
	void freeObjects(){//TODO: Is it right?
		nonFunctionalDescription = null;
		mappingsToRanges = null;
		representedServiceInstances = null;
		ancestor = null;
		descendants = null;
	}


	//Setters.
	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}
	
	public void addDescendant( NonFunctionalAbstraction descendant ){
		if( descendants == null ) descendants = new ArrayList<NonFunctionalAbstraction>();

		descendants.add( descendant );
	}
	
	public void setAncestor(NonFunctionalAbstraction cAncestor){

		ancestor = cAncestor;
	}

	//Getters.
	public int getDBKey(){

		return dbKey;
	}
	
	public NonFunctionalAbstraction getAncestor(){

		return ancestor;
	}
	
	public Integer getNonFunctionalAbstractionID(){
		return nonFunctionalAbstractionID;
	}

	public Hashtable<String, String> getNonFunctionalDescription(){
		return nonFunctionalDescription;
	}
	
	public ArrayList<NonFunctionalAbstraction> getDescendants(){
		return descendants;
	}

	@SuppressWarnings("unchecked")
	public Map<String, Interval> getMappingsToRanges(){

		return mappingsToRanges;
	}

	public Map<Integer, ServiceInstance> getRepresentedServiceInstances(){

		return representedServiceInstances;
	}

	public boolean isEmpty(){

		return empty;
	}

	//Visualization.
	DefaultMutableTreeNode visualize( ServiceRegistration serviceRegistration, Integer collectionID, DefaultMutableTreeNode node ){

		Integer labelID = nonFunctionalAbstractionID + 1;

		String label = new String("Non-functional abstraction (" + labelID + "): ");
		String mappings = null;

		if( !isEmpty() ) mappings = new String("Mappings to ranges: ");

		int i = 0;

		for( Enumeration<String> e = nonFunctionalDescription.keys(); e.hasMoreElements(); ){

			String property = e.nextElement();
			String qualifier = nonFunctionalDescription.get( property );

			if( i == 0 ){
				label += qualifier + " " + property;
				if( !isEmpty() ) mappings += property + " = (" + mappingsToRanges.get( property ).getStart() + ", " + mappingsToRanges.get( property ).getEnd() + ")"; 
			}

			else{
				label += ", " + qualifier + " " + property;
				if( !isEmpty() ) mappings += ", " + property + " = (" + mappingsToRanges.get( property ).getStart() + ", " + mappingsToRanges.get( property ).getEnd() + ")";
			}

			++i;
		}

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( label );

		if( node != null ) node.add( child1 );
		else node = child1;

		if( !isEmpty() ){
			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( mappings );
			child1.add(child11);
		}

		if( representedServiceInstances == null || ( representedServiceInstances != null && representedServiceInstances.isEmpty() ) ){

			DefaultMutableTreeNode child12 = new DefaultMutableTreeNode("Empty!");
			child1.add( child12 );
		}

		else{
			for( int k = 0; descendants != null && k < descendants.size(); ++k )

				child1 = descendants.get(k).visualize( serviceRegistration, collectionID, child1 );

			DefaultMutableTreeNode child12 = new DefaultMutableTreeNode("Represented service instances");
			child1.add( child12 );

			for( Enumeration<Integer> e = ( (Hashtable<Integer, ServiceInstance>) representedServiceInstances).keys(); e.hasMoreElements(); ){

				Integer k = e.nextElement();

				Integer label2 = k + 1;

				child12 = serviceRegistration.visualize( collectionID, label2, representedServiceInstances.get( k ), child12);
			}
		}

		return node;
	}

	DefaultMutableTreeNode visualize( DefaultMutableTreeNode node ){//This is the case of visualizing an independent non-functional hierarchy (mode = 3).

		Integer labelID = nonFunctionalAbstractionID + 1;

		String label = new String("Non-functional abstraction (" + labelID + "): ");

		int i = 0;

		for( Enumeration<String> e = nonFunctionalDescription.keys(); e.hasMoreElements(); ){

			String property = e.nextElement();
			String qualifier = nonFunctionalDescription.get( property );

			if( i == 0 ) label += qualifier + " " + property;

			else label += ", " + qualifier + " " + property;

			++i;
		}

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( label );
		node.add( child1 );

		for( int k = 0; descendants != null && k < descendants.size(); ++k )

			child1 = descendants.get(k).visualize( child1 );

		return node;
	}
	
	
	//------------------------------------------------------------------------------------
	public DefaultMutableTreeNode visualizeForQueryResults( DefaultMutableTreeNode node ){

		Integer labelID = nonFunctionalAbstractionID + 1;

		String label = new String("Non-functional abstraction (" + labelID + "): ");
		String mappings = null;

		if( !isEmpty() ) mappings = new String("Mappings to ranges: ");

		int i = 0;

		for( Enumeration<String> e = nonFunctionalDescription.keys(); e.hasMoreElements(); ){

			String property = e.nextElement();
			String qualifier = nonFunctionalDescription.get( property );

			if( i == 0 ){
				label += qualifier + " " + property;
				//System.out.println(nonFunctionalAbstractionID + "-" + property);
				if( !isEmpty() && mappingsToRanges != null){
					if (mappingsToRanges.get( property ) != null){
						mappings += property + " = (" + mappingsToRanges.get( property ).getStart() + ", " + mappingsToRanges.get( property ).getEnd() + ")";
					}
				}
				//System.out.println("OK " + nonFunctionalAbstractionID);
			}

			else{
				label += ", " + qualifier + " " + property;
				
				if( !isEmpty() && mappingsToRanges != null){
					if (mappingsToRanges.get( property ) != null){
						mappings += ", " + property + " = (" + mappingsToRanges.get( property ).getStart() + ", " + mappingsToRanges.get( property ).getEnd() + ")";
					}
				}
			}

			++i;
		}

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( label );

		if( node != null ) node.add( child1 );
		else node = child1;
		if( !isEmpty() ){
			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( mappings );
			child1.add(child11);
		}

		if( representedServiceInstances == null || ( representedServiceInstances != null && representedServiceInstances.isEmpty() ) ){

			DefaultMutableTreeNode child12 = new DefaultMutableTreeNode("Empty!");
			child1.add( child12 );
		}

		else{
			for( int k = 0; descendants != null && k < descendants.size(); ++k )

				child1 = descendants.get(k).visualizeForQueryResults( child1 );

				DefaultMutableTreeNode child12 = new DefaultMutableTreeNode("Represented service instances");
				child1.add( child12 );

				for( Enumeration<Integer> e = ( (Hashtable<Integer, ServiceInstance>) representedServiceInstances).keys(); e.hasMoreElements(); ){

					Integer k = e.nextElement();

					Integer label2 = k + 1;

					//child12 = serviceRegistration.visualize( collectionID, label2, representedServiceInstances.get( k ), child12);
					child12.add(representedServiceInstances.get( k ).visualizeForQueryResults( label2, ""));
				}
			}

		return node;
	}
	//------------------------------------------------------------------------------------
	
	
	//Setters used for xml transformation
	
	public void setxNonFunctionalAbstractionID( Integer cNonFunctionalAbstractionID ){
		nonFunctionalAbstractionID = cNonFunctionalAbstractionID;
	}
	
	public void setxNonFunctionalDescription( Hashtable<String, String> cNonFunctionalDescription ){
		nonFunctionalDescription = cNonFunctionalDescription;
	}
	
	public void setxMappingsToRanges( Map<String, Interval> cMappingsToRanges ){
		mappingsToRanges = cMappingsToRanges;
	}
	
	public void setxRepresentedServiceInstances( Map<Integer, ServiceInstance> cRepresentedServiceInstances ){
		representedServiceInstances = cRepresentedServiceInstances;
	}
	
	public void setxAncestor( NonFunctionalAbstraction cAncestor ){
		ancestor = cAncestor;
	}
	
	public void setxDescendants( ArrayList<NonFunctionalAbstraction> cDescendants ){
		descendants = cDescendants;
	}
	
	public void setxEmpty( boolean cEmpty ){
		empty = cEmpty;
	}
	
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
	
	//Getters used for xml transformation
	
	public Integer getxNonFunctionalAbstractionID(){
		return nonFunctionalAbstractionID;
	}
	
	public Hashtable<String, String> getxNonFunctionalDescription(){
		return nonFunctionalDescription;
	}
	
	public Map<String, Interval> getxMappingsToRanges(){
		return mappingsToRanges;
	}
	
	public Map<Integer, ServiceInstance> getxRepresentedServiceInstances(){
		return representedServiceInstances;
	}
	
	public NonFunctionalAbstraction getxAncestor(){
		return ancestor;
	}
	
	public ArrayList<NonFunctionalAbstraction> getxDescendants(){
		return descendants;
	}
	
	public boolean getxEmpty(){
		return empty;
	}
	
	public int getxDBKey(){
		return dbKey;
	}
}
