package org.ow2.choreos.abstractionBase;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


public class NonFunctionalAbstractionWithNestedFAbstractions extends NonFunctionalAbstraction {


	//Variables.
	private List<ServiceInterface> unRepresentedInterfaces;
	private Map<Integer, FunctionalAbstraction> functionalAbstractions;


	//Constructor.
	NonFunctionalAbstractionWithNestedFAbstractions( Integer nonFunctionalAbstractionID, JointGroup jointGroup, NonFunctionalAbstraction nonFunctionalAbstractionAncestor) {
		super(nonFunctionalAbstractionID, jointGroup, nonFunctionalAbstractionAncestor);
	}


	//Getters.
	public ArrayList<ServiceInterface> getInterfacesOfRepresentedServiceInstances( ServiceRegistration serviceRegistration, Integer collectionID ){//TODO: This function must be done protected and be provided by controller!

		ArrayList<ServiceInterface> interfaces = null;

		for( Enumeration<Integer> e = ( (Hashtable<Integer, ServiceInstance>) representedServiceInstances).keys(); e.hasMoreElements(); ){

			Integer k =  e.nextElement();

			if( interfaces == null ) interfaces = new ArrayList<ServiceInterface>();

			ServiceInterface i = serviceRegistration.getInterface( collectionID, representedServiceInstances.get( k ) );

			if( i != null && ! interfaces.contains( i ) ) interfaces.add( i );
		}

		return interfaces;
	}

	FunctionalAbstraction lookUp( Integer functionalAbstractionID ){

		if( functionalAbstractions == null || functionalAbstractions == null ) return null;

		return functionalAbstractions.get( functionalAbstractionID );
	}

	List<ServiceInterface> getUnrepresentedInterfaces(){

		return unRepresentedInterfaces;
	}


	//Setters.
	//Adding a functional abstraction (mode = 2).
	ServiceInterface addAbstraction( String namesDistanceCalculatorAlgorithm, Integer functionalAbstractionID, Integer descendant1, Integer descendant2, InterfacesMapping interfacesMapping, ArrayList<ServiceInterface> unRepresentedInterfaces ){

		this.unRepresentedInterfaces = unRepresentedInterfaces;//Update the 'unRepresentedInterfaces' list at each step!

		ArrayList<FunctionalAbstraction> descenadants = null;
		FunctionalAbstraction functionalAbstractionDescendant1 = null;
		FunctionalAbstraction functionalAbstractionDescendant2 = null;

		if( functionalAbstractions == null )
			functionalAbstractions = new Hashtable<Integer, FunctionalAbstraction>();

		else{
			functionalAbstractionDescendant1 = lookUp( descendant1 );//The descendants may refer to concrete interface, not to interface of abstraction.
			functionalAbstractionDescendant2 = lookUp( descendant2 );

			if( functionalAbstractionDescendant1 != null ){
				descenadants = new ArrayList<FunctionalAbstraction>();
				descenadants.add( functionalAbstractionDescendant1 );
			}

			if( functionalAbstractionDescendant2 != null ){
				if( descenadants == null ) descenadants = new ArrayList<FunctionalAbstraction>();
				descenadants.add( functionalAbstractionDescendant2 );
			}
		}

		FunctionalAbstraction addedFunctionalAbstraction = new FunctionalAbstraction( namesDistanceCalculatorAlgorithm, functionalAbstractionID, descenadants, interfacesMapping );

		functionalAbstractions.put( functionalAbstractionID, addedFunctionalAbstraction );

		if( functionalAbstractionDescendant1 != null ) functionalAbstractionDescendant1.setAncestor( addedFunctionalAbstraction );
		if( functionalAbstractionDescendant2 != null ) functionalAbstractionDescendant2.setAncestor( addedFunctionalAbstraction );

		return functionalAbstractions.get( functionalAbstractionID ).getInterface();
	}

	//Visualization.
	DefaultMutableTreeNode visualize( ServiceRegistration serviceRegistration, Integer collectionID, DefaultMutableTreeNode node ){

		DefaultMutableTreeNode nonFunctionalAbstraction = super.visualize(serviceRegistration, collectionID, node);

		if( nonFunctionalAbstraction == null ) return node;

		node.add( nonFunctionalAbstraction );

		if( functionalAbstractions == null && representedServiceInstances != null ){

			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "There is NO nested functional hierarchy" );
			nonFunctionalAbstraction.add(child11);
		}

		else if( functionalAbstractions != null && representedServiceInstances != null ){

			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "Nested functional hierarchy" );
			nonFunctionalAbstraction.add(child11);

			if( unRepresentedInterfaces != null && ! unRepresentedInterfaces.isEmpty() ){

				DefaultMutableTreeNode child111 = new DefaultMutableTreeNode("Unrepresented service interfaces");
				child11.add( child111 );

				int counter = 0;

				for( int k = 0; k < unRepresentedInterfaces.size(); ++k )

					child111 = unRepresentedInterfaces.get( k ).visualize( ++counter, child111 );
			}

			for( Enumeration<Integer> e = ( (Hashtable<Integer,FunctionalAbstraction>) functionalAbstractions).keys(); e.hasMoreElements(); ){

				Integer k =  e.nextElement();

				if( functionalAbstractions.get( k ).ancestor == null )

					child11 = functionalAbstractions.get( k ).visualize( functionalAbstractions.size(), child11 );
			}
		}

    	return node;
	}
}
