package org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.schema.hungarianAlgorithm;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.schema.BuiltInComponentsDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.assignmentProblem.AssignmentProblemController;
import org.ow2.choreos.assignmentProblem.AssignmentProblemSolver;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;


public class ComponentsDistanceCalculatorHungarianAlgorithm {


	public static ComponentsAndStructuresMapping calculate( String namesDistanceCalculator, Component c1, Component c2, Map<String, Object> indexOfMaterializedObjectsPath1, Map<String, Object> indexOfMaterializedObjectsPath2 ){

		return calculateLocally( namesDistanceCalculator, c1, c2, indexOfMaterializedObjectsPath1, indexOfMaterializedObjectsPath2 );
	}

	public static ComponentsAndStructuresMapping calculate( String namesDistanceCalculator, Component c1, Component c2 ){

		return calculateLocally( namesDistanceCalculator, c1, c2, null, null );
	}

	private static ComponentsAndStructuresMapping calculateLocally( String namesDistanceCalculator, Component c1, Component c2, Map<String, Object> indexOfTransformedObjectsPath1, Map<String, Object> indexOfTransformedObjectsPath2 ){

		ComponentsAndStructuresMapping componentsAndStructuresMapping = null;


		if( c1.hasReferencedComponent() && c2.hasReferencedComponent() ){

			if( c1.getReferencedComponent().isBuiltInType() && c2.getReferencedComponent().isBuiltInType() ){

				if( indexOfTransformedObjectsPath1 == null && indexOfTransformedObjectsPath2 == null ) componentsAndStructuresMapping = BuiltInComponentsDistanceCalculator.calculate( namesDistanceCalculator, c1, c2 );

				else componentsAndStructuresMapping = BuiltInComponentsDistanceCalculator.calculate( namesDistanceCalculator, c1, c2, indexOfTransformedObjectsPath1, indexOfTransformedObjectsPath2 );
			}

			else {

				//...
			}
		}

		else{

			ComponentsAndStructuresMapping[][] allComponentsAndStructuresMappings = null;

			if( c1.getComponents() != null && c2.getComponents() != null ) allComponentsAndStructuresMappings = calculateComponentsAndStructuresDistancesAndMappings( namesDistanceCalculator, c1.getComponents(), c2.getComponents(), indexOfTransformedObjectsPath1, indexOfTransformedObjectsPath2 );


			if( allComponentsAndStructuresMappings == null ){

				PrintMessages.print( PrintMessages.mappingPrint, "Components/Structures distance: d (" + c1.getName() + ", " + c2.getName() + ") : there is no component mapping" );

				return null;
			}

			PrintMessages.print( PrintMessages.mappingPrint, "Components/Structures distance: d (" + c1.getName() + ", " + c2.getName() + ") = component mapping exists" );


			int size = 0;

			List<ComponentsAndStructuresMapping> componentsAndStructuresMappings = null;

			if( allComponentsAndStructuresMappings != null ){

				printDistancesAndMappings( allComponentsAndStructuresMappings );

				componentsAndStructuresMappings = selectBestPossibleMapping( allComponentsAndStructuresMappings );

				size += componentsAndStructuresMappings.size();
			}


			double sum = 0.0;

			for( int i = 0; componentsAndStructuresMappings != null && i < componentsAndStructuresMappings.size(); ++i ) sum += componentsAndStructuresMappings.get( i ).getDistance();

			double avg = sum / size; PrintMessages.print( PrintMessages.mappingPrint, "Components distance: avg (" + c1.getName() + ", " + c2.getName() + ") = " + avg );


			double NED = NamesDistanceCalculator.calculate( namesDistanceCalculator, c1.getName(), c2.getName() ); PrintMessages.print( PrintMessages.mappingPrint, "Components distance: NED (" + c1.getName() + ", " + c2.getName() + ") = " + NED );

			double distance = ( NED + avg ) / 2; PrintMessages.print( PrintMessages.mappingPrint, "Message distance: d (" + c1.getName() + ", " + c2.getName() + ") = " + distance );


			Object originalObject1 = null, originalObject2 = null;

			if( indexOfTransformedObjectsPath1 == null && indexOfTransformedObjectsPath2 == null ){

				originalObject1 = c1;

				originalObject1 = c2;
			}

			else{

				originalObject1 = getOriginalObject( c1, indexOfTransformedObjectsPath1 );

				originalObject2 = getOriginalObject( c2, indexOfTransformedObjectsPath2 );
			}


			componentsAndStructuresMapping = new ComponentsAndStructuresMapping();

			componentsAndStructuresMapping.add( originalObject1 );
			componentsAndStructuresMapping.add( originalObject2 );
			componentsAndStructuresMapping.set( distance );
			componentsAndStructuresMapping.set( componentsAndStructuresMappings );
		}


		return componentsAndStructuresMapping;
	}

	private static ComponentsAndStructuresMapping[][] calculateComponentsAndStructuresDistancesAndMappings( String namesDistanceCalculator, List<Component> components1, List<Component> components2, Map<String, Object> indexOfTransformedObjectsPath1, Map<String, Object> indexOfTransformedObjectsPath2 ){

		ComponentsAndStructuresMapping[][] allComponentsMappings = new ComponentsAndStructuresMapping[ components1.size() ][ components2.size() ];

		boolean allNull = true;


		for( int i = 0; i < components1.size(); ++i ){

			for( int j = 0; j < components2.size(); ++j ){

				allComponentsMappings[i][j] = calculateLocally( namesDistanceCalculator, components1.get( i ), components2.get( j ), indexOfTransformedObjectsPath1, indexOfTransformedObjectsPath2 );

				if( allComponentsMappings[i][j] != null ) allNull = false;
			}
		}

		if( allNull == true ) return null; 


		return allComponentsMappings;//This function returns null when all the operations do not match!
	}

	private static void printDistancesAndMappings( ComponentsAndStructuresMapping[][] allComponentsMappings ){

		if( allComponentsMappings != null ){

			int rows = allComponentsMappings.length;
			int cols = allComponentsMappings[0].length;

			for( int i = 0; i < rows; ++i )

				for( int j = 0; j < cols; ++j )

					if( allComponentsMappings[ i ][ j ] != null && allComponentsMappings[ i ][ j ].getComponentsAndStructureMapping() != null ) PrintMessages.print( PrintMessages.mappingPrint, "componentsMappings = (" + XMLSchemaRepresentationController.getTag( allComponentsMappings[ i ][ j ].getComponentsAndStructureMapping().get( 0 ) ) + ", " + XMLSchemaRepresentationController.getTag( allComponentsMappings[ i ][ j ].getComponentsAndStructureMapping().get( 1 ) ) + ") = " + allComponentsMappings[i][j].getDistance() );
		}
	}

	private static List<ComponentsAndStructuresMapping> selectBestPossibleMapping( ComponentsAndStructuresMapping[][] allComponentsMappings ){

		int rows = allComponentsMappings.length;
		int cols = allComponentsMappings[0].length;

		int maxDimension = Math.max( rows, cols );

		double[][] allComponentsDistances = new double[rows][cols];//I construct the table with the distances in order to give it as input to the hungarian algorithm!

		for( int i = 0; i < rows; ++i ){

			for( int j = 0; j < cols; ++j ){

				if( allComponentsMappings[i][j] == null ) allComponentsDistances[i][j] = (double) (maxDimension + 1.0);//I want this value to be excluded by hungarian!

				else allComponentsDistances[i][j] = allComponentsMappings[i][j].getDistance();
			}
		}

		int[][] assignment = AssignmentProblemSolver.solve( AssignmentProblemController.getHungarianAlgorithm(), allComponentsDistances );


		if( assignment == null ) return null;


		List<ComponentsAndStructuresMapping> componentsMapping = new ArrayList<ComponentsAndStructuresMapping>();//I manipulate the output of the hungarian output!

		if( rows > cols ){
			for ( int i = 0; i < assignment.length; i++ ){
				if( allComponentsMappings[ assignment[i][1] ][ assignment[i][0] ] != null )//I exclude the cases of the non-matched operations: mapping = null, but hungarian includes them in its output!
					componentsMapping.add( allComponentsMappings[ assignment[i][1] ][ assignment[i][0] ] );
			}
		}

		else if( rows <= cols ){
			for (int i = 0; i < assignment.length; i++){
				if( allComponentsMappings[ assignment[i][0] ][ assignment[i][1] ] != null )//I exclude the cases of the non-matched types: mapping = null, but hungarian includes them in its output!
					componentsMapping.add( allComponentsMappings[ assignment[i][0] ][ assignment[i][1] ] );
			}
		}

		//for (int i = 0; i < operationsMappings.size(); i++)
		//	JOptionPane.showMessageDialog(null, "Operation mapping: d(" + operationsMappings.get(i).getOperations().get(0).getName() + ", " + operationsMappings.get(i).getOperations().get(1).getName() + ") = " + operationsMappings.get(i).getDistance());

		return componentsMapping;//It cannot be null because the hungarian accepts as input at least one mapping that has distance < 1.
	}


	private static Object getOriginalObject( Object comparedObject, Map<String, Object> indexOfTransformedObjectsPath ){

		Object originalObject = null;


		if( comparedObject != null ){

			String schemaPath = XMLSchemaRepresentationController.getSchemaPath( comparedObject );

			if( schemaPath != null ) originalObject = indexOfTransformedObjectsPath.get( schemaPath );
		}


		return originalObject;
	}
}
