package org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.hungarianAlgorithm;

import java.util.ArrayList;
import java.util.List;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.DistanceCalculatorController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.OperationsDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.OperationsMapping;
import org.ow2.choreos.assignmentProblem.AssignmentProblemController;
import org.ow2.choreos.assignmentProblem.AssignmentProblemSolver;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


public class InterfacesDistanceCalculatorHungarianAlgorithm {

	public static InterfacesMapping calculate( String namesDistanceCalculator, String schemaRepresentationModel, String abstractionDepth, Double similarityLintel, ServiceInterface i1, ServiceInterface i2 ){

		PrintMessages.print( PrintMessages.mappingPrint, "Interface distance: d (" + i1.getName() + ", " + i2.getName() + ")" );

		InterfacesMapping interfacesMapping = null;
		
		double distance = Double.MAX_VALUE;
		
		if(!abstractionDepth.equals("Interfaces")){
			OperationsMapping[][] allOperationsMappings = calculateDistancesAndMappings( namesDistanceCalculator, schemaRepresentationModel, abstractionDepth, similarityLintel, i1.getOperations(), i2.getOperations() );
	
	
			if( allOperationsMappings == null ){
	
				PrintMessages.print( PrintMessages.mappingPrint, "Interface distance: d (" + i1.getName() + ", " + i2.getName() + ") : there is no operation mapping!" );
	
				return null;
			}
	
			printDistancesAndMappings( allOperationsMappings );
	
	
			PrintMessages.print( PrintMessages.mappingPrint, "Interface distance: d (" + i1.getName() + ", " + i2.getName() + ") : exists operation mapping!" );
	
	
			List<OperationsMapping> operationsMappings = selectBestPossibleMapping( allOperationsMappings );
	
	
			double sum = 0.0;
	
			for( int i = 0; i < operationsMappings.size(); ++i ) sum += operationsMappings.get( i ).getDistance();
	
			double avg = sum / operationsMappings.size();
	
	
			double NED = NamesDistanceCalculator.calculate( namesDistanceCalculator, i1.getName(), i2.getName() );
	
	
			distance = ( NED + avg ) / 2;
	
			PrintMessages.print( PrintMessages.mappingPrint, "Interface distance: NED (" + i1.getName() + ", " + i2.getName() + ") = " + NED );
			PrintMessages.print( PrintMessages.mappingPrint, "Interface distance: avg (" + i1.getName() + ", " + i2.getName() + ") = " + avg );
			PrintMessages.print( PrintMessages.mappingPrint, "Interface distance: d (" + i1.getName() + ", " + i2.getName() + ") = " + distance );
			
			interfacesMapping = new InterfacesMapping();
			interfacesMapping.add( i1 );
			interfacesMapping.add( i2 );
			interfacesMapping.set( distance );
			interfacesMapping.set( operationsMappings );
		}
		else{
			distance = NamesDistanceCalculator.calculate( namesDistanceCalculator, i1.getName(), i2.getName() );
		    
		    interfacesMapping = new InterfacesMapping();
			interfacesMapping.add( i1 );
			interfacesMapping.add( i2 );
		    interfacesMapping.set( distance );
		    interfacesMapping.set( null );
	
		}
		
		if(distance >= similarityLintel) return null;
		else {
			System.out.println("d: " + distance + ", I: " + interfacesMapping.getServiceInterfaces().get(0).getName() + " - " + interfacesMapping.getServiceInterfaces().get(1).getName());
			return interfacesMapping;
		}
	}

	private static OperationsMapping[][] calculateDistancesAndMappings( String namesDistanceCalculator, String schemaRepresentationModel, String abstractionDepth, Double similarityLintel, List<Operation> operations1, List<Operation> operations2 ){

		OperationsMapping[][] allOperationsMappings = new OperationsMapping[ operations1.size() ][ operations2.size() ];

		boolean allNull = true;


		for(int i = 0; i < operations1.size(); ++i){

			for(int j = 0; j < operations2.size(); ++j){

				allOperationsMappings[i][j] = OperationsDistanceCalculator.calculate( DistanceCalculatorController.getHungarianAlgorithm(), namesDistanceCalculator, schemaRepresentationModel, abstractionDepth, similarityLintel, operations1.get(i), operations2.get(j) );

				if( allOperationsMappings[i][j] != null ) allNull = false;
			}
		}

		if( allNull == true ) return null; 


		return allOperationsMappings;//This function returns null when all the operations do not match!
	}

	private static void printDistancesAndMappings( OperationsMapping[][] allOperationsMappings ){

		if( allOperationsMappings != null ){

			int rows = allOperationsMappings.length;
			int cols = allOperationsMappings[0].length;

			for( int i = 0; i < rows; ++i )

				for( int j = 0; j < cols; ++j )

					if( allOperationsMappings[ i ][ j ] != null && allOperationsMappings[ i ][ j ].getOperations() != null ) PrintMessages.print( PrintMessages.mappingPrint, "operationMapping = (" + allOperationsMappings[ i ][ j ].getOperations().get( 0 ).getTag() + ", " + allOperationsMappings[ i ][ j ].getOperations().get( 1 ).getTag() + ") = " + allOperationsMappings[i][j].getDistance() );
		}
	}

	private static List<OperationsMapping> selectBestPossibleMapping( OperationsMapping[][] allOperationsMappings ){

		int rows = allOperationsMappings.length;
		int cols = allOperationsMappings[0].length;

		int maxDimension = Math.max( rows, cols );

		double[][] allOperationsDistances = new double[rows][cols];//I construct the table with the distances in order to give it as input to the hungarian algorithm!

		for(int i = 0; i < rows; ++i){

			for(int j = 0; j < cols; ++j){

				if( allOperationsMappings[i][j] == null ) allOperationsDistances[i][j] = (double) (maxDimension + 1.0);//I want this value to be excluded by hungarian!

				else allOperationsDistances[i][j] = allOperationsMappings[i][j].getDistance();
			}
		}

		int[][] assignment = AssignmentProblemSolver.solve( AssignmentProblemController.getHungarianAlgorithm(), allOperationsDistances );


		if( assignment == null ) return null;

		List<OperationsMapping> operationsMappings = new ArrayList<OperationsMapping>();//I manipulate the output of the hungarian output!

		if( rows > cols ){
			for (int i = 0; i < assignment.length; i++){
				if( allOperationsMappings[ assignment[i][1] ][ assignment[i][0] ] != null )//I exclude the cases of the non-matched operations: mapping = null, but hungarian includes them in its output!
					operationsMappings.add( allOperationsMappings[ assignment[i][1] ][ assignment[i][0] ] );
			}
		}

		else if( rows <= cols ){
			for (int i = 0; i < assignment.length; i++){
				if( allOperationsMappings[ assignment[i][0] ][ assignment[i][1] ] != null )//I exclude the cases of the non-matched types: mapping = null, but hungarian includes them in its output!
					operationsMappings.add( allOperationsMappings[ assignment[i][0] ][ assignment[i][1] ] );
			}
		}

		//for (int i = 0; i < operationsMappings.size(); i++)
		//	JOptionPane.showMessageDialog(null, "Operation mapping: d(" + operationsMappings.get(i).getOperations().get(0).getName() + ", " + operationsMappings.get(i).getOperations().get(1).getName() + ") = " + operationsMappings.get(i).getDistance());

		return operationsMappings;//It cannot be null because the hungarian accepts as input at least one mapping that has distance < 1.
	}
}
