package org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.hungarianAlgorithm;

import java.util.Map;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.schema.ComponentsDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.schema.ComponentsDistanceCalculatorController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.BuiltInMessageTypesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ReferencedComponent;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;


public class MessageTypesDistanceCalculatorHungarianAlgorithm {


	//Methods.
	public static MessageTypesMapping calculate( String namesDistanceCalculator, String targetSchemaRepresentationModel, MessageType mt1, MessageType mt2 ){

		MessageTypesMapping messageTypesMapping = null;

		if( mt1.getReferencedComponent().isBuiltInType() && mt2.getReferencedComponent().isBuiltInType() ) messageTypesMapping = BuiltInMessageTypesDistanceCalculator.calculate( namesDistanceCalculator, targetSchemaRepresentationModel, mt1, mt2 );

		else if( ! mt1.getReferencedComponent().isBuiltInType() && ! mt2.getReferencedComponent().isBuiltInType() ) {

			ReferencedComponent rc1 = mt1.getReferencedComponent();

			ReferencedComponent rc2 = mt2.getReferencedComponent();


			String primarySchemaRepresentationModel = null;

			String primarySchemaRepresentationModel1 = rc1.getSchemaRepresentationModel();

			String primarySchemaRepresentationModel2 = rc1.getSchemaRepresentationModel();


			if( ! primarySchemaRepresentationModel1.equals( primarySchemaRepresentationModel2 ) ){

				PrintMessages.print( PrintMessages.mappingPrint, "Message types distance: d (" + mt1.getTag() + ", " + mt2.getTag() + ") : there is no message type mapping because their referenced components are represented in different model." );

				return null;
			}

			else primarySchemaRepresentationModel = primarySchemaRepresentationModel1;


			Component component1 = mt1.getReferencedComponent().getMaterializedComponent( targetSchemaRepresentationModel );//Because of the message types have been transformed, the components have been materialized and are not equal to null.

			Component component2 = mt2.getReferencedComponent().getMaterializedComponent( targetSchemaRepresentationModel );


			ComponentsAndStructuresMapping componentsMapping = null;

			if( primarySchemaRepresentationModel.equals( targetSchemaRepresentationModel ) ) componentsMapping = ComponentsDistanceCalculator.calculate( ComponentsDistanceCalculatorController.getHungarianAlgorithm(), namesDistanceCalculator, component1, component2 );//In this case, the model in which the organization is performed is the same with the original model.

			else{

				Map<String, Object> indexOfTransformedObjectsPath1 = rc1.getIndexOfMaterializedComponentsPath( targetSchemaRepresentationModel );

				Map<String, Object> indexOfTransformedObjectsPath2 = rc2.getIndexOfMaterializedComponentsPath( targetSchemaRepresentationModel );

				MapPrinting.stdPrintObjectsIndex( PrintMessages.mappingPrint, indexOfTransformedObjectsPath1, "indexOfTransformedObjectsPath1" );

				MapPrinting.stdPrintObjectsIndex( PrintMessages.mappingPrint, indexOfTransformedObjectsPath2, "indexOfTransformedObjectsPath2" );


				componentsMapping = ComponentsDistanceCalculator.calculate( ComponentsDistanceCalculatorController.getHungarianAlgorithm(), namesDistanceCalculator, component1, component2, indexOfTransformedObjectsPath1, indexOfTransformedObjectsPath2 );
			}


			if( componentsMapping == null ){

				PrintMessages.print( PrintMessages.mappingPrint, "Message types distance: d (" + mt1.getTag() + ", " + mt2.getTag() + ") : there is no message type mapping." );

				return null;
			}


			PrintMessages.print( PrintMessages.mappingPrint, "Message types distance: d (" + mt1.getTag() + ", " + mt2.getTag() + ") : message type mapping exists." );

			double NED = NamesDistanceCalculator.calculate( namesDistanceCalculator, mt1.getTag(), mt2.getTag() ); PrintMessages.print( PrintMessages.mappingPrint, "Message type distance: NED (" + mt1.getTag() + ", " + mt2.getTag() + ") = " + NED );

			double distance = ( NED + componentsMapping.getDistance() ) / 2; PrintMessages.print( PrintMessages.mappingPrint, "Message type distance: d (" + mt1.getTag() + ", " + mt2.getTag() + ") = " + distance );


			messageTypesMapping = new MessageTypesMapping();

			messageTypesMapping.add( mt1 );
			messageTypesMapping.add( mt2 );
			messageTypesMapping.set( distance );
			messageTypesMapping.set( targetSchemaRepresentationModel );
			messageTypesMapping.set( componentsMapping );
		}


		return messageTypesMapping;
	}
}
