package org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.hungarianAlgorithm;

import java.util.ArrayList;
import java.util.List;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.DistanceCalculatorController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.MessageTypesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessagesMapping;
import org.ow2.choreos.assignmentProblem.AssignmentProblemController;
import org.ow2.choreos.assignmentProblem.AssignmentProblemSolver;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;


public class MessagesDistanceCalculatorHungarianAlgorithm {


	public static MessagesMapping calculate( String namesDistanceCalculator, String schemaRepresentationModel, String abstractionDepth, Double similarityLintel, Message m1, Message m2 ){//This function returns: (a) trivial mapping 'messagesMapping' (d = 0.0, typesMappings = null), (b) not match ('messagesMapping' = null), (c) match ('messagesMapping' != null).

		if( m1 == null || m2 == null ){

			PrintMessages.print( PrintMessages.mappingPrint, "Message distance: trivial message mapping without messages." );

			MessagesMapping messagesMapping = new MessagesMapping();

			messagesMapping.add( null );
			messagesMapping.add( null );
			messagesMapping.set( 0.0 );

			return messagesMapping;
		}


		List<MessageType> messageTypes1 = m1.getMessageTypes(); print( m1 );

		List<MessageType> messageTypes2 = m2.getMessageTypes(); print( m2 );
		
		double distance = Double.MAX_VALUE;
		MessagesMapping messagesMapping = null;
		
		if(!abstractionDepth.equals("Messages")){
			if( messageTypes1 == null || messageTypes2 == null ) return createTrivialMapping( namesDistanceCalculator, m1, m2 );//This is the case that one or both of the messages are void and the mapping is trivial!
	
			else{//This is the case in which both messages have types but we do not know if these types match!
	
				List<MessageType> builtInMessageTypes1 = new ArrayList<MessageType>(), nonBuiltInMessageTypes1 = new ArrayList<MessageType>(), builtInMessageTypes2 = new ArrayList<MessageType>(), nonBuiltInMessageTypes2 = new ArrayList<MessageType>();
	
				splitMessageTypesInBuiltInAndNonBuiltIn( messageTypes1, builtInMessageTypes1, nonBuiltInMessageTypes1 ); print( true, m1, builtInMessageTypes1 ); print( false, m1, nonBuiltInMessageTypes1 );
	
				splitMessageTypesInBuiltInAndNonBuiltIn( messageTypes2, builtInMessageTypes2, nonBuiltInMessageTypes2 ); print( true, m2, builtInMessageTypes2 ); print( false, m2, nonBuiltInMessageTypes2 );
	
	
				MessageTypesMapping[][] allBuiltInMessageTypesMappings = null, allNonBuiltInMessageTypesMappings = null;
	
				if( builtInMessageTypes1 != null && builtInMessageTypes2 != null ) allBuiltInMessageTypesMappings = calculateDistancesAndMappings( namesDistanceCalculator, schemaRepresentationModel, builtInMessageTypes1, builtInMessageTypes2 );
	
				if( nonBuiltInMessageTypes1 != null && nonBuiltInMessageTypes2 != null ) allNonBuiltInMessageTypesMappings = calculateDistancesAndMappings( namesDistanceCalculator, schemaRepresentationModel, nonBuiltInMessageTypes1, nonBuiltInMessageTypes2 );
	
	
				if( allBuiltInMessageTypesMappings == null && allNonBuiltInMessageTypesMappings == null ){
	
					PrintMessages.print( PrintMessages.mappingPrint, "Message distance: d (" + m1.getName() + ", " + m2.getName() + ") = there is no message mapping" );
	
					return null;
				}
	
				PrintMessages.print( PrintMessages.mappingPrint, "Message distance: d (" + m1.getName() + ", " + m2.getName() + ") = non trivial message mapping" );
	
	
				List<MessageTypesMapping> builtInMessageTypesMappings = null, nonBuiltInMessageTypesMappings = null;
	
				if( allBuiltInMessageTypesMappings != null ) builtInMessageTypesMappings = selectBestPossibleMapping( allBuiltInMessageTypesMappings );
	
				if( allNonBuiltInMessageTypesMappings != null ) nonBuiltInMessageTypesMappings = selectBestPossibleMapping( allNonBuiltInMessageTypesMappings );
	
	
				List<MessageTypesMapping> messageTypesMappings = merge( builtInMessageTypesMappings, nonBuiltInMessageTypesMappings );
	
	
				double sum = 0.0;
	
				for( int i = 0; i < messageTypesMappings.size(); ++i ) sum += messageTypesMappings.get( i ).getDistance();
	
				double avg = sum / messageTypesMappings.size(); PrintMessages.print( PrintMessages.mappingPrint, "Message distance: avg (" + m1.getName() + ", " + m2.getName() + ") = " + avg );
	
	
				double NED = NamesDistanceCalculator.calculate( namesDistanceCalculator, m1.getName(), m2.getName() ); PrintMessages.print( PrintMessages.mappingPrint, "Message distance: NED (" + m1.getName() + ", " + m2.getName() + ") = " + NED );
	
				if( NED == 1.0 ){
	
					PrintMessages.print( PrintMessages.mappingPrint, "Message distance: d (" + m1.getName() + ", " + m2.getName() + ") = there is no message mapping (There is no common message name)." );
	
					return null;
				}
	
				distance = ( NED + avg ) / 2; PrintMessages.print( PrintMessages.mappingPrint, "Message distance: d (" + m1.getName() + ", " + m2.getName() + ") = " + distance );
	
	
				messagesMapping = new MessagesMapping();
	
				messagesMapping.add( m1 );
				messagesMapping.add( m2 );
				messagesMapping.set( distance );
				messagesMapping.set( messageTypesMappings );

			}
		}
		else {
			distance = NamesDistanceCalculator.calculate( namesDistanceCalculator, m1.getName(), m2.getName() );
			
			messagesMapping = new MessagesMapping();
		
			messagesMapping.add( m1 );
			messagesMapping.add( m2 );
			messagesMapping.set( distance );
			messagesMapping.set( null );
			
		}
		
		if(distance >= similarityLintel) return null;
		else return messagesMapping;
	}

	private static MessagesMapping createTrivialMapping( String namesDistanceCalculator, Message m1, Message m2 ){

		double NED = NamesDistanceCalculator.calculate( namesDistanceCalculator, m1.getName(), m2.getName() );//The distance equals to the NED because ND = 0.0!

		PrintMessages.print( PrintMessages.mappingPrint, "Message distance: d (" + m1.getName() + ", " + m2.getName() + ") = trivial message mapping with messages but without types = " + NED );


		if( NED == 1.0 ) return null;//The trivial message mapping has NED = 1.0, so the messages do not match at all!

		else{

			MessagesMapping messagesMapping = new MessagesMapping();

			messagesMapping.add( m1 );
			messagesMapping.add( m2 );
			messagesMapping.set( NED );
			messagesMapping.set( null );

			return messagesMapping;
		}
	}

	private static void splitMessageTypesInBuiltInAndNonBuiltIn( List<MessageType> messageTypes, List<MessageType> builtInMessageTypes, List<MessageType> nonBuiltInMessageTypes ){

		for( int i = 0; messageTypes != null && i < messageTypes.size(); ++i ){//Splitting the message types of the message1 in built-in and non-built-in.

			if( messageTypes.get( i ).getReferencedComponent().isBuiltInType() ) builtInMessageTypes.add( messageTypes.get( i ) );

			else nonBuiltInMessageTypes.add( messageTypes.get( i ) );
		}
	}

	private static void print( boolean builtIn, Message m, List<MessageType> messageTypes ){

		String msg = builtIn ? "builtInMessageTypes" : "nonBuiltInMessageTypes";

		PrintMessages.print( PrintMessages.mappingPrint, msg + " of " + m.getTag() );

		for( int i = 0; messageTypes != null && i < messageTypes.size(); ++i ) PrintMessages.print( PrintMessages.mappingPrint, messageTypes.get( i ).getTag() );
	}

	private static void print( Message m ){

		PrintMessages.print( PrintMessages.mappingPrint, "messageTypes of " + m.getTag() );

		for( int i = 0; m.getMessageTypes() != null && i < m.getMessageTypes().size(); ++i ) PrintMessages.print( PrintMessages.mappingPrint, m.getMessageTypes().get( i ).getTag() );
	}

	private static List<MessageTypesMapping> merge( List<MessageTypesMapping> messageTypes1, List<MessageTypesMapping> messageTypes2 ){

		if( messageTypes1 != null && messageTypes2 != null ){

			messageTypes1.addAll( messageTypes2 );

			return messageTypes1;
		}

		else if( messageTypes1 == null && messageTypes2 != null ) return messageTypes2;

		else if( messageTypes1 != null && messageTypes2 == null ) return messageTypes1;

		else return null;
	}

	private static MessageTypesMapping[][] calculateDistancesAndMappings( String namesDistanceCalculator, String schemaRepresentationModel, List<MessageType> messageTypes1, List<MessageType> messageTypes2 ){

		MessageTypesMapping[][] allMessageTypesMappings = new MessageTypesMapping[ messageTypes1.size() ][ messageTypes2.size() ];

		boolean allNull = true;

		for( int i = 0; i < messageTypes1.size(); ++i ){

			for( int j = 0; j < messageTypes2.size(); ++j ){

				allMessageTypesMappings[i][j] = MessageTypesDistanceCalculator.calculate( DistanceCalculatorController.getHungarianAlgorithm(), namesDistanceCalculator, schemaRepresentationModel, messageTypes1.get(i), messageTypes2.get(j) );

				if( allMessageTypesMappings[i][j] != null )  allNull = false;  
			}
		}

		if( allNull == true ) return null; 


		return allMessageTypesMappings;//This function returns null when all the types do not match!
	}

	private static List<MessageTypesMapping> selectBestPossibleMapping( MessageTypesMapping[][] allMessageTypesMappings  ){

		int rows = allMessageTypesMappings.length;
		int cols = allMessageTypesMappings[0].length;

		int maxDimension = Math.max( rows, cols );

		double[][] allMessageTypesDistances = new double[rows][cols];//I construct the table with the distances in order to give it as input to the hungarian algorithm!

		for(int i = 0; i < allMessageTypesMappings.length; ++i){

			for(int j = 0; j < allMessageTypesMappings[0].length; ++j){

				if( allMessageTypesMappings[i][j] == null ) allMessageTypesDistances[i][j] = (double) (maxDimension + 1.0);//I want this value to be excluded by hungarian!

				else allMessageTypesDistances[i][j] = allMessageTypesMappings[i][j].getDistance();
			}
		}


		int[][] assignment = AssignmentProblemSolver.solve( AssignmentProblemController.getHungarianAlgorithm(), allMessageTypesDistances );


		List<MessageTypesMapping> messageTypesMapping = new ArrayList<MessageTypesMapping>();//I manipulate the output of the hungarian output!


		if( rows > cols ){
			for (int i = 0; i < assignment.length; i++){
				if( allMessageTypesMappings[ assignment[i][1] ][ assignment[i][0] ] != null )//I exclude the cases of the non-matched types: mapping = null, but hungarian includes them in its output!
					messageTypesMapping.add( allMessageTypesMappings[ assignment[i][1] ][ assignment[i][0] ] );
			}
		}

		else if( rows <= cols ){
			for (int i = 0; i < assignment.length; i++){
				if( allMessageTypesMappings[ assignment[i][0] ][ assignment[i][1] ] != null )//I exclude the cases of the non-matched types: mapping = null, but hungarian includes them in its output!
					messageTypesMapping.add( allMessageTypesMappings[ assignment[i][0] ][ assignment[i][1] ] );
			}
		}

		//for (int i = 0; i < typesMapping.size(); i++)

			//PopupMessages.organizationPrint( "Message mapping: d(" + typesMapping.get(i).getTypes().get(0).getName() + ", " + typesMapping.get(i).getTypes().get(1).getName() + ") = "  +typesMapping.get(i).getDistance() );


		return messageTypesMapping;//It cannot be null because the hungarian accepts as input at least one mapping that has distance < 1.
	}
}
