package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service;

import java.util.ArrayList;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.measurements.number.DecimalPrecision;
import org.ow2.choreos.serviceRepresentation.functional.Message;


@SuppressWarnings("serial")
public class MessagesMapping implements java.io.Serializable {


	//Variables.
	private double distance;

	private List<Message> messages;//It may be null in case two messages are totally empty!

	private List<MessageTypesMapping> messageTypesMappings;

	private boolean trivial = false;
	private boolean trivialWithoutMessages = false;

	private int dbAbstractionKey;

	private String inOrOut;


	//Empty constructor used for xml transformation
	public MessagesMapping(){
				
	}
	
	//Setters.
	public void add( Message m ){

		if( m == null ) trivialWithoutMessages = true;

		else{

			if( messages == null ) messages = new ArrayList<Message>();

			messages.add( m );
		}
	}

	public void set( double cDistance ){

		distance = cDistance;
	}

	public void set( List<MessageTypesMapping> cMessageTypesMappings ){

		if( cMessageTypesMappings == null ) trivial = true;

		else messageTypesMappings = cMessageTypesMappings;
	}

	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}

	public void setMessages(List<Message> representedMessages){

		messages = representedMessages;
	}
	
	public void setInOrOut( String cInOrOut ){

		inOrOut = cInOrOut;
	}


	//Getters.
	public List<Message> getMessages(){

		return messages;
	}

	public double getDistance(){

		return distance;
	}
	
	public int getDBAbstractionKey(){
		
		return dbAbstractionKey;
	}
	
	public String getType(){

		return inOrOut;
	}

	public List<MessageTypesMapping> getMessageTypesMapping(){

		return messageTypesMappings;
	}

	public boolean isTrivial(){

		return trivial;
	}

	public boolean isTrivialWithoutMessages(){

		return trivialWithoutMessages;
	}

	public Message getMessage( String messageName ){//It retrieves the message object which has name the parameter 'messageName'.

		Message message = null;


		for( int i = 0; messages != null && i < messages.size(); ++i ){

			if( messages.get( i ).getName().equals( messageName ) ){

				message = messages.get( i );

				break;
			}
		}


		return message;
	}

	public Message getMappedMessage( String messageName ){//It returns randomly one of them mapped messages.

		Message message = null;

		List<Integer> candidateIndices = null;


		boolean exist = false;

		for( int i = 0; messages != null && i < messages.size(); ++i ){

			if( messages.get( i ).getName().equals( messageName ) ) exist = true;

			else{

				if( candidateIndices == null ) candidateIndices = new ArrayList<Integer>();

				candidateIndices.add( i );
			}
		}


		if( exist ){

			int randomIndex = ListGetters.getElementRandomly( candidateIndices );

			message = messages.get( randomIndex ) ;
		}


		return message;
	}


	//Visualization.
	DefaultMutableTreeNode visualize( DefaultMutableTreeNode node ){

		if( trivialWithoutMessages ){

			DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Trivial message mapping (undefined messages)" );

			node.add( child1 );

			return node;
		}

		for( int k = 0; messages != null && k < messages.size(); ++k ){

			Integer label = k + 1;

			DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Message (" + label + "): " + messages.get( k ).getName() );

			node.add( child1 );
		}


		DefaultMutableTreeNode child2 = new DefaultMutableTreeNode( "Distance = " + DecimalPrecision.setFourDecimalsPrecisionHalfDown( distance ) );

		node.add( child2 );


		if( trivial ){

			DefaultMutableTreeNode child3 = new DefaultMutableTreeNode("Trivial message types mapping (undefined message types)");
			node.add( child3 );

			return node;
		}

		DefaultMutableTreeNode child3 = new DefaultMutableTreeNode( "Message types mapping" );

		node.add( child3 );

		for( int k = 0; k < messageTypesMappings.size(); ++k ){

			Integer label = k + 1;

			child3 = messageTypesMappings.get( k ).visualize( label, child3 );
		}

    	return node;
	}
	
	//Setters used for xml transformation
	
	public void setxDistance( double cDistance ){
		distance = cDistance;
	}
			
	public void setxMessages( List<Message> cMessages ){
		messages = cMessages;
	}
			
	public void setxMessageTypesMappings( List<MessageTypesMapping> cMessageTypesMappings ){
		messageTypesMappings = cMessageTypesMappings;
	}
			
	public void setxTrivial( boolean cTrivial ){
		trivial = cTrivial;
	}
			
	public void setxTrivialWithoutMessages( boolean cTrivialWithoutMessages ){
		trivialWithoutMessages = cTrivialWithoutMessages;
	}
			
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
			
	public void setxInOrOut( String cInOrOut ){
		inOrOut = cInOrOut;
	}
			
	//Getters used for xml transformation
			
	public double getxDistance(){
		return distance;
	}
			
	public List<Message> getxMessages(){
		return messages;
	}
			
	public List<MessageTypesMapping> getxMessageTypesMappings(){
		return messageTypesMappings;
	}
			
	public boolean getxTrivial(){
		return trivial;
	}
			
	public boolean getxTrivialWithoutMessages(){
		return trivialWithoutMessages;
	}
			
	public int getxDBAbstractionKey(){
		return dbAbstractionKey;
	}
			
	public String getxInOrOut(){
		return inOrOut;
	}
}
