package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service;

import java.util.ArrayList;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.measurements.number.DecimalPrecision;
import org.ow2.choreos.serviceRepresentation.functional.Operation;


@SuppressWarnings("serial")
public class OperationsMapping implements java.io.Serializable {


	//Variables.
	private double distance;

	private MessagesMapping inputMappings;//This cannot be null because in this case the operations do not match!
	private MessagesMapping outputMappings;//This cannot be null because in this case the operations do not match!

	private List<Operation> operations;

	private int dbAbstractionKey;


	//Empty constructor used for xml transformation
	public OperationsMapping(){
				
	}
	
	//Setters.
	public void add( Operation op ){

		if( operations == null ) operations = new ArrayList<Operation>();

		operations.add( op );
	}

	public void set( double cDistance ){

		distance = cDistance;
	}

	public void setInputMappings( MessagesMapping inputMappings ){

		this.inputMappings = inputMappings;
	}

	public void setOutputMappings( MessagesMapping outputMappings ){

		this.outputMappings = outputMappings;
	}
	
	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}

	public void setOperations(ArrayList<Operation> representedOperations){
		
		operations = representedOperations;
	}

	//Getters.
	public List<Operation> getOperations(){

		return operations;
	}

	public double getDistance(){

		return distance;
	}

	public MessagesMapping getInputMappings(){

		return inputMappings;
	}

	public MessagesMapping getOutputMappings(){

		return outputMappings;
	}
	
	public int getDBAbstractionKey(){

		return dbAbstractionKey;
	}

	public Operation getOperation( String operationName ){//It retrieves the operation object which has name the parameter 'operationName'.

		Operation operation = null;


		for( int i = 0; operations != null && i < operations.size(); ++i ){

			if( operations.get( i ).getName().equals( operationName ) ){

				operation = operations.get( i );

				break;
			}
		}


		return operation;
	}

	public Operation getMappedOperation( String operationName ){//It returns randomly one of them mapped operations.

		Operation operation = null;

		List<Integer> candidateIndices = null;


		boolean exist = false;

		for( int i = 0; operations != null && i < operations.size(); ++i ){

			if( operations.get( i ).getName().equals( operationName ) ) exist = true;

			else{

				if( candidateIndices == null ) candidateIndices = new ArrayList<Integer>();

				candidateIndices.add( i );
			}
		}


		if( exist ){

			int randomIndex = ListGetters.getElementRandomly( candidateIndices );

			operation = operations.get( randomIndex ) ;
		}


		return operation;
	}


	//Visualization.
	DefaultMutableTreeNode visualize( Integer label, DefaultMutableTreeNode node ){

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Operation mapping (" + label + ")" );

		node.add( child1 );


		for( int k = 0; k < operations.size(); ++k ){

			label = k + 1;

			DefaultMutableTreeNode child11 = new DefaultMutableTreeNode( "Operation (" + label + "): " + operations.get( k ).getName() );

			child1.add( child11 );
		}


		DefaultMutableTreeNode child12 = new DefaultMutableTreeNode( "Distance = " + DecimalPrecision.setFourDecimalsPrecisionHalfDown( distance ) );

		child1.add( child12 );


		DefaultMutableTreeNode child13 = new DefaultMutableTreeNode( "Input messages mapping" );

		child1.add( child13 );

		inputMappings.visualize( child13 );


		DefaultMutableTreeNode child14 = new DefaultMutableTreeNode( "Output messages mapping" );

		child1.add( child14 );

		outputMappings.visualize( child14 );


    	return node;
	}
	
	//Setters used for xml transformation
	
	public void setxDistance( double cDistance ){
		distance = cDistance;
	}
			
	public void setxInputMappings( MessagesMapping cInputMappings ){
		inputMappings = cInputMappings;
	}
			
	public void setxOutputMappings( MessagesMapping cOutputMappings ){
		outputMappings = cOutputMappings;
	}
			
	public void setxOperations( List<Operation> cOperations ){
		operations = cOperations;
	}
			
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
			
	//Getters used for xml transformation
			
	public double getxDistance(){
		return distance;
	}
				
	public MessagesMapping getxInputMappings(){
		return inputMappings;
	}
				
	public MessagesMapping getxOutputMappings(){
		return outputMappings;
	}
				
	public List<Operation> getxOperations(){
		return operations;
	}
				
	public int getxDBAbstractionKey(){
		return dbAbstractionKey;
	}
}
