package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.interfaceMappingsToJTrees;

import java.util.ArrayList;
import java.util.Random;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;


public class TreeUtils {

    /**This indicates that the source tree will be used */
    public static final int SOURCE_TREE = 0;
    /**This indicates that the destination tree will be used */
    public static final int DESTINATION_TREE = 1;

    /**
     * Returns the path having as last component <code>aNode</code>
     *
     * @param aNode A TreeNode
     * @return A path containing <code>aNode</code> as last component.
     * Returns <code>null</code> if <code>aNode</code> is null
     */
    public static TreePath ConvertTreeNodeToPath(DefaultMutableTreeNode aNode) {
        if (aNode != null) {
            return new TreePath(aNode.getPath());
        } else {
            return null;
        }
    }

    /**
     * Calculates and returns the total height in pixels of all the visible
     * descendants of <code>aNode</code>.
     * <strong>Warning:</strong> <code>aNode</code> should belong to <code>aTree</code>
     * @param aTree An instance of <code>Jtree</code>
     * @param aNode An instance of <code>DefaultMutableTreeNode</code>
     * @return An integer indicating the total height
     */
    public static int getTotalDescendantsHeight(JTree aTree, DefaultMutableTreeNode aNode) {
        if ((aTree != null) && (aNode != null)) {
            int ret = 0;
            if (aTree.isExpanded((TreeUtils.ConvertTreeNodeToPath((DefaultMutableTreeNode) aNode)))) {
                for (int i = 0; i < aNode.getChildCount(); i++) {
                    ret += aTree.getPathBounds(TreeUtils.ConvertTreeNodeToPath((DefaultMutableTreeNode) aNode.getChildAt(i))).height;
                    ret += getTotalDescendantsHeight(aTree, (DefaultMutableTreeNode) aNode.getChildAt(i));
                }
            }
            return ret;
        } else {
            return 0;
        }
    }

    /**
     * Expands <strong>all</strong> nodes of <code>aTree</code>
     * @param aTree An instance of <code>JTree</code>
     */
    public static void ExpandJTree(JTree aTree) {
        for (int row = 0; row < aTree.getRowCount(); row++) {
            aTree.expandRow(row);
        }
    }

    /**
     * Collapses all nodes of <code>aTree</code> and all his descendants
     * @param aTree An instance of <code>JTree</code>
     */
    public static void CollapseJTree(JTree aTree) {
        for (int row = aTree.getRowCount() - 1; row > 0; row--) {
            aTree.collapseRow(row);
        }
    }

    /**
     * <strong>This is a test method.</strong>
     * Returns a random {@code JTree} that has maximum tree depth {@code maxDepth}
     * @param maxDepth An integer indicating the maximum tree depth
     * @return An instance of {@code JTree}
     */
    public static JTree randomJTree(int maxDepth,int maxChildCount) {
        DefaultMutableTreeNode aNode = randomTreeNode(maxDepth,maxChildCount);
        JTree tree = new JTree(new DefaultTreeModel(aNode));
        ExpandJTree(tree);
        GenerateNodeNames(tree);
        return tree;
    }

    /**
     * <strong>This is a test method</strong>
     * Returns a random {@code DefaultMutableTreeNode} with random descendants.
     * Each node's name is numerical and it represents the number of its children
     * @param maxDepth An integer indicating the maximum node depth
     * @return An instance of {@code DefaultMutableTreeNode}
     */
    private static DefaultMutableTreeNode randomTreeNode(int maxDepth, int maxChildCount) {
        Random random = new Random();
        DefaultMutableTreeNode aNode = new DefaultMutableTreeNode(null);

        if (maxDepth > 1) {
            int childCount = random.nextInt(maxChildCount);
            for (int i = 0; i < childCount; i++) {
                aNode.add(randomTreeNode(maxDepth - 1,maxChildCount));
            }
        }
        return aNode;
    }

    private static void GenerateNodeNames(JTree tree) {
        for(int row =0; row<tree.getRowCount();row++)
            ((DefaultMutableTreeNode) tree.getPathForRow(row).getLastPathComponent()).setUserObject(Integer.toString(row));
    }
    
    /**
     * Finds and returns the {@link TreePath} in {@code tree} specified by given 
     * array of {@link DefaultMutableTreeNode} objects {@code nodes} 
     * @param tree An instance of {@link JTree}
     * @param nodes An array of DefaultMutableTreeNode objects characterizing the TreePath
     * @return A instance of {@code TreePath} from {@code tree} or null if 
     * no such TreePath exists
     */
    public static TreePath findTreePath(javax.swing.JTree tree, DefaultMutableTreeNode[] nodes){
        String[] nodeNames = new String[nodes.length];
        for(int i=0;i<nodes.length;i++){
            nodeNames[i] = nodes[i].getUserObject().toString();
        }
        return findTreePathByName(tree,nodeNames);
    }
    
    public static TreePath findTreePathByName(javax.swing.JTree tree, ArrayList<String> names) {
        TreeNode root = (TreeNode) tree.getModel().getRoot();
        return findTreePath(tree, new TreePath(root), names.toArray(), 0, true);
    }
    /**
     * Finds and returns the {@link TreePath} in {@code tree} specified by given 
     * array of String objects {@code names} 
     * @param tree An instance of {@link JTree}
     * @param names An array of String objects characterizing the TreePath
     * @return A instance of {@code TreePath} from {@code tree} or null if 
     * no such TreePath exists
     */
    public static TreePath findTreePathByName(javax.swing.JTree tree, String[] names) {
        TreeNode root = (TreeNode) tree.getModel().getRoot();
        return findTreePath(tree, new TreePath(root), names, 0, true);
    }

    @SuppressWarnings("unchecked")
	private static TreePath findTreePath(javax.swing.JTree tree, TreePath parent, Object[] nodes, int depth, boolean byName) {
        TreeNode node = (TreeNode) parent.getLastPathComponent();
        Object o = node;

        // If by name, convert node to a string
        if (byName) {
            o = o.toString();
        }

        // If equal, go down the branch
        if (o.equals(nodes[depth])) {
            // If at end, return match
            if (depth == nodes.length - 1) {
                return parent;
            }

            // Traverse children
            if (node.getChildCount() >= 0) {
                for (java.util.Enumeration e = node.children(); e.hasMoreElements();) {
                    TreeNode n = (TreeNode) e.nextElement();
                    TreePath path = parent.pathByAddingChild(n);
                    TreePath result = findTreePath(tree, path, nodes, depth + 1, byName);
                    // Found a match
                    if (result != null) {
                        return result;
                    }
                }
            }
        }

        // No match at this branch
        return null;
    }

}
