package org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.partitionalClustering.kMeans;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import org.ow2.choreos.abstractionBase.AbstractionBaseController;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstractionWithNestedFAbstractions;
import org.ow2.choreos.abstractionDrivenServiceOrganization.functionalOrganization.FunctionalOrganization;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroupsCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroupsCalculatorImplV1;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.PropertiesBasedGroupsCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.PropertiesBasedGroupsCalculatorImplV1;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.PropertyBasedGroup;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.gui.ServiceOrganizationVisualizer;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.gui.guiManipulation.JTreeManipulation;
import org.ow2.choreos.measurements.time.Delay;
import org.ow2.choreos.measurements.time.ElapsedTimeCalculator;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;


/**
 * 
 * @author dathanas
 *
 */
public class NonFunctionalOrganizationKmeans {


	//Setters.
	public static Integer produceNFH1( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, Integer functionalAbstractionID, List<ServiceInterface> interfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){//Mode = 1.

		Map<Integer, ServiceInstance> representedServiceInstances = findRepresentedServiceInstances( serviceRegistration, options.getCollectionID(), interfaces );

		if( representedServiceInstances == null ){

			PrintMessages.print( PrintMessages.organizationPrint, "The represented service instances are NOT characterized by non-functional descriptions!" );

			return 0;
		}


		//Pre-processing the input set of the service instances!
		QualityProperties Q = findCommonProperties( serviceRegistration, options.getCollectionID(), representedServiceInstances );//TODO: In this version, I assume that all the service instances provide the same set of properties!

		if( Q == null || ( Q != null && Q.getNumOfProperties() == 0 ) ){

			if( Q == null ) PrintMessages.print( PrintMessages.organizationPrint, "Quality properties = null" );

			PrintMessages.print( PrintMessages.organizationPrint, "The non-functional descriptions of the represented service instances do not provide common quality properties!" );

			return 0;
		}

		/*This functions will not be implemented because I will do the filtering on the fly!
		 * S = filterServiceInstances();
		 * L = filterAbstractDomains();
		 */

		List<NonFunctionalAbstraction> nonFunctionalAbstractions = new ArrayList<NonFunctionalAbstraction>();//The list of the produced non-functional abstractions!

		//Create a non-functional abstraction that describes all the service instances.
		Integer nonFunctionalAbstractionID = 0;

		PropertiesBasedGroupsCalculator propertiesBasedGroupsCalculator = new PropertiesBasedGroupsCalculatorImplV1( serviceRegistration );

		List<ArrayList<PropertyBasedGroup>> propertiesBasedGroups = propertiesBasedGroupsCalculator.produce( options.getDelay(), true, options.getCollectionID(), Q, representedServiceInstances, serviceOrganizationVisualizer );

		JointGroupsCalculator jointGroupsCalculator = new JointGroupsCalculatorImplV1( serviceOrganizationVisualizer );

		List<JointGroup> jointGroups = jointGroupsCalculator.produce( options.getDelay(), options.getCollectionID(), propertiesBasedGroups, serviceOrganizationVisualizer );

		for( int i = 0; i < jointGroups.size(); ++i ){//Even if the service instances do not provide values, I create joint groups!

			NonFunctionalAbstraction nonFunctionalAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), functionalAbstractionID, nonFunctionalAbstractionID, null, jointGroups.get(i) );//NULL is the ancestor ID!

			nonFunctionalAbstractions.add( nonFunctionalAbstraction );
		}

		Delay.delay( options.getDelay() );

		loadHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!
		
		Delay.delay( options.getDelay() );

		load2ndLevelProducedLabel( serviceOrganizationVisualizer, nonFunctionalAbstractionID );//GUI!


		do {

			//The candidate for division non-functional abstraction.
			NonFunctionalAbstraction candidateNonFunctionalAbstraction = nonFunctionalAbstractions.remove( nonFunctionalAbstractions.size() - 1);

			representedServiceInstances = candidateNonFunctionalAbstraction.getRepresentedServiceInstances();

			if( representedServiceInstances != null && representedServiceInstances.size() > abstractionBase.getThreshold( options.getCollectionID() ) ){

				Integer ancestorID = candidateNonFunctionalAbstraction.getNonFunctionalAbstractionID();

				propertiesBasedGroups = propertiesBasedGroupsCalculator.produce( options.getDelay(), false, options.getCollectionID(), Q, representedServiceInstances, serviceOrganizationVisualizer );

				jointGroups = jointGroupsCalculator.produce( options.getDelay(), options.getCollectionID(), propertiesBasedGroups, serviceOrganizationVisualizer );

				for( int i = 0; jointGroups != null && i < jointGroups.size(); ++i ){

					++nonFunctionalAbstractionID;

					NonFunctionalAbstraction nonFunctionalAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), functionalAbstractionID, nonFunctionalAbstractionID, ancestorID, jointGroups.get(i) );

					nonFunctionalAbstractions.add( nonFunctionalAbstraction );

					Delay.delay( options.getDelay() );

					loadHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!
				}

				Delay.delay( options.getDelay() );

				load2ndLevelProducedLabel( serviceOrganizationVisualizer, nonFunctionalAbstractionID );//GUI!
			}

		} while ( nonFunctionalAbstractions.size() > 0 );


		return nonFunctionalAbstractionID + 1;
	}

	private static Map<Integer, ServiceInstance> findRepresentedServiceInstances( ServiceRegistration serviceRegistration, Integer collectionID, List<ServiceInterface> interfaces ){

		Map<Integer, ServiceInstance> representedServiceInstances = null;

		for( int i = 0; interfaces != null && i < interfaces.size(); ++i ){

			ServiceInterface tmpInterface = interfaces.get( i ); 

			for( int j = 0; tmpInterface.getServiceInstances() != null && j < tmpInterface.getServiceInstances().size(); ++j ){

				ServiceInstance serviceInstance = tmpInterface.getServiceInstances().get( j );

				if( serviceInstance.hasNonFunctionalDescription( serviceRegistration.getRef2SourceOfInterface( collectionID, tmpInterface ) ) ){

					if( representedServiceInstances == null ) representedServiceInstances = new Hashtable<Integer, ServiceInstance>();

					representedServiceInstances.put( representedServiceInstances.size(), serviceInstance );
				}
			}
		}

		return representedServiceInstances;
	}


	private static QualityProperties findCommonProperties( ServiceRegistration serviceRegistration, Integer collectionID, Map<Integer, ServiceInstance> representedServiceInstances ){
		//TODO: I have to take from each instance the properties that it provides and then to find the intersection of them!

		//TODO: Here, I assume that all sources and all service instances provide the same set of properties!
		//So, I take only the properties from the first source, arbitrarily!

		return representedServiceInstances.get(0).getQualityProperties( serviceRegistration.getRef2SourceOfServiceInstance( collectionID, representedServiceInstances.get(0) ) );
	}


	private static void load2ndLevelProducedLabel( ServiceOrganizationVisualizer serviceOrganizationVisualizer, int numOfNFA ){

		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText("#nested non-functional abstractions: " + numOfNFA);
		serviceOrganizationVisualizer.repaint();
	}

	private static void loadHierarchy( ServiceOrganizationVisualizer serviceOrganizationVisualizer, AbstractionBaseController abstractionBase, Integer collectionID ){

		DefaultMutableTreeNode root = abstractionBase.visualize( collectionID );

		if( root == null ) return;


		serviceOrganizationVisualizer.setTreeOfHierarchy( root );

		serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView( serviceOrganizationVisualizer.getTreeOfHierarchy() );

		JTreeManipulation.expand( serviceOrganizationVisualizer.getTreeOfHierarchy(), new TreePath( root ) );

		serviceOrganizationVisualizer.repaint();
	}


	public static void produceNFH2( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, List<ServiceInterface> allInterfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){//Mode = 2.

		Map<Integer, ServiceInstance> representedServiceInstances = findRepresentedServiceInstances( serviceRegistration, options.getCollectionID(), allInterfaces );

		if( representedServiceInstances == null ){

			PrintMessages.print( PrintMessages.organizationPrint, "The service instances are NOT characterized by non-functional descriptions!" );

			return;
		}


		//Pre-processing the input set of the service instances!
		QualityProperties Q = findCommonProperties( serviceRegistration, options.getCollectionID(), representedServiceInstances );//TODO: In this version, I assume that all the service instances provide the same set of properties!

		if( Q == null || ( Q != null && Q.getNumOfProperties() == 0 )){

			if( Q == null ) PrintMessages.print( PrintMessages.organizationPrint, "Quality properties = null" );

			PrintMessages.print( PrintMessages.organizationPrint, "The non-functional descriptions of the represented service instances do not provide common quality properties!" );

			return ;
		}

		/*This functions will not be implemented because I will do the filtering on the fly!
		 * S = filterServiceInstances();
		 * L = filterAbstractDomains();
		 */

		List<NonFunctionalAbstraction> nonFunctionalAbstractions = new ArrayList<NonFunctionalAbstraction>();//The list of the produced non-functional abstractions!

		//Create a non-functional abstraction that describes all the service instances.
		Integer nonFunctionalAbstractionID = 0;
		Integer totalFunctionalAbstractionID = 0;

		PropertiesBasedGroupsCalculator propertiesBasedGroupsCalculator = new PropertiesBasedGroupsCalculatorImplV1( serviceRegistration );

		List<ArrayList<PropertyBasedGroup>> propertiesBasedGroups = propertiesBasedGroupsCalculator.produce( options.getDelay(), true, options.getCollectionID(), Q, representedServiceInstances, serviceOrganizationVisualizer );

		JointGroupsCalculator jointGroupsCalculator = new JointGroupsCalculatorImplV1( serviceOrganizationVisualizer );

		List<JointGroup> jointGroups = jointGroupsCalculator.produce( options.getDelay(), options.getCollectionID(), propertiesBasedGroups, serviceOrganizationVisualizer );//In this phase, only one joint group/abstraction is produced!

		NonFunctionalAbstraction nonFunctionalAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), nonFunctionalAbstractionID, null, jointGroups.get( 0 ) );//NULL is the ancestor ID and I take the first element of the 'jointGroups' list because in this phase, only one joint group/abstraction is produced!

		if( nonFunctionalAbstraction == null ) return;

		nonFunctionalAbstractions.add( nonFunctionalAbstraction );

		Delay.delay( options.getDelay() );

		loadProducedNFALabels( serviceOrganizationVisualizer, nonFunctionalAbstractionID );//GUI!

		Delay.delay( options.getDelay() );

		loadHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!


		do {

			//The candidate for division non-functional abstraction.
			NonFunctionalAbstraction candidateNonFunctionalAbstraction = nonFunctionalAbstractions.remove( nonFunctionalAbstractions.size() - 1);

			representedServiceInstances = candidateNonFunctionalAbstraction.getRepresentedServiceInstances();

			if( representedServiceInstances != null && representedServiceInstances.size() > abstractionBase.getThreshold( options.getCollectionID() ) ){

				Integer ancestorID = candidateNonFunctionalAbstraction.getNonFunctionalAbstractionID();

				propertiesBasedGroups = propertiesBasedGroupsCalculator.produce( options.getDelay(), false, options.getCollectionID(), Q, representedServiceInstances, serviceOrganizationVisualizer );

				jointGroups = jointGroupsCalculator.produce( options.getDelay(), options.getCollectionID(), propertiesBasedGroups, serviceOrganizationVisualizer );

				for( int i = 0; jointGroups != null && i < jointGroups.size(); ++i ){

					++nonFunctionalAbstractionID;

					nonFunctionalAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), nonFunctionalAbstractionID, ancestorID, jointGroups.get(i) );

					if( nonFunctionalAbstraction == null ) return;

					nonFunctionalAbstractions.add( nonFunctionalAbstraction );

					Delay.delay( options.getDelay() );

					loadHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!

					Delay.delay( options.getDelay() );

					loadProducedNFALabels( serviceOrganizationVisualizer, nonFunctionalAbstractionID );//GUI!
				}
			}


			if( representedServiceInstances != null && representedServiceInstances.size() > 0 ) totalFunctionalAbstractionID += FunctionalOrganization.produceFH2( serviceRegistration, abstractionBase, options, nonFunctionalAbstractionID, ((NonFunctionalAbstractionWithNestedFAbstractions) candidateNonFunctionalAbstraction).getInterfacesOfRepresentedServiceInstances(serviceRegistration, options.getCollectionID()), serviceOrganizationVisualizer );//Note that even if the service instances do not provide values, I create a joint group. If the group is empty, then the functional organization do not have meaning!


		} while ( nonFunctionalAbstractions.size() > 0 );


		loadFinalProducedLabelFNestedNF( serviceOrganizationVisualizer, nonFunctionalAbstractionID, totalFunctionalAbstractionID );//GUI!

		loadTerminationLabel( serviceOrganizationVisualizer );//GUI!

		changeButtonLabelToConstructHierarchy( serviceOrganizationVisualizer );//GUI!

		enableStoreButton( serviceOrganizationVisualizer );
	}

	private static void loadFinalProducedLabelFNestedNF( ServiceOrganizationVisualizer serviceOrganizationVisualizer, int numOfNFA, int numOfFA ){

		serviceOrganizationVisualizer.getLabelOfProducing1stAbstractions().setText("2. The produced hierarchy consists of...");

		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setText("#non-functional abstractions: " + numOfNFA);
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setForeground(new Color(0, 200, 80));
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setFont(new java.awt.Font("Tahoma", 0, 10));

		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText("#nested functional abstractions: " + numOfFA);
		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setForeground(new Color(0, 200, 80));
		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setFont(new java.awt.Font("Tahoma", 0, 10));

		serviceOrganizationVisualizer.repaint();
	}

	private static void loadTerminationLabel( ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfAlgorithmExecution().setText("Algorithm terminated.");
	}

	private static void changeButtonLabelToConstructHierarchy( ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getButtonOfConstructingHierarchy().setText("Re-produce Hierarchy");
		serviceOrganizationVisualizer.repaint();
	}

	private static void enableStoreButton( ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getButtonOfStoringHierarchy().setEnabled(true);
	}

	private static void loadProducedNFALabels( ServiceOrganizationVisualizer serviceOrganizationVisualizer, int numOfNFA ){

		serviceOrganizationVisualizer.getLabelOfProducing1stAbstractions().setText( "2. Producing..." );
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setText( "#non-functional abstractions: " + numOfNFA );

		serviceOrganizationVisualizer.repaint();
	}


	public static void produceNFH3( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, List<ServiceInterface> allInterfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){//Mode = 3.

		long startTime = System.currentTimeMillis();

		Map<Integer, ServiceInstance> representedServiceInstances = findRepresentedServiceInstances( serviceRegistration, options.getCollectionID(), allInterfaces );

		if( representedServiceInstances == null ){

			PrintMessages.print( PrintMessages.organizationPrint, "The service instances are NOT characterized by non-functional descriptions!" );

			return;
		}

		reset( serviceOrganizationVisualizer );//GUI!

		loadExecutionLabel( serviceOrganizationVisualizer );//GUI.

		Delay.delay( options.getDelay() );

		//Pre-processing the input set of the service instances!
		QualityProperties Q = findCommonProperties( serviceRegistration, options.getCollectionID(), representedServiceInstances );//TODO: In this version, I assume that all the service instances provide the same set of properties!

		if( Q == null || ( Q != null && Q.getNumOfProperties() == 0 ) ){

			if( Q == null ) PrintMessages.print( PrintMessages.organizationPrint, "Quality properties = null" );

			PrintMessages.print( PrintMessages.organizationPrint, "The non-functional descriptions of the represented service instances do not provide common quality properties!" );

			return;
		}

		/*This functions will not be implemented because I will do the filtering on the fly!
		 * S = filterServiceInstances();
		 * L = filterAbstractDomains();
		 */

		List<NonFunctionalAbstraction> nonFunctionalAbstractions = new ArrayList<NonFunctionalAbstraction>();//The list of the produced non-functional abstractions!

		//Create a non-functional abstraction that describes all the service instances.
		Integer nonFunctionalAbstractionID = 0;

		PropertiesBasedGroupsCalculator propertiesBasedGroupsCalculator = new PropertiesBasedGroupsCalculatorImplV1( serviceRegistration );

		List<ArrayList<PropertyBasedGroup>> propertiesBasedGroups = propertiesBasedGroupsCalculator.produce( options.getDelay(), true, options.getCollectionID(), Q, representedServiceInstances, serviceOrganizationVisualizer );

		JointGroupsCalculator jointGroupsCalculator = new JointGroupsCalculatorImplV1( serviceOrganizationVisualizer );

		List<JointGroup> jointGroups = jointGroupsCalculator.produce( options.getDelay(), options.getCollectionID(), propertiesBasedGroups, serviceOrganizationVisualizer );//In this phase, only one joint group/abstraction is produced!

		NonFunctionalAbstraction nonFunctionalAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), nonFunctionalAbstractionID, null, jointGroups.get( 0 ) );//NULL is the ancestor ID and I take the first element of the 'jointGroups' list because in this phase, only one joint group/abstraction is produced!

		if( nonFunctionalAbstraction == null ) return;

		nonFunctionalAbstractions.add( nonFunctionalAbstraction );

		loadInitializationCompleted( serviceOrganizationVisualizer );//GUI!

		Delay.delay( options.getDelay() );

		loadNonFunctionalHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!


		do {

			//The candidate for division non-functional abstraction.
			NonFunctionalAbstraction candidateNonFunctionalAbstraction = nonFunctionalAbstractions.remove( nonFunctionalAbstractions.size() - 1);

			representedServiceInstances = candidateNonFunctionalAbstraction.getRepresentedServiceInstances();

			if( representedServiceInstances != null && representedServiceInstances.size() > abstractionBase.getThreshold( options.getCollectionID() ) ){

				Integer ancestorID = candidateNonFunctionalAbstraction.getNonFunctionalAbstractionID();

				Delay.delay( options.getDelay() );

				loadCandidateForDivisionAbstraction( serviceOrganizationVisualizer, ancestorID + 1 );//GUI!

				propertiesBasedGroups = propertiesBasedGroupsCalculator.produce( options.getDelay(), false, options.getCollectionID(), Q, representedServiceInstances, serviceOrganizationVisualizer );

				jointGroups = jointGroupsCalculator.produce( options.getDelay(), options.getCollectionID(), propertiesBasedGroups, serviceOrganizationVisualizer);

				for( int i = 0; jointGroups != null && i < jointGroups.size(); ++i ){

					++nonFunctionalAbstractionID;

					nonFunctionalAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), nonFunctionalAbstractionID, ancestorID, jointGroups.get(i) );

					if( nonFunctionalAbstraction == null ) return;

					nonFunctionalAbstractions.add( nonFunctionalAbstraction );

					Delay.delay( options.getDelay() );

					loadNonFunctionalHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!
				}
			}

		} while ( nonFunctionalAbstractions.size() > 0 && ! options.hasOneHierarchyLevel() );


		long millis = System.currentTimeMillis() - startTime;

		loadNonFunctionalHierarchy( serviceOrganizationVisualizer, abstractionBase, options.getCollectionID() );//GUI!

		Integer numOfProducedNonFunctionalAbstractions = nonFunctionalAbstractionID + 1;

		Delay.delay( options.getDelay() );

		loadTerminationLabel( serviceOrganizationVisualizer, numOfProducedNonFunctionalAbstractions, ElapsedTimeCalculator.getTime( millis ) );//GUI!
	}

	private static void reset( ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfInitializationOfNonFunctionalAlgorithm().setText("");
		serviceOrganizationVisualizer.getLabelOfInitialNonFunctionalAbstraction().setText("");
		serviceOrganizationVisualizer.getLabelOfProducingNonFunctionalAbstractions().setText("");
		serviceOrganizationVisualizer.getLabelOfPropertyGroups().setText("");
		serviceOrganizationVisualizer.getLabelOfJointGroups().setText("");

		serviceOrganizationVisualizer.getScrollPanelOfNonFunctionalHierarchy().setViewportView( null );

		serviceOrganizationVisualizer.repaint();
	}

	private static void loadExecutionLabel( ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfExecutionOfNonFunctionalAlgorithm().setText("Algorithm execution...");
		serviceOrganizationVisualizer.getLabelOfInitializationOfNonFunctionalAlgorithm().setText("1. Initialization...");

		serviceOrganizationVisualizer.repaint();
	}

	private static void loadInitializationCompleted( ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfInitializationOfNonFunctionalAlgorithm().setText("1. Initialization completed.");
		serviceOrganizationVisualizer.getLabelOfInitialNonFunctionalAbstraction().setText("The initial non-functional abstraction has been produced.");

		serviceOrganizationVisualizer.repaint();
	}

	private static void loadTerminationLabel( ServiceOrganizationVisualizer serviceOrganizationVisualizer, Integer numOfNFA, String time ){

		serviceOrganizationVisualizer.getLabelOfExecutionOfNonFunctionalAlgorithm().setText("Algorithm terminated ( " + time + " ).");
		serviceOrganizationVisualizer.getLabelOfProducingNonFunctionalAbstractions().setText("2. The produced hierarchy consists of");

		serviceOrganizationVisualizer.getLabelOfPropertyGroups().setText("#non-functional abstractions: " + numOfNFA);
		serviceOrganizationVisualizer.getLabelOfPropertyGroups().setForeground( new Color(0, 200, 80) );
		serviceOrganizationVisualizer.getLabelOfPropertyGroups().setFont( new java.awt.Font("Tahoma", 0, 10) );

		serviceOrganizationVisualizer.repaint();
	}

	private static void loadCandidateForDivisionAbstraction( ServiceOrganizationVisualizer serviceOrganizationVisualizer, Integer nonFunctionalAbstractionID ){

		serviceOrganizationVisualizer.getLabelOfProducingNonFunctionalAbstractions().setText("2. Dividing the non-functional abstraction with ID: " + nonFunctionalAbstractionID);

		serviceOrganizationVisualizer.repaint();
	}

	private static void loadNonFunctionalHierarchy( ServiceOrganizationVisualizer serviceOrganizationVisualizer, AbstractionBaseController abstractionBase, Integer collectionID ){

		DefaultMutableTreeNode root = abstractionBase.visualizeNFH( collectionID );

		if( root == null ) return;


		serviceOrganizationVisualizer.setTreeOfNonFunctionalHierarchy( root );

		serviceOrganizationVisualizer.getTreeOfNonFunctionalHierarchy().addTreeSelectionListener( selectNonFunctionalAbstractionActionListener( serviceOrganizationVisualizer ) );
		serviceOrganizationVisualizer.getScrollPanelOfNonFunctionalHierarchy().setViewportView( serviceOrganizationVisualizer.getTreeOfNonFunctionalHierarchy() );

		JTreeManipulation.expand( serviceOrganizationVisualizer.getTreeOfNonFunctionalHierarchy(), new TreePath( root ) );

		serviceOrganizationVisualizer.repaint();
	}

	private static TreeSelectionListener selectNonFunctionalAbstractionActionListener( final ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

        return new TreeSelectionListener(){

        	public void valueChanged(TreeSelectionEvent ew){

        		serviceOrganizationVisualizer.setPath2 ( ew.getPath() );

        		int pathCount = ew.getPath().getPathCount();

            	String selection = ew.getPath().getPathComponent( pathCount - 1 ).toString();

            	if( selection != null && selection.contains( "Non-functional abstraction" ) && ! selection.contains( "Non-functional abstractions" ) ) serviceOrganizationVisualizer.getButtonOfVisualizingNonFunctional().setEnabled( true );
            	
            	else serviceOrganizationVisualizer.getButtonOfVisualizingNonFunctional().setEnabled( false );
        	}
        };
    }

	public void updateNFH(){

	}
}
