package org.ow2.choreos.dataStructures.tree.IntervalTree;


import org.ow2.choreos.dataStructures.array.arrayInstantiation.ArrayInstantiation;
import org.ow2.choreos.dataStructures.array.arrayManipulation.ArraySorting;
import org.ow2.choreos.dataStructures.array.arrayManipulation.ArrayTransformations;
import org.ow2.choreos.dataStructures.interval.Interval1D;


public class IntervalTree1D {


	//Variables.
    private IntervalTree1DNode root;


    //Constructor.
    public IntervalTree1D( Object[] intervals, Object[] values ){

    	int[] endpoints = determineEndPoints( intervals );

    	ArraySorting.sortIntegers( endpoints ); //ArrayPrinting.stdPrintln( endpoints ); 

    	root = insertElementaryIntervals( endpoints, Integer.MIN_VALUE, Integer.MAX_VALUE );

    	insertIntervals( intervals, values );
    }

    //Methods used by constructor.
    private int[] determineEndPoints( Object[] intervals ){

    	int[] endpoints = ArrayInstantiation.newIntArray( intervals, 2 );


    	for( int i = 0, j = -1; intervals != null && i < intervals.length; ++i ){

    		endpoints[ ++j ] = (int) ( (Interval1D) intervals[ i ] ).getLeftEndpoint();

    		endpoints[ ++j ] = (int) ( (Interval1D) intervals[ i ] ).getRightEndpoint();
    	}


    	return ArrayTransformations.removeRepetitions( endpoints );
    }

    private IntervalTree1DNode insertElementaryIntervals( int[] endpoints, int leftSpanEndpoint, int rightSpanEndpoint ){

    	 IntervalTree1DNode intervalTree1DNode = null;


    	if( endpoints != null ){

    		int midEndpoint = endpoints.length % 2 == 0 ? endpoints.length / 2 : ( endpoints.length - 1 ) / 2; //ArrayPrinting.stdPrintln( endpoints ); System.out.println( " midEndpoint= " + midEndpoint );

    		intervalTree1DNode = new IntervalTree1DNode( (Integer) endpoints[ midEndpoint ], leftSpanEndpoint, rightSpanEndpoint );


        	IntervalTree1DNode leftChild = null;

        	if( midEndpoint - 1 >= 0 ) leftChild = insertElementaryIntervals( ArrayTransformations.copyOfRange( endpoints, 0, midEndpoint - 1 ), leftSpanEndpoint, (Integer) endpoints[ midEndpoint ] );

        	else leftChild = insertElementaryIntervals( null, leftSpanEndpoint, (Integer) endpoints[ midEndpoint ] );

        	intervalTree1DNode.setLeftChild( leftChild );


        	IntervalTree1DNode rightChild = null;

        	if( midEndpoint + 1 < endpoints.length ) rightChild = insertElementaryIntervals( ArrayTransformations.copyOfRange( endpoints, midEndpoint + 1, endpoints.length - 1 ), (Integer) endpoints[ midEndpoint ], rightSpanEndpoint );

        	else rightChild = insertElementaryIntervals( null, (Integer) endpoints[ midEndpoint ], rightSpanEndpoint );

        	intervalTree1DNode.setRightChild( rightChild );
    	}

    	else intervalTree1DNode = new IntervalTree1DNode( leftSpanEndpoint, rightSpanEndpoint );


    	return intervalTree1DNode;
    }


    private void insertIntervals( Object[] intervals, Object[] values ){

    	for( int i = 0; intervals != null && i < intervals.length; ++i ) insertInterval( (Interval1D) intervals[ i ], values[ i ], root );
    }

    private void insertInterval( Interval1D interval, Object value, IntervalTree1DNode node ){

    	if( interval.contains( node.getSpan() ) || ( interval.isPoint() && interval.getLeftEndpoint() == node.getKey() ) ) node.addValue( interval, value );

    	else{

    		if( node.getLeftChild() != null && checkLeftChildTraversalCondition( interval, node ) ) insertInterval( interval, value, node.getLeftChild() );

    		if( node.getRightChild() != null && checkRightChildTraversalCondition( interval, node ) ) insertInterval( interval, value, node.getRightChild() );
    	}
    }

    private boolean checkLeftChildTraversalCondition( Interval1D interval, IntervalTree1DNode node ){

    	if( interval.contains( node.getKey() ) ) return true;

    	if( interval.isLeft( node.getKey() ) ) return true;


    	return false;
    }

    private boolean checkRightChildTraversalCondition( Interval1D interval, IntervalTree1DNode node ){

    	if( interval.contains( node.getKey() ) ) return true;

    	if( interval.isRight( node.getKey() ) ) return true;


    	return false;
    }


    //Getters.
    public IntervalTree1DNode getRoot(){

    	return root;
    }


    //Visualization.
    public void visualize(){

    	visualize( "", root );
    }

    private void visualize( String space, IntervalTree1DNode IntervalTree1DNode ){

    	System.out.println( space + IntervalTree1DNode.getKey() );

    	space += " ";

    	if( IntervalTree1DNode.getLeftChild() != null ) visualize( space, IntervalTree1DNode.getLeftChild() );

    	if( IntervalTree1DNode.getRightChild() != null ) visualize( space, IntervalTree1DNode.getRightChild() );
    }
}
