package org.ow2.choreos.dataStructures.tree.IntervalTree;

import java.util.Iterator;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;


public class IntervalTree1DNode {


	//Variables.
	private Double key;

	private Interval1D span;//It is the aggregation of the elementary intervals that are covered by the key of the node.

	private Map<Object, Object> values;//It is map with key the interval and value an object.

	private IntervalTree1DNode left, right; // left and right subtrees

	private boolean isLeaf;

	private int N;                    		// size of subtree rooted at this node

	private double max;               		// max endpoint in subtree rooted at this node


	//Constructor 1 : internal node.
    protected IntervalTree1DNode( double cKey, double leftSpanEndpoint, double rightSpanEndpoint ) {

    	key = cKey;

    	//max = interval.getRightEndpoint();

    	span = new Interval1D( leftSpanEndpoint, rightSpanEndpoint );

    	isLeaf = false;
    }

    //Constructor 2 : leaf node.
    protected IntervalTree1DNode( int leftSpanEndpoint, int rightSpanEndpoint ) {

    	span = new Interval1D( leftSpanEndpoint, rightSpanEndpoint );

    	isLeaf = true;
    }


    //Setters.
    protected void addValue( Interval1D interval, Object value ){

    	values = MapInsertion.put( interval, value, values, false );
    }

    protected void setLeftChild( IntervalTree1DNode cLeft ){

    	left = cLeft;
    }

    protected void setRightChild( IntervalTree1DNode cRight ){

    	right = cRight;
    }

    protected void setMax( double cMax ){

    	max = cMax;
    }

    protected void setN( int cN ){

    	N = cN;
    }


    //Getters.
    public Double getKey(){

    	return key;
    }

    public Object printKey(){

    	return key != null ? "key= " + key : "";
    }

    public Object printIntKey(){

    	return key != null ? "key= " + key.intValue() : "";
    }

    public Interval1D getSpan(){

    	return span;
    }

    public String printSpan(){

    	return " span= " + span.getIntEndpoints();
    }

    public IntervalTree1DNode getLeftChild(){

    	return left;
    }

    public IntervalTree1DNode getRightChild(){

    	return right;
    }

    protected double getMax(){

    	return max;
    }

    protected int getN(){

    	return N;
    }


    //Visualization.
    public DefaultMutableTreeNode visualize(){

    	DefaultMutableTreeNode node = new DefaultMutableTreeNode( printIntKey() + printSpan() );


    	Iterator<Object> iterator = values != null ? values.keySet().iterator() : null;

		while( iterator != null && iterator.hasNext() ){

			Object key = iterator.next();

			Object value = values.get( key );

			DefaultMutableTreeNode valueNode = new DefaultMutableTreeNode( "scope= " + ( (Interval1D) key ).getIntEndpoints() + " label= " + SchemaRepresentationController.getTag( value ) );

			node.add( valueNode );
		}


    	return node;
    }
}
