package org.ow2.choreos.gui;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.array.arrayManipulation.ArrayTransformations;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapRemoval.MapRemoval;


/**
 * 
 * @author dathanas
 * TODO
 */
public class UserSettings {


	//Constants.
	private static final String concreteElementsDataStructureKind = "kind of data structure for schema elements = ";

	private static final String interactiveMatchingProcess = "interactive matching process = ";

	private static final String minimumFrequency = "minimum frequency value = ";

	private static final String topK = "k = ";

	private static final String patternConfidenceThreshold = "pattern confidence threshold";

	private static final String correspondenceConfidenceThreshold = "correspondence confidence threshold";

	private static final String similarityRulesForElementKinds = " - similarity rules for element kinds= ";

	private static final String docsOfSchema = " - list of documents of schema= ";

	private static final String refs2DocsOfSchema = " - list of references to documents of schema= ";

	private static final String collectionIdentifier = " - collection ID= ";

	private static final String ref2Source = " - reference to source= ";

	private static final String primarySchemaRepresentationModel = " - primary schema representation model= ";

	private static final String targetSchemaRepresentationModel = " - target schema representation model= ";


	//Variables: generic settings.
	private Map<Object, Object> settings;//It is a generic way of storing different settings/variables.


	//Variables: log-in information.
	private String userCategory, curatorPassword;

	//Variables: Organization options.
	private Integer collectionID;
	private Integer mode;

	//Variables: Functional options.
	private String functionalOrganizationAlgorithm;//It takes values from the class 'abstractionDrivenServiceOrganization.functionalOrganization.FunctionalOrganizationController'.
	private String nonFunctionalOrganizationAlgorithm;//It takes values from the class 'abstractionDrivenServiceOrganization.nonFunctionalOrganization.NonFunctionalOrganizationController'.

	private String schemaRepresentationModel;

	private String interfacesDistanceCalculatorAlgorithm;//It takes values from the class 'abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.InterfacesDistanceCalculatorController'.
	private String namesDistanceCalculatorAlgorithm;//It takes from the class 'abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator'.

	//Variables: Non-functional options.
	private Integer threshold;
	private Boolean oneHierarchyLevel;
	private Boolean existingNonFuncDocs;

	//Variables: interactive abstractions generation.
	private String abstractionDepth;
	private Double similarityLintel;
	
	//Variables: Others.
	private Boolean delay;


	//Constructor.
	public UserSettings( Integer cCollectionID ){//It is used by the class 'QueryEngineController'.

		collectionID = cCollectionID;

		print();
	}

	public UserSettings( Integer cCollectionID, String cUserCategory, String cCuratorPassword ){

		collectionID = cCollectionID;

		userCategory = cUserCategory;

		curatorPassword = cCuratorPassword;

		print();
	}

	public UserSettings() {//It is used by the 'SchemaRegistrationVisualizer' class.

	}


	//Setters.
	void setSchemaElementsDataStructureKind( String cSchemaElementsDataStructureKind ){

		settings = MapInsertion.put( concreteElementsDataStructureKind, cSchemaElementsDataStructureKind, settings, true );
	}

	void setInteractiveMatchingProcess( Boolean cInteractiveMatchingProcess ){

		settings = MapInsertion.put( interactiveMatchingProcess, cInteractiveMatchingProcess, settings, true );
	}

	void setMinimumFrequency( Integer cMinimumFrequency ){

		settings = MapInsertion.put( minimumFrequency, cMinimumFrequency, settings, true );
	}

	protected void setTopK( int k ){

		settings = MapInsertion.put( topK, k, settings, true );
	}

	void addSimilarityRuleForElementKinds( Object cRule ){

		if( cRule == null ) return;


		Object[] cRules = null;

		if( ( cRules = getSimilarityRulesForElementKinds() ) == null ) cRules = new Object[] { cRule };

		else cRules = ArrayTransformations.concat( cRules , cRule );


		settings = MapInsertion.put( similarityRulesForElementKinds, cRules, settings, true );
	}

	protected void setOption( Object optionLabel, Object optionValue ){

		settings = MapInsertion.put( optionLabel, optionValue, settings, true );
	}

	protected void setPatternConfidenceThreshold( double cPatternConfidenceThreshold ){

		settings = MapInsertion.put( patternConfidenceThreshold, cPatternConfidenceThreshold, settings, true );
	}

	protected void setCorrespondenceConfidenceThreshold( double cCorrespondenceConfidenceThreshold ){

		settings = MapInsertion.put( correspondenceConfidenceThreshold, cCorrespondenceConfidenceThreshold, settings, true );
	}

	protected void setDocsOfSchema( Object[] cDocsOfSchema ){

		settings = MapInsertion.put( docsOfSchema, cDocsOfSchema, settings, true );
	}

	protected void setRefs2DocsOfSchema( List<String> cRefs2DocsOfSchema ){

		settings = MapInsertion.put( refs2DocsOfSchema, cRefs2DocsOfSchema, settings, true );
	}

	protected void resetRefs2DocsOfSchema(){

		MapRemoval.remove( refs2DocsOfSchema, settings );
	}

	void addRef2DocOfSchema( String cRef2DocOfSchema ){

		List<String> cRefs2DocsOfSchema = null;


		if( ( cRefs2DocsOfSchema = getRefs2DocsOfSchema() ) == null ){

			cRefs2DocsOfSchema = new ArrayList<String>();

			cRefs2DocsOfSchema.add( cRef2DocOfSchema );
		}

		else cRefs2DocsOfSchema.add( cRef2DocOfSchema );


		settings = MapInsertion.put( refs2DocsOfSchema, cRefs2DocsOfSchema, settings, true );
	}

	void setCollectionIdentifier( Integer cCollectionIdentifier ){

		settings = MapInsertion.put( collectionIdentifier, cCollectionIdentifier, settings, true ); print();
	}

	void setRef2Source( String cRef2Source ){

		settings = MapInsertion.put( ref2Source, cRef2Source, settings, true ); print();
	}

	void setPrimarySchemaRepresentationModel( String cPrimarySchemaRepresentationModel ){

		settings = MapInsertion.put( primarySchemaRepresentationModel, cPrimarySchemaRepresentationModel, settings, false );

		settings = MapInsertion.put( targetSchemaRepresentationModel, cPrimarySchemaRepresentationModel, settings, true ); print();
	}

	void setTargetSchemaRepresentationModel( String cTargetSchemaRepresentationModel ){

		settings = MapInsertion.put( targetSchemaRepresentationModel, cTargetSchemaRepresentationModel, settings, true );
	}

	public void setCollectionID( Integer cCollectionID ){

		collectionID = cCollectionID;
		mode = null;

		functionalOrganizationAlgorithm = null;
		schemaRepresentationModel = null;
		interfacesDistanceCalculatorAlgorithm = null;
		namesDistanceCalculatorAlgorithm = null;

		threshold = null;
		oneHierarchyLevel = null;
		existingNonFuncDocs = null;

		delay = null;

		print();
	}

	public void setMode( Integer cMode ){

		mode = cMode; print();
	}

	public void setAbstractionDepth( String cAbstractionDepth ) {

		abstractionDepth = cAbstractionDepth;
	}

	public void setSimilarityLintel( Double cSimilarityLintel ) {

		similarityLintel = cSimilarityLintel;
	}


	//Functional options.
	public void setFunctionalOrganizationAlgorithm( String cFunctionalOrganizationAlgorithm ){

		functionalOrganizationAlgorithm = cFunctionalOrganizationAlgorithm; print();
	}

	public void setNonFunctionalOrganizationAlgorithm( String cNonFunctionalOrganizationAlgorithm ){

		nonFunctionalOrganizationAlgorithm = cNonFunctionalOrganizationAlgorithm; print();
	}

	public void setSchemaRepresentationModel( String cSchemaRepresentationModel ){

		schemaRepresentationModel = cSchemaRepresentationModel; print();
	}

	public void setInterfacesDistanceCalculatorAlgorithm( String cInterfacesDistanceCalculatorAlgorithm ){

		interfacesDistanceCalculatorAlgorithm = cInterfacesDistanceCalculatorAlgorithm; print();
	}

	public void setNamesDistanceCalculatorAlgorithm( String cNamesDistanceCalculatorAlgorithm ){

		namesDistanceCalculatorAlgorithm = cNamesDistanceCalculatorAlgorithm; print();
	}


	//Non-functional options.
	public void setThreshold( Integer cThreshold ){

		threshold = cThreshold; print();
	}

	public void setOneHierarchyLevel( Boolean cOneHierarchyLevel ){

		oneHierarchyLevel = cOneHierarchyLevel; print();
	}

	public void setExistingNonFuncDocs( Boolean cExistingNonFuncDocs ){

		existingNonFuncDocs = cExistingNonFuncDocs; print();
	}

	public void setDelay( Boolean cDealy ){

		delay = cDealy; print();
	}


	//Getters.
	public Object getOptionValue( Object optionLabel ){

		return MapGetter.exists( optionLabel, settings ) ? MapGetter.getValue( optionLabel, settings ) : null;
	}

	public String getSchemaElementsDataStructureKind(){

		return settings != null && settings.get( concreteElementsDataStructureKind ) != null ? (String) settings.get( concreteElementsDataStructureKind ) : null;
	}

	public Boolean getInteractiveMatchingProcess(){

		return settings != null && settings.get( interactiveMatchingProcess ) != null ? (Boolean) settings.get( interactiveMatchingProcess ) : null;
	}

	public int getMinimumFrequency(){

		return settings != null && settings.get( minimumFrequency ) != null ? (Integer) settings.get( minimumFrequency ) : null;
	}

	public int getTopK(){

		return settings != null && settings.get( topK ) != null ? (Integer) settings.get( topK ) : null;
	}

	public double getMatchingConfidenceThreshold(){

		return settings != null && settings.get( correspondenceConfidenceThreshold ) != null ? (Double) settings.get( correspondenceConfidenceThreshold ) : null;
	}

	public double getPatternConfidenceThreshold(){

		return settings != null && settings.get( patternConfidenceThreshold ) != null ? (Double) settings.get( patternConfidenceThreshold ) : null;
	}

	public Object[] getSimilarityRulesForElementKinds(){

		return settings != null && settings.get( similarityRulesForElementKinds ) != null ? (Object[]) settings.get( similarityRulesForElementKinds ) : null;
	}

	@SuppressWarnings("unchecked")
	public List<String> getRefs2DocsOfSchema(){

		return settings != null && settings.get( refs2DocsOfSchema ) != null ? (List<String>) settings.get( refs2DocsOfSchema ) : null;
	}

	public Integer getCollectionIdentifier(){

		return settings != null && settings.get( collectionIdentifier ) != null ? (Integer) settings.get( collectionIdentifier ) : null;
	}

	public String getRef2Source(){

		return settings != null && settings.get( ref2Source ) != null ? (String) settings.get( ref2Source ) : null;
	}

	public String getPrimarySchemaRepresentationModel(){

		return settings != null && settings.get( primarySchemaRepresentationModel ) != null ? (String) settings.get( primarySchemaRepresentationModel ) : null;
	}

	public String getTargetSchemaRepresentationModel(){

		return settings != null && settings.get( targetSchemaRepresentationModel ) != null ? (String) settings.get( targetSchemaRepresentationModel ) : null;
	}

	public Object[] getDocsOfSchema(){

		return settings != null && settings.get( docsOfSchema ) != null ? (Object[]) settings.get( docsOfSchema ) : null;
	}


	public String getUserCategory(){

		return userCategory;
	}

	public String getCuratorPassword(){

		return curatorPassword;
	}

	public Integer getCollectionID(){

		return collectionID;
	}

	private Integer printCollectionID(){

		return getCollectionIdentifier();
	}

	public String getFunctionalOrganizationAlgorithm(){

		return functionalOrganizationAlgorithm;
	}

	public String getNonFunctionalOrganizationAlgorithm(){

		return nonFunctionalOrganizationAlgorithm;
	}

	public String getSchemaRepresentationModel(){

		return schemaRepresentationModel;
	}

	public String getNamesDistanceCalculatorAlgorithm(){

		return namesDistanceCalculatorAlgorithm;
	}

	public String getInterfacesDistanceCalculatorAlgorithm(){

		return interfacesDistanceCalculatorAlgorithm;
	}

	public Integer getThreshold(){

		return threshold;
	}

	public Boolean hasOneHierarchyLevel(){

		return oneHierarchyLevel;
	}

	public Integer getMode(){

		return mode;
	}

	public Boolean existNonFuncDescriptions(){

		return existingNonFuncDocs;
	}

	public Boolean getDelay(){

		return delay;
	}

	public String getAbstractionDepth() {

		return abstractionDepth;
	}

	public Double getSimilarityLintel() {

		return similarityLintel;
	}


	//Visualization.
	private void print(){

		PrintMessages.stdPrintLn( PrintMessages.guiPrint, "settings:" + collectionIdentifier + printCollectionID() + " - mode = " + mode + " - schemaRepresentationModel = " + schemaRepresentationModel + " - namingDistanceType = " + namesDistanceCalculatorAlgorithm + " - threshold = " + threshold + " - existingNonFuncDocs = " + existingNonFuncDocs + " - delay = " + delay  + " - oneHierarchyLevel= " + oneHierarchyLevel + " - userCategory = " + userCategory + " - curatorPassword = " + curatorPassword + ref2Source + getRef2Source() + primarySchemaRepresentationModel + getPrimarySchemaRepresentationModel() + targetSchemaRepresentationModel + getTargetSchemaRepresentationModel() );
	}
}
