package org.ow2.choreos.gui.interfaceMappingsVisualizer;


import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.ow2.choreos.gui.interfaceMappingsVisualizer.components.MappingsInfoDialog;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.components.MappingsPanel;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.InterfaceMappingsVisualizerListener;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnection;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnectionLineProperties;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnectionUtils;

/**
 * This class represents the main Container that extends {@link JFrame}.
 * <p>This Container contains a {@link JMenuBar} and a {@link MappingsPanel} 
 * which is used for representing Mappings(connections) between various
 * nodes of two {@link JTree}s.
 * @see MappingsPanel*/
public class InterfaceMappingsVisualizer extends javax.swing.JDialog {
    /**
	 * 
	 */
	private static final long serialVersionUID = -5199626050619561369L;
	/** Creates new form InterfaceMappingsVisualizer
	 * @wbp.parser.constructor*/
    public InterfaceMappingsVisualizer(java.awt.Frame owner, boolean modal,InterfaceMappingsVisualizerListener callbackOperations) {
    	super(owner,modal);
        initComponents(callbackOperations);
    }

    /**
     * Creates new form InterfaceMappingsVisualizer with {@code sourceTree} and {@code
     * destTree} as the source {@code JTree} and destination {@codeJTree} of the
     * container {@link MappingsPanel mappingsPanel}
     * @param sourceTree An instance of {@link JTree}
     * @param destTree An instance of {@link JTree}
     */
    public InterfaceMappingsVisualizer(java.awt.Frame owner, boolean modal,JTree sourceTree, JTree destTree, InterfaceMappingsVisualizerListener callbackOperations){
    	this(owner,modal,callbackOperations);
        setSourceTreeModel((DefaultTreeModel)sourceTree.getModel());
        setDestTreeModel((DefaultTreeModel)destTree.getModel());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * @param callbackOperations 
     */
    private void initComponents(final InterfaceMappingsVisualizerListener callbackOperations) {

//        mappingsColorChooser = new MappingsLineColorChooser();
        mappingsPanel = new MappingsPanel(callbackOperations);
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        completeMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        deleteMappingMenuItem = new javax.swing.JMenuItem();
        viewMenu = new javax.swing.JMenu();
        showDataMenuItem = new javax.swing.JMenuItem();
        lineTypeRadioButton = new javax.swing.JRadioButtonMenuItem();
        testMenu = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Mappings Demo");
        setMinimumSize(new java.awt.Dimension(1024, 600));

        fileMenu.setText("File");

        completeMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        completeMenuItem.setIcon(new javax.swing.ImageIcon("Icons/Icon.png")); // NOI18N
        completeMenuItem.setText("Complete Verification");
        completeMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                completeMenuItemActionPerformed(callbackOperations);
            }
        });
        fileMenu.add(completeMenuItem);

        menuBar.add(fileMenu);

        editMenu.setText("Edit");

        deleteMappingMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_DELETE, 0));
        deleteMappingMenuItem.setIcon(new javax.swing.ImageIcon("Icons/deleteIcon.png")); // NOI18N
        deleteMappingMenuItem.setText("Delete Mapping Between Selected Nodes");
        deleteMappingMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteMappingMenuItemActionPerformed(callbackOperations);
            }
        });
        editMenu.add(deleteMappingMenuItem);

        menuBar.add(editMenu);

        viewMenu.setText("View");

        showDataMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.CTRL_MASK));
        showDataMenuItem.setIcon(new javax.swing.ImageIcon("Icons/dataIcon.png")); // NOI18N
        showDataMenuItem.setText("Show data");
        showDataMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showDataMenuItemActionPerformed(evt);
            }
        });
        viewMenu.add(showDataMenuItem);

        lineTypeRadioButton.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_L, java.awt.event.InputEvent.CTRL_MASK));
        lineTypeRadioButton.setText("Straight Lines");
        lineTypeRadioButton.setIcon(new javax.swing.ImageIcon("Icons/lineIcon.png")); // NOI18N
        lineTypeRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lineTypeRadioButtonActionPerformed(evt);
            }
        });
        viewMenu.add(lineTypeRadioButton);

        menuBar.add(viewMenu);

        testMenu.setText("Testing");
        menuBar.add(testMenu);

        setJMenuBar(menuBar);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(mappingsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 1004, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(mappingsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGap(18, 18, 18))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void deleteMappingMenuItemActionPerformed(InterfaceMappingsVisualizerListener callbackOperations) {
        TreePath sourcePath = this.mappingsPanel.getSourceTree().getSelectionPath();
        TreePath destPath = this.mappingsPanel.getDestTree().getSelectionPath();

        ArrayList<VisualComponentConnection> foundMappings = VisualComponentConnectionUtils.findConnectionsWith(this.mappingsPanel,VisualComponentConnectionUtils.EXISTING_AND_FOR_ADDITION_CONNECTIONS,
        		(DefaultMutableTreeNode) sourcePath.getLastPathComponent(),
        		(DefaultMutableTreeNode) destPath.getLastPathComponent());
        if (callbackOperations != null) {
			callbackOperations.deleteMapping(foundMappings.get(0));
		}
        else
        	this.mappingsPanel.addToDeletedMappings(foundMappings.get(0));
    }

    private void completeMenuItemActionPerformed(InterfaceMappingsVisualizerListener callbackOperations) {
    	if(callbackOperations != null){
    		callbackOperations.complete();
    	}
    	else{
    		this.mappingsPanel.complete(callbackOperations);
    	}
    }

    private void lineTypeRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {
        if (lineTypeRadioButton.isSelected()) {
            this.mappingsPanel.setLineType(VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_SIMPLE);
        } else {
            this.mappingsPanel.setLineType(VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_RECTANGULAR);
        }
        this.repaint();
    }

    private void showDataMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        if (!mappingsPanel.getExistingMappings().isEmpty()
                || !mappingsPanel.getAddedMappings().isEmpty()
                || !mappingsPanel.getDeletedMappings().isEmpty()) {
            new MappingsInfoDialog(this,
                    mappingsPanel.getExistingMappings(),
                    mappingsPanel.getAddedMappings(),
                    mappingsPanel.getDeletedMappings()).setVisible(true);
        } else {
            JOptionPane.showMessageDialog(this, "No data to show!", "No Data", JOptionPane.ERROR_MESSAGE);
        }
    }
//Getters
    /**
	 * @return the {@link MappingsPanel mappingsPanel}
	 */
	public MappingsPanel getMappingsPanel() {
		return mappingsPanel;
	}
//Delegate Setters
    /**
     * Delegate method from {@link MappingsPanel}
     * @param treeModel An instance of {@link DefaultTreeModel}
     * @see MappingsPanel#setSourceTreeModel
     */
    public final void setSourceTreeModel(DefaultTreeModel treeModel) {
        mappingsPanel.setSourceTreeModel(treeModel);
    }
    
	/**
     * Delegate method from {@link MappingsPanel}
     * @param treeModel An instance of {@link DefaultTreeModel}
     * @see MappingsPanel#setSourceTreeModel
     */
    public final void setDestTreeModel(DefaultTreeModel treeModel) {
        mappingsPanel.setDestTreeModel(treeModel);
    }
    
    /**
     * Cover method that calls both 
     * {@link #setSourceTreeModel} and {@link #setDestTreeModel}
     * @param sourceModel An instance of {@link DefaultTreeModel} given to {@code setSourceTreeModel}
     * @param destModel An instance of {@link DefaultTreeModel} given to {@code setDestTreeModel}
     */
    public void setTreeModels(javax.swing.tree.DefaultTreeModel sourceModel, javax.swing.tree.DefaultTreeModel destModel){
        setSourceTreeModel(sourceModel);
        setDestTreeModel(destModel);
    }

    /**
     * Delegate method from {@link MappingsPanel}
     * @param source A {@code String}
     * @param dest A {@code String}
     * @see MappingsPanel#setViewLabels
     */
    public void setViewLabels(String source, String dest) {
        mappingsPanel.setViewLabels(source, dest);
    }

//Delegate Getters
    /**
     * Delegate method from {@link MappingsPanel}
	 * @return
	 * @see MappingsPanel#getDestTree()
	 */
	public JTree getDestTree() {
		return mappingsPanel.getDestTree();
	}

	/**
	 * Delegate method from {@link MappingsPanel}
	 * @return
	 * @see MappingsPanel#getSourceTree()
	 */
	public JTree getSourceTree() {
		return mappingsPanel.getSourceTree();
	}
	
    /**
     * Adds {@code connections} as existing {@link VisualComponentConnection}s to the {@link MappingsPanel}
     * of this Container
     * @param connections An {@link ArrayList} of {@link VisualComponentConnection} objects
     */
    public void addExistingConnections(ArrayList<VisualComponentConnection> connections){
        mappingsPanel.setExistingMappings(connections);
    }
    
    

	/**
    * @param args the command line arguments
    */
    public static void main(String args[]) throws UnsupportedLookAndFeelException{
        //UIManager.setLookAndFeel(new NimbusLookAndFeel());
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new InterfaceMappingsVisualizer(null,true,null).setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem completeMenuItem;

    private javax.swing.JMenuItem deleteMappingMenuItem;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenu fileMenu;
//    private gui.interfaceMappingsVisualizer.components.MappingsLineColorChooser mappingsColorChooser;
    private javax.swing.JRadioButtonMenuItem lineTypeRadioButton;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem showDataMenuItem;
    private javax.swing.JMenu testMenu;
    private javax.swing.JMenu viewMenu;
    private MappingsPanel mappingsPanel;
    // End of variables declaration//GEN-END:variables

}
