package org.ow2.choreos.matchingProcess.schemaRepresentation;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.matchingProcess.Matcher;
import org.ow2.choreos.matchingProcess.MatcherStatistics;
import org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing.ConcreteElementsDataStructureController;
import org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing.ConcreteElementsIndex;
import org.ow2.choreos.measurements.time.ElapsedTimeCalculator;


/**
 * 
 * @author dathanas
 *
 */
public abstract class SchemaMatcher extends Matcher {


	//Variables.
	protected ConcreteElementsIndex index;

	protected final String queryingSchemaElementsDataStructureKind = ConcreteElementsDataStructureController.mapOfMapOfMap;


	//Constructors.
	protected SchemaMatcher( UserSettings userSettings, String currentPhase, boolean makeIndex ){

		super( userSettings, currentPhase );


		initializeSchemaMatcher( userSettings, currentPhase, makeIndex, null, null );
	}

	protected SchemaMatcher( UserSettings userSettings, String currentPhase, boolean makeIndex, Object guiLabel, Object guiProgressBar ){

		super( userSettings, currentPhase );


		initializeSchemaMatcher( userSettings, currentPhase, makeIndex, guiLabel, guiProgressBar );
	}

	protected SchemaMatcher( UserSettings userSettings, String currentPhase, boolean makeIndex, Object guiLabel, Object guiProgressBar, String cLogFilePath ){

		super( userSettings, currentPhase, cLogFilePath );


		initializeSchemaMatcher( userSettings, currentPhase, makeIndex, guiLabel, guiProgressBar );
	}

	private void initializeSchemaMatcher( UserSettings userSettings, String currentPhase, boolean makeIndex, Object guiLabel, Object guiProgressBar ){

		if( makeIndex ){

			index = new ConcreteElementsIndex( userSettings, guiLabel, guiProgressBar );

			addIndexStatistics( guiLabel, guiProgressBar, ConcreteElementsIndex.getVisualizationRootLabelForAllElements(), ConcreteElementsIndex.getLabelOfDuration() );
		}
	}

	protected void addIndexStatistics( Object guiLabel, Object guiProgressBar, String rootLabel, String durationLabel ){

		if( matcherStatistics == null ) matcherStatistics = new MatcherStatistics();


		matcherStatistics.addStatisticKind( currentPhase, rootLabel );


		matcherStatistics.addStatistic( currentPhase, rootLabel, "#abstract elements =", Integer.toString( index.getNumberOfAbstractElements() ) );

		//matcherStatistics.addStatistic( currentPhase, rootLabel, "#schemas (on average per abstract element) = ", "" );

		matcherStatistics.addStatistic( currentPhase, rootLabel, "#concrete elements =", Integer.toString( index.getTotalNumberOfSchemaElements() ) );

		//matcherStatistics.addStatistic( currentPhase, rootLabel, "#concrete elements (on average per abstract element) = ", Integer.toString( index.getTotalNumberOfConcreteElements() ) );

		//matcherStatistics.addStatistic( currentPhase, rootLabel, "#concrete elements (min/max per abstract element) = ", Integer.toString( index.getTotalNumberOfConcreteElements() ) + "/" + Integer.toString( index.getTotalNumberOfConcreteElements() ) );

		matcherStatistics.addStatistic( currentPhase, rootLabel, durationLabel + " =", ElapsedTimeCalculator.getTime( index.getDuration() ) );


		matcherStatistics.addVisualizedStatistic( currentPhase, rootLabel, (DefaultMutableTreeNode) index.visualize( guiLabel, guiProgressBar, rootLabel, null ) );
	}
}
