package org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.gui.guiManipulation.JFrameManipulation;
import org.ow2.choreos.matchingProcess.MatcherStatistics;
import org.ow2.choreos.matchingProcess.schemaRepresentation.SchemaMatcher;
import org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing.AbstractElement;
import org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing.ConcreteElementsIndex;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.PatternBasedSchemaElementIndexing.PatternBasedConcreteElementsIndex;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.pattern.Pattern;
import org.ow2.choreos.measurements.time.ElapsedTimeCalculator;


/**
 * 
 * @author dathanas
 *
 */
public abstract class PatternBasedSchemaMatcher extends SchemaMatcher {


	//Variables.
	protected double closenessDegreeThreshold, patternConfidenceThreshold;//user-provided.

	protected String wayOfFormingMultiparititeCorrespondences;//user-provided.


	protected int lastPatternID = 0;

	protected Map<Object, Object> acceptedPatterns;//The first object is the pattern and the second one is artificial object.

	protected int numOfEnumeratedPatterns;//These are the patterns that are produced and then, checked whether they are accepted or promising.

	protected List<Object> uncheckedPatterns;//These are accepted or promising patterns.


	private int totalAbstractElementsNumber = 0;


	//Constructors.
	protected PatternBasedSchemaMatcher( UserSettings userSettings, String currentPhase ){

		super( userSettings, currentPhase, false );


		initializePatternBasedSchemaMatcher( userSettings, currentPhase, null, null );
	}

	protected PatternBasedSchemaMatcher( UserSettings userSettings, String currentPhase, Object guiLabel, Object guiComponent ){

		super( userSettings, currentPhase, false, guiLabel, guiComponent );


		initializePatternBasedSchemaMatcher( userSettings, currentPhase, guiLabel, guiComponent );
	}

	protected PatternBasedSchemaMatcher( UserSettings userSettings, String currentPhase, Object guiLabel, Object guiComponent, String cLogFilePath ){

		super( userSettings, currentPhase, false, guiLabel, guiComponent, cLogFilePath );


		initializePatternBasedSchemaMatcher( userSettings, currentPhase, guiLabel, guiComponent );
	}

	private void initializePatternBasedSchemaMatcher( UserSettings userSettings, String currentPhase, Object guiLabel, Object guiComponent ){

		index = new PatternBasedConcreteElementsIndex( userSettings, guiLabel, guiComponent );


		addIndexStatistics( guiLabel, guiComponent, ConcreteElementsIndex.getVisualizationRootLabelForAllElements(), ConcreteElementsIndex.getLabelOfDuration() );


		closenessDegreeThreshold = (Double) userSettings.getOptionValue( PatternBasedSchemaMatchingController.thresholdOfClosenessDensityValueUserOptionLabel );

		patternConfidenceThreshold = userSettings.getPatternConfidenceThreshold();

		wayOfFormingMultiparititeCorrespondences = (String) userSettings.getOptionValue( PatternBasedSchemaMatchingController.formingMultiparititeCorrespondencesUserOptionLabel );


		PrintMessages.stdPrintLn( PrintMessages.matchingPrint, "closenessDegreeThreshold = " + closenessDegreeThreshold );

		PrintMessages.stdPrintLn( PrintMessages.matchingPrint, "patternConfidenceThreshold = " + patternConfidenceThreshold );

		PrintMessages.stdPrintLn( PrintMessages.matchingPrint, "wayOfFormingMultiparititeCorrespondences = " + wayOfFormingMultiparititeCorrespondences );
	}


	//Setters.
	protected void initializeSingletonPatterns( Map<Object, Object> abstractElements, Object guiLabel, Object guiProgressBar ){

		String prevGUILabel = JFrameManipulation.getLabel( guiLabel );

		JFrameManipulation.appendLabel( guiLabel, " (" + getLabelOfSingletonPatternsConstruction() + ")" );

		JFrameManipulation.resetProgressBarLabel( guiProgressBar );


		int counter = 0;

		Iterator<Object> i = abstractElements != null && abstractElements.keySet() != null ? abstractElements.keySet().iterator() : null;

		while( i != null && i.hasNext() ){

			int abstractElementNumber = (Integer) i.next();

			AbstractElement abstractElement = (AbstractElement) abstractElements.get( abstractElementNumber );


			Map<Object, Object> schemaSizes = index.getSchemaSizes();

			Pattern singletonPattern = new Pattern( ++lastPatternID, abstractElement, schemaSizes, logFilePath );

			++numOfEnumeratedPatterns;

			totalAbstractElementsNumber += singletonPattern.getSize();


			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "The singleton pattern: " );

			if( PrintMessages.toPrint( PrintMessages.matchingPrint ) ) singletonPattern.visualize( logFilePath );


			addInAcceptedPatterns( singletonPattern );

			addInUncheckedPatterns( singletonPattern );


			JFrameManipulation.setProgressBarLabel( guiProgressBar, ++counter, abstractElements.size() );
		}


		JFrameManipulation.setLabel( guiLabel, prevGUILabel );
	}

	protected void addInAcceptedPatterns( Pattern pattern ){

		if( pattern.getConfidence() > patternConfidenceThreshold ){

			acceptedPatterns = MapInsertion.put( pattern, true, acceptedPatterns, false );

			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "Pattern " + numOfEnumeratedPatterns + " : ACCEPTED." );
		}
	}

	protected void addInUncheckedPatterns( Pattern pattern ){

		if( pattern.getConfidence() > patternConfidenceThreshold ) uncheckedPatterns = ListInsertion.add( pattern, uncheckedPatterns );

		else{

			--lastPatternID;


			if( pattern.getConfidenceUpperBound() > patternConfidenceThreshold ){

				uncheckedPatterns = ListInsertion.add( pattern, uncheckedPatterns );

				PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "Pattern " + numOfEnumeratedPatterns + " : PROMISING." );
			}

			else PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "Pattern " + numOfEnumeratedPatterns + " : PRUNED." );
		}
	}

	protected void addPatternStatistics( Object guiLabel, Object guiProgressBar ){

		if( matcherStatistics == null ) matcherStatistics = new MatcherStatistics();


		matcherStatistics.addStatisticKind( currentPhase, getVisualizationRootLabel() );


		matcherStatistics.addStatistic( currentPhase, getVisualizationRootLabel(), "#patterns =", Integer.toString( getNumberOfPatterns() ) );

		matcherStatistics.addStatistic( currentPhase, getVisualizationRootLabel(), "#abstract elements =", Integer.toString( totalAbstractElementsNumber ) );

		matcherStatistics.addStatistic( currentPhase, getVisualizationRootLabel(), "#concrete elements (on average per pattern) = ", Integer.toString( 0 ) );

		matcherStatistics.addStatistic( currentPhase, getVisualizationRootLabel(), "#concrete elements (min/max per pattern) = ", Integer.toString( 0 ) + "/" + Integer.toString( 0 ) );

		matcherStatistics.addStatistic( currentPhase, getVisualizationRootLabel(), "patterns enumeration duration =", ElapsedTimeCalculator.getTime( currentPhaseDuration ) );


		matcherStatistics.addVisualizedStatistic( currentPhase, getVisualizationRootLabel(), (DefaultMutableTreeNode) visualize( guiLabel, guiProgressBar, null ) );
	}

	protected Pattern extendPattern( Pattern currentPattern, Interval1D currentPatternAbstractElementScope, int patternGrowthAbstractElementID, Object patternGrowthSchemaElements, Map<Object, Object> schemaSizes ){//It makes a deep copy of the 'currentPattern' and, concurrently, extends it with a new node and extends the pattern instantiations with 'patternGrowthSchemaElements'.

		Pattern newPattern = new Pattern( ++lastPatternID, currentPattern, currentPatternAbstractElementScope.getIntLeftEndpoint(), patternGrowthAbstractElementID, patternGrowthSchemaElements, schemaSizes, logFilePath );

		totalAbstractElementsNumber += newPattern.getSize();


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "The extended pattern:");

		if( PrintMessages.toPrint( PrintMessages.matchingPrint ) ) newPattern.visualize( logFilePath );


		return newPattern;
	}


	//Getters.
	private int getNumberOfPatterns(){

		return acceptedPatterns != null ? acceptedPatterns.size() : 0;
	}


	//Visualization.
	static public String getLabelOfSingletonPatternsConstruction(){

		return "patterns enumeration";
	}

	static public String getLabelOfSingletonPatternsVisualization(){

		return "patterns visualization";
	}

	static public String getVisualizationRootLabel(){

		return "Frequent embedded structural patterns";
	}

	protected Object visualize( Object guiLabel, Object guiProgressBar, String logFilePath ){

		String prevGUILabel = JFrameManipulation.getLabel( guiLabel );

		JFrameManipulation.appendLabel( guiLabel, " (" + getLabelOfSingletonPatternsVisualization() + ")" );

		JFrameManipulation.resetProgressBarLabel( guiProgressBar );


		boolean logFile = logFilePath != null ? true : false;

		DefaultMutableTreeNode root = null;


		int counter = 0;

		Iterator<Object> i = acceptedPatterns != null && acceptedPatterns.keySet() != null ? acceptedPatterns.keySet().iterator() : null;

		while( i != null && i.hasNext() ){

			Object pattern = i.next();


			if( root == null ) root = new DefaultMutableTreeNode( getVisualizationRootLabel() );


			Object retValue = ((Pattern) pattern).visualize( logFilePath );

			if( ! logFile ) root.add( (MutableTreeNode) retValue );


			JFrameManipulation.setProgressBarLabel( guiProgressBar, ++counter, acceptedPatterns.size() );
		}


		if( root == null ) root = new DefaultMutableTreeNode( "No " + getVisualizationRootLabel() );


		JFrameManipulation.setLabel( guiLabel, prevGUILabel );


    	return root;
	}
}
