package org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.holisticMatching.patternGrowthMatching;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.matchingProcess.Matching;
import org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing.AbstractElement;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.PatternBasedSchemaMatchingController;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.PatternBasedSchemaElementIndexing.PatternBasedConcreteElementsIndex;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.holisticMatching.HolisticPatternBasedSchemaMatcher;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.pattern.Pattern;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.pattern.PatternAbstractElement;


/**
 * 
 * @author dathanas
 *
 */
public class PatternGrowthMatcher extends HolisticPatternBasedSchemaMatcher {


	//Variables.
	private static String phase0 = "0. Indexing phase";
	private static String phase1 = "1. Pattern-initialization phase";
	private static String phase2 = "2. Pattern-growth phase";
	private static String phase3 = "3. Matching pattern-instantiations phase";

	private List<Double> confidenceOfAcceptedMatchings;


	//Verifiers.
	public static boolean isPatternGrowthMatcher( String matcherKind ){

		return matcherKind != null && matcherKind.equals( getMatcherKind() ) ? true : false;
	}


	//Constructors.
	public PatternGrowthMatcher( UserSettings userSettings ){

		super( userSettings, phase0 );


		nextPhase = phase1;
	}

	public PatternGrowthMatcher( UserSettings userSettings, Object guiLabel, Object guiComponent ){

		super( userSettings, phase0, guiLabel, guiComponent );


		nextPhase = phase1;
	}

	public PatternGrowthMatcher( UserSettings userSettings, Object guiLabel, Object guiComponent, String cLogFilePath ){

		super( userSettings, phase0, guiLabel, guiComponent, cLogFilePath );


		nextPhase = phase1;
	}


	//Getters.
	public static String getMatcherKind(){

		return "Holistic Pattern-growth Generation Algorithm";
	}

	static public String getStartPhase(){

		return phase0;
	}


	//Setters.
	public void match() {

		match( null, null );
	}

	public void match( Object guiLabel, Object guiComponent ) {

		if( ! interactive ) performAllPhases( guiLabel, guiComponent );

		else{

			if( currentPhase == phase0 ) performPhase1( guiLabel, guiComponent );

			else if( currentPhase == phase1 ) performPhase2();

			else if( currentPhase == phase2 ) performPhase3();
		}
	}

	private void performAllPhases( Object guiLabel, Object guiComponent ) {

		performPhase1( guiLabel, guiComponent );

		performPhase2();

		performPhase3();
	}

	private void performPhase1( Object guiLabel, Object guiComponent ) {

		PrintMessages.stdPrintLn( PrintMessages.matchingPrint, getMatcherKind() );


		currentPhase = phase1;

		nextPhase = phase2;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n########## START - " + currentPhase + " ##########" );


		initialize( guiLabel, guiComponent );


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n########## END - " + currentPhase + " ##########" );
	}

	private void performPhase2() {

		currentPhase = phase2;

		nextPhase = phase3;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n########## START - " + currentPhase + " ##########" );


		long startTime = System.currentTimeMillis();


		while( uncheckedPatterns != null && ! uncheckedPatterns.isEmpty() ){

			Pattern currentPattern = (Pattern) uncheckedPatterns.remove( uncheckedPatterns.size() - 1 );


			int numberOfAcceptedPatterns = acceptedPatterns != null ? acceptedPatterns.size() : 0;

			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n( # accepted patterns, # enumerated patterns, # unchecked patterns ) = ( " + numberOfAcceptedPatterns + ", " + numOfEnumeratedPatterns + ", " + uncheckedPatterns.size() + " )" );

			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "The current pattern: " );

			if( PrintMessages.toPrint( PrintMessages.matchingPrint ) ) currentPattern.visualize( logFilePath );


			List<Object> rightmostPathAbstractElements = currentPattern.getRightmostPathAbstractElements();

			if( rightmostPathAbstractElements != null ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t# rightmost nodes = " + rightmostPathAbstractElements.size() );

			for( int i = rightmostPathAbstractElements.size() - 1; rightmostPathAbstractElements != null && i >= 0; --i ){

				PatternAbstractElement currentPatternAbstractElement = (PatternAbstractElement) rightmostPathAbstractElements.get( i );


				int counter = i + 1; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t( " + counter + " ) : Examined rightmost node (abstract element) : " + currentPatternAbstractElement.getScope().getIntEndpoints() );


				AbstractElement queryingAbstractElement = constructQueryingAbstractElement( currentPattern, currentPatternAbstractElement );


				Map<Object, Object> patternGrowthAbstractElements = ( (PatternBasedConcreteElementsIndex) index ).lookUpPatternGrowthAbstractElements( queryingAbstractElement );


				Iterator<Object> patternGrowthAbstractElementsIterator = patternGrowthAbstractElements != null && patternGrowthAbstractElements.keySet() != null ? patternGrowthAbstractElements.keySet().iterator() : null;


				//if( i == rightmostPathAbstractElements.size() - 1 && ! hasValidLeaf( j, currentPatternAbstractElement ) ) MapRemoval.remove( currentPattern , acceptedPatterns );


				while( patternGrowthAbstractElementsIterator != null && patternGrowthAbstractElementsIterator.hasNext() ){

					int patternGrowthAbstractElementID = (Integer) patternGrowthAbstractElementsIterator.next();

					Object patternGrowthSchemaElements = patternGrowthAbstractElements.get( patternGrowthAbstractElementID );


					Map<Object, Object> schemaSizes = index.getSchemaSizes();

					Pattern extendedPattern = extendPattern( currentPattern, currentPatternAbstractElement.getScope(), patternGrowthAbstractElementID, patternGrowthSchemaElements, schemaSizes );

					++numOfEnumeratedPatterns;


					addInAcceptedPatterns( extendedPattern );

					addInUncheckedPatterns( extendedPattern );
				}
			}
		}


		currentPhaseDuration = System.currentTimeMillis() - startTime;

		totalDuration += currentPhaseDuration;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n########## END - " + currentPhase + " ##########" );
	}

	private boolean hasValidLeaf( Iterator<Object> patternGrowthAbstractElements, PatternAbstractElement currentPatternAbstractElement ){

		if( patternGrowthAbstractElements == null && ! ( (PatternBasedConcreteElementsIndex) index ).isComponentDeclaration( currentPatternAbstractElement.getPatternAbstractElementID() ) ) return false;


		return true;
	}

	private AbstractElement constructQueryingAbstractElement( Pattern currentPattern, PatternAbstractElement currentPatternAbstractElement ){

		AbstractElement queryingAbstractElement = currentPattern.constructQueryingAbstractElement( currentPatternAbstractElement, queryingSchemaElementsDataStructureKind );

		if( PrintMessages.toPrint( PrintMessages.matchingPrint ) ) queryingAbstractElement.visualize( logFilePath );


		return queryingAbstractElement;
	}


	private void performPhase3() {

		currentPhase = phase3;

		nextPhase = null;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n########## START - " + currentPhase + " ##########" );

		int numberOfAcceptedPatterns = acceptedPatterns != null ? acceptedPatterns.size() : 0;

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n# accepted patterns = " + numberOfAcceptedPatterns );


		long startTime = System.currentTimeMillis();


		Iterator<Object> acceptedPatternsIterator = acceptedPatterns != null && acceptedPatterns.keySet() != null ? acceptedPatterns.keySet().iterator() : null;

		while( acceptedPatternsIterator != null && acceptedPatternsIterator.hasNext() ){

			Pattern currentPattern = (Pattern) acceptedPatternsIterator.next();


			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "The current pattern: " );

			if( PrintMessages.toPrint( PrintMessages.matchingPrint ) ) currentPattern.visualize( logFilePath );


			List<Object> currentMatchings = currentPattern.generateMatchings( closenessDegreeThreshold, wayOfFormingMultiparititeCorrespondences, logFilePath );


			for( int i = 0; currentMatchings != null && i < currentMatchings.size(); ++i ){

				Matching currentMatching = (Matching) acceptedMatchings.get( i );


				ListInsertion.addSorted( currentMatching.getConfidence(), currentMatching, confidenceOfAcceptedMatchings, acceptedMatchings );

				if( acceptedMatchings.size() > k ){

					confidenceOfAcceptedMatchings.remove( 0 );

					acceptedMatchings.remove( 0 );
				}
			}
		}


		currentPhaseDuration = System.currentTimeMillis() - startTime;

		totalDuration += currentPhaseDuration;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n########## END - " + currentPhase + " ##########" );
	}
}
