package org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.pattern;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import org.ow2.choreos.basicAlgorithms.CartesianProduct;
import org.ow2.choreos.dataStructures.array.arrayInstantiation.ArrayInstantiation;
import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapInstantiation.MapInstantiation;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.queue.queueInsertion.QueueInsertion;
import org.ow2.choreos.dataStructures.queue.queueRemoval.QueueRemoval;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing.AbstractElement;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.PatternBasedSchemaMatchingController;
import org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.PatternBasedSchemaElementIndexing.PatternBasedConcreteElementsIndex;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;


/**
 * 
 * @author dathanas
 *
 */
public class Pattern {


	//Variables.
	private int patternID;


	private PatternAbstractElement rootPatternAbstractElement;

	private int size = 0;//The total number of abstract elements.


	private List<Object> orderingOfRefs2DocOfSchemas;//This is the ordering of the schemas (user-provided) to which the instantiations of the pattern belong.

	private Map<Object, Object> patternInstantiationIDs;//The first Object is the 'ref2DocOfSchemas' and the second is the id of its pattern instantiations.

	private Map<Object, List<Object>> patternInstantiations;//The first Object is the 'ref2DocOfSchemas' and the second is a list of 'PatternInstantiation' objects. The latter objects are the roots of different pattern instantiations.

	private Map<Object, Object> patternInstantiationMaxDensity;//The first Object is the 'ref2DocOfSchemas' and the second is the maximum density value of all the pattern instantiations.


	private double confidence, confidenceUpperBound;


	//Constructors.
	public Pattern( int cPatternID, AbstractElement abstractElement, Map<Object, Object> schemaSizes, String logFilePath ){//It is used for initializing singleton patterns.

		if( abstractElement != null ){

			patternID = cPatternID; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "Pattern " + patternID );


			rootPatternAbstractElement = new PatternAbstractElement( abstractElement.getAbstractElementID() );

			size = 1;


			createPatternInstantiations( rootPatternAbstractElement.getScope(), abstractElement, schemaSizes, logFilePath );
		}

		else{

			PrintMessages.print( "Pattern: Illegal (null) abstract element." );

			System.exit( 0 );
		}
	}

	@SuppressWarnings("unchecked")
	private void createPatternInstantiations( Interval1D abstractElementScope, AbstractElement abstractElement, Map<Object, Object> schemaSizes, String logFilePath ){

		double patternDensity = 1, patternCoverage = 1, patternDensityUpperBound = 1, patternCoverageUpperBound = 1;


		Map<Object, Object> concreteElementsOfAllSchemas = abstractElement.getConcreteElements();

		orderingOfRefs2DocOfSchemas = abstractElement.getOrderingOfRefs2DocOfSchemas();

		for( int i = 0; orderingOfRefs2DocOfSchemas != null && i < orderingOfRefs2DocOfSchemas.size(); ++i ){

			String ref2DocOfSchemas = (String) orderingOfRefs2DocOfSchemas.get( i ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tref2DocOfSchemas = " + ref2DocOfSchemas );


			double patternInstantiationsDensity = 0, patternInstantiationsCoverage = 0, patternInstantiationsDensityUpperBound = 0, patternInstantiationsCoverageUpperBound = 0;


			Object existingDataStructure = MapGetter.getValue( ref2DocOfSchemas, concreteElementsOfAllSchemas );

			List<Object> concreteElementsOfASchema = (List<Object>) existingDataStructure;


			List<Object> patternInstantiationsInASchema = null;

			for( int j = 0; concreteElementsOfASchema != null && j < concreteElementsOfASchema.size(); ++j ){

				int patternInstantiationID = getPatternInstantionID( ref2DocOfSchemas ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\tpatternInstantiation = " + patternInstantiationID );


				PatternInstantiation patternInstantiation = new PatternInstantiation( patternInstantiationID, abstractElementScope.getIntLeftEndpoint(), concreteElementsOfASchema.get( j ) );

				patternInstantiationsInASchema = ListInsertion.add( patternInstantiation, patternInstantiationsInASchema );


				int schemaSize = (Integer) MapGetter.getValue( ref2DocOfSchemas, schemaSizes );

				double patternInstantiationDensity = patternInstantiation.getDensity( schemaSize, false ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationDensity = " + patternInstantiationDensity );

				if( patternInstantiationDensity > patternInstantiationsDensity ) patternInstantiationsDensity = patternInstantiationDensity;

				double patternInstantiationDensityUpperBound = patternInstantiation.getDensity( schemaSize, true ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationDensityUpperBound = " + patternInstantiationDensityUpperBound );

				if( patternInstantiationDensityUpperBound > patternInstantiationsDensityUpperBound ) patternInstantiationsDensityUpperBound = patternInstantiationDensityUpperBound;


				double patternInstantiationCoverage = patternInstantiation.getCoverage( schemaSize, false ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationCoverage = " + patternInstantiationCoverage );

				if( patternInstantiationCoverage > patternInstantiationsCoverage ) patternInstantiationsCoverage = patternInstantiationCoverage;

				double patternInstantiationCoverageUpperBound = patternInstantiation.getCoverage( schemaSize, true ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationCoverageUpperBound = " + patternInstantiationCoverageUpperBound );

				if( patternInstantiationCoverageUpperBound > patternInstantiationsCoverageUpperBound ) patternInstantiationsCoverageUpperBound = patternInstantiationCoverageUpperBound;
			}


			patternInstantiations = MapInsertion.putInMapOfList( ref2DocOfSchemas, patternInstantiationsInASchema, patternInstantiations, true );


			patternInstantiationMaxDensity = MapInsertion.put( ref2DocOfSchemas, patternInstantiationsDensity, patternInstantiationMaxDensity, true );


			patternDensity = patternDensity * patternInstantiationsDensity;

			patternCoverage = patternCoverage * patternInstantiationsCoverage;

			patternDensityUpperBound = patternDensityUpperBound * patternInstantiationsDensityUpperBound;

			patternCoverageUpperBound = patternCoverageUpperBound * patternInstantiationsCoverageUpperBound;
		}


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternDensity = " + patternDensity );

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternCoverage = " + patternCoverage );

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternDensityUpperBound = " + patternDensityUpperBound );

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternCoverageUpperBound = " + patternCoverageUpperBound );

		confidence = patternDensity * patternCoverage; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tconfidence = " + confidence );

		confidenceUpperBound = patternDensityUpperBound * patternCoverageUpperBound; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tconfidenceUpperBound = " + confidenceUpperBound );
	}


	public Pattern( int newPatternID, Pattern currentPattern, int currentPatternAbstractElementScopeLeftEndpoint, int patternGrowthAbstractElementID, Object patternGrowthSchemaElements, Map<Object, Object> schemaSizes ){//Copy constructor: it is used by the class 'PatternBasedSchemaMatcher' for creating a deep copy of a pattern and extending it.

		newPattern( newPatternID, currentPattern, currentPatternAbstractElementScopeLeftEndpoint, patternGrowthAbstractElementID, patternGrowthSchemaElements, schemaSizes, null );
	}

	public Pattern( int newPatternID, Pattern currentPattern, int currentPatternAbstractElementScopeLeftEndpoint, int patternGrowthAbstractElementID, Object patternGrowthSchemaElements, Map<Object, Object> schemaSizes, String logFilePath ){//Copy constructor: it is used by the class 'PatternBasedSchemaMatcher' for creating a deep copy of a pattern and extending it.

		newPattern( newPatternID, currentPattern, currentPatternAbstractElementScopeLeftEndpoint, patternGrowthAbstractElementID, patternGrowthSchemaElements, schemaSizes, logFilePath );
	}

	private void newPattern( int newPatternID, Pattern currentPattern, int currentPatternAbstractElementScopeLeftEndpoint, int patternGrowthAbstractElementID, Object patternGrowthSchemaElements, Map<Object, Object> schemaSizes, String logFilePath ){

		if( currentPattern != null ){

			patternID = newPatternID; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "Pattern " + patternID );

			size = currentPattern.size;


			rootPatternAbstractElement = new PatternAbstractElement( (PatternAbstractElement) currentPattern.rootPatternAbstractElement );

			int newPatternAbstractElementScopeLeftEndpoint = addPatternAbstractElement( currentPatternAbstractElementScopeLeftEndpoint, patternGrowthAbstractElementID, patternGrowthSchemaElements );


			orderingOfRefs2DocOfSchemas = currentPattern.getOrderingOfRefs2DocOfSchemas();


			extendPatternInstantiations( currentPattern.patternInstantiations, currentPatternAbstractElementScopeLeftEndpoint, newPatternAbstractElementScopeLeftEndpoint, patternGrowthSchemaElements, schemaSizes, logFilePath );
		}

		else{

			PrintMessages.print( "Pattern: Illegal (null) copy abstract element." );

			System.exit( 0 );
		}
	}

	private int addPatternAbstractElement( int currentPatternAbstractElementScopeLeftEndpoint, int patternGrowthAbstractElementID, Object patternGrowthSchemaElements ){

		int newPatternAbstractElementScopeLeftEndpoint = size;

		PatternAbstractElement newPatternAbstractElement = new PatternAbstractElement( patternGrowthAbstractElementID, size, size );

		++size;

		PatternAbstractElement newInstanceOfCurrentPatternAbstractElement = getNewInstanceOfCurrentPatternAbstractElement( currentPatternAbstractElementScopeLeftEndpoint );

		newInstanceOfCurrentPatternAbstractElement.addChild( newPatternAbstractElement );

		updateScopesInTheRightmostPath( newInstanceOfCurrentPatternAbstractElement );


		return newPatternAbstractElementScopeLeftEndpoint;
	}

	private PatternAbstractElement getNewInstanceOfCurrentPatternAbstractElement( int currentPatternAbstractElementScopeLeftEndpoint ){

		PatternAbstractElement tempPatternAbstractElement = rootPatternAbstractElement; 

		while( tempPatternAbstractElement != null ){

			if( tempPatternAbstractElement.getScope().getIntLeftEndpoint() == currentPatternAbstractElementScopeLeftEndpoint ) return tempPatternAbstractElement;

			tempPatternAbstractElement = tempPatternAbstractElement.getRightmostChild();
		}


		return null;
	}

	private void updateScopesInTheRightmostPath( PatternAbstractElement currentPatternAbstractElement ){

		PatternAbstractElement tmpPatternAbstractElement = rootPatternAbstractElement;

		while( tmpPatternAbstractElement != currentPatternAbstractElement ){

			tmpPatternAbstractElement.increaseRightEndpoint();

			tmpPatternAbstractElement = (PatternAbstractElement) tmpPatternAbstractElement.getRightmostChild();
		}
	}

	private void extendPatternInstantiations( Map<Object, List<Object>> cPatternInstantiations, int currentPatternAbstractElementScopeLeftEndpoint, int newPatternAbstractElementScopeLeftEndpoint, Object patternGrowthSchemaElements, Map<Object, Object> schemaSizes, String logFilePath ){

		double patternDensity = 1, patternCoverage = 1, patternDensityUpperBound = 1, patternCoverageUpperBound = 1;


		for( int i = 0; orderingOfRefs2DocOfSchemas != null && i < orderingOfRefs2DocOfSchemas.size(); ++i ){

			String ref2DocOfSchemas = (String) orderingOfRefs2DocOfSchemas.get( i ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tref2DocOfSchemas = " + ref2DocOfSchemas );

			List<Object> patternInstantiationsInASchema = MapGetter.getValueOfMapOfList( ref2DocOfSchemas, cPatternInstantiations );


			double patternInstantiationsDensity = 0, patternInstantiationsCoverage = 0, patternInstantiationsDensityUpperBound = 0, patternInstantiationsCoverageUpperBound = 0;


			List<Object> extendedPatternInstantiationsInASchema = null; 

			for( int j = 0; patternInstantiationsInASchema != null && j < patternInstantiationsInASchema.size(); ++j ){

				PatternInstantiation patternInstantiation = (PatternInstantiation) patternInstantiationsInASchema.get( j );


				Object currentSchemaElement = patternInstantiation.getConcreteElement( currentPatternAbstractElementScopeLeftEndpoint );

				Map<Object, Object> pairOfSchemaElementAndRightmostChildScopeRightEndpoint = patternInstantiation.getPairOfConcreteElementAndRightmostChildScopeRightEndpoint( currentSchemaElement );


				List<Object> patternGrowthSchemaElementsOfCurrentSchemaElement = PatternBasedConcreteElementsIndex.getPatternGrowthSchemaElements( ref2DocOfSchemas, pairOfSchemaElementAndRightmostChildScopeRightEndpoint, patternGrowthSchemaElements );

				for( int k = 0; patternGrowthSchemaElementsOfCurrentSchemaElement != null && k < patternGrowthSchemaElementsOfCurrentSchemaElement.size(); ++k ){

					Object patternGrowthSchemaElement = patternGrowthSchemaElementsOfCurrentSchemaElement.get( k );

					int patternInstantiationID = getPatternInstantionID( ref2DocOfSchemas );


					PatternInstantiation newPatternInstantiation = new PatternInstantiation( patternInstantiationID, patternInstantiation ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\tpatternInstantiation = " + newPatternInstantiation.getPatternInstantiationID() );

					newPatternInstantiation.add( newPatternAbstractElementScopeLeftEndpoint, patternGrowthSchemaElement, currentSchemaElement );


					extendedPatternInstantiationsInASchema = ListInsertion.add( newPatternInstantiation, extendedPatternInstantiationsInASchema );


					int schemaSize = (Integer) MapGetter.getValue( ref2DocOfSchemas, schemaSizes );

					double patternInstantiationDensity = newPatternInstantiation.getDensity( schemaSize, false ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationDensity = " + patternInstantiationDensity );

					if( patternInstantiationDensity > patternInstantiationsDensity ) patternInstantiationsDensity = patternInstantiationDensity;

					double patternInstantiationDensityUpperBound = newPatternInstantiation.getDensity( schemaSize, true ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationDensityUpperBound = " + patternInstantiationDensityUpperBound );

					if( patternInstantiationDensityUpperBound > patternInstantiationsDensityUpperBound ) patternInstantiationsDensityUpperBound = patternInstantiationDensityUpperBound;


					double patternInstantiationCoverage = newPatternInstantiation.getCoverage( schemaSize, false ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationCoverage = " + patternInstantiationCoverage );

					if( patternInstantiationCoverage > patternInstantiationsCoverage ) patternInstantiationsCoverage = patternInstantiationCoverage;

					double patternInstantiationCoverageUpperBound = newPatternInstantiation.getCoverage( schemaSize, true ); PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\tpatternInstantiationCoverageUpperBound = " + patternInstantiationCoverageUpperBound );

					if( patternInstantiationCoverageUpperBound > patternInstantiationsCoverageUpperBound ) patternInstantiationsCoverageUpperBound = patternInstantiationCoverageUpperBound;
				}
			}

			patternInstantiations = MapInsertion.putInMapOfList( ref2DocOfSchemas, extendedPatternInstantiationsInASchema, patternInstantiations, true );


			patternInstantiationMaxDensity = MapInsertion.put( ref2DocOfSchemas, patternInstantiationsDensity, patternInstantiationMaxDensity, true );


			patternDensity = patternDensity * patternInstantiationsDensity;

			patternCoverage = patternCoverage * patternInstantiationsCoverage;

			patternDensityUpperBound = patternDensityUpperBound * patternInstantiationsDensityUpperBound;

			patternCoverageUpperBound = patternCoverageUpperBound * patternInstantiationsCoverageUpperBound;
		}


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternDensity = " + patternDensity );

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternCoverage = " + patternCoverage );

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternDensityUpperBound = " + patternDensityUpperBound );

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tpatternCoverageUpperBound = " + patternCoverageUpperBound );

		confidence = patternDensity * patternCoverage; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tconfidence = " + confidence );

		confidenceUpperBound = patternDensityUpperBound * patternCoverageUpperBound; PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\tconfidenceUpperBound = " + confidenceUpperBound );
	}


	//Getters.
	public int getPatternID(){

		return patternID;
	}

	public int getSize(){

		return size;
	}

	private List<Object> getOrderingOfRefs2DocOfSchemas(){

		return orderingOfRefs2DocOfSchemas;
	}

	public AbstractElement constructQueryingAbstractElement( PatternAbstractElement currentPatternAbstractElement, String queryingSchemaElementsDataStructureKind ){

		AbstractElement abstractElement = null;


		Iterator<Object> i = patternInstantiations != null && patternInstantiations.keySet() != null ? patternInstantiations.keySet().iterator() : null;

		while( i != null && i.hasNext() ){

			String ref2DocOfSchema = (String) i.next();


			List<Object> patternInstantiationsInASchema = MapGetter.getValueOfMapOfList( ref2DocOfSchema, patternInstantiations );

			for( int j = 0; patternInstantiationsInASchema != null && j < patternInstantiationsInASchema.size(); ++j ){

				PatternInstantiation patternInstantiation = (PatternInstantiation) patternInstantiationsInASchema.get( j );

				Object concreteElement = patternInstantiation.getConcreteElement( currentPatternAbstractElement.getScope().getIntLeftEndpoint() );

				Map<Object, Object> pairOfSchemaElementAndRightmostChildScopeRightEndpoint = patternInstantiation.getPairOfConcreteElementAndRightmostChildScopeRightEndpoint( concreteElement );


				if( abstractElement == null ) abstractElement = new AbstractElement( currentPatternAbstractElement.getPatternAbstractElementID(), queryingSchemaElementsDataStructureKind );

				abstractElement.addConcreteElement( ref2DocOfSchema, pairOfSchemaElementAndRightmostChildScopeRightEndpoint );
			}
		}


		return abstractElement;
	}

	protected int getFrequency(){

		return patternInstantiations != null ? patternInstantiations.size() : 0;
	}

	private int getPatternInstantionID( String ref2DocOfSchemas ){

		Integer patternInstantiationID = MapGetter.exists( ref2DocOfSchemas, patternInstantiationIDs ) ? (Integer) MapGetter.getValue( ref2DocOfSchemas, patternInstantiationIDs ) : 0;

		patternInstantiationIDs = MapInsertion.put( ref2DocOfSchemas, ++patternInstantiationID, patternInstantiationIDs, true );


		return patternInstantiationID;
	}

	public List<Object> getRightmostPathAbstractElements(){

		List<Object> rightmostPathAbstractElements = null;


		rightmostPathAbstractElements = ListInsertion.add( rootPatternAbstractElement, rightmostPathAbstractElements );

		PatternAbstractElement currentAbstractElement = rootPatternAbstractElement; 

		while( currentAbstractElement != null ){

			currentAbstractElement = (PatternAbstractElement) currentAbstractElement.getRightmostChild();

			rightmostPathAbstractElements = ListInsertion.add( currentAbstractElement, rightmostPathAbstractElements );
		}


		return rightmostPathAbstractElements;
	}

	public double getConfidence(){

		return confidence;
	}

	public double getConfidenceUpperBound(){

		return confidenceUpperBound;
	}


	public List<Object> generateMatchings( double closenessDegreeThreshold, String wayOfFormingMultiparititeCorrespondences ){

		return generateMatchings( closenessDegreeThreshold, wayOfFormingMultiparititeCorrespondences, null );
	}


	private int repetitions;//TODO

	public List<Object> generateMatchings( double closenessDegreeThreshold, String wayOfFormingMultiparititeCorrespondences, String logFilePath ){

		List<Object> matchings = null;


		determinePromisingPatternInstantiations( closenessDegreeThreshold, logFilePath );


		boolean inGroupsOfSuccessiveSiblingOfSimilarKinds = wayOfFormingMultiparititeCorrespondences != null && wayOfFormingMultiparititeCorrespondences.equals( PatternBasedSchemaMatchingController.linkCorrespondingElementsUserOption ) ? false : true;

		int maximumNumberOfSiblings = inGroupsOfSuccessiveSiblingOfSimilarKinds ? setMaximumNumberOfSiblings( wayOfFormingMultiparititeCorrespondences ) : -1;


		calculateCartesianProductOfPatternInstantiationsAndMatchings( inGroupsOfSuccessiveSiblingOfSimilarKinds, maximumNumberOfSiblings, logFilePath );


		return matchings;
	}

	private void determinePromisingPatternInstantiations( double closenessDegreeThreshold, String logFilePath ){

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t---------- START - Determination of promising pattern-instantiations ----------" );


		for( int i = 0; orderingOfRefs2DocOfSchemas != null && i < orderingOfRefs2DocOfSchemas.size(); ++i ){

			String ref2DocOfSchemas = (String) orderingOfRefs2DocOfSchemas.get( i );

			double maxDensity = (Double) MapGetter.getValue( ref2DocOfSchemas, patternInstantiationMaxDensity );


			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t" + ref2DocOfSchemas + " ( maxDensity = " + maxDensity + " )" );


			List<Object> patternInstantiationsInASchema = MapGetter.getValueOfMapOfList( ref2DocOfSchemas, patternInstantiations );


			List<Object> promisingPatternInstantiations = null;

			for( int j = 0; patternInstantiationsInASchema != null && j < patternInstantiationsInASchema.size(); ++j ){

				PatternInstantiation patternInstantiation = (PatternInstantiation) patternInstantiationsInASchema.get( j );

				double density = patternInstantiation.getDensity();

				boolean nonPromising = maxDensity - density > closenessDegreeThreshold;

				if( ! nonPromising ) promisingPatternInstantiations = ListInsertion.add( patternInstantiation, promisingPatternInstantiations );

				PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t" + patternInstantiation.getPatternInstantiationID() + " ( density = " + density + " - nonPromising = " + nonPromising + " )" );
			}


			patternInstantiations = MapInsertion.putInMapOfList( ref2DocOfSchemas, promisingPatternInstantiations, patternInstantiations, true );
		}


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t---------- END - Determination of promising pattern-instantiations ----------" );
	}

	private void calculateCartesianProductOfPatternInstantiationsAndMatchings( boolean inGroupsOfSuccessiveSiblingOfSimilarKinds, int maximumNumberOfSiblings, String logFilePath ){

		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t---------- START - Calculation of the cartesian product of pattern instantiations ----------" );


		int numberOfSchemas = patternInstantiations != null ? patternInstantiations.size() : 0;

		Map<Object, List<Object>> copyOfPatternInstantiations = numberOfSchemas > 0 ? MapInstantiation.newMapOfLists() : null;

		int[] sizeOfPatternInstantiations = ArrayInstantiation.newIntArray( numberOfSchemas );

		boolean[] isOrderedPatternInstantiation = ArrayInstantiation.newbooleanArray( numberOfSchemas );

		int[] partialProductsOfSizeOfPatternInstantiations = ArrayInstantiation.newIntArray( numberOfSchemas );


		CartesianProduct.getPartialProductsOfSizeOfSets( true, patternInstantiations, copyOfPatternInstantiations, null, orderingOfRefs2DocOfSchemas, sizeOfPatternInstantiations, isOrderedPatternInstantiation, partialProductsOfSizeOfPatternInstantiations );


		int sizeOfCartesianProduct = partialProductsOfSizeOfPatternInstantiations != null ? partialProductsOfSizeOfPatternInstantiations[ partialProductsOfSizeOfPatternInstantiations.length - 1 ] : 0;

		boolean[] settedTuples = ArrayInstantiation.newbooleanArray( sizeOfCartesianProduct );


		Object[][] tupleOfPatternInstantiations = partialProductsOfSizeOfPatternInstantiations != null ? ArrayInstantiation.newArrayOfArray( sizeOfCartesianProduct, partialProductsOfSizeOfPatternInstantiations.length ) : null;


		for( int tupleCounter = 0; partialProductsOfSizeOfPatternInstantiations != null && tupleCounter < sizeOfCartesianProduct; ++tupleCounter ){

			CartesianProduct.setTuple( orderingOfRefs2DocOfSchemas, copyOfPatternInstantiations, tupleCounter, partialProductsOfSizeOfPatternInstantiations, sizeOfPatternInstantiations, tupleOfPatternInstantiations, settedTuples );

			generateMatchingsForATupleOfPatternInstantiations( tupleCounter, tupleOfPatternInstantiations[ tupleCounter ], inGroupsOfSuccessiveSiblingOfSimilarKinds, maximumNumberOfSiblings, logFilePath );
		}


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t---------- END - Calculation of the cartesian product of pattern instantiations ----------" );
	}

	private List<Object> generateMatchingsForATupleOfPatternInstantiations( int tupleCounter, Object[] tupleOfPatternInstantiations, boolean inGroupsOfSuccessiveSiblingOfSimilarKinds, int maximumNumberOfSiblings, String logFilePath ){

		List<Object> matchings = null;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t---------- START - Generate matchings for tuple " + tupleCounter + " of pattern instantiations ----------" );


		boolean exist = true;


		Queue<Object> queueOfNonPermutablePatternAbstractElements = QueueInsertion.add( rootPatternAbstractElement, null );

		while( ! queueOfNonPermutablePatternAbstractElements.isEmpty() && exist ){//While the whole pattern is not totally traversed.

			PatternAbstractElement currentNonPermutablePatternAbstractElement = (PatternAbstractElement) QueueRemoval.remove( queueOfNonPermutablePatternAbstractElements );

			int concreteElementScopeLeftEndpoint = currentNonPermutablePatternAbstractElement.getScope().getIntLeftEndpoint();

			PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\tPattern abstract element = " + concreteElementScopeLeftEndpoint );


			exist = checkTheExistenceOfMultipartiteCorrespondence( tupleOfPatternInstantiations, concreteElementScopeLeftEndpoint, logFilePath );

			if( exist ){

				if( ! inGroupsOfSuccessiveSiblingOfSimilarKinds ) queueOfNonPermutablePatternAbstractElements = QueueInsertion.addAll( currentNonPermutablePatternAbstractElement.getChildren(), queueOfNonPermutablePatternAbstractElements );

				else{

					Map<Object, List<Object>> groupsOfLeafChildren = currentNonPermutablePatternAbstractElement.getLeafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds();

					Map<Object, List<Object>> groupsOfNonLeafChildren = currentNonPermutablePatternAbstractElement.getNonLeafSuccessiveChildren();


					int numberOfGroupsOfLeafChildren = groupsOfLeafChildren != null ? groupsOfLeafChildren.size() : 0;

					int numberOfGroupsOfNonLeafChildren = groupsOfNonLeafChildren != null ? groupsOfNonLeafChildren.size() : 0;

					int numberOfGroupsOfChildren = numberOfGroupsOfLeafChildren + numberOfGroupsOfNonLeafChildren;


					PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\tnumberOfGroupsOfChildren = " + numberOfGroupsOfChildren );

					for( int i = 0; i < numberOfGroupsOfChildren; ++i ){

						if( MapGetter.existsInMapOfList( i, groupsOfLeafChildren ) ){

							PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\tGroup " + i + " leaves" );

							List<Object> iGroupOfLeafChildren = MapGetter.getValueOfMapOfList( i, groupsOfLeafChildren );

							if( iGroupOfLeafChildren.size() == 1 ) queueOfNonPermutablePatternAbstractElements = QueueInsertion.add( iGroupOfLeafChildren.get( 0 ), queueOfNonPermutablePatternAbstractElements );

							else generateMatchingsForLeafSiblingElementsOfSimilarKinds( concreteElementScopeLeftEndpoint, tupleOfPatternInstantiations, iGroupOfLeafChildren, logFilePath );
						}

						else if( MapGetter.existsInMapOfList( i, groupsOfNonLeafChildren ) ){

							PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\tGroup " + i + " non-leaves" );

							List<Object> iGroupOfNonLeafChildren = MapGetter.getValueOfMapOfList( i, groupsOfNonLeafChildren );

							queueOfNonPermutablePatternAbstractElements = QueueInsertion.addAll( iGroupOfNonLeafChildren, queueOfNonPermutablePatternAbstractElements );
						}
					}	
				}
			}
		}


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t---------- END - Generate matchings for tuple " + tupleCounter + " of pattern instantiations ----------" );


		return exist ? matchings : null;
	}

	private boolean checkTheExistenceOfMultipartiteCorrespondence( Object[] tupleOfPatternInstantiations, int concreteElementScopeLeftEndpoint, String logFilePath ){

		double product = 1;


		for( int schemaIndex1 = 0; product != 0 && tupleOfPatternInstantiations != null && schemaIndex1 < tupleOfPatternInstantiations.length; ++schemaIndex1 ){

			PatternInstantiation patternInstantiation1 = (PatternInstantiation) tupleOfPatternInstantiations[ schemaIndex1 ];

			Object concreteElement1 = patternInstantiation1.getConcreteElement( concreteElementScopeLeftEndpoint );


			for( int schemaIndex2 = schemaIndex1 + 1; product != 0 && schemaIndex2 < tupleOfPatternInstantiations.length; ++schemaIndex2 ){

				PatternInstantiation patternInstantiation2 = (PatternInstantiation) tupleOfPatternInstantiations[ schemaIndex2 ];

				Object concreteElement2 = patternInstantiation2.getConcreteElement( concreteElementScopeLeftEndpoint );


				PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t( " + Tokenizer.getSuffix( (String) orderingOfRefs2DocOfSchemas.get( schemaIndex1 ), "\\" ) + " : " + patternInstantiation1.getPatternInstantiationID() + " - concrete element " + concreteElementScopeLeftEndpoint + " : " + SchemaRepresentationController.getName( concreteElement1 )+ ", " + Tokenizer.getSuffix( (String) orderingOfRefs2DocOfSchemas.get( schemaIndex2 ), "\\" ) + " : " + patternInstantiation2.getPatternInstantiationID() + " - concrete element " + concreteElementScopeLeftEndpoint + " : " + SchemaRepresentationController.getName( concreteElement2 )+ " )" );

				//Calculate product == 0


				PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\trepetition = " + ++repetitions );//TODO
			}
		}


		return product > 0 ? true : false; 
	}

	private int setMaximumNumberOfSiblings( String wayOfFormingMultiparititeCorrespondences ){

		int hintMaximumNumberOfSiblings = 0;//TODO: Calculate.


		int intUserProvidedMaximumNumberOfSiblings = hintMaximumNumberOfSiblings;

		if( wayOfFormingMultiparititeCorrespondences != null && ( wayOfFormingMultiparititeCorrespondences.equals( PatternBasedSchemaMatchingController.greedilyLinkCorrespondingElementsUserOption ) || wayOfFormingMultiparititeCorrespondences.equals( PatternBasedSchemaMatchingController.cartesianProductAndGredyCombinationOfElementsUserOption ) ) ){

			String userProvidedMaximumNumberOfSiblings;

			userProvidedMaximumNumberOfSiblings = JOptionPane.showInputDialog ( "The hint value for the threshold in the number of sibling elements is: " + hintMaximumNumberOfSiblings + "\nEnter the value for this threshold that you prefer:" );

			if( userProvidedMaximumNumberOfSiblings != null && ! userProvidedMaximumNumberOfSiblings.isEmpty() ) intUserProvidedMaximumNumberOfSiblings = Integer.parseInt ( userProvidedMaximumNumberOfSiblings );

			JOptionPane.showMessageDialog ( null, "Your value is " + intUserProvidedMaximumNumberOfSiblings, "Results", JOptionPane.PLAIN_MESSAGE ); 
		}


		return intUserProvidedMaximumNumberOfSiblings;
	}

	private List<Object> generateMatchingsForLeafSiblingElementsOfSimilarKinds( int abstractElementScopeLeftEndpoint, Object[] tupleOfPatternInstantiations, List<Object> leafSiblingAbstractElementsOfSimilarKinds, String logFilePath ){

		List<Object> matchings = null;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t---------- Generate matchings for the leaf children of abstract element " + abstractElementScopeLeftEndpoint + " ----------" );


		Map<Object, List<Object>> leafSiblingConcreteElementsOfSimilarKinds = null;//The first Object is a 'ref2DocOfSchemas'.

		Map<Object, Object> orderedLeafSiblingConcreteElementsOfSimilarKinds = null;//The first Object is a 'ref2DocOfSchemas'.


		for( int i = 0; leafSiblingAbstractElementsOfSimilarKinds != null && i < leafSiblingAbstractElementsOfSimilarKinds.size(); ++i ){

			PatternAbstractElement leafSiblingAbstractElementOfSimilarKinds = (PatternAbstractElement) leafSiblingAbstractElementsOfSimilarKinds.get( i );


			for( int schemaIndex = 0; tupleOfPatternInstantiations != null && schemaIndex < tupleOfPatternInstantiations.length; ++schemaIndex ){

				Object ref2DocOfSchemas = orderingOfRefs2DocOfSchemas.get( schemaIndex );

				PatternInstantiation patternInstantiation = (PatternInstantiation) tupleOfPatternInstantiations[ schemaIndex ];


				if( i == 0 ){//It suffices to check the ordering property only for the first sibling! The same holds for the other siblings.

					int parentConcreteElementScopeLeftEndpoint = abstractElementScopeLeftEndpoint;

					Object parentConcreteElement = patternInstantiation.getConcreteElement( parentConcreteElementScopeLeftEndpoint );

					orderedLeafSiblingConcreteElementsOfSimilarKinds = MapInsertion.put( ref2DocOfSchemas, SchemaRepresentationController.hasOrderedChildren( parentConcreteElement ), orderedLeafSiblingConcreteElementsOfSimilarKinds, true );
				}


				int concreteElementScopeLeftEndpoint = leafSiblingAbstractElementOfSimilarKinds.getScope().getIntLeftEndpoint();

				Object concreteElement = patternInstantiation.getConcreteElement( concreteElementScopeLeftEndpoint );

				List<Object> leafSiblingConcreteElementsOfSimilarKindsOfARef2DocOfSchemas = MapGetter.getValueOfMapOfList( ref2DocOfSchemas, leafSiblingConcreteElementsOfSimilarKinds );

				leafSiblingConcreteElementsOfSimilarKindsOfARef2DocOfSchemas = ListInsertion.add( concreteElement, leafSiblingConcreteElementsOfSimilarKindsOfARef2DocOfSchemas );

				leafSiblingConcreteElementsOfSimilarKinds = MapInsertion.putInMapOfList( ref2DocOfSchemas, leafSiblingConcreteElementsOfSimilarKindsOfARef2DocOfSchemas, leafSiblingConcreteElementsOfSimilarKinds, true );


				PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\trepetition = " + ++repetitions );//TODO
			}
		}


		calculateCartesianProductOfLeafSiblingConcreteElementsOfSimilarKindsAndMatchings( leafSiblingConcreteElementsOfSimilarKinds, orderedLeafSiblingConcreteElementsOfSimilarKinds, logFilePath );


		return matchings;
	}

	/**
	 * 
	 * @param leafSiblingConcreteElementsOfSimilarKinds
	 * @param logFilePath
	 * @return
	 * 
	 * This function calculates the cartesian product in conjunction with the check of the existence of a multipartite correspondence at once for efficiency reasons. 
	 */
	private List<Object> calculateCartesianProductOfLeafSiblingConcreteElementsOfSimilarKindsAndMatchings( Map<Object, List<Object>> leafSiblingConcreteElementsOfSimilarKinds, Map<Object, Object> orderedLeafSiblingConcreteElementsOfSimilarKinds, String logFilePath ){

		List<Object> matchings = null;


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t---------- START - Calculation of the cartesian product of leaf sibling concrete elements of similar kinds ----------" );


		int numberOfSschemas = leafSiblingConcreteElementsOfSimilarKinds != null ? leafSiblingConcreteElementsOfSimilarKinds.size() : 0;

		int[] numberOfLeafSiblingConcreteElementsOfSimilarKinds = ArrayInstantiation.newIntArray( numberOfSschemas );

		boolean[] areOrderedLeafSiblingConcreteElementsOfSimilarKinds = ArrayInstantiation.newbooleanArray( numberOfSschemas );

		int[] partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds = ArrayInstantiation.newIntArray( numberOfSschemas );


		//CartesianProduct.getPartialProductsOfSizeOfSets( leafSiblingConcreteElementsOfSimilarKinds, orderedLeafSiblingConcreteElementsOfSimilarKinds, efs2DocOfSchemas, numberOfLeafSiblingConcreteElementsOfSimilarKinds, areOrderedLeafSiblingConcreteElementsOfSimilarKinds, partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds );


		int sizeOfCartesianProduct = partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds != null ? partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds[ partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds.length - 1 ] : 0;


		boolean exist = false; 

		/*for( int tupleCounter = 0; partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds != null && tupleCounter < sizeOfCartesianProduct; ++tupleCounter ){

			double product = 1;


			for( int schemaIndex1 = 0; product != 0 && partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds != null && schemaIndex1 < partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds.length; ++schemaIndex1 ){

				int repetition = partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds[ schemaIndex1 ] / numberOfLeafSiblingConcreteElementsOfSimilarKinds [ schemaIndex1 ];

				int valueIndex = ( tupleCounter / repetition ) % numberOfLeafSiblingConcreteElementsOfSimilarKinds [ schemaIndex1 ];

				Object concreteElement1 = CartesianProduct.getValue( schemaIndex1, refs2DocOfSchemas, valueIndex, leafSiblingConcreteElementsOfSimilarKinds );


				for( int schemaIndex2 = schemaIndex1 + 1; product != 0 && partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds != null && schemaIndex2 < partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds.length; ++schemaIndex2 ){

					repetition = partialProductsOfNumberOfLeafSiblingConcreteElementsOfSimilarKinds[ schemaIndex2 ] / numberOfLeafSiblingConcreteElementsOfSimilarKinds [ schemaIndex2 ];

					valueIndex = ( tupleCounter / repetition ) % numberOfLeafSiblingConcreteElementsOfSimilarKinds [ schemaIndex2 ];

					Object concreteElement2 = CartesianProduct.getValue( schemaIndex2, refs2DocOfSchemas, valueIndex, leafSiblingConcreteElementsOfSimilarKinds );


					PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\t( " + Tokenizer.getSuffix( (String) refs2DocOfSchemas[ schemaIndex1 ], "\\" ) + " - concrete element " + SchemaRepresentationController.getName( concreteElement1 )+ ", " + Tokenizer.getSuffix( (String) refs2DocOfSchemas[ schemaIndex2 ], "\\" ) + " - concrete element " + SchemaRepresentationController.getName( concreteElement2 )+ " )" );


					//Calculate product == 0


					PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t\trepetition = " + ++repetition );//TODO
				}
			}


			if( product != 0 ) exist = true;
		*/


		PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t---------- END - Calculation of the cartesian product of leaf sibling concrete elements of similar kinds ----------" );


		return exist ? matchings : null;
	}


	//Visualization.
	public Object visualize( String logFilePath ){

		boolean logfile = logFilePath != null ? true : false;

		DefaultMutableTreeNode root = null;


		String msg = "Pattern " + patternID + " (fr = " + getFrequency() + ")";

		if( logfile ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, msg );

		else root = new DefaultMutableTreeNode( msg );


		DefaultMutableTreeNode abstractElementsNode = null;

		msg = "\tAbstract elements";

		if( logfile ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, msg );

		else{

			abstractElementsNode = new DefaultMutableTreeNode( msg );

			root.add( abstractElementsNode );
		}


		if( rootPatternAbstractElement != null ){

			Object retValue = ((PatternAbstractElement) rootPatternAbstractElement).visualize( logFilePath );

			if( ! logfile ) abstractElementsNode.add( (MutableTreeNode) retValue );
		}


		DefaultMutableTreeNode patternInstatiationsNode = null;

		msg = "\t\tPattern instantiations (confidence, confidenceUpperBound) = (" + confidence + ", " + confidenceUpperBound + ")";

		if( logfile ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, msg );

		else{

			patternInstatiationsNode = new DefaultMutableTreeNode( msg );

			root.add( patternInstatiationsNode );
		}


		for( int i = 0; orderingOfRefs2DocOfSchemas != null && i < orderingOfRefs2DocOfSchemas.size(); ++i ){

			String ref2DocOfSchemas = (String) orderingOfRefs2DocOfSchemas.get( i );


			DefaultMutableTreeNode ref2DocOfSchemasNode = null;

			if( logfile ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\n\t\t\t" + ref2DocOfSchemas );

			else{

				ref2DocOfSchemasNode = new DefaultMutableTreeNode( ref2DocOfSchemas );

				patternInstatiationsNode.add( ref2DocOfSchemasNode );
			}


			List<Object> patternInstantiationsInASchema = MapGetter.getValueOfMapOfList( ref2DocOfSchemas, patternInstantiations );


			for( int j = 0; patternInstantiationsInASchema != null && j < patternInstantiationsInASchema.size(); ++j ){

				PatternInstantiation patternInstantiation = (PatternInstantiation) patternInstantiationsInASchema.get( j );


				Object retValue = patternInstantiation.visualize( logFilePath );

				if( ! logfile ) ref2DocOfSchemasNode.add( (MutableTreeNode) retValue );
			}
		}


    	return root;
	}
}
