package org.ow2.choreos.parser;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.omg.CORBA.DoubleHolder;
import org.omg.CORBA.LongHolder;
import org.omg.CORBA.StringHolder;
import org.ow2.choreos.gui.DocumentVisualizer;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.measurements.memory.MemoryConsumption;
import org.ow2.choreos.measurements.time.ElapsedTimeCalculator;
import org.ow2.choreos.parser.functionalDocuments.schema.SchemaParser;
import org.ow2.choreos.parser.functionalDocuments.schema.xml.v1.apache.XMLSchemaApacheParser;
import org.ow2.choreos.parser.functionalDocuments.service.behavioral.BehavioralParser;
import org.ow2.choreos.parser.functionalDocuments.service.wsdl.WSDLParser;
import org.ow2.choreos.parser.functionalDocuments.service.wsdl.v1.ibm.IBMWSDLv1Parser;
import org.ow2.choreos.parser.functionalDocuments.service.wsdl.v2.apacheWooden.ApacheWoodenWSDLv2Parser;
import org.ow2.choreos.parser.nonFunctionalDocuments.text.QoSParser;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.serviceRepresentation.behavioral.BehavioralServiceRepresentationController;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.FunctionalServiceRepresentationController;
import org.ow2.choreos.serviceRepresentation.nonFunctional.DocOfNonFuncProperties;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFunctionalServiceRepresentationController;


public class Parser{


	//WSDLParser sawsdlv1Parser; sawsdlv1Parser = new IBMSAWSDLv1Parser();


	static WSDLParser wsdlv1Parser = new IBMWSDLv1Parser();

	static WSDLParser wsdlv2Parser;

	static SchemaParser xmlParser = new XMLSchemaApacheParser();


	//Variables.
	private static String defaultSchemaRepresentationModel = XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();


	//Methods.
	public static Map<String, DocOfFuncProperties> parse( String ref2Source, List<String> refs2DocsOfFuncProperties, List<String> refs2DocsOfNonFuncProperties, List<String> refs2DocsOfBehavioralSpecification ){//This methods concerns the parsing of service documents.

		DocumentVisualizer documentVisualizer = null;

		long millis = 0;

		boolean stopParsing = false;


		int numOfDocs = 0, numOfFuncDocs = 0, parsedFuncDocs = 0, numOfServiceTypes = 0, numOfInterfaces = 0, numOfInstances = 0, numOfNonFuncDocs = 0, parsedNonFuncDocs = 0;


		//Functional documents.
		Map<String, DocOfFuncProperties> docsOfFuncProperties = null;

		for( int i = 0; ! stopParsing && refs2DocsOfFuncProperties != null && i < refs2DocsOfFuncProperties.size(); ++i ){

			numOfDocs++;
			numOfFuncDocs++;


			Object docOfFuncProperties = FunctionalServiceRepresentationController.getDocOfFunProperties( refs2DocsOfFuncProperties.get( i ) );

			Long duration = parseDocOfFuncProperties( ref2Source, docOfFuncProperties, defaultSchemaRepresentationModel );

			millis += duration;


			if( ! ( (DocOfFuncProperties) docOfFuncProperties).hasConstraintViolation() ){

				if( docsOfFuncProperties == null ) docsOfFuncProperties = new Hashtable<String, DocOfFuncProperties>();


				docsOfFuncProperties.put( refs2DocsOfFuncProperties.get( i ), (DocOfFuncProperties) docOfFuncProperties );

				parsedFuncDocs++;

				numOfServiceTypes += ( (DocOfFuncProperties) docOfFuncProperties).getNumOfServiceTypes();
				numOfInterfaces += ( (DocOfFuncProperties) docOfFuncProperties).getNumOfInterfaces();
				numOfInstances += ( (DocOfFuncProperties) docOfFuncProperties).getNumOfInstances();

				int size = 1;

				int depth = 1;

				int memory = 1;
				//TODO: size, depth, memory.


				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ElapsedTimeCalculator.getTime( duration ), size, depth, String.valueOf( memory ), ( (DocOfFuncProperties) docOfFuncProperties).visualize( ref2Source ) );

				else documentVisualizer.setTextOfComponentsForFunctionalDocuments( refs2DocsOfFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ElapsedTimeCalculator.getTime( duration ), size, depth, String.valueOf( memory ), ( (DocOfFuncProperties) docOfFuncProperties).visualize( ref2Source ) );


				PrintMessages.print( PrintMessages.parserPrint, "The parsing results of the document '" + refs2DocsOfFuncProperties.get( i ) + "' have been visualized.");
			}

			else{

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ( (DocOfFuncProperties) docOfFuncProperties).getConstraintViolation() );

				else documentVisualizer.setTextOfComponentsForFunctionalDocuments( refs2DocsOfFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ( (DocOfFuncProperties) docOfFuncProperties).getConstraintViolation() );


				if( PrintMessages.toPrint( PrintMessages.parserPrint ) ) stopParsing = true;
			}
		}


		//Non-Functional documents.
		Map<String, DocOfNonFuncProperties> docsOfNonFuncProperties = null;

		for( int i = 0; ! stopParsing && refs2DocsOfNonFuncProperties != null && i < refs2DocsOfNonFuncProperties.size(); ++i ){

			numOfDocs++;
			numOfNonFuncDocs++;


			Object docOfNonFuncProperties = NonFunctionalServiceRepresentationController.getDocOfNonFunProperties( refs2DocsOfNonFuncProperties.get( i ) );

			Long duration = parseDocOfNonFuncProperties( ref2Source, docOfNonFuncProperties );

			millis += duration;


			if( ! ( (DocOfNonFuncProperties) docOfNonFuncProperties).hasConstraintViolation() ){

				if( docsOfNonFuncProperties == null ) docsOfNonFuncProperties = new Hashtable<String, DocOfNonFuncProperties>();


				docsOfNonFuncProperties.put( refs2DocsOfNonFuncProperties.get( i ), (DocOfNonFuncProperties) docOfNonFuncProperties );

				parsedNonFuncDocs++;

				int size = 0;//It is intentionally equal to 0.

				int depth = 0;//It is intentionally equal to 0.

				int memory = 0;
				//TODO: size, depth, memory.

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfNonFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ElapsedTimeCalculator.getTime( duration ), size, depth, String.valueOf( memory ), ( (DocOfNonFuncProperties) docOfNonFuncProperties).visualize() );

				else documentVisualizer.setTextOfComponentsForFunctionalDocuments( refs2DocsOfNonFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ElapsedTimeCalculator.getTime( duration ), size, depth, String.valueOf( memory ), ( (DocOfNonFuncProperties) docOfNonFuncProperties).visualize() );


				PrintMessages.print( PrintMessages.parserPrint, "The parsing results of the document '" + refs2DocsOfNonFuncProperties.get( i ) + "' have been visualized.");
			}

			else{

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfNonFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ( (DocOfNonFuncProperties) docOfNonFuncProperties).getConstraintViolation() );

				else documentVisualizer.setTextOfComponentsForFunctionalDocuments( refs2DocsOfNonFuncProperties.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ( (DocOfNonFuncProperties) docOfNonFuncProperties).getConstraintViolation() );


				if( PrintMessages.toPrint( PrintMessages.parserPrint ) ) stopParsing = true;
			}
		}


		connectFuncAndNonFuncDocuments( docsOfFuncProperties, docsOfNonFuncProperties );


		//Behavioral specification documents.
		Map<String, Object> docsOfBehSpecification = null;

		for( int i = 0; ! stopParsing && refs2DocsOfBehavioralSpecification != null && i < refs2DocsOfBehavioralSpecification.size(); ++i ){

			numOfDocs++;
			numOfFuncDocs++;


			Object docOfBehSpecification = BehavioralServiceRepresentationController.getDocOfBehSpecification( refs2DocsOfBehavioralSpecification.get( i ) );

			Long duration = parseDocOfBehSpecification( docOfBehSpecification );

			millis += duration; 


			if( ! BehavioralServiceRepresentationController.hasConstraintViolation( docOfBehSpecification ) ){

				if( docsOfBehSpecification == null ) docsOfBehSpecification = new Hashtable<String, Object>();


				docsOfBehSpecification.put( refs2DocsOfBehavioralSpecification.get( i ), docOfBehSpecification );

				parsedFuncDocs++;


				int size = 0;//It is intentionally equal to 0.

				int depth = 0;//It is intentionally equal to 0.

				int memory = 0;
				//TODO: memory.

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfBehavioralSpecification.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ElapsedTimeCalculator.getTime( duration ), size, depth, String.valueOf( memory ), BehavioralServiceRepresentationController.visualize( docOfBehSpecification ) );

				else documentVisualizer.setTextOfComponentsForFunctionalDocuments( refs2DocsOfBehavioralSpecification.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, ElapsedTimeCalculator.getTime( duration ), size, depth, String.valueOf( memory ), BehavioralServiceRepresentationController.visualize( docOfBehSpecification ) );


				PrintMessages.print( PrintMessages.parserPrint, "The parsing results of the document '" + refs2DocsOfBehavioralSpecification.get( i ) + "' have been visualized.");
			}

			else{

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfBehavioralSpecification.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, BehavioralServiceRepresentationController.getConstraintViolation( docOfBehSpecification ) );

				else documentVisualizer.setTextOfComponentsForFunctionalDocuments( refs2DocsOfBehavioralSpecification.get( i ), numOfDocs, numOfFuncDocs, parsedFuncDocs, numOfServiceTypes, numOfInterfaces, numOfInstances, numOfNonFuncDocs, parsedNonFuncDocs, BehavioralServiceRepresentationController.getConstraintViolation( docOfBehSpecification ) );


				if( PrintMessages.toPrint( PrintMessages.parserPrint ) ) stopParsing = true;
			}
		}


		connectFuncAndBehDocuments( docsOfFuncProperties, docsOfBehSpecification );


		if( ! stopParsing && documentVisualizer != null ) documentVisualizer.setVisible( false );


		return docsOfFuncProperties;
	}

	private static long parseDocOfFuncProperties( String ref2Source, Object docOfFuncProperties, String schemaRepresentationModel ){

		long startTime = System.currentTimeMillis(), millis = 0;


		DocOfFuncProperties docOfFuncPropertiesWSDLv1 = wsdlv1Parser.parse( (DocOfFuncProperties) docOfFuncProperties );

		DocOfFuncProperties docOfFuncPropertiesWSDLv2 = null;

		if( docOfFuncPropertiesWSDLv1.hasConstraintViolation() ){

			if( wsdlv2Parser == null ) wsdlv2Parser = new ApacheWoodenWSDLv2Parser();//TODO: user selected.

			docOfFuncPropertiesWSDLv2 = wsdlv2Parser.parse( (DocOfFuncProperties) docOfFuncProperties );
		}


		if( ! docOfFuncPropertiesWSDLv1.hasConstraintViolation() && ( docOfFuncPropertiesWSDLv2 == null || ( docOfFuncPropertiesWSDLv2 != null && docOfFuncPropertiesWSDLv2.hasConstraintViolation() ) ) ) {

			PrintMessages.print( PrintMessages.parserPrint, "The document '" + ( (DocOfFuncProperties) docOfFuncProperties).getRef2DocOfFuncProperties() + "' is specified in the WSDLv1.1 language.");

			docOfFuncProperties = docOfFuncPropertiesWSDLv1;
		}

		else if( docOfFuncPropertiesWSDLv1.hasConstraintViolation() && ! docOfFuncPropertiesWSDLv2.hasConstraintViolation() ) {

			PrintMessages.print( PrintMessages.parserPrint, "The document '" + ( (DocOfFuncProperties) docOfFuncProperties).getRef2DocOfFuncProperties() + "' is specified in the WSDLv2.0 language.");

			docOfFuncProperties = docOfFuncPropertiesWSDLv2;
		}

		else{

			String message = "WSDLv1.1 parser comment:\n" + ( (DocOfFuncProperties) docOfFuncPropertiesWSDLv1).getConstraintViolation();

			message += "\n\nWSDLv2.0 parser comment:\n" + ( (DocOfFuncProperties) docOfFuncPropertiesWSDLv2).getConstraintViolation();

			message += "\n\nThe document '" + ( (DocOfFuncProperties) docOfFuncProperties).getRef2DocOfFuncProperties() + "' is NOT specified neither in the WSDLv1.0 nor the WSDLv2.0 language.";

			( (DocOfFuncProperties) docOfFuncProperties).setConstraintViolation( message );
		}


		if( ! ( (DocOfFuncProperties) docOfFuncProperties).hasConstraintViolation() ){

			List<Object> inlineSchemas = new ArrayList<Object>();

			Map<String, String> globalNamespaceQualifiers = new Hashtable<String, String>();

			StringHolder docTargetNamespace = new StringHolder();


			xmlParser.parse( ( (DocOfFuncProperties) docOfFuncProperties).getRef2DocOfFuncProperties(), schemaRepresentationModel, inlineSchemas, globalNamespaceQualifiers, docTargetNamespace );


			for( int i = 0; inlineSchemas != null && i < inlineSchemas.size(); ++i ) FunctionalServiceRepresentationController.set( inlineSchemas.get( i ), docOfFuncProperties );

			( (DocOfFuncProperties) docOfFuncProperties).set( globalNamespaceQualifiers );

			( (DocOfFuncProperties) docOfFuncProperties).set( docTargetNamespace.value );

			( (DocOfFuncProperties) docOfFuncProperties).connect();

			millis = System.currentTimeMillis() - startTime;


			if( ! ( (DocOfFuncProperties) docOfFuncProperties).hasFinalConstraintViolation() ){

				PrintMessages.print( PrintMessages.parserPrint, "The 'XMLSchema' of the document '" + ( (DocOfFuncProperties) docOfFuncProperties).getRef2DocOfFuncProperties() + "' has been parsed.");

				PrintMessages.print( PrintMessages.parserPrint, "The connection between the WSDL and the XML part of the document '" + ( (DocOfFuncProperties) docOfFuncProperties).getRef2DocOfFuncProperties() + "' has been completed.");
			}
		}


		return millis; 
	}

	private static Long parseDocOfNonFuncProperties( String ref2Source, Object docOfNonFuncProperties ){

		long startTime = System.currentTimeMillis();


		QoSParser.parse( ref2Source, (DocOfNonFuncProperties) docOfNonFuncProperties );


		return System.currentTimeMillis() - startTime;
	}

	private static void connectFuncAndNonFuncDocuments( Map<String, DocOfFuncProperties> docsOfFuncProperties, Map<String, DocOfNonFuncProperties> docsOfNonFuncProperties ){

		if( docsOfFuncProperties != null && docsOfNonFuncProperties != null ){

			for( Enumeration<String> e = ( (Hashtable<String,DocOfNonFuncProperties>) docsOfNonFuncProperties).keys(); e.hasMoreElements(); ){

				String i = e.nextElement();

				List<NonFuncDescription> nonFuncDescriptions = docsOfNonFuncProperties.get(i).getNonFuncDescriptions();

				for( int j = 0; nonFuncDescriptions != null && j < nonFuncDescriptions.size(); ++j ){

					if( nonFuncDescriptions.get(j).getRef2DocOfFuncProperties() != null ){//If reference to functional document is provided!

						DocOfFuncProperties docOfFuncProperties = docsOfFuncProperties.get( nonFuncDescriptions.get(j).getRef2DocOfFuncProperties());

						if( docOfFuncProperties != null ) docOfFuncProperties.set( nonFuncDescriptions.get( j ), docsOfNonFuncProperties.get(i).getQualityProperties() );//Functional document is found!
					}
				}
			}
		}
	}

	private static Long parseDocOfBehSpecification( Object docOfBehSpecification ){

		long startTime = System.currentTimeMillis();


		BehavioralParser.parse( docOfBehSpecification );


		return System.currentTimeMillis() - startTime;
	}

	private static void connectFuncAndBehDocuments( Map<String, DocOfFuncProperties> docsOfFuncProperties, Map<String, Object> docsOfBehSpecifications ){

		if( docsOfFuncProperties != null && docsOfBehSpecifications != null ){

			for( Enumeration<String> enumeration1 = ( (Hashtable<String, Object>) docsOfBehSpecifications ).keys(); enumeration1.hasMoreElements(); ){
	
				String i = enumeration1.nextElement();
	

				String ref2DocOfFuncProperties = BehavioralServiceRepresentationController.getRef2DocOfFuncProperties( docsOfBehSpecifications.get( i ) );//TODO: Here, I have to take info about the docOfFuncProperties in order the service interface name to be unique.

				String behavioralSpecification = BehavioralServiceRepresentationController.getContent( docsOfBehSpecifications.get( i ) );


				for( Enumeration<String> enumeration2 = ( (Hashtable<String, DocOfFuncProperties>) docsOfFuncProperties ).keys(); enumeration2.hasMoreElements(); ){
	
					String j = enumeration2.nextElement();


					DocOfFuncProperties docOfFuncProperties = docsOfFuncProperties.get( j );

					if( docOfFuncProperties != null ) docOfFuncProperties.set( behavioralSpecification, ref2DocOfFuncProperties );
				}
			}
		}
	}


	//Schema documents.
	public static List<Object> parse( String ref2Source, List<String> refs2DocsOfSchema ){

		DocumentVisualizer documentVisualizer = null;

		boolean stopParsing = false;

		int numOfDocs = 0, numOfParsedDocs = 0;


		List<Object> docsOfSchemas = null;

		for( int i = 0; ! stopParsing && refs2DocsOfSchema != null && i < refs2DocsOfSchema.size(); ++i ){

			numOfDocs++;


			Object docOfSchemas = SchemaRepresentationController.newDocOfSchemas( refs2DocsOfSchema.get( i ) );

			LongHolder duration = new LongHolder( 0 );

			DoubleHolder memory = new DoubleHolder( 0 );

			parseDocOfSchemas( duration, memory, docOfSchemas, defaultSchemaRepresentationModel );

			SchemaRepresentationController.setDuration( duration.value, docOfSchemas );

			SchemaRepresentationController.setMemory( memory.value, docOfSchemas );


			if( ! SchemaRepresentationController.hasFinalConstraintViolation( docOfSchemas ) ){

				if( docsOfSchemas == null ) docsOfSchemas = new ArrayList<Object>();

				docsOfSchemas.add( docOfSchemas );

				numOfParsedDocs++;


				int size = SchemaRepresentationController.getSize( docOfSchemas );

				int depth = 0;

				//TODO: depth.

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfSchema.get( i ), refs2DocsOfSchema.size(), numOfDocs, numOfParsedDocs, ElapsedTimeCalculator.getTime( duration.value ), String.valueOf( size ), String.valueOf( depth ), MemoryConsumption.print( memory.value ), SchemaRepresentationController.visualize( docOfSchemas ) );

				else documentVisualizer.setTextOfComponentsForSchemaDocuments( refs2DocsOfSchema.get( i ), refs2DocsOfSchema.size(), numOfDocs, numOfParsedDocs, ElapsedTimeCalculator.getTime( duration.value ), String.valueOf( size ), String.valueOf( depth ), MemoryConsumption.print( memory.value ), SchemaRepresentationController.visualize( docOfSchemas ) );


				PrintMessages.print( PrintMessages.parserPrint, "The parsing results of the document '" + refs2DocsOfSchema.get( i ) + "' have been visualized." );
			}

			else{

				if( documentVisualizer == null ) documentVisualizer = new DocumentVisualizer( ref2Source, refs2DocsOfSchema.get( i ), refs2DocsOfSchema.size(), numOfDocs, numOfParsedDocs, SchemaRepresentationController.getConstraintViolation( docOfSchemas ) );

				else documentVisualizer.setTextOfComponentsForSchemaDocuments( refs2DocsOfSchema.get( i ), refs2DocsOfSchema.size(), numOfDocs, numOfParsedDocs, SchemaRepresentationController.getConstraintViolation( docOfSchemas ) );


				if( PrintMessages.toPrint( PrintMessages.parserPrint ) ) stopParsing = true;
			}
		}

		if( ! stopParsing && documentVisualizer != null ) documentVisualizer.setVisible( false );


		return docsOfSchemas;
	}

	private static void parseDocOfSchemas( LongHolder duration, DoubleHolder memory, Object docOfSchemas, String schemaRepresentationModel ){

		long startTime = System.currentTimeMillis(), initUsedMemory = MemoryConsumption.getUsedMemory();


		List<Object> inlineSchemas = new ArrayList<Object>();

		xmlParser.parse( SchemaRepresentationController.getRef2DocOfSchemas( docOfSchemas ), schemaRepresentationModel, inlineSchemas );


		for( int i = 0; inlineSchemas != null && i < inlineSchemas.size(); ++i ) SchemaRepresentationController.set( inlineSchemas.get( i ), docOfSchemas );

		SchemaRepresentationController.connect( docOfSchemas );


		duration.value = System.currentTimeMillis() - startTime;

		memory.value = MemoryConsumption.measure( initUsedMemory );
	}
}
