package org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation;

import java.util.Arrays;

import org.ow2.choreos.abstractionBase.AbstractionBaseController;
import org.ow2.choreos.schemaRegistration.SchemaRegistrationController;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.serviceRegistration.ServiceRegistrationController;
import org.ow2.choreos.serviceRepresentation.functional.FunctionalServiceRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
public class ObjectConstraintsManipulation {


	public static String setEmptyAttribute( Object object, String attributeName ){

		boolean serviceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( object );

		boolean schemaKind = SchemaRepresentationController.isSchemaRepresentationKind( object );


		if( ! serviceKind && ! schemaKind ) return null;


		ObjectInfo objectInfo = manipulateObject( object );


		String constraintViolation = setEmptyAttribute( objectInfo.getQuotedVisualizationTagName(), attributeName );


		return setConstraintViolation( object, constraintViolation );
	}

	public static String setErrorAttribute( Object object, String[] attributeNames, String message ){

		String attributeName = Arrays.toString( attributeNames );

		return setErrorAttribute( object, attributeName, message );
	}

	public static String setErrorAttribute( Object object, String attributeName, String message ){

		boolean serviceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( object );

		boolean schemaKind = SchemaRepresentationController.isSchemaRepresentationKind( object );


		if( ! serviceKind && ! schemaKind ) return null;


		ObjectInfo objectInfo = manipulateObject( object );


		String constraintViolation = null;

		if( message != null && ! message.isEmpty() ){

			message = "\n" + message + ".";


			constraintViolation = setErrorAttribute( objectInfo.getQuotedVisualizationTagName(), attributeName, message );
		}


		return setConstraintViolation( object, constraintViolation );
	}

	public static String setUnknownTag( Object externalObject, String tagName ){
		
		boolean serviceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( externalObject );

		boolean schemaKind = SchemaRepresentationController.isSchemaRepresentationKind( externalObject );

		if( ! serviceKind && ! schemaKind ) return null;


		ObjectInfo objectInfo = manipulateObject( externalObject );


		String constraintViolation = setUnknownTag( objectInfo.getQuotedVisualizationTagName(), tagName );


		return setConstraintViolation( externalObject, constraintViolation );
	}

	public static String setUnconnectedComponent( Object externalObject, String internalObjectConstraintViolation ){

		boolean serviceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( externalObject );

		boolean schemaKind = SchemaRepresentationController.isSchemaRepresentationKind( externalObject );

		if( ! serviceKind && ! schemaKind ) return null;


		ObjectInfo objectInfo = manipulateObject( externalObject );


		String constraintViolation = setUnconnectedComponent( objectInfo.getQuotedVisualizationTagName(), internalObjectConstraintViolation  );


		return setConstraintViolation( externalObject, constraintViolation );
	}

	public static String setUnknownObject( Object rootObject, String[] targetObjectClassNames, String sourceSchemaRepresentationModel, String targetSchemaRepresentationModel ){

		boolean organizationKind = ServiceRegistrationController.isServiceRegistrationKinds( rootObject );

		boolean serviceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( rootObject );

		boolean schemaKind = SchemaRepresentationController.isSchemaRepresentationKind( rootObject );

		boolean abstractionBaseKind = AbstractionBaseController.isAbstractionBaseKind( rootObject );


		if( ! serviceKind && ! schemaKind && ! organizationKind && ! abstractionBaseKind ) return null;


		ObjectInfo objectInfo = manipulateObject( rootObject );


		String constraintViolation = setUnknownObject( objectInfo.getQuotedVisualizationTagName(), targetObjectClassNames, sourceSchemaRepresentationModel, targetSchemaRepresentationModel );


		return setConstraintViolation( rootObject, constraintViolation );
	}

	public static String setUnknownSchemaModel( Object externalObject, String sourceSchemaModel, String targetSchemaModel ){

		boolean organizationKind = ServiceRegistrationController.isServiceRegistrationKinds( externalObject );

		boolean serviceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( externalObject );

		boolean schemaKind = SchemaRepresentationController.isSchemaRepresentationKind( externalObject );

		boolean abstractionBaseKind = AbstractionBaseController.isAbstractionBaseKind( externalObject );


		if( ! serviceKind && ! schemaKind && ! organizationKind && ! abstractionBaseKind ) return null;


		ObjectInfo objectInfo = manipulateObject( externalObject );


		String constraintViolation = setUnknownSchemaModel( objectInfo.getQuotedVisualizationTagName(), sourceSchemaModel, targetSchemaModel );


		return setConstraintViolation( externalObject, constraintViolation );
	}

	public static String setContainmentError( Object externalObject, Object internalObject ){

		if( UnknownClassesController.isUnknownTag( internalObject ) ) return setUnknownTag( externalObject, UnknownClassesController.getUnknownTagName( internalObject ) );

		else if( UnknownClassesController.isUnknownObject( internalObject ) ) return setUnknownObject( externalObject, UnknownClassesController.getUnknownSourceObjectName( internalObject ), UnknownClassesController.getUnknownObjectSourceSchemaRepresentationModel( internalObject ), UnknownClassesController.getUnknownObjectTargetSchemaRepresentationModel( internalObject ) );

		else if( XMLSchemaRepresentationController.isUnknownSchemaModelKind( internalObject ) ) return setUnknownSchemaModel( externalObject, XMLSchemaRepresentationController.getUnknownSourceSchemaModel( internalObject ), XMLSchemaRepresentationController.getUnknownTargetSchemaModel( internalObject ) );


		boolean externalSchemaRegistrationKind = SchemaRegistrationController.isSchemaRegistrationKinds( externalObject );

		boolean externalOrganizationKind = ServiceRegistrationController.isServiceRegistrationKinds( externalObject );

		boolean externalServiceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( externalObject );

		boolean externalSchemaKind = SchemaRepresentationController.isSchemaRepresentationKind( externalObject );

		boolean externalAbstractionBaseKind = AbstractionBaseController.isAbstractionBaseKind( externalObject );


		boolean internalOrganizationKind = ServiceRegistrationController.isServiceRegistrationKinds( internalObject );

		boolean internalServiceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( internalObject );

		boolean internalSchemaKind = SchemaRepresentationController.isSchemaRepresentationKind( internalObject );

		boolean internalAbstractionBaseKind = AbstractionBaseController.isAbstractionBaseKind( internalObject );


		if( ! externalSchemaRegistrationKind && ! externalServiceKind && ! externalSchemaKind &&! internalServiceKind && ! internalSchemaKind && ! externalOrganizationKind && ! internalOrganizationKind && ! externalAbstractionBaseKind && ! internalAbstractionBaseKind ) return null;


		ObjectInfo externalObjectInfo = manipulateObject( externalObject );


		String constraintViolation = null;

		if( internalObject == null ) constraintViolation = setEmptyObject( externalObjectInfo.getQuotedVisualizationTagName(), "" );

		else{

			ObjectInfo internalObjectInfo = manipulateObject( internalObject );


			constraintViolation = internalObjectInfo.getConstraintViolation();
		}


		return setConstraintViolation( externalObject, constraintViolation );
	}

	public static String setConnectionError( Object externalObject, Object internalObject ){

		return setConnectionError( externalObject, internalObject, "" );
	}

	public static String setConnectionError( Object externalObject, Object internalObject, String message ){

		if( UnknownClassesController.isUnknownTag( internalObject ) ) return setUnknownTag( externalObject, UnknownClassesController.getUnknownTagName( internalObject ) + makeMessage( message ) );

		else if( UnknownClassesController.isUnknownObject( internalObject ) ) return setUnknownObject( externalObject, UnknownClassesController.getUnknownTargetObjectName( internalObject ), UnknownClassesController.getUnknownObjectSourceSchemaRepresentationModel( internalObject ), UnknownClassesController.getUnknownObjectTargetSchemaRepresentationModel( internalObject ) );

		else if( XMLSchemaRepresentationController.isUnknownSchemaModelKind( internalObject ) ) return setUnknownSchemaModel( externalObject, XMLSchemaRepresentationController.getUnknownSourceSchemaModel( internalObject ), XMLSchemaRepresentationController.getUnknownTargetSchemaModel( internalObject ) );


		boolean externalOrganizationKind = ServiceRegistrationController.isServiceRegistrationKinds( externalObject );

		boolean externalServiceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( externalObject );

		boolean externalSchemaKind = SchemaRepresentationController.isSchemaRepresentationKind( externalObject );

		boolean externalAbstractionBaseKind = AbstractionBaseController.isAbstractionBaseKind( externalObject );


		boolean internalOrganizationKind = ServiceRegistrationController.isServiceRegistrationKinds( internalObject );

		boolean internalServiceKind = FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( internalObject );

		boolean internalSchemaKind = SchemaRepresentationController.isSchemaRepresentationKind( internalObject );

		boolean internalAbstractionBaseKind = AbstractionBaseController.isAbstractionBaseKind( internalObject );


		if( ! externalServiceKind && ! externalSchemaKind &&! internalServiceKind && ! internalSchemaKind && ! externalOrganizationKind && ! internalOrganizationKind && ! externalAbstractionBaseKind && ! internalAbstractionBaseKind ) return null;


		ObjectInfo externalObjectInfo = manipulateObject( externalObject );


		String constraintViolation = null;

		if( internalObject == null ) constraintViolation = setEmptyObject( externalObjectInfo.getQuotedVisualizationTagName(), makeMessage( message ) );

		else{

			ObjectInfo internalObjectInfo = manipulateObject( internalObject );


			constraintViolation = setConnectionError( externalObjectInfo.getQuotedVisualizationTagName(), internalObjectInfo.getQuotedVisualizationTagName(), makeMessage( message ) );
		}


		return setConstraintViolation( externalObject, constraintViolation );
	}


	private static ObjectInfo manipulateObject( Object object ){

		if( SchemaRegistrationController.isSchemaRegistrationKinds( object ) ) return manipulateSchemaRegistrationObject( object );

		if( ServiceRegistrationController.isServiceRegistrationKinds( object ) ) return manipulateServiceRegistrationObject( object );

		if( FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind( object ) ) return manipulateServiceRepresentationObject( object );

		if( AbstractionBaseController.isAbstractionBaseKind( object ) ) return manipulateAbstractionBaseObject( object );


		return manipulateSchemaReprensentationObject( object );
	}

	private static ObjectInfo manipulateAbstractionBaseObject( Object object ){

		String visualizationTagName = AbstractionBaseController.getTag( object );

		String quotedVisualizationTagName = " '" + visualizationTagName + "'";

		String constraintViolation = AbstractionBaseController.getConstraintViolation( object );


		return new ObjectInfo( quotedVisualizationTagName, constraintViolation );
	}

	private static ObjectInfo manipulateSchemaRegistrationObject( Object object ){

		String visualizationTagName = SchemaRegistrationController.getTag( object );

		String quotedVisualizationTagName = " '" + visualizationTagName + "'";

		String constraintViolation = SchemaRegistrationController.getConstraintViolation( object );


		return new ObjectInfo( quotedVisualizationTagName, constraintViolation );
	}

	private static ObjectInfo manipulateServiceRegistrationObject( Object object ){

		String visualizationTagName = ServiceRegistrationController.getTag( object );

		String quotedVisualizationTagName = " '" + visualizationTagName + "'";

		String constraintViolation = ServiceRegistrationController.getConstraintViolation( object );


		return new ObjectInfo( quotedVisualizationTagName, constraintViolation );
	}

	private static ObjectInfo manipulateServiceRepresentationObject( Object object ){

		String visualizationTagName = FunctionalServiceRepresentationController.getTag( object );

		String quotedVisualizationTagName = " '" + visualizationTagName + "'";

		String constraintViolation = FunctionalServiceRepresentationController.getConstraintViolation( object );


		return new ObjectInfo( quotedVisualizationTagName, constraintViolation );
	}

	private static ObjectInfo manipulateSchemaReprensentationObject( Object object ){

		String visualizationTagName = SchemaRepresentationController.getTag( object );

		String schemaRepresentationModel = SchemaRepresentationController.getSchemaRepresentationModel( object );


		String quotedVisualizationTagName;

		if( schemaRepresentationModel == null ) quotedVisualizationTagName = " '" + visualizationTagName + "'";

		else quotedVisualizationTagName = " '" + visualizationTagName + " (" + schemaRepresentationModel + ")'";


		String constraintViolation = SchemaRepresentationController.getConstraintViolation( object );


		return new ObjectInfo( quotedVisualizationTagName, constraintViolation );
	}

	private static String setConstraintViolation( Object object, String constraintViolation ){

		if( SchemaRegistrationController.isSchemaRegistrationKinds( object ) ) return SchemaRegistrationController.setConstraintViolation( object, constraintViolation );

		if( ServiceRegistrationController.isServiceRegistrationKinds( object ) ) return ServiceRegistrationController.setConstraintViolation( object, constraintViolation );

		if( FunctionalServiceRepresentationController.isFunctionalServiceRepresentationKind(object ) ) return FunctionalServiceRepresentationController.setConstraintViolation( object, constraintViolation );

		if( AbstractionBaseController.isAbstractionBaseKind( object ) ) return AbstractionBaseController.setConstraintViolation( object, constraintViolation );

		return SchemaRepresentationController.setConstraintViolation( object, constraintViolation );
	}

	private static String makeMessage( String message ){

		if( message != null && ! message.isEmpty() ) message = "\n" + message + ".";

		else message = "";

		return message;
	}

	private static String makeExplanation( String message ){

		if( message != null && ! message.isEmpty() ) message = message + ".";

		else message = ".";

		return message;
	}


	private static String setEmptyAttribute( String externalObjectClassName, String attributeName ){

		return "ERROR in the:" + externalObjectClassName + ".\n\nIt contains an EMPTY definition of the attribute '" + attributeName + "'.";
	}

	private static String setErrorAttribute( String externalObjectClassName, String attributes, String message ){

		return "ERROR in the:" + externalObjectClassName + ".\n\nIt contains the ERROR attribute(s) '" + attributes + "'.\n" + message;
	}

	private static String setUnknownTag( String externalObjectClassName, String tagName ){

		return "ERROR in the:" + externalObjectClassName + ".\n\nIt contains the UNKNOWN tag '" + tagName + "'.";
	}

	private static String setUnconnectedComponent( String externalObjectClassName, String internalObjectConstraintViolation ){

		internalObjectConstraintViolation = ( internalObjectConstraintViolation = makeExplanation( internalObjectConstraintViolation ) ).equals( "" ) ? internalObjectConstraintViolation : " - " + internalObjectConstraintViolation;

		return "Connection ERROR in the:" + externalObjectClassName + ".\n\nIt is NOT CONNECTED with the corresponding referenced component in the (inline/imported) schema(s)" + internalObjectConstraintViolation;
	}

	private static String setUnknownSchemaModel( String externalObjectClassName, String sourceSchemaModel, String targetSchemaModel ){

		return targetSchemaModel == null ? "ERROR in the:" + externalObjectClassName + ".\n\nIt cannot be represented by the UNKNOWN schema model '" + sourceSchemaModel + "'." : "Transformation ERROR in the component: " + externalObjectClassName + ".\n\nIt cannot be tranformed from the model '" + sourceSchemaModel + "' to the model '" + targetSchemaModel + "' (No transformation rules).";
	}

	private static String setUnknownObject( String rootObjectClassName, String[] targetObjectClassNames, String sourceSchemaRepresentationModel, String targetSchemaRepresentationModel ){

		String targetObjectClassName = Arrays.toString( targetObjectClassNames );

		return "Transformation / Extraction ERROR in the component: " + rootObjectClassName + ".\n\nThe class(es) '" + targetObjectClassName + "' of the '" + sourceSchemaRepresentationModel + "' cannot be transformed / used for extraction in the schema model '" + targetSchemaRepresentationModel + "' (No target class).";
	}

	private static String setEmptyObject( String externalObjectClassName, String message ){

		return "ERROR in the:" + externalObjectClassName + ".\n\nIt contains an EMPTY definition of an inline component/structure.\n" + message;
	}

	private static String setConnectionError( String externalObjectClassName, String internalObjectClassName, String message ){

		return "ERROR in the:" + externalObjectClassName + ".\n\nIt WRONGLY contains the " + internalObjectClassName + ".\n" + message;
	}
}
