package org.ow2.choreos.parser.functionalDocuments.schema.xml.v1.apache;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.omg.CORBA.StringHolder;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.parser.functionalDocuments.schema.SchemaParser;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchema;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.TreeWalker;
import org.xml.sax.SAXException;


/**
 * 
 * @author dathanas
 * 
 * This class parses the XML components in the XSD language based on the read nodes returned by the 'apache' reader of the XML schema.
 * The parsing is performed in a generic way, namely, it does not depend on the each concrete schema representation.
 * On the contrary, it depends on the abstract classes: Component and Structure.  
 */
public class XMLSchemaApacheParser implements SchemaParser {


	public void parse( String ref2DocOfSchemas, String schemaRepresentationModel, List<Object> inlineSchemas ){//It is used by the files that contain only an XML schema and these schemas are not part of a service document (e.g., WSDL). 

		parse( ref2DocOfSchemas, schemaRepresentationModel, inlineSchemas, null, null );
	}

	public void parse( String ref2DocOfSchemas, String schemaRepresentationModel, List<Object> inlineSchemas, Map<String, String> globalNamespaceQualifiers, StringHolder docTargetNamespace ){//It is used for parsing service documents.

		List<ObjectInfo> objectInfos = globalNamespaceQualifiers != null && docTargetNamespace != null ? parseXMLSchema( false, ref2DocOfSchemas, globalNamespaceQualifiers, docTargetNamespace, schemaRepresentationModel ) : parseXMLSchema( false, ref2DocOfSchemas, schemaRepresentationModel );//In the first time, the 'schemaLocation' is the same with the URI of the WSDL document.


		for( int i = 0; objectInfos != null && i < objectInfos.size(); ++i ){

			Object mainXMLSchema = objectInfos.get( i ).getObject();

			inlineSchemas.add( mainXMLSchema );


			if( objectInfos.get( i ).getImportedSchemaLocations() != null ){


				List<String> cImportedSchemaLocations = new ArrayList<String>();

				cImportedSchemaLocations.addAll( objectInfos.get( i ).getImportedSchemaLocations() );


				while( ! cImportedSchemaLocations.isEmpty() ){

					String schemaLocation = cImportedSchemaLocations.remove( 0 ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parse): Parsing schemaLocation= " +schemaLocation );

					List<ObjectInfo> importedObjectInfos = globalNamespaceQualifiers != null && docTargetNamespace != null ? parseXMLSchema( true, schemaLocation, globalNamespaceQualifiers, docTargetNamespace, schemaRepresentationModel ) : parseXMLSchema( true, schemaLocation, schemaRepresentationModel );


					for( int j = 0; importedObjectInfos != null && j < importedObjectInfos.size(); ++j ){

						Object importedXMLSchema = importedObjectInfos.get( j ).getObject();


						XMLSchemaRepresentationController.set( importedXMLSchema, mainXMLSchema );


						List<String> importedXMLSchemaLocations = importedObjectInfos.get( j ).getImportedSchemaLocations();

						if( importedXMLSchemaLocations != null ) cImportedSchemaLocations.addAll( importedXMLSchemaLocations );
					}
				}
			}
		}
	}

	private List<ObjectInfo> parseXMLSchema( boolean imported, String schemaLocation, String schemaRepresentationModel ){//This function is for parsing the inline schema(s), not the imported schemas (It is used by the files that contain only a XML schema and these schemas are not part of a service document (e.g., WSDL)).

		List<ObjectInfo> objectInfos = null;

		PrintMessages.print( PrintMessages.xmlParserPrint, "schemaLocation= " + schemaLocation );


		try{

			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder loader = factory.newDocumentBuilder();
			Node docOfXMLSchema = loader.parse( schemaLocation );
			DocumentTraversal traversal = (DocumentTraversal) docOfXMLSchema;
			TreeWalker walker = traversal.createTreeWalker( ( (Document) docOfXMLSchema).getDocumentElement(), NodeFilter.SHOW_ELEMENT, null, true);


			if( imported ){

				ObjectInfo objectInfo = parseXMLSchema( walker, null, schemaRepresentationModel );

				objectInfos = addObjectInfo( objectInfo, objectInfos );
			}

			else{

				String tagName = getTagName( walker ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): tag= " + tagName );

				if( tagName.equals( XMLSchema.schemaTagName ) ){

					PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): Exists inline schema." );

					ObjectInfo objectInfo = parseXMLSchema( walker, null, schemaRepresentationModel );

					objectInfos = addObjectInfo( objectInfo, objectInfos );
				}

				else PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): There is no inline schema." );
			}
		}

		catch( IOException ex ){ objectInfos = getErrorObjectInfo( schemaLocation, null, schemaRepresentationModel, ex.getMessage() ); }
		catch( SAXException ex ){ objectInfos = getErrorObjectInfo( schemaLocation, null, schemaRepresentationModel, ex.getMessage() ); }
		catch( ParserConfigurationException ex ){ objectInfos = getErrorObjectInfo( schemaLocation, null, schemaRepresentationModel, ex.getMessage() ); }


		return objectInfos;
	}

	private List<ObjectInfo> parseXMLSchema( boolean imported, String schemaLocation, Map<String, String> globalNamespaceQualifiers, StringHolder docTargetNamespace, String schemaRepresentationModel ){//This function is for parsing the inline schema(s), not the imported schemas.

		List<ObjectInfo> objectInfos = null;

		PrintMessages.print( PrintMessages.xmlParserPrint, "schemaLocation= " + schemaLocation );


		try{

			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder loader = factory.newDocumentBuilder();
			Node docOfXMLSchema = loader.parse( schemaLocation );
			DocumentTraversal traversal = (DocumentTraversal) docOfXMLSchema;
			TreeWalker walker = traversal.createTreeWalker( ( (Document) docOfXMLSchema).getDocumentElement(), NodeFilter.SHOW_ELEMENT, null, true);


			if( imported ){

				ObjectInfo objectInfo = parseXMLSchema( walker, globalNamespaceQualifiers, schemaRepresentationModel );

				objectInfos = addObjectInfo( objectInfo, objectInfos );
			}

			else{

				String tagName = getTagName( walker ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): tag= " + tagName );

				if( tagName.equals( "definitions" ) ){

					if( globalNamespaceQualifiers == null || ( globalNamespaceQualifiers != null && globalNamespaceQualifiers.isEmpty() ) ) getNamespaceQualifiersAndTargetNamespace( globalNamespaceQualifiers, docTargetNamespace, walker );


					Node node = walker.firstChild();

					if( node != null ){

						tagName = getTagName( walker ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): tag= " + tagName );

						if( tagName.equals( "types" ) ){

							for( node = walker.firstChild(); node != null; node = walker.nextSibling() ){	

								tagName = getTagName( walker ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): tag= " + tagName );

								if( tagName.equals( XMLSchema.schemaTagName ) ){

									PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): There is inline schema." );

									ObjectInfo objectInfo = parseXMLSchema( walker, globalNamespaceQualifiers, schemaRepresentationModel );

									objectInfos = addObjectInfo( objectInfo, objectInfos );
								}

								else PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): There is NO inline schema." );


								walker.setCurrentNode( node );
							}
						}

						else PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): There is no 'types' tag." );
					}
				}

				else PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): There is no 'definitions' tag." );
			}
		}

		catch( IOException ex ){ objectInfos = getErrorObjectInfo( schemaLocation, globalNamespaceQualifiers, schemaRepresentationModel, ex.getMessage() ); }
		catch( SAXException ex ){ objectInfos = getErrorObjectInfo( schemaLocation, globalNamespaceQualifiers, schemaRepresentationModel, ex.getMessage() ); }
		catch( ParserConfigurationException ex ){ objectInfos = getErrorObjectInfo( schemaLocation, globalNamespaceQualifiers, schemaRepresentationModel, ex.getMessage() ); }


		return objectInfos;
	}

	private void getNamespaceQualifiersAndTargetNamespace( Map<String, String> namespaceQualifiers, StringHolder docTargetNamespace, TreeWalker walker ){//This function is used for the qualifiers of the namespaces (URIs) in the 'definition' tag.

		Map<String, String> attributes = getAttributes( walker );


		if( attributes != null ){

			for ( Enumeration<String> e = ( (Hashtable<String, String>) attributes ).keys(); e.hasMoreElements(); ){

				String attributeName = e.nextElement();

				String attributeValue = attributes.get( attributeName );


				if( Tokenizer.getPrefix( attributeName, ":" ) != null && Tokenizer.getPrefix( attributeName, ":" ).equals( "xmlns" ) ){

					String qualifier = Tokenizer.getSuffix( attributeName, ":" );

					String namespace = attributeValue;

					namespaceQualifiers.put( namespace, qualifier );
				}
			}
		}


		docTargetNamespace.value = MapGetter.getValueByKeyToLowerCase( "targetNamespace", attributes );
	}

	private List<ObjectInfo> addObjectInfo( ObjectInfo objectInfo, List<ObjectInfo> objectInfos ){//add 'objectInfo' to 'objectInfos'.

		if( objectInfo != null ){

			if( objectInfos == null ) objectInfos = new ArrayList<ObjectInfo>();

			objectInfos.add( objectInfo );
		}


		return objectInfos;
	}

	private List<ObjectInfo> getErrorObjectInfo( String schemaLocation, Map<String, String> namespaceQualifiers, String schemaRepresentationModel, String errorMessage ){

		List<ObjectInfo> objectInfos = null;


		Map<String, String> attributes = new Hashtable<String, String>();

		attributes.put( "targetNamespace", schemaLocation );


		String qualifier = (String) MapGetter.getValueByStringKey( schemaLocation, namespaceQualifiers );

		ObjectInfo objectInfo = new ObjectInfo( XMLSchemaRepresentationController.getErrorSchema( XMLSchema.schemaTagName, qualifier, attributes, schemaRepresentationModel, errorMessage ) );


		return addObjectInfo( objectInfo, objectInfos );
	}

	private ObjectInfo parseXMLSchema( TreeWalker walker, Map<String, String> globalNamespaceQualifiers, String schemaRepresentationModel ){

		Map<String, String> attributes = getTag( walker );//These are the attributes of the 'schema' tag.


		String targetNamespace = MapGetter.getValueByKeyToLowerCase( "targetNamespace", attributes );

		String qualifier = (String) MapGetter.getValueByStringKey( targetNamespace, globalNamespaceQualifiers ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): Qualifier (from the 'definitions' tag)= " + qualifier + " for namespace= " + targetNamespace );

		MapGetter.removeValueByKeyToLowerCase( targetNamespace, globalNamespaceQualifiers );


		Object xmlSchema = XMLSchemaRepresentationController.getSchema( XMLSchema.schemaTagName, qualifier, attributes, schemaRepresentationModel );

		Node initNode = walker.getCurrentNode();


		List<String> importedSchemaLocations = determineImportedSchemaLocations( walker );

		if( importedSchemaLocations != null ) PrintMessages.print( PrintMessages.xmlParserPrint, importedSchemaLocations.toString() );

		else PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseXMLSchema): It does not have imported schemas" );


		walker.setCurrentNode( initNode );


		List<Node> nodes = determineRootNodesOfComponents( walker );

		parseComponents( walker, nodes, xmlSchema, schemaRepresentationModel );


		return new ObjectInfo( xmlSchema, importedSchemaLocations );
	}

	private List<String> determineImportedSchemaLocations( TreeWalker walker ){

		List<String> importedSchemaLocations = null;

		Node node = walker.getCurrentNode();

		Node initNode = node;

		String tagName = getTagName( walker );

		if( tagName.equals( XMLSchema.schemaTagName ) ){

			for( Node n = walker.firstChild(); n != null; n = walker.nextSibling() ){
	
				Map<String, String> tag = getTag( walker );
	
				tagName = (String) MapGetter.getValueByStringKey( SchemaRepresentationController.getTagNameKey(), tag );

				if( tagName.equals( "import" ) ){

					String importedSchemaLocation = (String) (MapGetter.getValueByKeyToLowerCase( "schemaLocation", tag ) != null ? MapGetter.getValueByStringKey( "namespace", tag ) : MapGetter.getValueByKeyToLowerCase( "namespace", tag ));


					if( importedSchemaLocations == null ) importedSchemaLocations = new ArrayList<String>();

					importedSchemaLocations.add( importedSchemaLocation );
				}
			}
		}

		walker.setCurrentNode( initNode );


		return importedSchemaLocations;
	}

	private List<Node> determineRootNodesOfComponents( TreeWalker walker ){

		List<Node> nodes = null;

		Node node = walker.getCurrentNode();

		Node initNode = node;

		String tagName = getTagName( walker );

		if( tagName.equals( XMLSchema.schemaTagName ) ){

			for( Node n = walker.firstChild(); n != null; n = walker.nextSibling() ){

				Node dataComponent = walker.getCurrentNode();

				tagName = getTagName( walker );

				if( ! tagName.equals( "import" ) ){

					if( nodes == null ) nodes = new ArrayList<Node>();

					nodes.add( dataComponent );
				}
			}
		}

		walker.setCurrentNode( initNode );

		return nodes;
	}

	private void parseComponents( TreeWalker walker, List<Node> nodes, Object xmlSchema, String schemaRepresentationModel ){

		for( int i = 0; nodes != null && i < nodes.size(); i++ ){

			Node node = nodes.get( i );

			walker.setCurrentNode( node );

			Object component = parseComponent( walker, schemaRepresentationModel );

			set( component, xmlSchema );
		}
	}

	private Object parseComponent( TreeWalker walker, String schemaRepresentationModel ){

		List<Map<String, String>> path = null;


		Map<String, String> tag = getTag( walker );

		path = add( path, tag ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseComponent): current tagName= " + ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path ) );


		return parse( walker, path, true, schemaRepresentationModel );
	}

	private Object parse( TreeWalker walker, List<Map<String, String>> path, boolean isGlobal, String schemaRepresentationModel ){

		Object externalObject = XMLSchemaRepresentationController.getTagObject( path, isGlobal, schemaRepresentationModel );


		for( Node n = walker.firstChild(); n != null; n = walker.nextSibling() ){

			Map<String, String> tag = getTag( walker );

			path = add( path, tag ); PrintMessages.print( PrintMessages.xmlParserPrint, "XMLSchemaApacheParser (parseComponent): current tagName= " + ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path ) );

			Node prevNode = walker.getCurrentNode();

			Object internalObject = parse( walker, path, false, schemaRepresentationModel );

			externalObject = XMLSchemaRepresentationController.set( internalObject, externalObject );

			walker.setCurrentNode( prevNode );

			path = removeLastElement( path );
		}


		return externalObject;
	}

	private List<Map<String, String>> add( List<Map<String, String>> path, Map<String, String> tag ){

		if( path == null ) path = new ArrayList<Map<String, String>>();

		if( tag != null ) path.add( tag );


		return path;
	}

	private List<Map<String, String>> removeLastElement( List<Map<String, String>> path ){

		if( path != null && ! path.isEmpty() ) path.remove( path.size() - 1 );


		return path;
	}

	private void set( Object component, Object xmlSchema ){

		XMLSchemaRepresentationController.set( component, xmlSchema );
	}

	private String getTagName( TreeWalker walker ){

		return Tokenizer.getSuffix( ( (org.w3c.dom.Element) walker.getCurrentNode()).getTagName().toLowerCase(), ":" );
	}

	private Map<String, String> getTag( TreeWalker walker ){

		Map<String, String> tag = getAttributes( walker );

		String tagName = getTagName( walker );


		if( tag == null ) tag = new Hashtable<String, String>();

		tag.put( SchemaRepresentationController.getTagNameKey(), tagName );


		return tag;
	}

	private Map<String, String> getAttributes( TreeWalker walker ){//I take the the lower case of the names of the attributes (this is the first string) and their values (this is the second string).

		Map<String, String> attributes = null;


		Node node = walker.getCurrentNode();

		NamedNodeMap attributesNamedNodeMap = ( (org.w3c.dom.Element) node).getAttributes();

		for( int i = 0; attributesNamedNodeMap != null && i < attributesNamedNodeMap.getLength(); ++i ){

			if( attributes == null ) attributes = new Hashtable<String, String>();

			Node attributeNode = attributesNamedNodeMap.item( i );

			String attributeName = getAttributeName( attributeNode.toString() );

			String attributeValue = ( (org.w3c.dom.Element) node).getAttribute( attributeName );

			attributes.put( attributeName.toLowerCase(), attributeValue );
		}


		return attributes;
	}

	private String getAttributeName( String attribute ){

		if( attribute != null ){

			int semicolPos = attribute.lastIndexOf( "=" );

			if( semicolPos > 0 ) return attribute.substring( 0, semicolPos );
		}

		return attribute;
	}
}
