package org.ow2.choreos.pluginManager;

import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.Parser;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.parser.freeTextDocuments.TxtParser;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;


public class LocalFileSystemPluginImpl implements LocalFileSystemPlugin {


	//Variables.
	private int numOfDocuments;

	private Map<String, String> extensionsOfFuncDoc;
	private Map<String, String> extensionsOfDocContainsRef2FuncDoc;
	private Map<String, String> extensionsOfNonFuncDoc;
	private Map<String, String> extensionsOfBehavioralSpecificationDoc;
	private Map<String, String> extensionsOfSchemaDoc;


	//Constructor.
	public LocalFileSystemPluginImpl(){

		extensionsOfFuncDoc = new Hashtable<String, String>();
		extensionsOfFuncDoc.put( "wsdl", "wsdl" );
		extensionsOfFuncDoc.put( "xml", "xml" );
		extensionsOfFuncDoc.put( "asmx", "asmx" );

		extensionsOfDocContainsRef2FuncDoc = new Hashtable<String, String>();
		extensionsOfDocContainsRef2FuncDoc.put( "wsdls", "wsdls" );

		extensionsOfNonFuncDoc = new Hashtable<String, String>();
		extensionsOfNonFuncDoc.put( "qos", "qos" );

		extensionsOfBehavioralSpecificationDoc = new Hashtable<String, String>();
		extensionsOfBehavioralSpecificationDoc.put( "lts", "lts" );

		extensionsOfSchemaDoc = new Hashtable<String, String>();
		extensionsOfSchemaDoc.put( "xsd", "xsd" );


		PrintMessages.print( PrintMessages.pluginPrint, this );
	}


	//Getters.
	public Objects retrieve( String ref2Source ){//TODO: I retrieve documents even if the source id is the same with a previous one! Is it right?

		List<String> refs2DocsOfFuncProperties = new ArrayList<String>();
		List<String> refs2DocsOfNonFuncProperties = new ArrayList<String>();
		List<String> refs2DocsOfBehavioralSpecification = new ArrayList<String>();
		List<String> refs2DocsOfSchema = new ArrayList<String>();


       	File source = new File( ref2Source );

   		retrieve( source, refs2DocsOfFuncProperties, refs2DocsOfNonFuncProperties, refs2DocsOfBehavioralSpecification, refs2DocsOfSchema );


   		Objects objects = new Objects();

   		Map<String, DocOfFuncProperties> docsOfFuncProperties = Parser.parse( ref2Source, refs2DocsOfFuncProperties, refs2DocsOfNonFuncProperties, refs2DocsOfBehavioralSpecification );

   		List<Object> docsOfSchemas = Parser.parse( ref2Source, refs2DocsOfSchema );


   		objects.setDocsOfFuncProperties( docsOfFuncProperties );

   		objects.setSchemas( docsOfSchemas );


   		return objects;
	}

	public int getNumOfDocuments( String ref2Source ){

		numOfDocuments = 0;

		File source = new File( ref2Source );

		count( source );


		return numOfDocuments;
	}

	private void count( File source ){

		if( source.isFile() ){

			//System.out.println( "[FILE] " + source.getName() );

			String extension = Tokenizer.findExtension( source.getName() );

			if( extension != null && ( ( (Hashtable<String, String>) extensionsOfFuncDoc ).contains( extension.toLowerCase() ) ) ) numOfDocuments++;
		}

		else if( source.isDirectory() ){

			//System.out.println( "[DIR] " + source.getName() );

			File[] listOfFiles = source.listFiles();

			for( int i = 0; listOfFiles != null && i < listOfFiles.length; i++ ) count( listOfFiles[ i ] );
		}
	}

	private void retrieve( File source, List<String> refs2DocsOfFuncProperties, List<String> refs2DocsOfNonFuncProperties, List<String> refs2DocsOfBehavioralSpecification, List<String> refs2DocsOfSchema ){

		if( source.isFile() ){

			//System.out.println(spcs + "[FILE] " + source.getName());

			String extension = Tokenizer.findExtension( source.getName() );

			if( extension != null ){

				if( ( (Hashtable<String, String>) extensionsOfFuncDoc ).contains( extension.toLowerCase() ) ) refs2DocsOfFuncProperties.add( source.getAbsolutePath() );

				else if( ( (Hashtable<String, String>) extensionsOfNonFuncDoc ).contains( extension.toLowerCase() ) ) refs2DocsOfNonFuncProperties.add( source.getAbsolutePath() );

				else if( ( (Hashtable<String, String>) extensionsOfBehavioralSpecificationDoc ).contains( extension.toLowerCase() ) ) refs2DocsOfBehavioralSpecification.add( source.getAbsolutePath() );

				else if( ( (Hashtable<String, String>) extensionsOfDocContainsRef2FuncDoc ).contains( extension.toLowerCase() ) ) parseFile( source.getAbsolutePath(), refs2DocsOfFuncProperties );

				else if( ( (Hashtable<String, String>) extensionsOfSchemaDoc ).contains( extension.toLowerCase() ) ) add( source.getAbsolutePath(), refs2DocsOfSchema );

				else PrintMessages.print( PrintMessages.pluginPrint, "The '" + source.getAbsolutePath() + "' document has unknown extension!" );
			}
		}

		else if( source.isDirectory() ){

			//System.out.println(spcs + "[DIR] " + source.getName());

			File[] listOfFiles = source.listFiles();

			for(int i = 0; listOfFiles != null && i < listOfFiles.length; i++) retrieve( listOfFiles[i], refs2DocsOfFuncProperties, refs2DocsOfNonFuncProperties, refs2DocsOfBehavioralSpecification, refs2DocsOfSchema );
		}
	}

	private List<String> add( String ref2Doc, List<String> refs2Docs ){

		if( refs2Docs == null ) refs2Docs = new ArrayList<String>();

		if( ref2Doc != null ) refs2Docs.add( ref2Doc );


		return refs2Docs;
	}
	private void parseFile( String filePath, List<String> refs2DocsOfFuncProperties ){

		List<ArrayList<String>> allTerms = TxtParser.parse( filePath, false, "#", ',' );

		for( int i = 0; allTerms != null && i < allTerms.size(); ++i ) if( allTerms.get(i) != null ) refs2DocsOfFuncProperties.add( allTerms.get(i).get(1) );
	}
}
