package org.ow2.choreos.queryEngineFacade;

import java.util.List;
import java.io.Serializable;

import org.ow2.choreos.serviceRepresentation.functional.Operation;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Stores simple query information.
 * A simple query consists of a list of names of service operations, plus requirements of dependencies on other services.
 * 
 * @author pilios
 *
 */

@XmlRootElement
public class SimpleQuery implements Serializable{

	private List<String> operationNames;
	private String RQs;
	
	/**
	 * Creates a complete SimpleQuery object.
	 * Formal parameters must conform to some rules.For more information see {@link #checkWellFormedNess()}
	 * 
	 * @param operationNames A list of names of service operations.
	 * @param RQs Requirements of dependencies on other services.Requirements are separated by commas.
	 */
	public SimpleQuery(List<String> operationNames, String RQs){
		this.operationNames = operationNames;
		this.RQs = RQs;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 */
	public SimpleQuery(){
		
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Returns a list of names of service operations.
	 * 
	 * @return A list of names of service operations.
	 */
	public List<String> getOperationNames(){
		return operationNames;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Returns requirements of dependencies on other services.
	 * 
	 * @return Requirements of dependencies on other services.
	 */
	public String getRQs(){
		return RQs;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets the list of names of service operations.
	 * 
	 * @param operationNames A list of names of service operations.
	 */
	public void setOperationNames(List<String> operationNames){
		this.operationNames = operationNames;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets the requirements of dependencies on other services.
	 *  
	 * @param RQs Requirements of dependencies on other services.
	 */
	public void setRQs(String RQs){
		this.RQs = RQs;
	}
	
	/**
	 * <pre>
	 * Checks whether the SimpleQuery object is well-formed or not.
	 * If the SimpleQuery object is not well-formed, an exception is thrown.
	 * 
	 * Well-formedness rules:
	 * 
	 * The list of names of service operations must not be null or empty.
	 *  
	 * @throws QueryEngineFacadeException If the SimpleQuery object is not well-formed.
	 */
	public void checkWellFormedNess() throws QueryEngineFacadeException{
		
		if(operationNames == null || operationNames.size() == 0){
			throw(new QueryEngineFacadeException("SimpleQuery object not well formed: operations must not be null or empty"));
		}
	}
}
