package org.ow2.choreos.queryEngineServiceDeployment;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.apache.cxf.jaxrs.client.WebClient;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.queryEngineFacade.ConcreteService;
import org.ow2.choreos.queryEngineFacade.SimpleQuery;
import org.ow2.choreos.queryEngineFacade.AdvancedQuery;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.Marshaller;

import java.io.*;
import javax.swing.*;
import java.awt.*;

import java.util.List;
import java.util.ArrayList;

/**
 * A RESTful service client made for consuming methods of the query engine service ({@link QueryEngineService}).
 * To do so, it communicates with the {@link QueryEngineServiceServer} server thread that has been set running.
 * The server is addressed by the <b>URL</b> attribute of {@link QueryEngineServiceServer} class.
 * 
 * @author pilios
 *
 */

public class QueryEngineServiceClient {

	private String host;
	private WebClient client;
	
	/**
	 * Creates an instance of a RESTful service client, that is bind to (can communicate with) a server thread
	 * 
	 * @param host The server 's address
	 */
	public QueryEngineServiceClient(String host) {
		this.host = host;
	}
	
	private String toString(InputStream in) {
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		String result = null;
		
		try {
			String line = reader.readLine();
			result = line; 
			
			while((line=reader.readLine()) != null) {
			    result += line;
			}
		} catch(IOException ioe) {
			System.out.println(ioe.getMessage());
			JOptionPane.showMessageDialog(null, ioe.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
		}
		
		return result;
	}
	
	
	private WebClient setupClient() {
	
		WebClient client = WebClient.create(host);
		
		HTTPConduit http = (HTTPConduit)WebClient.getConfig(client).getConduit();
		HTTPClientPolicy httpClientPolicy = new HTTPClientPolicy();
		http.setClient(httpClientPolicy);
		http.getClient().setReceiveTimeout(2147483647);
		
		return client;
	}
	
	
	public void run() {
	
		client = setupClient();
		
		QueryEngineServiceClientGUI frame = new QueryEngineServiceClientGUI(this);
			
	}
	    
	public Response executeSBQLQuery(String query) {
		
		client.reset();
		client.path("queryengine/SBQLQuery/dimitris/pilios");
		client.accept(MediaType.APPLICATION_XML);
		
		Response r = client.post(query);
		return r;	
	}
	
	public Response executeSimpleQueryOverAbstractions(SimpleQuery query) {
		
		client.reset();
		client.path("queryengine/SimpleQueryOverAbstractions/dimitris/pilios");
		client.accept(MediaType.APPLICATION_XML);
		
		Response r = client.post(query);
		
		return r;	
	}
	
	public Response executeSimpleQueryOverInstances(SimpleQuery query) {
		
		client.reset();
		client.path("queryengine/SimpleQueryOverInstances/d/p");
		client.accept(MediaType.APPLICATION_XML);
		
		Response r = client.post(query);
		
		return r;	
	}
	
	public Response executeAdvancedQueryOverAbstractions(AdvancedQuery query) {
		
		client.reset();
		client.path("queryengine/AdvancedQueryOverAbstractions/dimitris/pilios");
		client.accept(MediaType.APPLICATION_XML);
		
		Response r = client.post(query);
		
		return r;	
	}
	
	public Response executeAdvancedQueryOverInstances(AdvancedQuery query) {
		
		client.reset();
		client.path("queryengine/AdvancedQueryOverInstances/dimitris/pilios");
		client.accept(MediaType.APPLICATION_XML);
		
		Response r = client.post(query);
		
		return r;	
	}

}


