package org.ow2.choreos.queryEngineServiceDeployment;

import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.apache.cxf.jaxrs.JAXRSServerFactoryBean;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.io.*;

/**
 * <pre>
 * 
 * A server for the (RESTful) query engine service.It is bind to the {@link QueryEngineService} class.When <b>start</b> method is called, 
 * a new server thread is created, which handles requests about the query engine service 's methods.
 * The <b>URL</b> attribute is used by a client to get access to this server.
 * 
 * </pre>
 * 
 * @author pilios
 *
 */
public class QueryEngineServiceServer implements Runnable {

    private static Logger logger;
        
    public static String URL;
    private static boolean running = false;

	static {
	    String port = "8080";
	    URL = "http://localhost:" + port + "/servicedeployer/";
	}
	
    public static void start() {
                
        logger = Logger.getLogger(QueryEngineServiceServer.class);
        new Thread(new QueryEngineServiceServer()).start();
            
        while (!running) {
            try {
                Thread.sleep(1);
            } catch (InterruptedException e) {
                System.out.println(e.getMessage());
                JOptionPane.showMessageDialog(null, e.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
            }
        }

    }

    public static void stop() {
        running = false;
    }

    public void run() {
        	
        JAXRSServerFactoryBean sf = new JAXRSServerFactoryBean();
        sf.setResourceClasses(QueryEngineService.class);
        sf.setAddress(URL);
        sf.create();
        logger.info("Enactment Engine has STARTED [" + URL + "]");
        running = true;
        
        ServerFrame frame = new ServerFrame();
        frame.setVisible(true);
        frame.appendText("Enactment Engine has STARTED [" + URL + "]");
        
        while (running) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
            	System.out.println(e.getMessage());
            	JOptionPane.showMessageDialog(null, e.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
            }
        }
            
        logger.info("Enactment Engine has STOPPED");
        frame.appendText("Enactment Engine has STOPPED");

    }
    
}

/**
 * A simple window, acting as a GUI for the server.
 * 
 * @author pilios
 *
 */
class ServerFrame extends JFrame {
	
	private JTextArea textArea;
	
	public ServerFrame() {
		
		setTitle("Query Engine Service Server");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(550, 300);
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int top = (screen.height - getHeight()) / 2;
		int left = (screen.width - getWidth()) / 2;
		setBounds(left, top, getWidth(), getHeight()); 
		
		JScrollPane scrollPane = new JScrollPane();
		textArea = new JTextArea();
		textArea.setEditable(false);
		textArea.setBackground(new Color(0, 0, 0));
		textArea.setForeground(new Color(250, 0, 0));
		textArea.setFont(new java.awt.Font("Tahoma", 0, 12)); 
		scrollPane.setViewportView(textArea);
		add(scrollPane);
	    
	    addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
            	
            	File logfile = new File("logfile.log");
            	if(logfile.exists()) {
            		
            		try {
		            	Writer output;
		            	output = new BufferedWriter(new FileWriter("logfile.log", true));
		            	output.append("\r\nTERMINATED");
		            	output.append("\r\n\r\n____________________________________________________________________");
		            	output.append("________________________________________________________________________\r\n\r\n\r\n");
		            	output.close();
            		} catch(IOException ioe) {
            			System.out.println(ioe.getMessage());
            			JOptionPane.showMessageDialog(null, ioe.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
            		}
            	}
            }
        });
	}
	
	public void appendText(String text) {
		textArea.setCaretPosition(textArea.getDocument().getLength());
		textArea.append(text);
	}

}
