package org.ow2.choreos.schemaRegistration;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.pluginManager.Objects;
import org.ow2.choreos.pluginManager.PluginManager;


/**
 * 
 * @author dathanas
 * 
 */
class SchemaRegistration {


	//Variables.
	private Map<Integer, SchemaCollection> collections;
	private boolean hasCollections;

	private Integer currentCollectionID;

	private Map<String, Boolean> hasTransformedCollections;//The 'String' is the 'schemaRepresentationmodel'.

	private double sumDuration, depth, sumSize, sumMemory;//Statistics.

	private int numOfParsedSchemaDocs;

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	protected SchemaRegistration(){

		hasCollections = false;

		hasConstraintViolation = false;

		currentCollectionID = 1;

		sumDuration = 0.0;

		sumMemory = 0.0;

		sumSize = 0.0;


		PrintMessages.print( PrintMessages.registrationPrint, hasConstraintViolation, this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	/*public void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedCollections= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedCollections ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasCollections && ! MapManipulation.getBooleanValueByKey( targetModel, hasTransformedCollections ) && ! hasConstraintViolation ){

			for( Enumeration<Integer> e = ( (Hashtable<Integer, SchemaCollection>) collections).keys(); e.hasMoreElements(); ){

				Integer collectionID = e.nextElement();


				collections.get( collectionID ).transform( targetModel );


				if( ! collections.get( collectionID ).hasTransformedSources( targetModel ) ) return;
			}

			if( hasTransformedCollections == null ) hasTransformedCollections = new Hashtable<String, Boolean>();

			hasTransformedCollections.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedCollections= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedCollections ) + " hasConstraintViolation= " + hasConstraintViolation );
	}*/

	public void increaseCurrentCollectionID(){

		currentCollectionID++;
	}

	public void register( String ref2Source ){

		Objects objects = PluginManager.retrieve( ref2Source );

		List<Object> docsOfSchemas = objects != null ? objects.getDocsOfSchemas() : null;


		SchemaCollection schemaCollection = lookUp( currentCollectionID );

		if( schemaCollection == null ){

			schemaCollection = new SchemaCollection( currentCollectionID );

			add( schemaCollection );
		}


		schemaCollection.add( ref2Source, docsOfSchemas );

		sumDuration += schemaCollection.getDuration();

		sumMemory += schemaCollection.getMemory();

		sumSize += schemaCollection.getSize();

		numOfParsedSchemaDocs += schemaCollection.getNumOfParsedSchemaDocs();
	}

	private void add( SchemaCollection schemaCollection ){

		if( ObjectConstraintsManipulation.setContainmentError( this, schemaCollection ) == null ){

			if( ! hasCollections ){

				collections = new Hashtable<Integer, SchemaCollection>();

				hasCollections = true;
			}

			collections.put( currentCollectionID, schemaCollection );
		}
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		if( hasCollections )

			for ( Enumeration<Integer> e = ( (Hashtable<Integer, SchemaCollection>) collections ).keys(); e.hasMoreElements(); ){

				SchemaCollection collection = collections.get( e.nextElement() );

				if( collection.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, collection );
			}


		return constraintViolation;
	}

	public Map<Integer, SchemaCollection> getCollections(){

		return collections;
	}

	public boolean hasCollections(){

		return hasCollections;
	}

	public Integer getCurrentCollectionID(){

		return currentCollectionID;
	}

	String getSingleRef2Source(){

		SchemaCollection schemaCollection = lookUp( currentCollectionID );


		return schemaCollection.getSingleRef2Source();
	}

	public SchemaCollection getCurrentCollection(){

		return lookUp( currentCollectionID );
	}

	private boolean isEmpty(){//It checks all the collections.

		if( hasCollections ) for( Enumeration<Integer> e = ( (Hashtable<Integer,SchemaCollection>) collections).keys(); e.hasMoreElements(); ) if( ! collections.get( e.nextElement() ).isEmpty() ) return false;


		return true;
	}

	public String getTag(){

		return "schema registration";
	}

	public boolean hasTransformedCollections( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedCollections );
	}

	SchemaCollection lookUp( Integer collectionID ){

		return collections != null ? collections.get( collectionID ) : null;
	}

	double getDuration(){

		return hasCollections ? sumDuration / (double) collections.size() : 0.0;
	}

	double getMemory(){

		return hasCollections ? sumMemory / (double) collections.size() : 0.0;
	}

	double getSize(){

		return hasCollections ? sumSize / (double) collections.size() : 0.0;
	}

	double getDuration( Integer collectionID ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.getDuration() : 0.0;
	}

	double getDuration( Integer collectionID, String ref2Source, String ref2DocOfSchemas ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.getDuration( ref2Source, ref2DocOfSchemas ) : 0.0;
	}

	int getNumOfParsedSchemaDocs(){

		return numOfParsedSchemaDocs;
	}

	int getNumOfParsedSchemaDocsOfCollection( Integer collectionID ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.getNumOfParsedSchemaDocs() : 0;
	}

	Object lookUp( Integer collectionID, String ref2Source, String ref2DocOfSchemas ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.lookUpDocOfSchema( ref2Source, ref2DocOfSchemas ) : null;
	}

	Object lookUp( Integer collectionID, String ref2DocOfSchemas ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.lookUpDocOfSchema( ref2DocOfSchemas ) : null;
	}

	List<String> getNamesOfDocOfSchemas( Integer collectionID ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.getNamesOfDocOfSchemas() : null;
	}

	List<String> getRefs2DocsOfSchemas( Integer collectionID ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.getRefs2DocsOfSchemas() : null;
	}

	String getSchemaRepresentationModel( Integer collectionID ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.getSchemaRepresentationModel() : null;
	}


	//Visualization.
	DefaultMutableTreeNode visualize( boolean withContents ){//It visualizes all the collections.

		DefaultMutableTreeNode root;

		if( isEmpty() ) root = new DefaultMutableTreeNode( "You have not stored any collection yet!" );

		else{

			root = new DefaultMutableTreeNode( "Schema Collections" );

			for( Enumeration<Integer> e = ((Hashtable<Integer,SchemaCollection>) collections ).keys(); e.hasMoreElements();) root = collections.get( e.nextElement() ).visualize( root, withContents );
		}


		return root;
	}

	DefaultMutableTreeNode visualize( Integer collectionID ){

		SchemaCollection schemaCollection = null;


		return ( schemaCollection = lookUp( collectionID ) ) != null ? schemaCollection.visualize( null, true ) : null;
	}
}
