package org.ow2.choreos.schemaRepresentation;

import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
public class SchemaRepresentationController {


	//Variables.
	private static final String xmlSchema = "schema.xml";//This is the package of the classes that model this schema kind.

	private static final String tagName = "tagname";//This is the key of the tag names in the created hash tables.


	//Verifiers.
	public static boolean isDocOfSchemasKind( Object object ){

		return object instanceof DocOfSchemas;
	}

	public static boolean isSchemaKind( Object object ){

		return object instanceof Schema;
	}

	public static boolean isDocOfSchemasTagName( String tagName ){

		return tagName != null && tagName.equals( DocOfSchemas.docOfSchemasTagName );
	}

	public static boolean isSchemaTagName( String tagName ){

		return tagName != null && tagName.equals( Schema.schemaTagName );
	}

	public static boolean isSchemaRepresentationKind( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return true;

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return true;


		return false;
	}

	public static boolean equalClassPath( String[] classPath, Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( XMLSchemaRepresentationController.equalClassPath( classPath, object ) ) return true;


		return false;
	}

	public static boolean isKnownSchemaRepresentation( String model ){

		return model != null && XMLSchemaRepresentationController.isKnownXMLSchemaRepresentation( model ) ? true : false;
	}

	public static Boolean isKindDescr( Object kindDescr ){

		return kindDescr != null ? kindDescr instanceof KindDescr : null;
	}

	public static Boolean canBeRootKindDescr( Object kindDescr ){

		if( kindDescr == null ) return null;


		return isKindDescr( kindDescr ) && ( SchemaRepresentationController.isDocOfSchemasTagName( ((KindDescr) kindDescr).getKind() ) || XMLSchemaRepresentationController.canBeRootKindDescr( kindDescr ) );
	}

	public static boolean isRule( Object rule ){

		return rule != null ? rule instanceof Rule : false;
	}


	//Getters.
	public static String getSchemaKind(){

		return xmlSchema;
	}

	public static String getTagNameKey(){

		return tagName;
	}

	public static Object newDocOfSchemas( String ref2DocOfSchemas ){

		return new DocOfSchemas( ref2DocOfSchemas );
	}

	public static String getRef2DocOfSchemas( Object object ){

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getRef2DocOfSchemas();


		return UnknownClassesController.getUnknownTagName();
	}

	public static String getNameOfDocOfSchemas( Object object ){

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getName();


		return UnknownClassesController.getUnknownTagName();
	}

	public static String getConstraintViolation( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getConstraintViolation();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getConstraintViolation( object );


		return UnknownClassesController.getUnknownObjectName();
	}

	public static String getTag( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getTag();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getTag( object );


		return UnknownClassesController.getTagName( object );
	}

	public static String getTagName( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getTagName();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getTagName( object );


		return UnknownClassesController.getTagName( object );
	}

	public static String getName( Object object ){

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getName();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getName( object );


		return UnknownClassesController.getTagName( object );
	}

	public static String getSchemaRepresentationModel( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getSchemaRepresentationModel();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getSchemaRepresentationModel( object );


		return null;
	}

	public static boolean hasConstraintViolation( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).hasConstraintViolation();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.hasConstraintViolation( object );


		return true;
	}

	public static boolean hasFinalConstraintViolation( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).hasFinalConstraintViolation();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.hasFinalConstraintViolation( object );


		return true;
	}

	public static DefaultMutableTreeNode visualize( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).visualize();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.visualize( object );


		return null;
	}

	static public Object visualize( Object object, String logFilePath ){

		return visualize( object, null, null, logFilePath );
    }

	static public Object visualize( Object object, Integer rightmostChildScopeRightEndpoint, String logFilePath ){

		return visualize( object, null, rightmostChildScopeRightEndpoint, logFilePath );
    }

	static public Object visualize( Object object, Integer leftEndpointOfConcreteElementScope, Integer rightmostChildScopeRightEndpoint, String logFilePath ){

		boolean filePrint = logFilePath != null ? true : false;

		Object retValue = null;


		String msg = "\t\t\t\t\tconcrete element";

		if( leftEndpointOfConcreteElementScope != null ) msg += " " + leftEndpointOfConcreteElementScope;

		msg += " : " + getScope( object ).getIntEndpoints() + " - " + getTag( object );

		if( rightmostChildScopeRightEndpoint != null ) msg += " - right endpoint of the scope of its rightmost child = " + rightmostChildScopeRightEndpoint;


		if( filePrint ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, msg );

		else retValue = new DefaultMutableTreeNode( msg );


		return retValue;
    }

	public static boolean belongsToCycle( Component materializedComponent, Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.belongsToCycle( materializedComponent, object );


		return false;
	}

	public static long getDuration( Object object ){

		return isDocOfSchemasKind( object ) ? ( (DocOfSchemas) object).getDuration() : 0;
	}

	public static double getMemory( Object object ){

		return isDocOfSchemasKind( object ) ? ( (DocOfSchemas) object).getMemory() : 0.0;
	}

	public static int getSize( Object object ){

		return isDocOfSchemasKind( object ) ? ( (DocOfSchemas) object).getSize() : 0;
	}

	public static List<?> getChildren( Object object, String targetSchemaRepresentationModel ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getSchemas();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getChildren( object, targetSchemaRepresentationModel );


		return null;
	}

	public static List<?> getChildrenWithoutAnnotation( Object object, String targetSchemaRepresentationModel ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getSchemas();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getChildrenWithoutAnnotation( object, targetSchemaRepresentationModel );


		return null;
	}

	public static Object newRule( String schemaRepresentationModel, String category, String elementKind1, boolean elementKindIsDecl1, boolean elementKindHasBuiltInType1, String elementKind2, boolean elementKindIsDecl2, boolean elementKindHasBuiltInType2, String quantitativeSimilarityValue ){

		if( XMLSchemaRepresentationController.isKnownXMLSchemaRepresentation( schemaRepresentationModel ) ) return XMLSchemaRepresentationController.newRuleForSimilarElementKinds( category, elementKind1, elementKindIsDecl1, elementKindHasBuiltInType1, elementKind2, elementKindIsDecl2, elementKindHasBuiltInType2, quantitativeSimilarityValue );


		return null;
	}

	public static Object newRule( String schemaRepresentationModel, String category, String elementKind1, boolean elementKindIsDecl1, boolean elementKindHasBuiltInType1, String elementKind2, boolean elementKindIsDecl2, boolean elementKindHasBuiltInType2, String qualitativeSimilarityValue, String quantitativeSimilarityValue ){

		if( XMLSchemaRepresentationController.isKnownXMLSchemaRepresentation( schemaRepresentationModel ) ) return XMLSchemaRepresentationController.newRuleForSimilarElementKinds( category, elementKind1, elementKindIsDecl1, elementKindHasBuiltInType1, elementKind2, elementKindIsDecl2, elementKindHasBuiltInType2, qualitativeSimilarityValue, quantitativeSimilarityValue );


		return null;
	}

	public static String[] getTagNames( String model ){

		if( XMLSchemaRepresentationController.isKnownXMLSchemaRepresentation( model ) ) return XMLSchemaRepresentationController.getTagNames();


		return null;
	}

	public static KindDescr getKindDescr( Object object ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object ).getKindDescr();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getElementKindDescr( object );


		return null;
	}

	public static boolean isContent( Object parentOfCurrentObject, Object currentObject, Object nextObject, Object parentOfNextObject ){

		if( isDocOfSchemasKind( currentObject ) ) return ( (DocOfSchemas) currentObject).hasContent( nextObject );

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( currentObject ) ) return XMLSchemaRepresentationController.isContent( getTagName( parentOfCurrentObject ), getTagName( currentObject ), getTagName( nextObject ), getTagName( parentOfNextObject ) );


		return false;
	}

	static public Interval1D getScope( Object object ){

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getScope();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getScope( object );


		return null;
	}

	static public Object getParent( Object object ){

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).getParent();

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.getParent( object );


		return null;
	}

	static public boolean hasOrderedChildren( Object object ){

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.hasOrderedChildren( getTagName( object ) );


		return false;
	}


	//Setters.
	public static Object set( Object internal, Object external ){

		PrintMessages.print( PrintMessages.controllerPrint, "SchemaRepresentationController1: Set the internal= " + ClassInfo.getClass( internal ) + " to external= " + ClassInfo.getClass( external ) );


		if( isDocOfSchemasKind( external ) ) return set( internal, (DocOfSchemas) external );


		return external;
	}


	private static Object set( Object internal, DocOfSchemas docOfSchemas ){//TODO: to be extended with other types of schemas...(more if cases).

		PrintMessages.print( PrintMessages.controllerPrint, "SchemaRepresentationController2: Set the internal= " + ClassInfo.getClass( internal ) + " to docOfSchemas= " + ClassInfo.getClass( docOfSchemas ) );


		if( isSchemaKind( internal ) ) docOfSchemas.add( (Schema) internal );

		else if( UnknownClassesController.isUnknownTag( internal ) ) ObjectConstraintsManipulation.setUnknownTag( docOfSchemas, UnknownClassesController.getUnknownTagName( internal ) );

		else if( UnknownClassesController.isUnknownObject( internal ) ) ObjectConstraintsManipulation.setUnknownObject( docOfSchemas, UnknownClassesController.getUnknownSourceObjectName( internal ), UnknownClassesController.getUnknownObjectSourceSchemaRepresentationModel( internal ), UnknownClassesController.getUnknownObjectTargetSchemaRepresentationModel( internal ) );

		else if( XMLSchemaRepresentationController.isUnknownSchemaModelKind( internal ) ) ObjectConstraintsManipulation.setUnknownSchemaModel( docOfSchemas, XMLSchemaRepresentationController.getUnknownSourceSchemaModel( internal ), XMLSchemaRepresentationController.getUnknownTargetSchemaModel( internal ) );


		return docOfSchemas;
	}

	public static String setConstraintViolation( Object object, String constraintViolation ){//TODO: to be extended with other types of schemas...(more if cases).

		if( isDocOfSchemasKind( object ) ) return ( (DocOfSchemas) object).setConstraintViolation( constraintViolation );

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) return XMLSchemaRepresentationController.setConstraintViolation( object, constraintViolation );


		return UnknownClassesController.getUnknownTagName();
	}

	public static void connect( Object object ){

		if( isDocOfSchemasKind( object ) ) ( (DocOfSchemas) object).connect();
	}

	public static void setDuration( long duration, Object object ){

		if( isDocOfSchemasKind( object ) ) ( (DocOfSchemas) object).setDuration( duration );
	}

	public static void setMemory( double memory, Object object ){

		if( isDocOfSchemasKind( object ) ) ( (DocOfSchemas) object).setMemory( memory );
	}

	public static void setMyOrderID( Object object, int myOrderID ){//TODO: to be extended with other types of schemas...(more if cases).

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) XMLSchemaRepresentationController.setMyOrderID( object, myOrderID );
	}

	public static void setScope( Object object, Interval1D scope ){

		if( isDocOfSchemasKind( object ) ) ( (DocOfSchemas) object).setScope( scope );

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) XMLSchemaRepresentationController.setScope( object, scope );
	}

	public static void setParent( Object object, Object parent ){

		if( isDocOfSchemasKind( object ) ) ( (DocOfSchemas) object).setParent( parent );

		if( XMLSchemaRepresentationController.isSchemaRepresentationKind( object ) ) XMLSchemaRepresentationController.setParent( object, parent );
	}
}
