package org.ow2.choreos.schemaRepresentation.xml;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.list.listManipulation.ListTransformations;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.KindDescr;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.schemaTagsTransformation.SchemaTags2JavaClassesTransformation;
import org.ow2.choreos.schemaRepresentation.schemaTagsTransformation.SchemaTags2SchemaPathTransformation;


/**
 * 
 * @author dathanas
 *
 */
public abstract class Component implements java.io.Serializable {


	//Variables.
	private static final long serialVersionUID = 9005071179960058980L;

	private Map<String, String> tagAttributes;

	private String[] schemaPath;
	private boolean hasSchemaPath;

	private String[] classPath;
	private boolean hasClassPath;


	private String name;
	private boolean isAnonymous;

	private ReferencedComponent referencedComponent;
	private boolean hasReferencedComponent;

	private String id;
	private boolean hasID;

	private boolean isGlobal;


	private int myOrderID = 0;//This is the order in which the component appears in the list of component/structure children of a component/structure.

	private int myChildrenOrderID = 0;//This is the order in which the components/structures appear in the list of its children.


	private boolean hasUnmaterializedReferencedComponents;


	private int size = 1;//This is the total number of the building blocks (without following the references).


	private List<Component> components;
	private boolean hasComponents;

	private List<Structure> structures;
	private boolean hasStructures;


	private Annotation annotation;
	private boolean hasAnnotation;


	private Interval1D scope;

	private Object parent;


	private String constraintViolation;
	private boolean hasConstraintViolation;

	private int dbKey;

	private boolean concrete; //actual (true) or representative (false)

	
	//Constructor.
	public Component( List<Map<String, String>> path, boolean cIsGlobal ){

		initVariables();


		setTagAttributes( path );


		if( path != null && ! path.isEmpty() ){

			setSchemaPath( path );

			setClassPath( path );
		}

		else{

			hasClassPath = false;

			hasSchemaPath = false;
		}


		isGlobal = cIsGlobal;


		setName( (String) MapGetter.getValueByStringKey( "name", tagAttributes ) );


		String cID = (String) MapGetter.getValueByStringKey( "id", tagAttributes );

		if( cID != null && ! cID.isEmpty() ){

			id = cID;

			hasID = true;
		}

		checkConstraint();


		PrintMessages.print( PrintMessages.xmlPrint, hasConstraintViolation(), this );
	}
	
	
	//Empty constructor used for XML transformation.
	public Component(){
		
	}

	//Method used by the constructor.
	private void initVariables(){

		hasID = false;

		hasReferencedComponent = false;
	
		hasComponents = false;
	
		hasUnmaterializedReferencedComponents = false;

		hasStructures = false;

		hasAnnotation = false;

		hasConstraintViolation = false;

		concrete = true;
	}


	//Constraint checkers.
	private String checkConstraint(){

		return isGlobal && isAnonymous ? ObjectConstraintsManipulation.setErrorAttribute( this, "name", "A global component cannot be anonymous" ) : null;
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}


		return constraintViolation;
	}

	protected void setMyOrderID( int cMyOrderID ){

		myOrderID = cMyOrderID;
	}

	public void setScope( Interval1D cScope ){

		scope = cScope;
	}

	public void setParent( Object cParent ){

		parent = cParent;
	}

	public void setName( String cName ){

		if( cName != null && ! cName.isEmpty() ){

			name = cName;

			isAnonymous = false;
		}

		else isAnonymous = true;
	}

	public Object add( Object object ) {

		ObjectConstraintsManipulation.setConnectionError ( this, object );


		return this;
	}


	public void addStructure( Structure structure ){

		if( structure.getSchemaRepresentationModel().equals( getSchemaRepresentationModel() ) ){

			if( ! hasStructures ){
	
				structures = new ArrayList<Structure>();
	
				hasStructures = true;
			}
	
			structures.add( structure );
	
			setMaterializedReferencedComponents( true );


			size += structure.getSize();


			structure.setMyOrderID( ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, structure, "Its structure does NOT have the SAME schema representation model" );
	}

	protected void setReferencedComponent( ReferencedComponent cReferencedComponent ){

		if( cReferencedComponent.hasConstraintViolation() ) ObjectConstraintsManipulation.setErrorAttribute( this, cReferencedComponent.getTypeKind(), cReferencedComponent.getConstraintViolation() );

		else{

			if( ! hasReferencedComponent ){

				referencedComponent = cReferencedComponent;

				hasReferencedComponent = true;
					
				setMaterializedReferencedComponents( referencedComponent.hasUnmaterializedComponent() );
			}

			else ObjectConstraintsManipulation.setConnectionError ( this, cReferencedComponent, "Its referenced type has already been set" );
		}
	}

	public void addComponent( Component component ){

		if( component.getSchemaRepresentationModel().equals( getSchemaRepresentationModel() ) ){

			if( ! hasComponents ){
	
				components = new ArrayList<Component>();
	
				hasComponents = true;
			}
	
			components.add( component );
	
			setMaterializedReferencedComponents( true );


			size += component.getSize();


			component.setMyOrderID( ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, component, "Its component does NOT have the SAME schema representation model" );
	}

	protected void setAnnotation( Annotation cAnnotation ){

		if( ! hasAnnotation ){

			annotation = cAnnotation;

			hasAnnotation = true;


			size += cAnnotation.getSize();


			cAnnotation.setMyOrderID( ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, cAnnotation, "Its annotation has already been set" );
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}

	public void connect( Map<String, Object> indexLocalComponentsWithoutTargetNamespaces, Map<String, String> schemaQualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaDefinitionNamespace ){//The first 'List' consists of the components of the previous level.

		if( hasUnmaterializedReferencedComponents && ! hasConstraintViolation ){

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "START - connect: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedReferencedComponents + " hasConstraintViolation= " + hasConstraintViolation );


			for( int i = 0; hasStructures && i < structures.size(); ++i ){
	
				structures.get( i ).connect( makeIndexLocalComponentsWithoutTargetNamespaces(), schemaQualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaDefinitionNamespace );
	
				if( structures.get( i ).hasUnmaterializedReferencedComponents() ){

					if( structures.get( i ).hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, structures.get( i ) );

					return;
				}
			}
	
			for( int i = 0; hasComponents && i < components.size(); ++i ){

				components.get( i ).connect( makeIndexRemainingLocalComponentsWithoutTargetNamespaces( components.get( i ) ), schemaQualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaDefinitionNamespace );
	
				if( components.get( i ).hasUnmaterializedReferencedComponents() ){

					if( components.get( i ).hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, components.get( i ) );

					return;
				}
			}
	
			if( hasReferencedComponent ){

				Map<String, String> tagQualifierNamespaces = MapGetter.getFilteredMapValues( "xmlns", ":", tagAttributes );

				referencedComponent.connect( this, schemaQualifierNamespaces, tagQualifierNamespaces, indexLocalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, getSchemaRepresentationModel(), schemaDefinitionNamespace );


				if( referencedComponent.hasUnmaterializedComponent() ){

					if( referencedComponent.hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, referencedComponent );

					return;
				}
			}


			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "END - connect: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedReferencedComponents + " hasConstraintViolation= " + hasConstraintViolation );
		}

		hasUnmaterializedReferencedComponents = false;
	}


	public void setMaterializedReferencedComponents( boolean cHasUnmaterializedReferencedComponennts ){

		hasUnmaterializedReferencedComponents = cHasUnmaterializedReferencedComponennts;
	}

	public boolean hasBuiltInType(){

		return hasReferencedComponent && referencedComponent.isBuiltInType() ? true : false;
	}

	public boolean isDeclaration(){

		return hasComponents || hasStructures ? false : true;
	}

	private Map<String, Object> makeIndexRemainingLocalComponentsWithoutTargetNamespaces( Component component ){

		Map<String, Object> indexComponents = new Hashtable<String, Object>();

		for( int i = 0; hasComponents && i < components.size(); ++i ){

			if( ! components.get( i ).equals( component ) ){

				String componentName = components.get( i ).getName();

				String componentKind = ClassInfo.getClass( components.get( i ) );

				indexComponents.put( componentName + "," + componentKind, components.get( i ) );
			}
		}


		return indexComponents;
	}

	private Map<String, Object> makeIndexLocalComponentsWithoutTargetNamespaces(){

		Map<String, Object> indexComponents = new Hashtable<String, Object>();


		for( int i = 0; hasComponents && i < components.size(); ++i ){

			String componentName = components.get( i ).getName();

			String componentKind = ClassInfo.getClass( components.get( i ) );

			indexComponents.put( componentName + "," + componentKind, components.get( i ) );
		}


		return indexComponents;
	}

	public void setConcrete( boolean cIsConcrete ){

		concrete = cIsConcrete;
	}

	protected void setTagAttributes( List<Map<String, String>> path ){

		Map<String, String> attributes = ListGetters.getLastListElement( path );

		if( attributes != null && ! attributes.isEmpty() ){

			tagAttributes = attributes;

			attributes.put( SchemaRepresentationController.getTagNameKey(), getTagName() );
		}
	}

	private void setSchemaPath( List<Map<String, String>> path ){

		schemaPath = SchemaTags2SchemaPathTransformation.transform( path );

		hasSchemaPath = schemaPath != null ? true : false;
	}

	protected void setSchemaPath( String[] path ){

		schemaPath = path;

		hasSchemaPath = schemaPath != null ? true : false;
	}

	private void setClassPath( List<Map<String, String>> path ){

		classPath = SchemaTags2JavaClassesTransformation.transform( path );

		hasClassPath = classPath != null ? true : false;
	}
	
	protected void setClassPath( String[] path ){

		classPath = path;

		hasClassPath = classPath != null ? true : false;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	protected String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; hasStructures && i < structures.size(); ++i ) if( structures.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, structures.get( i ) );

		for( int i = 0; hasComponents && i < components.size(); ++i ) if( components.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, components.get( i ) );

		if( hasReferencedComponent && referencedComponent.hasFinalConstraintViolation( this ) ) ObjectConstraintsManipulation.setContainmentError( this, referencedComponent );

		if( hasAnnotation && annotation.hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, annotation );


		return constraintViolation;
	}

	public boolean hasUnmaterializedReferencedComponents(){

		return hasUnmaterializedReferencedComponents;
	}

	protected boolean hasID(){

		return hasID;
	}

	public String getID(){

		return id;
	}

	public boolean isAnonymous(){

		return isAnonymous;
	}

	public String getName(){

		return name;
	}

	public boolean hasReferencedComponent(){

		return hasReferencedComponent;
	}

	protected boolean hasMaterializedType(){

		return hasReferencedComponent && ! referencedComponent.isBuiltInType() && ! referencedComponent.hasUnmaterializedComponent() ? true : false;
	}

	public ReferencedComponent getReferencedComponent(){

		return referencedComponent;
	}

	public String getReferenceBuiltInType(){

		return hasReferencedComponent ? referencedComponent.getType() : null;
	}

	public Component getMaterializedType(){

		return hasReferencedComponent && ! referencedComponent.isBuiltInType() && ! referencedComponent.hasUnmaterializedComponent() ? referencedComponent.getMaterializedComponent() : null;
	}

	protected boolean hasComponents(){

		return hasComponents;
	}

	public List<Component> getComponents(){

		return components;
	}

	List<Object> getLeafComponentsAndStructures( String primarySchemaRepresentationModel, String targetSchemaRepresentationModel, List<Object> leaves, Map<String, Object> indexOfTransformedObjectsPath ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - getLeafComponentsAndStructures: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedReferencedComponents + " hasConstraintViolation= " + hasConstraintViolation );


		for( int i = 0; hasStructures && i < structures.size(); ++i ) leaves = structures.get( i ).getLeafComponentsAndStructures( primarySchemaRepresentationModel, targetSchemaRepresentationModel, leaves, indexOfTransformedObjectsPath );
	
		for( int i = 0; hasComponents && i < components.size(); ++i ) leaves = components.get( i ).getLeafComponentsAndStructures( primarySchemaRepresentationModel, targetSchemaRepresentationModel, leaves, indexOfTransformedObjectsPath );


		if( hasReferencedComponent ){

			if( ! referencedComponent.isBuiltInType() ) leaves = referencedComponent.getLeafComponentsAndStructures( primarySchemaRepresentationModel, targetSchemaRepresentationModel, leaves, indexOfTransformedObjectsPath );

			else{

				if( isTheSameSchemaRepresentationModel( primarySchemaRepresentationModel ) ) leaves = ListInsertion.add( this, leaves );

				else{

					Object object = getOriginalObject( this, indexOfTransformedObjectsPath );

					leaves = ListInsertion.add( object, leaves );
				}
			}
		}


		PrintMessages.print( PrintMessages.transformationPrint, "END - getLeafComponentsAndStructures: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedReferencedComponents + " hasConstraintViolation= " + hasConstraintViolation );


		return leaves;
	}

	List<Object> getLeafComponentsOrStructures( String schemaRepresentationModel, List<Object> leaves ){//It returns the components/structures that are leaves, i.e., they have referencedComponent(s) to components.

		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		PrintMessages.print( printKinds, "START - getLeafComponentsOrStructures: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedReferencedComponents + " hasConstraintViolation= " + hasConstraintViolation );


		for( int i = 0; hasStructures && i < structures.size(); ++i ) leaves = structures.get( i ).getLeafComponentsOrStructures( schemaRepresentationModel, leaves );
	
		for( int i = 0; hasComponents && i < components.size(); ++i ) leaves = components.get( i ).getLeafComponentsOrStructures( schemaRepresentationModel, leaves );

		if( hasReferencedComponent && ! referencedComponent.isBuiltInType() ) leaves = ListInsertion.add( this, leaves );


		PrintMessages.print( printKinds, "END - getLeafComponentsOrStructures: " + getTag() + " - hasUnmaterializedComponents= " + hasUnmaterializedReferencedComponents + " hasConstraintViolation= " + hasConstraintViolation );


		return leaves;
	}

	public boolean isTheSameSchemaRepresentationModel( String schemaRepresentationModel ){

		return schemaRepresentationModel != null ? schemaRepresentationModel.equals( getSchemaRepresentationModel() ) : false;
	}

	private static Object getOriginalObject( Object comparedObject, Map<String, Object> indexOfTransformedObjectsPath ){

		Object originalObject = null;


		if( comparedObject != null ){

			String schemaPath = XMLSchemaRepresentationController.getSchemaPath( comparedObject );

			if( schemaPath != null && indexOfTransformedObjectsPath != null ) originalObject = indexOfTransformedObjectsPath.get( schemaPath );
		}


		return originalObject;
	}


	protected boolean hasAnnotation(){

		return hasAnnotation;
	}

	protected Annotation getAnnotation(){

		return annotation;
	}

	public boolean hasStructures(){

		return hasStructures;
	}

	public List<Structure> getStructures(){

		return structures;
	}

	public Map<String, String> getTagAttributes(){

		return tagAttributes;
	}

	public boolean isGlobal(){

		return isGlobal;
	}

	public int getDBKey(){
		
		return dbKey;
	}

	public boolean isConcrete(){

		return concrete;
	}

	public String[] getClassPath(){

		return classPath;
	}

	public boolean equalClassPath( String[] cClassPath ){
		
		boolean equal = true;

		int size = 0, cSize = 0;


		if( hasClassPath ) size = classPath.length;

		if( cClassPath != null ) cSize = cClassPath.length;

		
		for( int i = 0; size == cSize && i < size; ++i ){

			if( i == size - 1 ){
				
				String name = Tokenizer.getPrefix( classPath[ i ], "," );
				
				String referenceComponent = Tokenizer.getSuffix( classPath[ i ], "," );
				
				String cName = Tokenizer.getPrefix( cClassPath[ i ], "," );
				
				String cReferenceComponent = Tokenizer.getSuffix( cClassPath[ i ], "," );


				if( ( name != null && cName != null && name.equals( cName ) && referenceComponent != null && cReferenceComponent != null && referenceComponent.equals( cReferenceComponent ) ) ||
					( name != null && cName != null && name.equals( cName ) && referenceComponent == null && cReferenceComponent == null ) )
					 {

				}

				else
				{

					equal = false;

					break;
				}
			}

			else if( ! classPath[ i ].equals( cClassPath[ i ] ) ){
				
				equal = false;

				break;
			}
		}

	
		return equal;
	}

	private String getClassPathToString(){

		return hasClassPath ? Arrays.toString( getClassPath() ) : null;
	}

	private String printClassPath(){

		return hasClassPath && PrintMessages.toPrint( PrintMessages.xmlPrint ) ? " - classPath: " + getClassPathToString() : "";
		//return hasClassPath ? " - classPath: " + getClassPathToString() : "";
	}

	public String[] getSchemaPath(){

		return schemaPath;
	}

	protected boolean belongsToCycle( Component materializedComponent ){

		if( isRootOfSchemaPath( materializedComponent ) ) return true;


		List<Object> leafComponentsOrStructures = materializedComponent.getLeafComponentsOrStructures( getSchemaRepresentationModel(), null );


		for( int i = 0; leafComponentsOrStructures != null && i < leafComponentsOrStructures.size(); ++i ){

			Object componentOrStructure = leafComponentsOrStructures.get( i );

			List<Component> materializedComponents = getMaterializedComponents( componentOrStructure );

			for( int j = 0; materializedComponents != null && j < materializedComponents.size(); ++j )

				if( ! SchemaRepresentationController.belongsToCycle( materializedComponents.get( j ), componentOrStructure ) && belongsToCycle( materializedComponents.get( j ) ) ) return true;
		}


		return false;
	}

	private boolean isRootOfSchemaPath( Component selectedComponent ){

		String rootSchemaPathToString = selectedComponent != null ? selectedComponent.getSchemaPathToString() : null;

		String schemaPathToString = getRootOfSchemaPath();


		return rootSchemaPathToString != null && schemaPathToString != null ? rootSchemaPathToString.equals( schemaPathToString ) : false; 
	}

	private String getRootOfSchemaPath(){

		if( hasSchemaPath ){

			String[] root = { schemaPath[ 0 ] };


			return Arrays.toString( root  );
		}


		return null;
	}

	private List<Component> getMaterializedComponents( Object componentOrStructure ){

		List<Component> materializedComponents = null;


		if( XMLSchemaRepresentationController.isComponentKind( componentOrStructure ) ){

			ReferencedComponent referencedComponent = ( (Component) componentOrStructure).getReferencedComponent();

			Component materializedComponent = referencedComponent.getMaterializedComponent( getSchemaRepresentationModel() );


			if( materializedComponents == null && materializedComponent != null ) materializedComponents = new ArrayList<Component>();

			if( materializedComponent != null ) materializedComponents.add( materializedComponent );
		}


		else if( XMLSchemaRepresentationController.isStructureKind( componentOrStructure ) ){

			for( int i = 0; ( (Structure) componentOrStructure).hasReferencedComponents() && i < ( (Structure) componentOrStructure).getReferencedComponents().size(); ++i ){

				ReferencedComponent referencedComponent = ( (Structure) componentOrStructure).getReferencedComponents().get( i );

				Component materializedComponent = referencedComponent.getMaterializedComponent( getSchemaRepresentationModel() );


				if( materializedComponents == null && materializedComponent != null ) materializedComponents = new ArrayList<Component>();

				if( materializedComponent != null ) materializedComponents.add( materializedComponent );
			}
		}


		return materializedComponents;
	}

	public String getSchemaPathToString(){

		return hasSchemaPath ? Arrays.toString( getSchemaPath() ) : null;
	}

	private String printSchemaPath(){

		return hasSchemaPath && PrintMessages.toPrint( PrintMessages.xmlPrint ) ? " - schemaPath: " + getSchemaPathToString() : "";
		//return hasSchemaPath ? " - schemaPath: " + getSchemaPathToString() : "";
	}

	public double getSize(){

		return size;
	}

	protected int getMyOrderID(){

		return myOrderID;
	}

	protected String printMyOrderID(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return " (orderID= " + getMyOrderID() + ")";
	}

	private String printScope(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return scope != null ? scope.getIntEndpoints() + " : " : "";
	}

	public List<?> getChildrenWithoutAnnotation( String targetSchemaRepresentationModel ){

		List<?> children = ListTransformations.merge( structures, components );

		if( hasReferencedComponent && ! referencedComponent.isBuiltInType() && ! referencedComponent.belongsToCycle() ) children = ListTransformations.merge( children, referencedComponent.getMaterializedComponent( targetSchemaRepresentationModel ) );


		return children;
	}

	public List<?> getChildren( String targetSchemaRepresentationModel ){

		return ListTransformations.merge( getChildrenWithoutAnnotation( targetSchemaRepresentationModel ), annotation );
	}

	public KindDescr getKindDescr(){

		return new ElementKindDescr( "component", getTagName(), isDeclaration(), hasBuiltInType() );
	}

	Interval1D getScope(){

		return scope;
	}

	Object getParent(){

		return parent;
	}


	public abstract String getSchemaRepresentationModel();

	public abstract String getSchemaKind();

	public abstract String getTagName();

	public abstract String getTag();


	//Visualization.
	public DefaultMutableTreeNode visualize(){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( printScope() + getTag() + printClassPath() + printSchemaPath() );


		for( int i = 0; hasStructures && i < structures.size(); ++i ) node.add( structures.get( i ).visualize() );

		for( int i = 0; hasComponents && i < components.size(); ++i ) node.add( components.get( i ).visualize() );

		if( hasReferencedComponent && ! referencedComponent.isBuiltInType() && ! referencedComponent.belongsToCycle() ) node.add( referencedComponent.visualize() );

		if( hasAnnotation ) node.add( annotation.visualize() );


		return node;
	}
	
	
	//Setters used for xml transformation
	
	public void setxTagAttributes( Map<String, String> cTagAttributes ){
		tagAttributes = cTagAttributes;
	}
	
	public void setxSchemaPath( String[] cSchemaPath ){
		schemaPath = cSchemaPath;
	}
	
	public void setxHasSchemaPath( boolean cHasSchemaPath ){
		hasSchemaPath = cHasSchemaPath;
	}
	
	public void setxClassPath( String[] cClassPath ){
		classPath = cClassPath;
	}
	
	public void setxHasClassPath( boolean cHasClassPath ){
		hasClassPath = cHasClassPath;
	}
	
	public void setxName( String cName ){
		name = cName;
	}
	
	public void setxIsAnonymous( boolean cIsAnonymous ){
		isAnonymous = cIsAnonymous;
	}
	
	public void setxReferencedComponent( ReferencedComponent cReferencedComponent ){
		referencedComponent = cReferencedComponent;
	}
	
	public void setxHasReferencedComponent( boolean cHasReferencedComponent ){
		hasReferencedComponent = cHasReferencedComponent;
	}
	
	public void setxID( String cID ){
		id = cID;
	}
	
	public void setxHasID( boolean cHasID ){
		hasID = cHasID;
	}
	
	public void setxIsGlobal( boolean cIsGlobal ){
		isGlobal = cIsGlobal;
	}
	
	public void setxMyOrderID( int cMyOrderID ){
		myOrderID = cMyOrderID;
	}
	
	public void setxMyChildrenOrderID( int cMyChildrenOrderID ){
		myChildrenOrderID = cMyChildrenOrderID;
	}
	
	public void setxHasUnmaterializedReferencedComponents( boolean cHasUnmaterializedReferencedComponents ){
		hasUnmaterializedReferencedComponents = cHasUnmaterializedReferencedComponents;
	}
	
	public void setxSize( int cSize ){
		size = cSize;
	}
	
	public void setxComponents( List<Component> cComponents ){
		components = cComponents;
	}
	
	public void setxHasComponents( boolean cHasComponents ){
		hasComponents = cHasComponents;
	}
	
	public void setxStructures( List<Structure> cStructures ){
		structures = cStructures;
	}
	
	public void setxHasStructures( boolean cHasStructures ){
		hasStructures = cHasStructures;
	}
	
	public void setxAnnotation( Annotation cAnnotation ){
		annotation = cAnnotation;
	}
	
	public void setxHasAnnotation( boolean cHasAnnotation ){
		hasAnnotation = cHasAnnotation;
	}
	
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
	
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
	
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
	
	public void setxConcrete( boolean cConcrete ){
		concrete = cConcrete;
	}
	
	//Getters used for xml transformation
	
	public Map<String, String> getxTagAttributes(){
		return tagAttributes;
	}
	
	public String[] getxSchemaPath(){
		return schemaPath;
	}
	
	public boolean getxHasSchemaPath(){
		return hasSchemaPath;
	}
	
	public String[] getxClassPath(){
		return classPath;
	}
	
	public boolean getxHasClassPath(){
		return hasClassPath;
	}
	
	public String getxName(){
		return name;
	}
	
	public boolean getxIsAnonymous(){
		return isAnonymous;
	}
	
	public ReferencedComponent getxReferencedComponent(){
		return referencedComponent;
	}
	
	public boolean getxHasReferencedComponent(){
		return hasReferencedComponent;
	}
	
	public String getxID(){
		return id;
	}
	
	public boolean getxHasID(){
		return hasID;
	}
	
	public boolean getxIsGlobal(){
		return isGlobal;
	}
	
	public int getxMyOrderID(){
		return myOrderID;
	}
	
	public int getxMyChildrenOrderID(){
		return myChildrenOrderID;
	}
	
	public boolean getxHasUnmaterializedReferencedComponents(){
		return hasUnmaterializedReferencedComponents;
	}
	
	public int getxSize(){
		return size;
	}
	
	public List<Component> getxComponents(){
		return components;
	}
	
	public boolean getxHasComponents(){
		return hasComponents;
	}
	
	public List<Structure> getxStructures(){
		return structures;
	}
	
	public boolean getxHasStructures(){
		return hasStructures;
	}
	
	public Annotation getxAnnotation(){
		return annotation;
	}
	
	public boolean getxHasAnnotation(){
		return hasAnnotation;
	}
	
	public String getxConstraintViolation(){
		return constraintViolation;
	}
	
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
	
	public int getxDBKey(){
		return dbKey;
	}
	
	public boolean getxConcrete(){
		return concrete;
	}
}
