package org.ow2.choreos.schemaRepresentation.xml;

import java.util.Hashtable;
import java.util.Map;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class XMLBuiltInTypes {


	//Variables.
	private static String[] builtInTypes = {"anyType",    		  //0
											"anySimpleType",      //1
											"string",             //2
											"boolean",            //3
											"base64Binary",       //4
											"hexBinary",          //5
											"float",              //6
											"decimal",            //7
											"double",             //8
											"anyURI",             //9
											"QName",              //10
											"NOTATION",           //11
											"duration",           //12
											"dateTime",           //13
											"time",               //14
											"date",               //15
											"gYearMonth",         //16
											"gYear",              //17
											"gMonthDay",          //18
											"gDay",               //19
											"gMonth",             //20
											"normalizedString",   //21
											"token",			  //22
											"language",			  //23
											"Name",				  //24
											"NMTOKEN",			  //25
											"NCName",			  //26
											"NMTOKENS",			  //27
											"ID",				  //28
											"IDREF",			  //29
											"ENTITY",			  //30
											"IDREFS",			  //31
											"ENTITIES",			  //32
											"integer",			  //33
											"nonPositiveInteger", //34
											"long",				  //35
											"nonNegativeInteger", //36
											"negativeInteger",	  //37
											"int",				  //38
											"unsignedLong",		  //39
											"positiveInteger",	  //40
											"short",			  //41
											"unsignedInt",		  //42
											"byte",				  //43
											"unsignedShort",	  //44
											"unsignedByte"};	  //45

	private static int[] parentBasicType = {	0,//anyType                 has parent = anyType. 
												0,//anySimpleType           has parent = anyType.
												1,//string					has parent = anySimpleType.
												1,//boolean					has parent = anySimpleType.
												1,//base64Binary			has parent = anySimpleType.
												1,//hexBinary				has parent = anySimpleType.
												1,//float					has parent = anySimpleType.
												1,//decimal					has parent = anySimpleType.
												1,//double					has parent = anySimpleType.
												1,//anyURI					has parent = anySimpleType.
												1,//QName					has parent = anySimpleType.
												1,//NOTATION				has parent = anySimpleType.
												1,//duration				has parent = anySimpleType.
												1,//dateTime				has parent = anySimpleType.
												1,//time					has parent = anySimpleType.
												1,//date					has parent = anySimpleType.
												1,//gYearMonth				has parent = anySimpleType.
												1,//gYear					has parent = anySimpleType.
												1,//gMonthDay				has parent = anySimpleType.
												1,//gDay					has parent = anySimpleType.
												1,//gMonth					has parent = anySimpleType.
												2,//normalizedString		has parent = string.
												21,//token					has parent = normalizedString.
												22,//language				has parent = token.
												22,//Name					has parent = token.
												22,//NMTOKEN				has parent = token.
												24,//NCNAME					has parent = Name.
												25,//NMTOKENS				has parent = NMTOKEN.
												26,//ID						has parent = NCNAME.
												26,//IDREF					has parent = NCNAME.
												26,//ENTITY					has parent = NCNAME.
												29,//IDREFS					has parent = IDREF.
												30,//ENTITIES				has parent = ENTITY.
												7,//integer					has parent = decimal.
												33,//nonPositiveInteger		has parent = integer.
												33,//long					has parent = integer.
												33,//nonNegativeInteger		has parent = integer.
												34,//negativeInteger		has parent = nonPositiveInteger.
												35,//int					has parent = long.
												36,//unsignedLong			has parent = nonNegativeInteger.
												36,//positiveInteger		has parent = nonNegativeInteger.
												38,//short					has parent = int.
												39,//unsignedInt			has parent = unsignedLong.
												41,//byte					has parent = short.
												42,//unsignedShort			has parent = unsignedInt.
												44};//unsignedByte			has parent = unsignedShort.


	private static final Map<String, String> tranformedXML2JavaBuiltInTypes = new Hashtable<String, String>() { { put( "string", "String" );
	 																											  put( "integer", "BigInteger" );
	 																											  put( "decimal", "BigDemical" );
	 																											  put( "anySimpleType", "String" );
	 																											  put( "dateTime", "Calendar" );
	 																											  put( "base64Binary", "byte[]" );
	 																											  put( "hexBinary", "byte[]" );
	 																											  put( "unsignedInt", "long" );
	 																											  put( "unsignedShort", "int" );
	 																											  put( "unsignedByte", "short" );
	 																											  put( "time", "Calendar" );
	 																											  put( "date", "Calendar" );
																											  } };


	//Methods.
	public static int getParentBasicType( int i ){

		return parentBasicType[i];
	}


	public static String getBasicType( int i ){

		return builtInTypes[i];
	}


	public boolean ContainsBasicType( String name ){

		for(int i = 0; i < builtInTypes.length; ++i){

			int index = name.toLowerCase().indexOf( builtInTypes[i].toLowerCase() );

			if( index > 0 ) return true;
		}

		return false;
	}


	public static boolean isBuiltInType( String type ){

		for( int i = 0; i < builtInTypes.length; ++i ) if( type != null && type.toLowerCase().equals( builtInTypes[i].toLowerCase() ) ) return true;


		return false;
	}

	public static String transformToJavaBuiltInType( String xmlBuiltInType ){

		String javaBuiltInType = xmlBuiltInType;


		return xmlBuiltInType != null && ( javaBuiltInType = tranformedXML2JavaBuiltInTypes.get( xmlBuiltInType ) ) != null ? javaBuiltInType : xmlBuiltInType;
	}


	private static int getIndexOfBasicType( String type ){

		int i = 0;

		for( i = 0; i < builtInTypes.length; ++i ) if( builtInTypes[i].equals( type ) ) return i;

		return 0;
	}


	private static String[] getPath( String type ){

		String[] path = null;
		int len = 1;
		
		int i = getIndexOfBasicType( type );

		while( i != 0 ){

			i = parentBasicType[ i ];
			++len;
		}

		path = new String[len];
		int c = 0;

		path[c] = type;

		i = getIndexOfBasicType(type);

		while( i != 0 ){

			i = parentBasicType[i];
			++c;

			path[c] = new String( builtInTypes[i] );
		}


		return path;
	}

	private static boolean belongToTheSamePath( String t1, String t2 ){

		String[] maxPath = findMax( getPath( t1 ), getPath( t2 ) );
		String[] minPath = findMin( getPath( t1 ), getPath( t2 ) );

		for( int i = 0; maxPath != null && i < maxPath.length; ++i ) if( maxPath[i].equals( minPath[0] ) ) return true;

		return false;
	}

	public static double getDistance( String t1, String t2 ){

		if( t1.equals( t2 ) ) return 0.0;

		if( ! belongToTheSamePath( t1, t2 ) ) return 1.0;

		String[] maxPath = findMax( getPath( t1 ), getPath( t2 ) );
		String[] minPath = findMin( getPath( t1 ), getPath( t2 ) );

		String bottom = maxPath[0];
		String up = minPath[0];

		int distance = 0;

		int i = getIndexOfBasicType(bottom);

		while ( ! bottom.equals( up ) ){

			i = getParentBasicType(i);

			bottom = getBasicType(i);

			++distance;
		}

		double height = findHeightOfHierarchy();

		return (double) distance / (double) height;
	}

	public static String getSpecificInputType( String[] types ){

		if( types == null ) return null;

		if( types.length > 2 ) return null;


		String t1 = types[ 0 ], t2 = types[ 1 ];

		String[] path1 = getPath( t1 );
		String[] path2 = getPath( t2 );

		String[] maxPath = findMax( path1, path2 );


		return maxPath[ 0 ];
	}

	public static String getGenericOutputType( String[] types ){

		if( types == null ) return null;

		if( types.length > 2 ) return null;


		String t1 = types[ 0 ], t2 = types[ 1 ];

		String[] path1 = getPath( t1 );
		String[] path2 = getPath( t2 );

		String[] minPath = findMin( path1, path2 );


		return minPath[ 0 ];
	}

	private static double findHeightOfHierarchy(){

		String theMostSpecificType = new String( "ENTITIES" );

		String[] path = getPath( theMostSpecificType );

		return path.length - 1;
	}

	private static String[] findMax( String[] a, String[] b ){

		return a.length >= b.length ? a : b;
	}

	private static String[] findMin(String[] a, String[] b){

		return a.length < b.length ? a : b;
	}
}