package org.ow2.choreos.schemaRepresentation.xml.completelyStructured;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.Schema;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.ComponentController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute.AttributeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.dataType.DataTypeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.StructureController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class XMLSchemaCompletelyStructuredRepresentationController {


	//Variables.
	private static Map<String, String> extractionRules = new Hashtable<String, String>() { { put( "SimpleType,false", "ComplexType" );

	 																						 put( "ComplexType,false", "ComplexType" );

	 																						 put( "AttributeDefinition,false", "ComplexType" );

	 																						 put( "AttributeDeclaration,false", "ComplexType" );

	 																						 put( "ElementDefinition,false", "ComplexType" );

	 																						 put( "ElementDeclaration,false", "ComplexType" );

	 																						 put( "AttributeGroupDefinition,false", "ComplexType" );

	 																						 put( "AttributeGroupDeclaration,false", "ComplexType" );

	 																						 put( "ElementGroupDefinition,false", "ComplexType" );

	 																						 put( "ElementGroupDeclaration,false", "ComplexType" );

	 																						 put( "AttributeDeclaration,true", "AttributeDeclaration" );

	 																						 put( "ElementDeclaration,true", "AttributeDeclaration" );

	 																						 put( "AttributeGroupDeclaration,true", "AttributeDeclaration" );

	 																						 put( "ElementGroupDeclaration,true", "AttributeDeclaration" );


	 																						 put( "SimpleContent,true", "AttributeDeclaration" );

																							 put( "SimpleContent,false", "ComplexType" );

  																							 put( "ComplexContent,true", "AttributeDeclaration" );

																							 put( "ComplexContent,false", "ComplexType" );

																							 put( "SimpleTypeRestriction,true", "AttributeDeclaration" );

																							 put( "SimpleTypeRestriction,false", "ComplexType" );
																						} };


	//Verifiers.
	public static boolean isXMLCompleteSchemaKind( Object object ){

		return object != null ? object.getClass().equals( XMLSchemaCompletelyStructured.class ) : false;
	}

	public static boolean isComponentKind( Object object ){

		return ComponentController.isComponentKind( object );
	}

	public static boolean isBuiltInType( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).hasBuiltInType();

		if( isStructureKind( object ) ) return ( (Structure) object).hasBuiltInType();


		return false;
	}
	

	public static boolean isStructureKind( Object object ){

		return StructureController.isStructureKind( object );
	}

	public static boolean isSchemaTagName( String tagName ){

		return tagName != null && tagName.equals( "schema" ) ? true : false;
	}


	//Getters.
	public static Object getSchema( String tagName, String qualifier, Map<String, String> attributes ){

		if( isSchemaTagName( tagName ) ) return new XMLSchemaCompletelyStructured( qualifier, attributes );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Object getErrorSchema( String tagName, String qualifier, Map<String, String> attributes, String constraintViolation ){

		if( isSchemaTagName( tagName ) ){

			Schema xmlSchemaCompleteRepresentation = new XMLSchemaCompletelyStructured( qualifier, attributes );

			xmlSchemaCompleteRepresentation.setConstraintViolation( constraintViolation );
		}


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}


	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( ComponentController.isComponentTagName( tagName ) ) return ComponentController.getTagObject( path, isGlobal );

		if( StructureController.isStructureTagName( tagName ) ) return StructureController.getTagObject( path );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static List<String> getCompatibleComponentKinds( Object object ){//It returns the compatible component kinds with the 'object'.

		return ComponentController.getCompatibleComponentKinds( object );
	}


	public static Object extractObject( Object[] sourceObjects, List<Map<String, String>> path, boolean belongsToInputMessage, boolean isGlobal, String namesDistanceCalculatorAlgorithm ){//It is used for extraction purposes.

		String[] sourceObjectKinds = null, extractedObjectKinds = null;//It is used only by the method 'getUnknownObject()'.

		String extractedObjectKind = null;


		if( sourceObjects != null ){

			String extractionRule = ClassInfo.getClass( sourceObjects[ 0 ] ) + "," + XMLSchemaRepresentationController.isBuiltInType( sourceObjects[ 0 ] );//I have quarranted that the objectKinds are compatible by construction, and so, I check only the first kind.

			extractedObjectKind = extractionRules.get( extractionRule );
		}

		if( DataTypeController.isDataTypeKind( extractedObjectKind ) ) return DataTypeController.extractObject( sourceObjects, extractedObjectKind, path, isGlobal, namesDistanceCalculatorAlgorithm );

		if( AttributeController.isAttributeKind( extractedObjectKind ) ) return AttributeController.extractObject( sourceObjects, extractedObjectKind, path, belongsToInputMessage, isGlobal, namesDistanceCalculatorAlgorithm );


		if( extractedObjectKind != null ){

			String[] tmpExtractedObjectKinds = { extractedObjectKind };

			extractedObjectKinds = tmpExtractedObjectKinds;
		}

		return UnknownClassesController.getUnknownObject( sourceObjectKinds, XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation(), extractedObjectKinds, XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation() );
	}


	//Setters.
	public static Object add( Object internal, Object external ){

		PrintMessages.print( PrintMessages.controllerPrint, "XMLSchemaCompletelyStructuredRepresentationController: Set the internal= " + ClassInfo.getClass( internal ) + " to external= " + ClassInfo.getClass( external ));


		if( ComponentController.isComponentKind( external ) ) return ( (Component) external).add( internal );

		if( StructureController.isStructureKind( external ) ) return ( (Structure) external).add( internal );
		

		return external;
	}
}
