package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute;

import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;


/**
 * 
 * @author dathanas
 *
 */
abstract class Attribute extends Component {


	//Variables.
	private static final long serialVersionUID = 342872409440616349L;

	private String defaultValue;
	protected boolean hasDefaultValue;

	private String fixed;
	protected boolean hasFixed;

	private String form;
	private boolean hasForm;

	private String use;
	private boolean hasUse;


	//Constructor.
	protected Attribute( List<Map<String, String>> path, boolean isGlobal ){

		super( path, isGlobal );


		Map<String, String> attributes = ListGetters.getLastListElement( path );

		String cUse = (String) MapGetter.getValueByStringKey( "use", attributes );

		if( cUse != null && ! cUse.isEmpty() ){

			if ( checkConstraint3( cUse ) == null ){

				use = cUse;

				hasUse = true;
			}
		}

		else hasUse = false;


		String cDefault = (String) MapGetter.getValueByStringKey( "default", attributes );

		if( cDefault != null && ! cDefault.isEmpty() ){

			defaultValue = cDefault;

			hasDefaultValue = true;
		}

		else hasDefaultValue = false;


		String cFixed = (String) MapGetter.getValueByStringKey( "fixed", attributes );

		if( cFixed != null && ! cFixed.isEmpty() ){

			fixed = cFixed;

			hasFixed = true;
		}

		else hasFixed = false;


		String cForm = MapGetter.getValueByKeyToLowerCase( "form", attributes );

		if( cForm != null && ! cForm.isEmpty() && checkConstraint4( cForm ) == null ){

			form = cForm;

			hasForm = true;
		}

		else hasForm = false;


		checkConstraint1();

		checkConstraint2();
	}


	//Constraint checkers.
	private String checkConstraint1(){

		String[] attributeNames = { "fixed", "default" };

		return hasFixed && hasDefaultValue ? ObjectConstraintsManipulation.setErrorAttribute( this, attributeNames, "Values in both 'default' and 'fixed' attributes are not allowed" ) : null;
	}

	private String checkConstraint2(){

		String[] attributeNames = { "use", "default" };

		return hasDefaultValue && ! use.equals( "optional" ) ? ObjectConstraintsManipulation.setErrorAttribute( this, attributeNames, "If the 'default' and 'use' attributes are both present, the 'use' attribute must have the value 'optional'" ) : null;
	}

	private String checkConstraint3( String use ){

		return ! use.equals( "optional" ) && ! use.equals( "prohibited" ) && ! use.equals( "required" ) ? ObjectConstraintsManipulation.setErrorAttribute( this, "use", "use= (optional | prohibited | required)" ) : null;
	}

	private String checkConstraint4( String form ){

		return ! form.equals( "qualified" ) && ! form.equals( "unqualified" ) ? ObjectConstraintsManipulation.setErrorAttribute( this, "form", "form= ( qualified | unqualified )" ) : null;
	}


	//Getters.
	boolean hasDefaultValue(){

		return hasDefaultValue;
	}

	String getDefaultValue(){

		return defaultValue;
	}

	boolean hasFixed(){

		return hasFixed;
	}

	String getFixed(){

		return fixed;
	}

	boolean hasForm(){

		return hasForm;
	}

	String getForm(){

		return form;
	}

	boolean hasUse(){

		return hasUse;
	}

	String getUse(){

		return use;
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public String getTagName(){

		return ComponentTagNames.attributeTagName;
	}

	public String getTag(){

		return getTagName() + MapPrinting.printSpaceSeparated( SchemaRepresentationController.getTagNameKey(), getTagAttributes() ) + printMyOrderID();
	}
}
