package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLBuiltInTypes;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.ComponentController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class AttributeController {


	//Variables.
	private static Map<String, String> attributeKinds = new Hashtable<String, String>() { { put( "AnyAttribute", "AnyAttribute" ); put( "AttributeDefinition", "AttributeDefinition" ); put("AttributeDeclaration", "AttributeDeclaration"); } };


	//Verifiers.
	public static boolean isAttributeKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, getAttributeKinds() );
	}

	public static boolean isAttributeKind( String objectKind ){

		return MapGetter.keyBelongsToMapStrings( objectKind, getAttributeKinds() );
	}

	public static boolean isAttributeDefinitionKind( Object object ){

		return object != null ? object.getClass().equals( AttributeDefinition.class ) : false;
	}

	public static boolean isAnyAttributeKind( Object object ){

		return object != null ? object.getClass().equals( AnyAttribute.class ) : false;
	}


	public static boolean isAttributeTagName( String tagName ){

		return tagName != null && ( tagName.equals( "attribute" ) || tagName.toLowerCase().equals( "anyattribute" ) ) ? true : false;
	}

	public static boolean isConcreteAttributeTagName( String tagName ){

		return tagName != null && tagName.equals( "attribute" ) ? true : false;
	}

	public static boolean isAnyAttributeTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "anyattribute" ) ? true : false;
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isConcreteAttributeTagName( tagName ) ){

			Map<String, String> attributes = ListGetters.getLastListElement( path );

			String ref = (String) MapGetter.getValueByStringKey( "ref", attributes );

			if( ref != null && ! ref.isEmpty() ) return new AttributeDeclaration( path, ref, "ref", isGlobal );


			String type = (String) MapGetter.getValueByStringKey( "type", attributes );

			if( type != null && ! type.isEmpty() ) return new AttributeDeclaration( path, type, "type", isGlobal );


			return new AttributeDefinition( path, isGlobal );
		}

		else if( isAnyAttributeTagName( tagName ) ) return new AnyAttribute( path, isGlobal );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}


	public static Object extractObject( Object[] sourceObjects, String extractedObjectKind, List<Map<String, String>> path, boolean belongsToInputMessage, boolean isGlobal, String namesDistanceCalculatorAlgorithm ){

		String[] sourceObjectKinds = null, sourceObjectNames = null, sourceObjectBuiltInTypes = null, extractedObjectKinds = null;//It is used only by the method 'getUnknownObject()'.


		for( int i = 0; sourceObjects != null && i < sourceObjects.length; ++i ){

			if( sourceObjectKinds == null ) sourceObjectKinds = new String[ sourceObjects.length ];

			if( sourceObjectNames == null ) sourceObjectNames = new String[ sourceObjects.length ];

			if( sourceObjectBuiltInTypes == null ) sourceObjectBuiltInTypes = new String[ sourceObjects.length ];


			sourceObjectKinds[ i ] = ClassInfo.getClass( sourceObjects[ i ] );

			if( ComponentController.isComponentKind( sourceObjects[ i ] ) ) sourceObjectNames[ i ] = ( (Component) sourceObjects[ i ] ).getName();

			else sourceObjectNames[ i ] = "";


			sourceObjectBuiltInTypes[ i ] = XMLSchemaRepresentationController.getReferencedBuiltInType( sourceObjects[ i ] );
		}


		if( sourceObjects != null ){

			String extractedObjectType = extractAttributeTag( sourceObjectNames, sourceObjectBuiltInTypes, path,  belongsToInputMessage, namesDistanceCalculatorAlgorithm );


			return new AttributeDeclaration( path, extractedObjectType, "type", isGlobal );
		}


		if( extractedObjectKind != null ){

			String[] tmpExtractedObjectKinds = { extractedObjectKind };

			extractedObjectKinds = tmpExtractedObjectKinds;
		}

		return UnknownClassesController.getUnknownObject( sourceObjectKinds, XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation(), extractedObjectKinds, XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation() );
	}

	private static String extractAttributeTag( String[] sourceObjectNames, String[] sourceObjectBuiltInTypes, List<Map<String, String>> path, boolean belongsToInputMessage, String namesDistanceCalculatorAlgorithm ){

		String extractedObjectType = null;

		if( belongsToInputMessage ) extractedObjectType = XMLBuiltInTypes.getSpecificInputType( sourceObjectBuiltInTypes );

		else extractedObjectType = XMLBuiltInTypes.getGenericOutputType( sourceObjectBuiltInTypes );


		extractElementTag( sourceObjectNames, extractedObjectType, path, namesDistanceCalculatorAlgorithm );


		return extractedObjectType;
	}

	private static void extractElementTag( String[] sourceObjectNames, String extractedObjectType, List<Map<String, String>> path, String namesDistanceCalculatorAlgorithm ){

		String extractedName = NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, sourceObjectNames );


		Map<String, String> tagAttributes = ListGetters.getLastListElement( path );

		tagAttributes.put( SchemaRepresentationController.getTagNameKey(), ComponentTagNames.attributeTagName );

		tagAttributes.put( "name", extractedName );

		tagAttributes.put( "type", extractedObjectType );
	}


	public static Map<String, String> getAttributeKinds(){

		return attributeKinds;
	}

	public static String getFlatAttributeKinds(){

		return MapPrinting.printCommaSeparated( getAttributeKinds() );
	}
}
