package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.dataType;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute.AttributeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.group.GroupController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.ComplexTypeStructureController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.composition.ComplexTypeCompositionController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.specialization.SpecializationController;


/**
 * 
 * @author dathanas
 *
 */
class ComplexType extends Component {


	//Variables.
	private static final long serialVersionUID = 1456147225200468779L;

	private boolean isAbstract;

	private List<String> block;
	private boolean hasBlock;

	private List<String> finalValues;
	private boolean hasFinal;

	private boolean mixed;


	//Constructor.
	ComplexType( List<Map<String, String>> path, boolean isGlobal ){

		super( path, isGlobal );


		hasFinal = false;
		hasBlock = false;


		isAbstract = Boolean.parseBoolean( (String) MapGetter.getValueByStringKey( "abstract", getTagAttributes() ) );

		mixed = Boolean.parseBoolean( (String) MapGetter.getValueByStringKey( "mixed", getTagAttributes() ) );


		parse( "block", (String) MapGetter.getValueByStringKey( "block", getTagAttributes() ) );

		parse( "final", (String) MapGetter.getValueByStringKey( "final", getTagAttributes() ) );
	}


	//Methods used by the constructor.
	void parse( String label, String values ){

		if( values == null || ( values != null && values.isEmpty() ) ) return;


		int pos = values.indexOf(" ");

		if( pos <= 0 ){//Only one value exists.

			if( checkConstraint4( label, values ) == null ) add( label, values );
		}

		else{

			while( pos > 0 ){

				String tmpValue = values.substring( 0, pos );

				if( checkConstraint4( label, tmpValue ) == null ) add( label, tmpValue );

				values = values.substring( pos + 1, values.length() );

				pos = values.indexOf(" ");
			}

			if( checkConstraint4( label, values ) == null ) add( label, values );
		}

		checkConstraint9( label );
	}

	private void add( String label, String value ){

		if( label.equals( "block" ) ){

			if( ! hasBlock ){

				block = new ArrayList<String>();

				hasBlock = true;
			}

			block.add( value );
		}

		else{

			if( ! hasFinal ){

				finalValues = new ArrayList<String>();

				hasFinal = true;
			}

			finalValues.add( value );
		}
	}


	//Constraint checkers.
	private String checkConstraints123( Object object ){

		return ( hasComponents() ) ? ObjectConstraintsManipulation.setConnectionError( this, object, "The declaration of the '" + ( (Structure) object).getTag() + "' should not follow the ( declaration | definition ) of an attribute, attributeGroup or anyAttribute" ) : null;
	}

	private String checkConstraint4( String label, String value ){

		return ( value != null && ! value.equals( "#all" ) && ! value.equals( "extension" ) && ! value.equals( "restriction" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, label + "= '#all', 'extension' and 'restriction'" ) : null;
	}

	private String checkConstraint9( String label ){

		int occurrences = 0;

		String message = "The '#all' value cannot appear in a list of values";

		if( label.equals( "block" ) ){

			for( int i = 0; hasBlock && i < block.size(); ++i ) if( block.get( i ).equals( "#all" ) ) ++occurrences;

			return ( hasBlock && block.size() > 1 && occurrences > 0 ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, message ) : null;
		}

		else{

			for( int i = 0; hasFinal && i < finalValues.size(); ++i ) if( finalValues.get( i ).equals( "#all" ) ) ++occurrences;

			return ( hasFinal && finalValues.size() > 1 && occurrences > 0 ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, message ) : null;
		}
	}

	private String checkConstraint8( Object object ){

		if( GroupController.isElementGroupKind( object ) && hasStructures() ) return ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + ( (Component) object).getTag() + "' cannot co-exist with a structure node" );

		if( ( ComplexTypeCompositionController.isCompositionKind( object ) || SpecializationController.isSpecializationKind( object ) ) && hasElementGroup() ) return ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + ( (Structure) object).getTag() + "' cannot co-exist with a group node" );

		return null;
	}


	//Setters.
	private void setGroup( Component cGroup ){

		if( hasElementGroup() ) ObjectConstraintsManipulation.setConnectionError( this, cGroup, "Its group has already been set" );

		else super.addComponent( cGroup );
	}

	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( ( ComplexTypeCompositionController.isCompositionKind( object ) || SpecializationController.isSpecializationKind( object ) ) && checkConstraints123( object ) == null && checkConstraint8( object ) == null ) addStructure( (Structure) object );

			else if( GroupController.isElementGroupKind( object ) && checkConstraints123( object ) == null && checkConstraint8( object ) == null ) setGroup( (Component) object );

			else if( GroupController.isAttributeGroupKind( object ) || AttributeController.isAttributeKind( object ) ) addComponent( (Component) object );

			else if( XMLSchemaRepresentationController.isAnnotationKind( object ) ) setAnnotation( (Annotation) object );

			else{

				String message1 = "Valid content:\n1. complexType structures: " + ComplexTypeStructureController.getFlatComplexTypeStructureKinds();

				String message2 = "\n2. (Element and attribute) groups: " + GroupController.getFlatGroupKinds() + " and attributes: " + AttributeController.getFlatAttributeKinds();

				String message3 = "\n3. Annotation";

				ObjectConstraintsManipulation.setConnectionError ( this, object, message1 + message2 + message3 );
			}
		}

		return this;
	}


	//Getters.
	boolean isAbstract(){

		return isAbstract;
	}

	boolean hasBlock(){

		return hasBlock;
	}

	List<String> getBlock(){

		return block;
	}

	boolean isMixed(){

		return mixed;
	}

	boolean hasElementGroup(){

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i ) if( GroupController.isElementGroupKind( getComponents().get( i ) ) ) return true;


		return false;
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public String getTagName(){

		return ComponentTagNames.complexTypeTagName;
	}

	public String getTag(){

		return getTagName() + MapPrinting.printSpaceSeparated( SchemaRepresentationController.getTagNameKey(), getTagAttributes() ) + printMyOrderID();
	}
}
