package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.element;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;

import org.ow2.choreos.unknownClasses.UnknownClassesController;


@SuppressWarnings("serial")
public class ElementController {


	//Variables.
	private static Map<String, String> elementKinds = new Hashtable<String, String>() { { put( "Any", "Any" ); put( "ElementDefinition", "ElementDefinition" ); put( "ElementDeclaration", "ElementDeclaration" ); } };
	private static Map<String, String> elementTagNames = new Hashtable<String, String>() { { put( "any", "any" ); put( ComponentTagNames.elementTagName, ComponentTagNames.elementTagName ); } };


	//Verifiers.
	public static boolean isElementKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, elementKinds );
	}

	public static boolean isElementDefinitionKind( Object object ){

		return object != null ? object.getClass().equals( ElementDefinition.class ) : false;
	}

	public static boolean isElementDeclarationKind( Object object ){

		return object != null ? object.getClass().equals( ElementDeclaration.class ) : false;
	}


	public static boolean isElementTagName( String tagName ){

		return MapGetter.keyBelongsToMapStrings( tagName, elementTagNames );
	}

	private static boolean isConcreteElementTagName( String tagName ){

		return tagName != null && tagName.equals( ComponentTagNames.elementTagName );
	}

	private static boolean isAnyElementTagName( String tagName ){

		return tagName != null && tagName.equals( ComponentTagNames.anyTagName );
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isConcreteElementTagName( tagName ) ){

			Map<String, String> attributes = ListGetters.getLastListElement( path );

			String ref = (String) MapGetter.getValueByStringKey( "ref", attributes );
	
			if( ref != null && ! ref.isEmpty() ) return new ElementDeclaration( path, ref, "ref", isGlobal );
	
			String type = (String) MapGetter.getValueByStringKey( "type", attributes );
	
			if( type != null && ! type.isEmpty() ) return new ElementDeclaration( path, type, "type", isGlobal );
	
			return new ElementDefinition( path, isGlobal );
		}

		else if( isAnyElementTagName( tagName ) ) return new Any( path, isGlobal );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}


	public static Map<String, String> getElementKinds(){

		return elementKinds;
	}

	public static String getFlatElementKinds(){

		return MapPrinting.printCommaSeparated( elementKinds );
	}
}
