package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.group;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.Occurrences;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.composition.ComplexTypeCompositionController;


/**
 * 
 * @author dathanas
 *
 */
class ElementGroupDefinition extends GroupDefinition {


	//Variables.
	private static final long serialVersionUID = 957588977471604045L;

	private Occurrences occurs;


	//Constructor.
	ElementGroupDefinition( List<Map<String, String>> path, boolean isGlobal ) {

		super( path, isGlobal );


		occurs = new Occurrences( MapGetter.getValueByKeyToLowerCase( "minOccurs", getTagAttributes() ), MapGetter.getValueByKeyToLowerCase( "maxOccurs", getTagAttributes() ) );

		if( occurs.hasConstraintViolation() ) ObjectConstraintsManipulation.setErrorAttribute( this, "minOccurs/maxOccurs", occurs.getConstraintViolation() );
	}


	//Constraint checkers.
	private String checkConstraint( Object object ){

		return hasStructures() ? ObjectConstraintsManipulation.setConnectionError( this, object, "A group cannot contain more than one compositions") : null;
	}


	//Setters.
	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( ComplexTypeCompositionController.isCompositionKind ( object ) && checkConstraint( object ) == null ) addStructure( (Structure) object );

			else if( XMLSchemaRepresentationController.isAnnotationKind( object ) ) setAnnotation( (Annotation) object );

			else ObjectConstraintsManipulation.setConnectionError ( this, object, "Valid content:\n\n1. complexType structures: " + ComplexTypeCompositionController.getFlatComplexTypeCompositionKinds() + "\n2. Annotation" );
		}


		return this;
	}


	//Getters.
	public String getTagName(){

		return ComponentTagNames.groupTagName;
	}

	@SuppressWarnings("serial")
	public String getTag(){

		Map<String, String> exceptedKeys = new Hashtable<String, String>() { { put( SchemaRepresentationController.getTagNameKey(), SchemaRepresentationController.getTagNameKey() );
																			   put( "minOccurs", "minOccurs" );
																			   put( "maxOccurs", "maxOccurs" ); } };

		return getTagName() + MapPrinting.printSpaceSeparated( exceptedKeys, getTagAttributes() ) + occurs.visualize() + printMyOrderID();
	}
}
