package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.specialization;

import java.util.List;
import java.util.Map;

import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute.AttributeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.group.GroupController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.composition.ComplexTypeCompositionController;


/**
 * 
 * @author dathanas
 *
 */
class ComplexContentSpecializationKind extends SpecializationKind {


	//Variables.
	private static final long serialVersionUID = -7318799269153405099L;


	//Constructor.
	protected ComplexContentSpecializationKind( List<Map<String, String>> path ){

		super( path );
	}


	//Constraint checkers.
	private String checkConstraint1( Object object ){

		return ( GroupController.isElementGroupKind( object ) && hasStructures() ) || ( ComplexTypeCompositionController.isCompositionKind( object ) && hasElementGroup() ) ? ObjectConstraintsManipulation.setConnectionError( this, object, "It should not contain a group and composition node concurrently" ) : null;
	}

	private String checkConstraint2( Object object ){

		return hasElementGroup() ? ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + getTag() + "' cannot contain more than one groups" ) : null;
	}

	private String checkConstraint3( Object object ){

		return hasComposition() ? ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + getTag() + "' cannot contain more than one compositions" ) : null;
	}

	protected String checkConstraint4( Object object ){

		return AttributeController.isAnyAttributeKind( object ) && hasAnyAttribute() ? ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + getTag() + "' cannot contain more than one anyAttributes" ) : null;
	}

	protected String checkConstraint5( Object object ){

		return hasAnnotation() ? ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + getTag() + "' cannot contain more than one annotations" ) : null;
	}


	//Setters.
	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( ( AttributeController.isAttributeKind( object ) && checkConstraint4( object ) == null ) || GroupController.isAttributeGroupKind( object ) || ( GroupController.isElementGroupKind( object ) && checkConstraint1( object ) == null && checkConstraint2( object ) == null ) ) addComponent( (Component) object );

			else if( XMLSchemaRepresentationController.isAnnotationKind( object ) && checkConstraint5( object ) == null ) setAnnotation( (Annotation) object );

			else if( ComplexTypeCompositionController.isCompositionKind( object ) && checkConstraint1( object ) == null && checkConstraint3( object ) == null ) addStructure( (Structure) object );

			else{

				String message1 = "Valid content:\n1. For structures, the complexType structures: " + ComplexTypeCompositionController.getFlatComplexTypeCompositionKinds();

				String message2 = "\n2. For components, the attributes: " + AttributeController.getFlatAttributeKinds() + ", the attribute groups: " + GroupController.getFlatAttributeGroupKinds() + "and the element groups: " + GroupController.getElementGroupKinds();

				String message3 = "\n3. Annotation";

				ObjectConstraintsManipulation.setConnectionError ( this, object, message1 + message2 + message3 );
			}
		}

		return this;
	}


	//Getters.
	private boolean hasElementGroup(){

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i ) if( GroupController.isElementGroupKind( getComponents().get( i ) ) ) return true;

		return false;
	}

	private boolean hasComposition(){

		for( int i = 0; hasStructures() && i < getStructures().size(); ++i ) if( ComplexTypeCompositionController.isCompositionKind( getComponents().get( i ) ) ) return true;

		return false;
	}

	private boolean hasAnyAttribute(){

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i ) if( AttributeController.isAnyAttributeKind( getComponents().get( i ) ) ) return true;


		return false;
	}
}
