package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.complexType.specialization;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.StructureTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class SpecializationController {


	//Variables.
	private static Map<String, String> specializationKinds = new Hashtable<String, String>() { { put( "Extension", "Extension" );
																								 put( "Restriction", "Restriction" );
																								 put( "ComplexContentSpecializationKind", "ComplexContentSpecializationKind" );
																								 put( "ComplexContent", "ComplexContent" );
																								 put( "SimpleContent", "SimpleContent" ); } };


	//Verifiers.
	public static boolean isSimpleContentSpecializationKind( Object object ){

		return object != null ? object instanceof SimpleContentSpecializationKind : null;
	}

	public static boolean isComplexContentSpecializationKind( Object object ){

		return object != null ? object instanceof ComplexContentSpecializationKind : null;
	}

	public static boolean isSpecializationKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, specializationKinds );
	}


	private static boolean isSimpleContentTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "simplecontent" ) ? true : false;
	}

	private static boolean isComplexContentTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "complexcontent" ) ? true : false;
	}

	private static boolean isRestrictionTagName( String tagName ){

		return tagName != null && tagName.equals( "restriction" ) ? true : false;
	}

	private static boolean isExtensionTagName( String tagName ){

		return tagName != null && tagName.equals( "extension" ) ? true : false;
	}

	private static boolean isContentTagName( String tagName ){

		return tagName != null && ( tagName.toLowerCase().equals( "simplecontent" ) || tagName.toLowerCase().equals( "complexcontent" ) ) ? true : false;
	}

	private static boolean isSimpleContentRestrictionTagName( List<Map<String, String>> path, String tagName ){

		String prevPrevTagName = ListGetters.getPrevPrevElementValue( SchemaRepresentationController.getTagNameKey(), path );


		return tagName != null && tagName.equals( "restriction" ) && prevPrevTagName != null && prevPrevTagName.equals( StructureTagNames.simpleContentTagName ) ? true : false;
	}

	private static boolean isSimpleContentExtensionTagName( List<Map<String, String>> path, String tagName ){

		String prevPrevTagName = ListGetters.getPrevPrevElementValue( SchemaRepresentationController.getTagNameKey(), path );


		return tagName != null && tagName.equals( "extension" ) && prevPrevTagName != null && prevPrevTagName.equals( StructureTagNames.simpleContentTagName ) ? true : false;
	}

	private static boolean isComplexContentRestrictionTagName( List<Map<String, String>> path, String tagName ){

		String prevPrevTagName = ListGetters.getPrevPrevElementValue( SchemaRepresentationController.getTagNameKey(), path );


		return tagName != null && tagName.equals( "restriction" ) && prevPrevTagName != null && prevPrevTagName.equals( StructureTagNames.complexContentTagName ) ? true : false;
	}

	private static boolean isRestrictionTagName( List<Map<String, String>> path, String tagName ){

		String prevPrevTagName = ListGetters.getPrevPrevElementValue( SchemaRepresentationController.getTagNameKey(), path );


		return tagName != null && ( tagName.equals( "extension" ) || tagName.equals( "restriction" ) ) && prevPrevTagName != null && ( prevPrevTagName.equals( StructureTagNames.complexContentTagName ) || prevPrevTagName.equals( StructureTagNames.simpleContentTagName ) ) ? true : false;
	}

	private static boolean isComplexContentExtensionTagName( List<Map<String, String>> path, String tagName ){

		String prevPrevTagName = ListGetters.getPrevPrevElementValue( SchemaRepresentationController.getTagNameKey(), path );


		return tagName != null && tagName.equals( "extension" ) && prevPrevTagName != null && prevPrevTagName.equals( StructureTagNames.complexContentTagName ) ? true : false;
	}

	public static boolean isSpecializationTagName( String tagName ){

		if( isContentTagName( tagName ) ) return true;

		if( isRestrictionTagName( tagName ) ) return true;

		if( isExtensionTagName( tagName ) ) return true;


		return false;
	}

	public static boolean isSpecializationTagName( List<Map<String, String>> path ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isContentTagName( tagName ) ) return true;

		if( isRestrictionTagName( path, tagName ) ) return true;

		if( isExtensionTagName( tagName ) ) return true;


		return false;
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isSimpleContentTagName( tagName ) ) return new SimpleContent( path );

		if( isSimpleContentRestrictionTagName( path, tagName ) ) return new Restriction( path );

		if( isSimpleContentExtensionTagName( path, tagName ) ) return new Extension( path );


		if( isComplexContentTagName( tagName ) ) return new ComplexContent( path );

		if( isComplexContentRestrictionTagName( path, tagName ) || isComplexContentExtensionTagName( path, tagName ) ) return new ComplexContentSpecializationKind( path );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Map<String, String> getSpecializationKinds(){

		return specializationKinds;
	}

	public static String getFlatSpecializationKinds(){

		return MapPrinting.printCommaSeparated( getSpecializationKinds() );
	}
}
