package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.restrictionFacet;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class RestrictionFacetController {


	//Variables.
	private static Map<String, String> restrictionFacetKinds = new Hashtable<String, String>() { { put("NonFixedFacet", "NonFixedFacet"); put("PositiveValueFixedFacet", "PositiveValueFixedFacet"); put("PositiveValueFixedFacet", "PositiveValueFixedFacet"); put("NonNegativeValueFixedFacet", "NonNegativeValueFixedFacet"); put("AnyValueFixedFacet", "AnyValueFixedFacet"); } };


	//Verifiers.
	private static boolean isNonFixedFacetTagName( String tagName ){

		return tagName != null && ( tagName.equals( "pattern" ) || tagName.equals( "enumeration" ) ) ? true : false;
	}

	private static boolean isPositiveValueFixedFacetTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "totaldigits" ) ? true : false;
	}

	private static boolean isNonNegativeValueFixedFacetTagName( String tagName ){

		return tagName != null && ( tagName.toLowerCase().equals( "fractiondigits" ) || tagName.equals( "length" ) || tagName.toLowerCase().equals( "minlength" ) || tagName.toLowerCase().equals( "maxlength" ) ) ? true : false;
	}

	private static boolean isAnyValueFixedFacetTagName( String tagName ){

		return tagName != null && ( tagName.toLowerCase().equals( "minexclusive" ) || tagName.toLowerCase().equals( "mininclusive" ) || tagName.toLowerCase().equals( "maxexclusive" ) || tagName.toLowerCase().equals( "maxinclusive" ) ) ? true : false;
	}

	private static boolean isEnumeratedValueFacetTagName( String tagName ){

		return tagName != null && tagName.equals( "whitespace" ) ? true : false;
	}

	public static boolean isRestrictionFacetKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, restrictionFacetKinds );
	}

	public static boolean isRestrictionFacetTagName( String tagName ){

		return isNonFixedFacetTagName( tagName ) || isPositiveValueFixedFacetTagName( tagName ) || isNonNegativeValueFixedFacetTagName( tagName ) || isAnyValueFixedFacetTagName( tagName ) || isEnumeratedValueFacetTagName( tagName ) ? true : false;
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isNonFixedFacetTagName( tagName ) ) return new NonFixedFacet( path );

		if( isPositiveValueFixedFacetTagName( tagName ) ) return new PositiveValueFixedFacet( path );

		if( isNonNegativeValueFixedFacetTagName( tagName ) ) return new NonNegativeValueFixedFacet( path );

		if( isAnyValueFixedFacetTagName( tagName ) ) return new AnyValueFixedFacet( path );

		if( isEnumeratedValueFacetTagName( tagName ) ) return new EnumeratedValueFacet( path );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Map<String, String> getFestrictionFacetKinds(){

		return restrictionFacetKinds;
	}

	public static String getFlatFestrictionFacetKinds(){

		return MapPrinting.printCommaSeparated( getFestrictionFacetKinds() );
	}
}
