package org.ow2.choreos.serviceBase;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessagesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.OperationsMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.dataStructures.interval.Interval;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;

/**
 * 
 * @author dathanas
 *
 * This class contains methods to retrieve the Abstraction model objects (Abstractions, mappings etc) from the DBMS
 */
public class AbstractionsHierarchyRetriever{
	private Statement selectStatement;
	private int resultSetType;
	
	public AbstractionsHierarchyRetriever(){
		//do nothing at the moment
	}
	//reconstructs and returns the 'Range' objects of a NonFunctionalAbstraction specified by the 'whereClause' argument
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public Hashtable<String, Interval> retrieveNFARanges(Connection dbConnection, String whereClause){
		Hashtable<String, Interval> retrievedMappingsToRanges = new Hashtable<String, Interval>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM nfabstractionsRanges " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			String property = rs.getString(2);
  	  			Double valueFrom = rs.getDouble(3);
  	  			Double valueTo = rs.getDouble(4);
  	  			int cDBKey = rs.getInt(5);
  	  			Interval tmpRange = new Interval(valueFrom, valueTo);
  	  			tmpRange.setDBAbstractionKey(cDBKey);
  	  			
  	  			retrievedMappingsToRanges.put(property, tmpRange);
  	  		}
  	  		if (retrievedMappingsToRanges.isEmpty()){
  	  			retrievedMappingsToRanges = null;
  	  		}
  	  		return retrievedMappingsToRanges;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveNFARanges: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the Hashtable 'NonFunctionalDescriptions' of a NonFunctionalAbstraction specified by the 'whereClause' argument
	public Hashtable<String, String> retrieveNFANonFunctionalDescriptions(Connection dbConnection, String whereClause){
		Hashtable<String, String> retrievedNFANonFunctionalDescriptions = new Hashtable<String, String>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM nfanonfuncdescriptions " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			String property = rs.getString(2);
  	  			String value = rs.getString(3);
  	  			retrievedNFANonFunctionalDescriptions.put(property, value);
  	  		}
  	  		return retrievedNFANonFunctionalDescriptions;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveNFANonFunctionalDescriptions: " + e.getMessage());
	    }
		return null;
	}
	

	//TODO: reconstructs and returns the 'ComponentsMapping' objects specified by the 'whereClause' argument
	public ArrayList<ComponentsAndStructuresMapping> retrieveComponentsAndStructuresMappings(Connection dbConnection, String whereClause){
		ArrayList<ComponentsAndStructuresMapping> retrievedComponentsMappings = new ArrayList<ComponentsAndStructuresMapping>();
			
		try {
			selectStatement = dbConnection.createStatement();
	  		String upd = new String ("SELECT * FROM componentsandstructuresmappings " + whereClause);
	  		ResultSet rs = selectStatement.executeQuery(upd);
	  		while (rs.next()) {
	  			double distance = rs.getDouble(2);
	  			int id = rs.getInt(1);
		  			
	  			ComponentsAndStructuresMapping currComponentsMapping = new ComponentsAndStructuresMapping();
	  			currComponentsMapping.set(distance);
		 				  			
	  			
	  			retrievedComponentsMappings.add(currComponentsMapping);
	  	  	}
	  		return retrievedComponentsMappings;
		}
		catch(Exception e) {
			System.err.println("Exception retrieveComponentsMappings: " + e.getMessage());
	    }
		return null;
	}
	
	//TODO: reconstructs and returns the mapped 'Component' objects specified by the 'whereClause' argument
	/*public ArrayList<Component> retrieveMappedComponents(Connection dbConnection, String whereClause){
		ArrayList<Component> retrievedMappedComponents = new ArrayList<Component>();
		try {
			selectStatement = dbConnection.createStatement();
			//retrieve actual messagetypes
	  		String upd = new String ("SELECT * FROM mappedcomponents, components where MMT_MT_ID = MT_ID " + whereClause);
	  		ResultSet rs = selectStatement.executeQuery(upd);
	  		while (rs.next()) {
	  			String name = rs.getString(7);
	 			int id = rs.getInt(1);
	 			String type = rs.getString(10);
	 			String typeKind = rs.getString(9);
			  			
	  			MessageType currMessageType = new MessageType(name, type, typeKind);
	  			
	  			retrievedMappedComponents.add(currMessageType);
	  	  	}
		  		
	  		Statement selectStatement2 = dbConnection.createStatement();
	  		//retrieve representative messagetypes
	 		upd = new String ("SELECT * FROM mappedcomponents, representativecomponents where MMT_RMST_ID = RMT_ID " + whereClause);
	  		ResultSet rs2 = selectStatement2.executeQuery(upd);
	  		while (rs2.next()) {
	  			String name = rs2.getString(7);
	 			int id = rs2.getInt(1);
	 			String type = rs2.getString(10);
	 			String typeKind = rs2.getString(9);
		  			
	  			MessageType currMessageType = new MessageType(name, type, typeKind);
		  				  				  			
	  			retrievedMappedComponents.add(currMessageType);
	  	  	}
	  	  		  	  		
	  	  	//System.out.println("MappedMessageTypes: " + retrievedMappedMessageTypes.size());
	 	  	return retrievedMappedComponents;
			}		
			catch(Exception e) {
	        	System.err.println("Exception retrieveMappedComponents: " + e.getMessage());
		    }
		return null;
				
	}*/
	

	//reconstructs and returns the mapped 'MessageType' objects specified by the 'whereClause' argument
	public ArrayList<MessageType> retrieveMappedMessageTypes(Connection dbConnection, String whereClause){
		ArrayList<MessageType> retrievedMappedMessageTypes = new ArrayList<MessageType>();
		try {
			selectStatement = dbConnection.createStatement();
			//retrieve actual messagetypes
	  		String upd = new String ("SELECT * FROM mappedmessagetypes, messagetypes where MMT_MT_ID = MT_ID " + whereClause);
	  		ResultSet rs = selectStatement.executeQuery(upd);
	  		while (rs.next()) {
	  			String name = rs.getString(7);
	 			int id = rs.getInt(1);
	 			String type = rs.getString(10);
	 			String typeKind = rs.getString(9);
			  			
	  			MessageType currMessageType = new MessageType(name, type, typeKind);
	  			
	  			retrievedMappedMessageTypes.add(currMessageType);
	  	  	}
		  		
	  		Statement selectStatement2 = dbConnection.createStatement();
	  		//retrieve representative messagetypes
	  		upd = new String ("SELECT * FROM mappedmessagetypes, representativemessagetypes where MMT_RMST_ID = RMT_ID " + whereClause);
	  		ResultSet rs2 = selectStatement2.executeQuery(upd);
	  		while (rs2.next()) {
	  			String name = rs2.getString(7);
	 			int id = rs2.getInt(1);
	 			String type = rs2.getString(10);
	 			String typeKind = rs2.getString(9);
		  			
	  			MessageType currMessageType = new MessageType(name, type, typeKind);
		  				  				  			
	  			retrievedMappedMessageTypes.add(currMessageType);
	  	  	}
	  	  		  	  		
	  	  	return retrievedMappedMessageTypes;
		}		
		catch(Exception e) {
			System.err.println("Exception retrieveMappedMessageTypes: " + e.getMessage());
	    }
		return null;		
	}
	
	
	//reconstructs and returns the mapped 'Message' objects specified by the 'whereClause' argument
	public ArrayList<Message> retrieveMappedMessages(Connection dbConnection, String whereClause){
		ArrayList<Message> retrievedMappedMessages = new ArrayList<Message>();
		try {
			selectStatement = dbConnection.createStatement();
			//retrieve actual messages
  	  		String upd = new String ("SELECT * FROM mappedmessages, messages where MM_MS_ID = MS_ID " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			String name = rs.getString(7);
  	  			int id = rs.getInt(1);
	  			String cType = rs.getString(9);
	  			int mappedMessageId = rs.getInt(2);
	  			
	  			Message currMessage = new Message(name);
	  			currMessage.setDBKey(rs.getInt(10));
	  			currMessage.setInOrOut(cType);
	  			
	  			ServiceRepresentationRetriever myRetriever = new ServiceRepresentationRetriever();
  	  			String myWhereClause = new String("WHERE MT_MS_ID = " + mappedMessageId);
  	  			ArrayList<MessageType> myRetrievedMessageTypes = new ArrayList<MessageType>();
  	  			myRetrievedMessageTypes = myRetriever.retrieveMessageTypes(dbConnection, myWhereClause);
	  			//add MessagesTypes to Message
	  			for (int i = 0; i < myRetrievedMessageTypes.size(); i++){
	  				currMessage.add(myRetrievedMessageTypes.get(i));
	  			}

	  			retrievedMappedMessages.add(currMessage);
  	  		}
  	  		
  	  		Statement selectStatement2 = dbConnection.createStatement();
  	  		//retrieve representative messages
	  		upd = new String ("SELECT * FROM mappedmessages, representativemessages where MM_RM_ID = RMS_ID " + whereClause);
	  		ResultSet rs2 = selectStatement2.executeQuery(upd);
	  		while (rs2.next()) {
	  			String name = rs2.getString(7);
	  			int id = rs2.getInt(1);
	  			String cType = rs2.getString(9);
	  			int mappedMessageId = rs2.getInt(2);
  			
	  			Message currMessage = new Message(name);
	  			currMessage.setDBKey(rs2.getInt(10));
	  			currMessage.setInOrOut(cType);
  			
	  			AbstractionsHierarchyRetriever myRetriever = new AbstractionsHierarchyRetriever();
	  			String myWhereClause = new String("WHERE RMT_RMS_ID = " + mappedMessageId);
	  			ArrayList<MessageType> myRetrievedMessageTypes = new ArrayList<MessageType>();
	  			myRetrievedMessageTypes = myRetriever.retrieveRepresentativeMessageTypes(dbConnection, myWhereClause);
	  			//add MessagesTypes to Message
	  			for (int i = 0; i < myRetrievedMessageTypes.size(); i++){
	  				currMessage.add(myRetrievedMessageTypes.get(i));
	  			}

	  			retrievedMappedMessages.add(currMessage);
	  		}
  	  		
  	  		return retrievedMappedMessages;
		}		
		catch(Exception e) {
	          System.err.println("Exception retrieveMappedMessages: " + e.getMessage());
	    }
		return null;
	}
	
	
	//reconstructs and returns the 'MessageTypesMapping' objects specified by the 'whereClause' argument
	public ArrayList<MessageTypesMapping> retrieveMessageTypesMappings(Connection dbConnection, String whereClause){
		ArrayList<MessageTypesMapping> retrievedMessageTypesMappings = new ArrayList<MessageTypesMapping>();
			
		try {
			selectStatement = dbConnection.createStatement();
	  		String upd = new String ("SELECT * FROM messagetypesmappings " + whereClause);
	  		
	  		ResultSet rs = selectStatement.executeQuery(upd);
	  		while (rs.next()) {
	  			double distance = rs.getDouble(2);
	 			int id = rs.getInt(1);
		  			
		 		MessageTypesMapping currMessageTypesMapping = new MessageTypesMapping();
		 		currMessageTypesMapping.set(distance);
			  			
		 		//TODO: add componentMappings to MessageTypesmapping
		 		String myWhereClause = new String("");
		 		//List<ComponentsAndStructuresMapping> myMapping = retrieveComponentsAndStructuresMappings(dbConnection, whereClause);
		 		
		 		//add representedMessageTypes to MessageTypeMapping
		  		myWhereClause = "AND MMT_MTM_ID = " + id;
		  		ArrayList<MessageType> myMappedMessageTypes = new ArrayList<MessageType>();
		  		if (resultSetType == 1){
		  			myMappedMessageTypes = retrieveMappedMessageTypes(dbConnection, myWhereClause);	
		  			for (int i = 0; i < myMappedMessageTypes.size(); i++){
		  				currMessageTypesMapping.add(myMappedMessageTypes.get(i));
		  			}
		  		}
		  		else{
		  			currMessageTypesMapping.setMessageTypes(myMappedMessageTypes);
		  		}
		  		retrievedMessageTypesMappings.add(currMessageTypesMapping);
	  	  	}

	  		return retrievedMessageTypesMappings;
		}
		catch(Exception e) {
			System.err.println("Exception retrieveMessageTypesMappings: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the 'MessagesMapping' objects specified by the 'whereClause' argument
	public ArrayList<MessagesMapping> retrieveMessagesMappings(Connection dbConnection, String whereClause){
		ArrayList<MessagesMapping> retrievedMessagesMappings = new ArrayList<MessagesMapping>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM messagesmappings " + whereClause);
  	  		
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			double distance = rs.getDouble(2);
	  			int id = rs.getInt(1);
	  			String cType = rs.getString(3);
	  			
	  			MessagesMapping currMessagesMapping = new MessagesMapping();
	  			currMessagesMapping.set(distance);
	  			currMessagesMapping.setInOrOut(cType);
	  				  			
	  			String myWhereClause = new String ("WHERE MTM_MM_ID = " + id);
	  			ArrayList<MessageTypesMapping> myMessageTypesMappings = retrieveMessageTypesMappings(dbConnection, myWhereClause);
	  			currMessagesMapping.set(myMessageTypesMappings);
	  			
	  			//add representedMessages to MessageMapping
	  			myWhereClause = new String ("AND MM_MM_ID = " + id);
	  			ArrayList<Message> myMappedMessages = new ArrayList<Message>();
	  			if (resultSetType == 1){
	  				myMappedMessages = retrieveMappedMessages(dbConnection, myWhereClause);
	  				for (int i = 0; i < myMappedMessages.size(); i++){
	  					currMessagesMapping.add(myMappedMessages.get(i));
	  				}
	  			}
	  			else{
	  				currMessagesMapping.setMessages(myMappedMessages);
	  			}
	  			
	  			retrievedMessagesMappings.add(currMessagesMapping);
	  				
  	  		}

  	  		return retrievedMessagesMappings;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveMessagesMappings: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the mapped 'Operation' objects specified by the 'whereClause' argument
	public ArrayList<Operation> retrieveMappedOperations(Connection dbConnection, String whereClause){
		ArrayList<Operation> retrievedMappedOperations = new ArrayList<Operation>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM mappedoperations, operations where MO_OP_ID = OP_ID " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			int mappedOperationId = rs.getInt(2);
  	  			String name = rs.getString(7);
	  				  			
	  			Message inMessage = null;
  	  			Message outMessage = null;
  	  			
  	  			//add Messages to Operation
  	  			ServiceRepresentationRetriever myRetriever = new ServiceRepresentationRetriever();
  	  			String myWhereClause = new String("WHERE MS_OP_ID = " + mappedOperationId);
  	  			ArrayList<Message> myRetrievedMessages = new ArrayList<Message>();
  	  			myRetrievedMessages = myRetriever.retrieveMessages(dbConnection, myWhereClause);
  	  			  	  			
  	  			for (int i = 0; i < myRetrievedMessages.size(); i++){
  	  				Message tmpMessage = myRetrievedMessages.get(i);
  	  				if (tmpMessage.getInOrOut().equals("IN")){
  	  					inMessage = tmpMessage;
  	  				}
  	  				else{
  	  					outMessage = tmpMessage;
  	  				}
  	  			}
	  			  	  			
	  			Operation currOperation = new Operation(name, inMessage, outMessage);
	  			currOperation.setDBKey(rs.getInt(11));
	  			
	  			retrievedMappedOperations.add(currOperation);
  	  		}
  	  		
  	  		Statement selectStatement2 = dbConnection.createStatement();
  	  		// ZAS changed MO_OP_ID to MO_ROP_ID
	  		upd = "SELECT * FROM mappedoperations, representativeoperations where MO_ROP_ID = ROP_ID " + whereClause;

	  		ResultSet rs2 = selectStatement2.executeQuery(upd);
	  		
	  		while (rs2.next()) {
	  			int id = rs2.getInt(1);
	  			int mappedOperationId = rs2.getInt(2);
	  			String name = rs2.getString(7);
  				  			
	  			Message inMessage = null;
	  			Message outMessage = null;
	  			
	  			//add Messages to Operation
	  			AbstractionsHierarchyRetriever myRetriever = new AbstractionsHierarchyRetriever();
	  			String myWhereClause = new String("WHERE RMS_ROP_ID = " + mappedOperationId);
	  			ArrayList<Message> myRetrievedMessages = new ArrayList<Message>();
	  			myRetrievedMessages = myRetriever.retrieveRepresentativeMessages(dbConnection, myWhereClause);
	  			  	  			
	  			for (int i = 0; i < myRetrievedMessages.size(); i++){
	  				Message tmpMessage = myRetrievedMessages.get(i);
	  				if (tmpMessage.getInOrOut().equals("IN")){
	  					inMessage = tmpMessage;
	  				}
	  				else{
	  					outMessage = tmpMessage;
	  				}
	  			}
  			  	  			
	  			Operation currOperation = new Operation(name, inMessage, outMessage);
	  			currOperation.setDBKey(rs2.getInt(11));
  
	  			retrievedMappedOperations.add(currOperation);
	  		}

  	  		return retrievedMappedOperations;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveMappedOperations: " + e.getMessage());
	    }
		return null;
	}

	//reconstructs and returns the 'OperationsMapping' objects specified by the 'whereClause' argument
	public ArrayList<OperationsMapping> retrieveOperationsMappings(Connection dbConnection, String whereClause){
		ArrayList<OperationsMapping> retrievedOperationsMappings = new ArrayList<OperationsMapping>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM operationsmappings " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			double distance = rs.getDouble(2);
	  				  			
	  			OperationsMapping currOperationsMapping = new OperationsMapping();
	  			currOperationsMapping.set(distance);
	  			
	  			//add representedOperations to OperationsMapping
	  			String myWhereClause = new String(" AND MO_OM_ID = " + id);
	  			ArrayList<Operation> myMappedOperations = new ArrayList<Operation>();
	  			if (resultSetType == 1){
	  				myMappedOperations = retrieveMappedOperations(dbConnection, myWhereClause);
	  				for (int i = 0; i < myMappedOperations.size(); i++){
	  					currOperationsMapping.add(myMappedOperations.get(i));
	  				}
	  			}
	  			else{
	  				currOperationsMapping.setOperations(myMappedOperations);
	  			}
	  			
	  			//add MessageMappings to OperationsMapping
	  			myWhereClause = "WHERE MM_OM_ID = " + id;
	  			ArrayList<MessagesMapping> myMessagesMappings = new ArrayList<MessagesMapping>();
	  			myMessagesMappings = retrieveMessagesMappings(dbConnection, myWhereClause);
	  			for (int i = 0; i < myMessagesMappings.size(); i++){
	  				MessagesMapping tmpMessageMapping = myMessagesMappings.get(i);
	  				if (tmpMessageMapping.getType().equals("IN")){	
	  					currOperationsMapping.setInputMappings(tmpMessageMapping);
	  				}
	  				else if (tmpMessageMapping.getType().equals("OUT")){
	  					currOperationsMapping.setOutputMappings(tmpMessageMapping);
	  				}
	  			}
	  			
	  			retrievedOperationsMappings.add(currOperationsMapping);
  	  		}

  	  		return retrievedOperationsMappings;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveOperationsMappings: " + e.getMessage());
	    }
		return null;
		
	}
	
	//reconstructs and returns the mapped 'ServiceInterface' objects of a FunctionalAbstraction specified by the 'whereClause' argument
	public ArrayList<ServiceInterface> retrieveMappedInterfaces(Connection dbConnection, String whereClause){
		ArrayList<ServiceInterface> retrievedMappedInterfaces = new ArrayList<ServiceInterface>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM mappedinterfaces, serviceinterfaces where MI_SI_ID = SI_ID " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			String name = rs.getString(7);
	  			int mappedInterfaceId = rs.getInt(6);
	  			String b = rs.getString(9);
	  			
	  			ServiceInterface currInterface = new ServiceInterface(name);
	  			currInterface.setDBKey(rs.getInt(11));
	  			currInterface.seteBehavioralSpecification(b);

	  			ServiceRepresentationRetriever myRetriever = new ServiceRepresentationRetriever();
  	  			String myWhereClause = new String(" WHERE OP_SI_ID = " + mappedInterfaceId);
  	  			ArrayList<Operation> myRetrievedOperations = new ArrayList<Operation>();
  	  			myRetrievedOperations = myRetriever.retrieveOperations(dbConnection, myWhereClause);
  	  			for (int i = 0; i < myRetrievedOperations.size(); i++) {
  	  				currInterface.add(myRetrievedOperations.get(i));
  	  			}
  	  			
  	  			myWhereClause = " WHERE SIS_SI_ID = " + mappedInterfaceId;
	  			ArrayList<ServiceInstance> myRetrievedInstances = new ArrayList<ServiceInstance>();
	  			myRetrievedInstances = myRetriever.retrieveInstances(dbConnection, myWhereClause);
	  			for (int i = 0; i < myRetrievedInstances.size(); i++) {
	  				currInterface.addServiceInstance(myRetrievedInstances.get(i));
	  			}
  	  			
	  			retrievedMappedInterfaces.add(currInterface);
  	  		}
  	  		
  	  		Statement selectStatement2 = dbConnection.createStatement();
  	  		upd = new String ("SELECT * FROM mappedinterfaces, representativeinterfaces where MI_RI_ID = RSI_ID " + whereClause);
	  		ResultSet rs2 = selectStatement2.executeQuery(upd);
	  		
	  		while (rs2.next()) {
	  			String name = rs2.getString(7);
	  			int mappedInterfaceId = rs2.getInt(6);
	  			String b = rs2.getString(9);
  				  			
	  			ServiceInterface currInterface = new ServiceInterface(name);
	  			currInterface.setDBKey(rs2.getInt(11));
	  			currInterface.seteBehavioralSpecification(b);

	  			AbstractionsHierarchyRetriever myRetriever = new AbstractionsHierarchyRetriever();
	  			String myWhereClause = new String(" WHERE ROP_RSI_ID = " + mappedInterfaceId);
	  			ArrayList<Operation> myRetrievedOperations = new ArrayList<Operation>();
	  			myRetrievedOperations = myRetriever.retrieveRepresentativeOperations(dbConnection, myWhereClause);
	  			for (int i = 0; i < myRetrievedOperations.size(); i++) {
	  				currInterface.add(myRetrievedOperations.get(i));
	  			}
	  				  			
	  			retrievedMappedInterfaces.add(currInterface);
	  		}
  	  	
  	  		return retrievedMappedInterfaces;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveMappedInterfaces: " + e.getMessage());
	    }
		return null;
		
	}
	
	//reconstructs and returns the 'InterfacesMapping' objects of a FunctionalAbstraction specified by the 'whereClause' argument
	public ArrayList<InterfacesMapping> retrieveInterfacesMappings(Connection dbConnection, String whereClause){
		ArrayList<InterfacesMapping> retrievedInterfacesMappings = new ArrayList<InterfacesMapping>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM interfacesmappings " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			double distance = rs.getDouble(2);
	  				  			
	  			InterfacesMapping currInterfacesMapping = new InterfacesMapping();
	  			currInterfacesMapping.setDBAbstractionKey(rs.getInt(3));
	  			currInterfacesMapping.set(distance);
	  			
	  			//add representedInterfaces to InterfacesMapping
	  			String myWhereClause = new String ("AND MI_IM_ID = " + id);
	  			ArrayList<ServiceInterface> myServiceInterfaces = new ArrayList<ServiceInterface>();
	  			if (resultSetType == 1){
	  				myServiceInterfaces = retrieveMappedInterfaces(dbConnection, myWhereClause);
	  				for (int i = 0; i < myServiceInterfaces.size(); i++){
	  					currInterfacesMapping.add(myServiceInterfaces.get(i));
	  				}
	  			}
	  			else{
	  				currInterfacesMapping.setInterfaces(myServiceInterfaces);
	  			}
	  			
	  			//add OperationsMappings to InterfacesMapping
	  			myWhereClause = new String ("WHERE OM_IM_ID = " + id);
	  			ArrayList<OperationsMapping> myOperationsMappings = new ArrayList<OperationsMapping>();
	  			myOperationsMappings = retrieveOperationsMappings(dbConnection, myWhereClause);
	  			currInterfacesMapping.set(myOperationsMappings);
	  			retrievedInterfacesMappings.add(currInterfacesMapping);
  	  		}

  	  		return retrievedInterfacesMappings;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveInterfacesMappings: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the 'RepresentativeServiceInterfaces' objects of a FunctionalAbstraction specified by the 'whereClause' argument
	public ArrayList<ServiceInterface> retrieveRepresentativeServiceInterfaces(Connection dbConnection, String whereClause){
		ArrayList<ServiceInterface> retrievedRepresentativeServiceInterfaces = new ArrayList<ServiceInterface>();
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM representativeinterfaces " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			String name = rs.getString(2);
	  			int id = rs.getInt(1);
	  			String b = rs.getString(4);
	  			
	  			ServiceInterface currInterface = new ServiceInterface(name);
	  			currInterface.setDBAbstractionKey(rs.getInt(6));
	  			currInterface.seteBehavioralSpecification(b);
	  			
	  			//to reconstruct the 'ServiceInterface' we must retrieve its 'Operations' too
  	  			String myWhereClause = new String(" where ROP_RSI_ID = " + id);
	  			ArrayList<Operation> myOperations = retrieveRepresentativeOperations(dbConnection, myWhereClause);
	  			for (int i = 0; i < myOperations.size(); i++){
	  				currInterface.add(myOperations.get(i));
	  			}
	  			
	  			retrievedRepresentativeServiceInterfaces.add(currInterface);
  	  		}

  	  		return retrievedRepresentativeServiceInterfaces;
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveRepresentativeServiceInterfaces: " + e.getMessage());
	    }
		return null;
	}
	
	public ArrayList <Operation> retrieveRepresentativeOperations(Connection dbConnection, String whereClause){
		ArrayList <Operation> operationsRetrieved = new ArrayList <Operation>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM representativeoperations " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			String name = rs.getString(2);
  	  			int id = rs.getInt(1);
  	  			Message inMessage = null;
  	  			Message outMessage = null;
  	  			
  	  			//to reconstruct the 'Operation' we must retrieve its 'Messages' (IN and OUT) too
  	  			String myWhereClause = new String("where RMS_ROP_ID = " + id + " AND RMS_TYPE = 'IN'");
  	  			ArrayList<Message> messagesRetrieved = retrieveRepresentativeMessages(dbConnection, myWhereClause);
  	  			if (messagesRetrieved != null){
  	  				inMessage = messagesRetrieved.get(0);
  	  			}
  	  			messagesRetrieved.clear();
  	  			
  	  			myWhereClause = new String("where RMS_ROP_ID = " + id + " AND RMS_TYPE = 'OUT'");
	  			messagesRetrieved = retrieveRepresentativeMessages(dbConnection, myWhereClause);
	  			if (messagesRetrieved != null){
	  				outMessage = messagesRetrieved.get(0);
	  			}
  	  			  	  			
  	  			Operation currOperation = new Operation(name, inMessage, outMessage);
  	  			currOperation.setDBKey(rs.getInt(6));
  	  			operationsRetrieved.add(currOperation);	
  	  		}
  	  		
  	  		return operationsRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveRepresentativeOperations Exception: " + e.getMessage());
	    }	
		
		return null;
	}
	
	
	public ArrayList<Message> retrieveRepresentativeMessages(Connection dbConnection, String whereClause){
		ArrayList<Message> messagesRetrieved = new ArrayList<Message>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM representativemessages " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String name = rs.getString(2);
  	  			String type = rs.getString(4);
  	  			
  	  			Message currMessage = new Message(name);
  	  			currMessage.setDBKey(rs.getInt(5));
  	  			currMessage.setInOrOut(type);
  	  			
  	  			//to reconstruct the 'Message' we must retrieve its 'MessageTypes' too  			
  	  			String myWhereClause = new String("where RMT_RMS_ID = " + id);
	  			ArrayList<MessageType> myRepresentativeMessageTypes = retrieveRepresentativeMessageTypes(dbConnection, myWhereClause);
	  			for (int i = 0 ; i < myRepresentativeMessageTypes.size() ; i++){
	  				currMessage.add(myRepresentativeMessageTypes.get(i));
	  			}
  	  			
  	  			messagesRetrieved.add(currMessage);
  	  		}

  	  		return messagesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveRepresentativeMessages Exception: " + e.getMessage());
	    }
		
		return null;
	}
	
	//reconstructs and returns the 'MessageType' objects specified by the 'whereClause' argument
	public ArrayList<MessageType> retrieveRepresentativeMessageTypes(Connection dbConnection, String whereClause){
		ArrayList<MessageType> messageTypesRetrieved = new ArrayList<MessageType>();
			
		try {
			selectStatement = dbConnection.createStatement();
		   	String upd = new String ("SELECT * FROM representativemessagetypes " + whereClause);
		   	ResultSet rs = selectStatement.executeQuery(upd);
		   	while (rs.next()) {
		   		int id = rs.getInt(1);
	  	  		String cName = rs.getString(2);
	  	  		String cType = rs.getString(5);
	  	  		String typeKind = rs.getString(4);
	  	  		
	  	  		MessageType currMessageType;
	  	  		  	  		
	  	  		String myWhereClause = new String(" where RCOM_ISGLOBAL = 1 and RCOM_TAG_ATTRIBUTES <> '' and RCOM_MT_ID = " + id);
	  	  		ArrayList<Component> myMaterializedComponents = retrieveRepresentativeMaterializedComponents(dbConnection, myWhereClause, id);
	  	  		if (myMaterializedComponents != null){
	  	  			currMessageType = new MessageType(cName, cType, typeKind, myMaterializedComponents.get(0));
	  	  			currMessageType.setDBKey(rs.getInt(6));
	  	  		}
	  	  		else{
	  	  			currMessageType = new MessageType(cName, cType, typeKind);
	  	  			currMessageType.setDBKey(rs.getInt(6));
	  	  		}
	  	  			  	  		
	  	  		messageTypesRetrieved.add(currMessageType);
	  	  	}
	  		return messageTypesRetrieved;
		}
		catch(Exception e) {
			System.err.println("retrieveRepresentativeMessageTypes Exception: " + e.getMessage());
	    }
		return null;
	}
	

	//reconstructs and returns the 'MaterializedComponents' objects specified by the 'whereClause' argument
	public ArrayList<Component> retrieveRepresentativeMaterializedComponents(Connection dbConnection, String whereClause, int msgTypeId){
		ArrayList<Component> componentsRetrieved = new ArrayList<Component>();
		
		try {
			selectStatement = dbConnection.createStatement();
	 	  	String upd = new String ("SELECT * FROM representativecomponentsandstructures " + whereClause);
	  	  	ResultSet rs = selectStatement.executeQuery(upd);
	  	  	while (rs.next()) {
	  	  		//String cName = rs.getString(2);
	  	  		String cType = rs.getString(3);
	  	  		String cTypeKind = rs.getString(4);
		  	  	String concatenatedSchemaPath = rs.getString(7);
	  	  		String concatenatedClassPath = rs.getString(8);
	  	  		boolean cIsGlobal = rs.getBoolean(5);
	  	  		String tagAttributesString = rs.getString(6);
	  	  		String model = rs.getString(9);
	  	  		
	  	  		Map<String, String> tagAttributes = new Hashtable<String, String>();
	  	  		String[] schemaPath = null;
	  	  		String[] classPath = null;
	  	  			  	  		
	  	  		if (!tagAttributesString.equals("")){
	  	  			String[] tmp = tagAttributesString.split(",");
	  	  			for (int i = 0; i < tmp.length; i++){
	  	  				String[] tmp2 = tmp[i].split("-");
	  	  				String name = tmp2[0];
	  	  				String attr = tmp2[1];
	  	  				tagAttributes.put(name, attr);
	  	  			}
	  	  		}
	  	  		
	  	  		if (!concatenatedSchemaPath.equals("")){
	  	  			schemaPath = concatenatedSchemaPath.split("-");
	  	  		}
		  		
	  	  		if (!concatenatedClassPath.equals("")){
	  	  			classPath = concatenatedClassPath.split("-");
	  	  		}
	  	  		
	  	  		List<Map<String, String>> tagAttributesList = new ArrayList<Map<String, String>>();
	  	  		tagAttributesList.add(tagAttributes);
	  	  		
	  	  		String myWhereClause = new String(" where RCOM_ISGLOBAL = 0 and RCOM_IS_COMPONENT = 1 and RCOM_TAG_ATTRIBUTES <> '' and RCOM_MT_ID = " + msgTypeId);
	  	  		ArrayList<Component> myComponents = retrieveRepresentativeComponents(dbConnection, myWhereClause, tagAttributesList, msgTypeId);
	  	  		
	  	  		myWhereClause = " where RCOM_ISGLOBAL = 0 and RCOM_IS_COMPONENT = 0 and RCOM_TAG_ATTRIBUTES <> '' and RCOM_MT_ID = " + msgTypeId;
	  	  		ArrayList<Structure> myStructures = retrieveRepresentativeStructures(dbConnection, myWhereClause, tagAttributesList, msgTypeId);
	  	  		
	  	  		Component currMaterializedComponent = (Component) XMLSchemaRepresentationController.getTagObject(tagAttributesList, cIsGlobal, model);
	  	  		if (myComponents != null){
	  	  			for (int i = 0; i < myComponents.size(); i++){
	  	  				currMaterializedComponent.addComponent(myComponents.get(i));
	  	  			}
	  	  		}
	  	  		
	  	  		if (myStructures != null){
	  	  			for (int i = 0; i < myStructures.size(); i++){
	  	  				currMaterializedComponent.addStructure(myStructures.get(i));
	  	  			}
	  	  		}
	  	  		
	  	  		componentsRetrieved.add(currMaterializedComponent);
	  	  	}//end of while
	  	  	
	  	  	if (componentsRetrieved.isEmpty()){
	  	  		return null;
	  	  	}
	  	  	else{
	  	  		return componentsRetrieved;
	  	  	}
	  	  	
		}
		catch(Exception e) {
			System.err.println("retrieveRepresentativeMaterializedComponents Exception: " + e.getMessage());
	    }
		return null;
	}
	
	
	//reconstructs and returns the 'RepresentativeLeafComponents' objects specified by the 'whereClause' argument
	public ArrayList<Component> retrieveRepresentativeComponents(Connection dbConnection, String whereClause, List<Map<String, String>> tagAttributesList, int msgTypeId){
		ArrayList<Component> componentsRetrieved = new ArrayList<Component>();
		
		try {
			selectStatement = dbConnection.createStatement();
	 	  	String upd = new String ("SELECT * FROM representativecomponentsandstructures " + whereClause);
	  	  	ResultSet rs = selectStatement.executeQuery(upd);
	  	  	while (rs.next()) {
	  	  		Map<String, String> tagAttributes = new Hashtable<String, String>();
	  	  		String[] schemaPath = null;
	  	  		String[] classPath = null;
	  	  		String concatenatedSchemaPath = rs.getString(7);
	  	  		String concatenatedClassPath = rs.getString(8);
	  	  		boolean cIsGlobal = rs.getBoolean(5);
	  	  		String model = rs.getString(9);
	  	  		
	  	  		String tagAttributesString = rs.getString(6);
	  	  		if (!tagAttributesString.equals("")){
	  	  			String[] tmp = tagAttributesString.split(",");
	  	  			for (int i = 0; i < tmp.length; i++){
	  	  				String[] tmp2 = tmp[i].split("-");
	  	  				String name = tmp2[0];
	  	  				String attr = tmp2[1];
	  	  				tagAttributes.put(name, attr);
	  	  			}
	  	  		}
	  	  		
	  	  		if (!concatenatedSchemaPath.equals("")){
	  	  			schemaPath = concatenatedSchemaPath.split("-");
	  	  		}
  	  		
	  	  		if (!concatenatedClassPath.equals("")){
	  	  			classPath = concatenatedClassPath.split("-");
	  	  		}
	  	  		
	  	  		ArrayList<Map<String, String>> myTagAttributesList = (ArrayList<Map<String, String>>) ((ArrayList<Map<String, String>>) tagAttributesList).clone();
	  	  		myTagAttributesList.add(tagAttributes);
	  	  		Component currComponent = (Component) XMLSchemaRepresentationController.getTagObject(myTagAttributesList, cIsGlobal, model);
	  	  		
	  	  		componentsRetrieved.add(currComponent);
	  	  	}
	  	  	
	  	  	if (componentsRetrieved.isEmpty()){
	  	  		return null;
	  	  	}
	  	  	else{
	  	  		return componentsRetrieved;
	  	  	}
	  	  	
		}
		catch(Exception e) {
			System.err.println("retrieveRepresentativeComponents Exception: " + e.getMessage());
	    }
		return null;
	}

	
	//reconstructs and returns the 'RepresentativeStructures' objects specified by the 'whereClause' argument
	public ArrayList<Structure> retrieveRepresentativeStructures(Connection dbConnection, String whereClause, List<Map<String, String>> tagAttributesList, int msgTypeId){
		ArrayList<Structure> structuresRetrieved = new ArrayList<Structure>();
		
		try {
			selectStatement = dbConnection.createStatement();
	 	  	String upd = new String ("SELECT * FROM representativecomponentsandstructures " + whereClause);
	  	  	ResultSet rs = selectStatement.executeQuery(upd);
	  	  	while (rs.next()) {
	  	  		Map<String, String> tagAttributes = new Hashtable<String, String>();
	  	  		String[] schemaPath = null;
	  	  		String[] classPath = null;
	  	  		String concatenatedSchemaPath = rs.getString(7);
	  	  		String concatenatedClassPath = rs.getString(8);
	  	  		boolean cIsGlobal = rs.getBoolean(5);
	  	  		String model = rs.getString(9);
	  	  		
	  	  		String tagAttributesString = rs.getString(6);
	  	  		if (!tagAttributesString.equals("")){
	  	  			String[] tmp = tagAttributesString.split(",");
	  	  			for (int i = 0; i < tmp.length; i++){
	  	  				String[] tmp2 = tmp[i].split("-");
	  	  				String name = tmp2[0];
	  	  				String attr = tmp2[1];
	  	  				tagAttributes.put(name, attr);
	  	  			}
	  	  		}
	  	  		
	  	  		if (!concatenatedSchemaPath.equals("")){
	  	  			schemaPath = concatenatedSchemaPath.split("-");
	  	  		}
  	  		
	  	  		if (!concatenatedClassPath.equals("")){
	  	  			classPath = concatenatedClassPath.split("-");
	  	  		}
	  	  		
		  	  	ArrayList<Map<String, String>> myTagAttributesList = (ArrayList<Map<String, String>>) ((ArrayList<Map<String, String>>) tagAttributesList).clone();
	  	  		myTagAttributesList.add(tagAttributes);
	  	  		
	  	  		Structure currStructure = (Structure) XMLSchemaRepresentationController.getTagObject(myTagAttributesList, cIsGlobal, model);
	  	  		
	  	  		structuresRetrieved.add(currStructure);
	  	  		
	  	  	}
	  	  	
	  	  	if (structuresRetrieved.isEmpty()){
	  	  		return null;
	  	  	}
	  	  	else{
	  	  		return structuresRetrieved;
	  	  	}
	 		
		}
		catch(Exception e) {
			System.err.println("retrieveRepresentativeStructures Exception: " + e.getMessage());
	    }
		return null;
	}
	
	
	
	public Hashtable<Integer, ServiceInstance> retrieveRepresentedInstances(Connection dbConnection, String whereClause){
		ArrayList<ServiceInstance> retrievedRepresentedInstancesList = new ArrayList<ServiceInstance>();
		Hashtable<Integer, ServiceInstance> retrievedRepresentedInstancesTable = new Hashtable<Integer, ServiceInstance>();
		ServiceRepresentationRetriever tmp = new ServiceRepresentationRetriever();
		String myWhereClause = new String(" , representedinstances where ris_id = sis_id " + whereClause);
		retrievedRepresentedInstancesList = tmp.retrieveInstances(dbConnection, myWhereClause);
		for (int i = 0 ; i < retrievedRepresentedInstancesList.size(); i++){
			retrievedRepresentedInstancesTable.put(i, retrievedRepresentedInstancesList.get(i));
		}
		
		return retrievedRepresentedInstancesTable;
	}
	
	
	public ArrayList<ServiceInterface> retrieveRepresentedInterfaces(Connection dbConnection, String whereClause){
		//not used at the moment  --  if used need to check if duplicate tuples are returned!!!!
		ArrayList<ServiceInterface> retrievedRepresentedInterfaces = new ArrayList<ServiceInterface>();
		ServiceRepresentationRetriever tmp = new ServiceRepresentationRetriever();
		String myWhereClause = new String(" , representedinterfaces where ri_id = si_id " + whereClause);
		retrievedRepresentedInterfaces = tmp.retrieveInterfaces(dbConnection, myWhereClause);
		
		return retrievedRepresentedInterfaces;
	}
	
	public ArrayList<FunctionalAbstraction> retrieveFunctionalChilds(Connection dbConnection, int abstractionId, int resultSetType){
		ArrayList<FunctionalAbstraction> childs = new ArrayList<FunctionalAbstraction>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM hier_edges where AD_ANC_ID = " + abstractionId);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		//for each descendant (which is a functional abstraction) retrieve it!
  	  		while (rs.next()) {
  	  			ArrayList<FunctionalAbstraction> tmpChilds = new ArrayList<FunctionalAbstraction>();
	  			int childId = rs.getInt(3);
	  			String myWhereClause = new String (" and A_ID = " + childId);		
	  			tmpChilds = retrieveFunctionalAbstractions(dbConnection, myWhereClause, resultSetType);
	  			childs.add(tmpChilds.get(0));
  	  		}
  	  		return childs;
  	  		  	  	
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveFunctionalChilds: " + e.getMessage());
	    }		
		return null;
	}

	public ArrayList <FunctionalAbstraction> retrieveFunctionalAbstractions(Connection dbConnection, String whereClause, int resultSetType){
		ArrayList <FunctionalAbstraction> retrievedAbstractions = new ArrayList <FunctionalAbstraction>();
		this.resultSetType = resultSetType;
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM abstractions where A_NFA_ID = -1 " + whereClause);

  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
	  			int id = rs.getInt(1);
	  			Integer faId = rs.getInt(2);
	  			
	  			int hierarchyId = rs.getInt(5);
		  			
	  			Statement statement2 = dbConnection.createStatement();
	  			ResultSet rs2 = statement2.executeQuery("SELECT * FROM hierarchies where H_ID = " + hierarchyId);
	  			String namesDistanceCalculatorAlgorithm = new String();
	  			
	  			while (rs2.next()){
	  				namesDistanceCalculatorAlgorithm = rs2.getString(4);
	  			}
	  			
	  			//retrieve InterfacesMappings
	  			String myWhereClause = new String (" where IM_FA_ID = " + id);
	  			ArrayList<InterfacesMapping> myInterfacesMappings = new ArrayList<InterfacesMapping>();
	  			myInterfacesMappings = retrieveInterfacesMappings(dbConnection, myWhereClause);
	  			InterfacesMapping myInterfacesMapping = myInterfacesMappings.get(0);
	  			
	  			//retrieve childs
	  			ArrayList<FunctionalAbstraction> myChilds = new ArrayList<FunctionalAbstraction>();
	  			myChilds = retrieveFunctionalChilds(dbConnection, id, resultSetType);
	  			
	  			//retrieve representative interface
	  			myWhereClause = " where RSI_A_ID = " + id;
	  			ArrayList<ServiceInterface> retrievedRepresentativeInterfaces = retrieveRepresentativeServiceInterfaces(dbConnection, myWhereClause);
	  			ServiceInterface representativeInterface = retrievedRepresentativeInterfaces.get(0);
	  			
	  			FunctionalAbstraction tmpFA = null;
	  			tmpFA = new FunctionalAbstraction(representativeInterface, namesDistanceCalculatorAlgorithm, faId, myChilds, myInterfacesMapping);
	  			tmpFA.setAncestorId(-1);
	  					
	  			retrievedAbstractions.add(tmpFA);
	  			
  	  		}
  	  		return retrievedAbstractions;
  	  	
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveFunctionalAbstractions: " + e.getMessage());
	    }
		
		return null;
	}
	
	
	public ArrayList<NonFunctionalAbstraction> retrieveNonFunctionalChilds(Connection dbConnection, int abstractionId, NonFunctionalAbstraction ancestor, int resultSetType){

		ArrayList<NonFunctionalAbstraction> childs = new ArrayList<NonFunctionalAbstraction>();
		boolean flag = false;
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM hier_edges where AD_ANC_ID = " + abstractionId);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		//for each descendant (which is a non-functional abstraction) retrieve it!
  	  		while (rs.next()) {
  	  			ArrayList<NonFunctionalAbstraction> tmpDescendants = new ArrayList<NonFunctionalAbstraction>();
	  			int descendantId = rs.getInt(3);
	 	  		flag = true;
	  			String myWhereClause = new String (" and A_ID = " + descendantId);		
	  			tmpDescendants = retrieveNonFunctionalAbstractions(dbConnection, myWhereClause, ancestor, resultSetType);
	  			childs.add(tmpDescendants.get(0));
  	  		}
  	  		return childs;
  	  		  	  	
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveNonFunctionalDescendants: " + e.getMessage());
	    }		
		return null;
	}
	
	
	@SuppressWarnings("rawtypes")
	public ArrayList <NonFunctionalAbstraction> retrieveNonFunctionalAbstractions(Connection dbConnection, String whereClause, NonFunctionalAbstraction ancestor, int resultSetType){
		ArrayList <NonFunctionalAbstraction> retrievedAbstractions = new ArrayList <NonFunctionalAbstraction>();
		this.resultSetType = resultSetType;
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM abstractions where A_FA_ID = -1 " + whereClause);

  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
	  			int id = rs.getInt(1);
	  			int nfaId = rs.getInt(3);
	  			
	  			upd = new String ("SELECT * FROM abstractions , hier_edges where a_id = " + id + " and ad_des_id = " + id);
	  			Statement selectStatement2 = dbConnection.createStatement();
	  	  		ResultSet rs2 = selectStatement2.executeQuery(upd);
	  	  		int ancestorId = -1;
	  	  		if (rs2.next()){
	  	  			ancestorId = rs2.getInt(7);
	  	  		}
	  	  		 			
	  			NonFunctionalAbstraction tmpNFA = null;
	  			String myWhereClause = new String(" where NFR_NFA_ID = " + id);
	  			Hashtable<String, Interval> nfaRanges = retrieveNFARanges(dbConnection, myWhereClause);
	  			
	  			myWhereClause = " where NFD_NFA_ID = " + id;
	  			Hashtable<String, String> qualifiers = retrieveNFANonFunctionalDescriptions(dbConnection, myWhereClause);
	  			
	  			myWhereClause = " and RIS_NFA_ID = " + id;
	  			Hashtable<Integer, ServiceInstance> representedInstances = retrieveRepresentedInstances(dbConnection, myWhereClause);
	  			JointGroup x = new JointGroup();
	  			
	  			if( nfaRanges != null ){
					
					Set nfaRangesSet = nfaRanges.entrySet();
				    Iterator nfaRangesIterator = nfaRangesSet.iterator();
				    while (nfaRangesIterator.hasNext()) {
				      Map.Entry entry = (Map.Entry) nfaRangesIterator.next();
				      x.addMappingsToRanges(entry.getKey().toString(), (Interval)(entry.getValue()));
				    }
				}
	  			
	  			if( qualifiers != null ){
					
					Set nfaRangesSet = qualifiers.entrySet();
				    Iterator nfaRangesIterator = nfaRangesSet.iterator();
				    while (nfaRangesIterator.hasNext()) {
				      Map.Entry entry = (Map.Entry) nfaRangesIterator.next();
				      x.addQualifier(entry.getKey().toString(), (String)(entry.getValue()));
				    }
				} 
	  			
	  			x.addRepresentedServiceInstances("newProperty", representedInstances);
	  			
	  			if (ancestorId == -1){
	  				
	  				tmpNFA = new NonFunctionalAbstraction(nfaId, x, null);
	  				//retrieve descendants
		  			ArrayList<NonFunctionalAbstraction> myChilds = new ArrayList<NonFunctionalAbstraction>();
		  			myChilds = retrieveNonFunctionalChilds(dbConnection, id, tmpNFA, resultSetType);
		  			for (int i = 0; i < myChilds.size(); i++){
		  				tmpNFA.addDescendant(myChilds.get(i));
		  			}
		  			
	  			}
	  			else{
	  				
	  				myWhereClause = " and A_ID = " + ancestorId;
	  				//retrieve my ancestor
	  				NonFunctionalAbstraction myParent = ancestor;
	  				tmpNFA = new NonFunctionalAbstraction(nfaId , x, myParent);
	  				//retrieve descendants
		  			ArrayList<NonFunctionalAbstraction> myChilds = new ArrayList<NonFunctionalAbstraction>();
		  			myChilds = retrieveNonFunctionalChilds(dbConnection, id, tmpNFA, resultSetType);
		  			for (int i = 0; i < myChilds.size(); i++){
		  				tmpNFA.addDescendant(myChilds.get(i));
		  			}
		  			
	  			}
	  				  			
	  			retrievedAbstractions.add(tmpNFA);
  	  		}
  	  		
  	  		return retrievedAbstractions;
  	  	
		}
		catch(Exception e) {
	          System.err.println("Exception retrieveNonFunctionalAbstractions: " + e.getMessage());
	    }
		return null;
	}
	
	
	
}