package org.ow2.choreos.serviceBase;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.choreos.abstractionBase.AbstractionBaseController;
import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.Hierarchy;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessagesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.OperationsMapping;
import org.ow2.choreos.dataStructures.interval.Interval;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ReferencedComponent;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.serviceRegistration.ServiceCollection;
import org.ow2.choreos.serviceRegistration.ServiceSource;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.serviceRepresentation.functional.ServiceType;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;

/**
 * 
 * @author dathanas
 *
 * This class contains methods to parse Service and Abstraction objects in the main memory and store them to the DBMS
 */
public class ServiceBaseController {
		
	//parses the selected Collection and its containing objects and calls the appropriate methods to store them to the DBMS 
	@SuppressWarnings({ "unchecked", "rawtypes" })
	static public void storeCollection(ServiceCollection currentServiceCollection, String dBase){
		Connection dbConnection = null;  // create connection to the DMBS
		ServiceRepresentationStorage storeServicesCollection= new ServiceRepresentationStorage();
		if (dBase.equals("")){
    		dBase = "localhost/default";
    	}
		
        try {
          Class.forName("com.mysql.jdbc.Driver").newInstance(); // load SQL driver to connect to DBMS
          //open connection to database
          
    	  dbConnection = DriverManager.getConnection("jdbc:mysql://" + dBase,"root", "root");
    	  //all object insertions in DBMS will be done here!
    	  
    	  if (currentServiceCollection == null){
      		  System.out.println("No collection to store!");
      		  return;
      	  }
    	  
      	  storeServicesCollection.storeCurrCollection(currentServiceCollection, dbConnection);
      	  
      	  //start of object parsing
      	  Map<String, ServiceSource> sourcesToStore = currentServiceCollection.getSources();
      	  if( sourcesToStore != null ){
      		int sourceCounter = -1;
      		int docsOfFuncPropertiesCounter = -1;
      		int typeCounter = -1;
      		int interfaceCounter = -1;
      		int instanceCounter = -1;
      		int nfDescriptionCounter = -1;
      		int operationCounter = -1;
      		int messageCounter = -1;
      		int messageTypeCounter = -1;
      		int propValueCounter = -1;
      		
      		//parse and store ServiceSources
			for( Enumeration<String> e1 = ( (Hashtable<String, ServiceSource>) sourcesToStore).keys(); e1.hasMoreElements(); ){
				ServiceSource tmpServiceSource = sourcesToStore.get( e1.nextElement() );
				sourceCounter = storeServicesCollection.storeServiceSource(tmpServiceSource, dbConnection, currentServiceCollection.getCollectionID());
				
				//parse and store DocOfFuncProperties
				Map<String, DocOfFuncProperties> docsOfFuncPropertiesToStore = tmpServiceSource.getDocsOfFuncProperties();
				if( docsOfFuncPropertiesToStore != null ){
					
					for( Enumeration<String> e2 = ((Hashtable<String,DocOfFuncProperties>) docsOfFuncPropertiesToStore).keys(); e2.hasMoreElements(); ){
						DocOfFuncProperties tmpDoc = docsOfFuncPropertiesToStore.get( e2.nextElement() );
						docsOfFuncPropertiesCounter = storeServicesCollection.storeDocOfFuncProperties(dbConnection, tmpDoc, sourceCounter);
						
						Map<String, ServiceType> serviceTypesToStore = tmpDoc.getServiceTypes();
						if( serviceTypesToStore != null ){
							
							//parse and store ServiceTypes
							for( Enumeration<String> e3 = ( (Dictionary<String, ServiceType>) serviceTypesToStore).keys(); e3.hasMoreElements(); ){
								ServiceType tmpServiceType = serviceTypesToStore.get( e3.nextElement() );

								typeCounter = storeServicesCollection.storeServiceType(tmpServiceType, dbConnection, docsOfFuncPropertiesCounter);	
								tmpServiceType.setDBKey(docsOfFuncPropertiesCounter);
								
								// ZAS ISSUE seems here we store for each service type all the interfaces no matter if these are provided by this type
								// ZAS Removed List<ServiceInterface> interfacesToStore = tmpServiceSource.getAllInterfaces();
								// ZAS Added
								List<ServiceInterface> interfacesToStore = tmpServiceType.getInterfaces();
								if( interfacesToStore != null ){

									//parse and store ServiceInterfaces
									for( int e4 = 0; e4 < interfacesToStore.size(); e4++ ){
										ServiceInterface tmpServiceInterface = interfacesToStore.get(e4);
										interfaceCounter = storeServicesCollection.storeServiceInterface(tmpServiceInterface, dbConnection, typeCounter);
										tmpServiceInterface.setDBKey(typeCounter);
										
										List<ServiceInstance> serviceInstancesToStore = tmpServiceInterface.getServiceInstances();
										if( serviceInstancesToStore != null ){
											
											//parse and store ServiceInstances
											for( int e5 = 0; e5 < serviceInstancesToStore.size(); e5++ ){
												ServiceInstance tmpServiceInstance = serviceInstancesToStore.get(e5);
												instanceCounter = storeServicesCollection.storeServiceInstance(tmpServiceInstance, dbConnection, interfaceCounter);
												tmpServiceInstance.setDBKey(interfaceCounter);
												
												Map<String, NonFuncDescription> nonFuncDescriptionsToStore = tmpServiceInstance.getNonFuncDescriptions();
												if( nonFuncDescriptionsToStore != null ){
													
													//parse and store NonFuncDescriptions
													for( Enumeration<String> e6 = ( (Hashtable<String, NonFuncDescription>) nonFuncDescriptionsToStore).keys(); e6.hasMoreElements(); ){
														NonFuncDescription tmpNonFuncDescription = nonFuncDescriptionsToStore.get( e6.nextElement() );
														nfDescriptionCounter = storeServicesCollection.storeNonFuncDescription(tmpNonFuncDescription, dbConnection, instanceCounter);
														
														Hashtable<String, Double> propValuesToStore = tmpNonFuncDescription.getPropertiesValues();
														if( propValuesToStore != null ){
															//parse and store Properties and Values
															Set propsSet = propValuesToStore.entrySet();
														    Iterator propsIterator = propsSet.iterator();
														    while (propsIterator.hasNext()) {
														      Map.Entry entry = (Map.Entry) propsIterator.next();
														      propValueCounter = storeServicesCollection.storePropertyValue(entry.getKey().toString(),(Double)(entry.getValue()), dbConnection, nfDescriptionCounter);
														      
														    }
														} 
														
													}
												}
											
												//parse and store QualityProperties
												Map<String, QualityProperties> qualityPropertiesToStore = tmpServiceInstance.getQualityProperties();
												if( qualityPropertiesToStore != null ){
													
													Set qualitiesSet = qualityPropertiesToStore.entrySet();
												    Iterator qualitiesIterator = qualitiesSet.iterator();
												    while (qualitiesIterator.hasNext()) {
												      Map.Entry entry = (Map.Entry) qualitiesIterator.next();
												      storeServicesCollection.storeQualityProperties(dbConnection, entry.getKey().toString(),(QualityProperties)(entry.getValue()),  instanceCounter);
												    }
												} 
											}
										}
										
										List<Operation> operationsToStore = tmpServiceInterface.getOperations();
										if( operationsToStore != null ){
										
											//parse and store Operations
											for( int e7 = 0; e7 < operationsToStore.size(); e7++ ){
												Operation tmpOperation = operationsToStore.get(e7);
																								
												Message inputMessage = tmpOperation.getInMessage();
												Message outputMessage = tmpOperation.getOutMessage();
												
												operationCounter = storeServicesCollection.storeOperation(tmpOperation, dbConnection, interfaceCounter);
												tmpOperation.setDBKey(interfaceCounter);
												
												if (inputMessage != null) {
													//store inMessage of the Operation
													messageCounter = storeServicesCollection.storeMessage(inputMessage, dbConnection, operationCounter, "IN");
													inputMessage.setDBKey(operationCounter);
													
													ArrayList<MessageType> inMessageTypesToStore = (ArrayList<MessageType>) inputMessage.getMessageTypes();
													if (inMessageTypesToStore != null){
														//parse and store MessageTypes
														for( int e8 = 0; e8 < inMessageTypesToStore.size(); e8++ ){
															MessageType tmpMessageType = inMessageTypesToStore.get(e8);
															messageTypeCounter = storeServicesCollection.storeMessageType(tmpMessageType, dbConnection, messageCounter);
															tmpMessageType.setDBKey(messageCounter);
															
															//store ReferencedComponent, MaterializedComponent and Leaf Components
															List<Object> componentsAndStructuresToStore = new ArrayList<Object>();
															componentsAndStructuresToStore = tmpMessageType.getReferencedComponent().getLeafComponentsAndStructures();
															ReferencedComponent referencedComponent = tmpMessageType.getReferencedComponent();
															if (referencedComponent != null){
																storeServicesCollection.storeReferencedComponent(referencedComponent, dbConnection, messageTypeCounter);
															}
															
															Component materializedComponent = tmpMessageType.getReferencedComponent().getMaterializedComponent();
															if (materializedComponent != null){
																storeServicesCollection.storeLeafComponent(materializedComponent, dbConnection, messageTypeCounter);
																materializedComponent.setDBKey(messageTypeCounter);
															}
															
															
															if (componentsAndStructuresToStore != null){
																//System.out.println(tmpMessageType.getName() + "--" + componentsAndStructuresToStore.size());
																for( int e9 = 0; e9 < componentsAndStructuresToStore.size(); e9++ ){
																	if (componentsAndStructuresToStore.get(e9).getClass().toString().indexOf("component") != -1){
																		Component tmpComponent = (Component) componentsAndStructuresToStore.get(e9);
																		storeServicesCollection.storeLeafComponent(tmpComponent, dbConnection, messageTypeCounter);
																		tmpComponent.setDBKey(messageTypeCounter);
																	}
																	else{
																		Structure tmpStructure = (Structure) componentsAndStructuresToStore.get(e9);
																		storeServicesCollection.storeLeafStructure(tmpStructure, dbConnection, messageTypeCounter);
																		tmpStructure.setDBKey(messageTypeCounter);
																	}
																	
																}
															}//end of if componentsAndStructures
															
														}
													}
													
												}
												if (outputMessage != null) {
													//store inMessage of the Operation
													messageCounter = storeServicesCollection.storeMessage(outputMessage, dbConnection, operationCounter, "OUT");
													outputMessage.setDBKey(operationCounter);
													
													ArrayList<MessageType> outMessageTypesToStore = (ArrayList<MessageType>) outputMessage.getMessageTypes();
													if (outMessageTypesToStore != null){
														//parse and store MessageTypes
														for( int e10 = 0; e10 < outMessageTypesToStore.size(); e10++ ){
															MessageType tmpMessageType = outMessageTypesToStore.get(e10);
															messageTypeCounter = storeServicesCollection.storeMessageType(tmpMessageType, dbConnection, messageCounter);
															tmpMessageType.setDBKey(messageCounter);
															
															//store ReferencedComponent, MaterializedComponent and Leaf Components
															List<Object> componentsAndStructuresToStore = new ArrayList<Object>();
															componentsAndStructuresToStore = tmpMessageType.getReferencedComponent().getLeafComponentsAndStructures();
															ReferencedComponent referencedComponent = tmpMessageType.getReferencedComponent();
															if (referencedComponent != null){
																storeServicesCollection.storeReferencedComponent(referencedComponent, dbConnection, messageTypeCounter);
															}
															
															Component materializedComponent = tmpMessageType.getReferencedComponent().getMaterializedComponent();
															if (materializedComponent != null){
																storeServicesCollection.storeLeafComponent(materializedComponent, dbConnection, messageTypeCounter);
																materializedComponent.setDBKey(messageTypeCounter);
															}
															
															if (componentsAndStructuresToStore != null){
																
																for( int e9 = 0; e9 < componentsAndStructuresToStore.size(); e9++ ){
																	if (componentsAndStructuresToStore.get(e9).getClass().toString().indexOf("component") != -1){
																		Component tmpComponent = (Component) componentsAndStructuresToStore.get(e9);
																		storeServicesCollection.storeLeafComponent(tmpComponent, dbConnection, messageTypeCounter);
																		tmpComponent.setDBKey(messageTypeCounter);
																	}
																	else{
																		Structure tmpStructure = (Structure) componentsAndStructuresToStore.get(e9);
																		storeServicesCollection.storeLeafStructure(tmpStructure, dbConnection, messageTypeCounter);
																		tmpStructure.setDBKey(messageTypeCounter);
																	}
																	
																}
															}//end of if componentsAndStructures
															
														}
													}
													
												}
												
											}
										}	
										
									}
								}
								
							}
						}
						
					}
					
				}
				
			}	
		 } //end of (services) object parsing 
      	
      	//retrieveCollection(dbConnection);
      
        } catch(Exception e) {
          System.err.println("Exception storeCollection: " + e.getMessage());
          e.printStackTrace();
        } finally {
          try {
            if(dbConnection != null)
            	dbConnection.close();
          } catch(SQLException e) {}
        }
        
    }// end of storeCollection
	
	//used for debugging
	static public void retrieveCollection(Connection dbConnection){
		
		ServiceRepresentationRetriever serviceRetriever = new ServiceRepresentationRetriever();
		serviceRetriever.retrieveCollection(dbConnection,"");
		
    }
	
	//used for debugging
	static public void retrieveAbstractions(Connection dbConnection){
		AbstractionsHierarchyRetriever abstractionRetriever = new AbstractionsHierarchyRetriever();
	
		abstractionRetriever.retrieveFunctionalAbstractions(dbConnection,"", -1);
		
		ArrayList <NonFunctionalAbstraction> retrievedAbstractions = new ArrayList <NonFunctionalAbstraction> ();
		retrievedAbstractions = abstractionRetriever.retrieveNonFunctionalAbstractions(dbConnection,"", null, -1);
	}
	
	//parses the 'Abstraction' objects (both Functional and Nonfunctional) of the hierarchies and its containing objects and calls the appropriate methods to store them to the DBMS 
	@SuppressWarnings({ "rawtypes" })
	static public void storeAbstractions(AbstractionBaseController abstractionBase, String dBase){
    	
    	Connection dbConnection = null;  // create connection to the DMBS
    	AbstractionsStorage storeAbstractions = new AbstractionsStorage();
    	if (dBase.equals("")){
    		dBase = "localhost/default";
    	}
    	
        try {
          Class.forName("com.mysql.jdbc.Driver").newInstance(); // load SQL driver to connect to DBMS
          //open connection to database
    	  dbConnection = DriverManager.getConnection("jdbc:mysql://" + dBase,"root", "root");
                
    	  int abstractionsCounter = -1;
    	  int representativeInterfaceCounter = -1;
    	  int interfaceMappingsCounter = -1;
    	  int operationsMappingsCounter = -1;
    	  int messageMappingsCounter = -1;
    	  int messageTypesMappingsCounter = -1;
    	  int componentsAndStructuresMappingsCounter = -1;
    	  int hierachiesCounter = -1;
    	  int reprOperationsCounter = -1;
    	  int reprMessagesCounter = -1;
    	  int reprMessageTypesCounter = -1;
    	  int reprComponentsCounter = -1;
    	  
    	  
    	  Map<Integer, Hierarchy> hierarchiesToStore = abstractionBase.getHierarchies();
    	  
    	  //parse and store Hierarchies
    	  for( Enumeration<Integer> e = ( (Hashtable<Integer, Hierarchy>) hierarchiesToStore).keys(); e.hasMoreElements(); ){
    		  Hierarchy tmpHierarchy = hierarchiesToStore.get(e.nextElement());
    		  hierachiesCounter = storeAbstractions.storeHierarchy(dbConnection, tmpHierarchy);
    	  
    		  Map<Integer, FunctionalAbstraction> functionalAbstractionsToStore = new Hashtable<Integer, FunctionalAbstraction>();
    		  Hashtable<Integer, NonFunctionalAbstraction> nonFunctionalAbstractionsToStore = new Hashtable<Integer, NonFunctionalAbstraction>();
    		  
    		  if (abstractionBase == null){
    			  System.out.println("No abstractions!!");
    			  return;
    		  }
    		  else{
    			  functionalAbstractionsToStore = abstractionBase.get(tmpHierarchy.getCollectionID());
    			  nonFunctionalAbstractionsToStore = abstractionBase.getNonFunctional(tmpHierarchy.getCollectionID());
    		  }
    	      	  
    		  if( functionalAbstractionsToStore != null ){
    			  ArrayList<FunctionalAbstraction> functionalAbstractionsToStoreList = new ArrayList<FunctionalAbstraction>();
    			  for( Enumeration<Integer> e1 = ( (Hashtable<Integer,FunctionalAbstraction>) functionalAbstractionsToStore).keys(); e1.hasMoreElements(); ){
    				  functionalAbstractionsToStoreList.add(functionalAbstractionsToStore.get(e1.nextElement()));
    			  }
    			  ArrayList<ArrayList<Integer>> childs = constructChildsList(functionalAbstractionsToStoreList);
    			  
    			  ArrayList<Integer> insertedAbstractions = new ArrayList<Integer>();
    			  //parse and store FunctionalAbstractions - parsing starts from leafs and ends to the root of the abstraction hierarchy
    			     			  
    			  while (abstractionsCounter != functionalAbstractionsToStoreList.size()){
    				  
    				  int id = chooseAbstractionToStore(childs, functionalAbstractionsToStoreList, insertedAbstractions);
    				  
    				  FunctionalAbstraction tmpFunctionalAbstraction = functionalAbstractionsToStoreList.get( id );
    				  insertedAbstractions.add(id);
    				  
    				  abstractionsCounter = storeAbstractions.storeAbstraction(dbConnection, tmpFunctionalAbstraction, null, hierachiesCounter);
    				  tmpFunctionalAbstraction.setDBKey(abstractionsCounter);
    				  
    				  //store the RepresentativeInterface of the Abstraction
    				  representativeInterfaceCounter = storeAbstractions.storeRepresentativeInterface(dbConnection, tmpFunctionalAbstraction.getInterface(), abstractionsCounter);
    				  tmpFunctionalAbstraction.getInterface().setDBKey(representativeInterfaceCounter);
    				  //store the InterfacesMapping of the Abstraction
    				  interfaceMappingsCounter = storeAbstractions.storeInterfacesMapping(dbConnection, tmpFunctionalAbstraction.getInterfacesMapping(), abstractionsCounter);
    				  
    				 
    				  //parse and store MappedInterfaces of the InterfacesMapping
    				  InterfacesMapping interfacesMapping = tmpFunctionalAbstraction.getInterfacesMapping();
    				  List<ServiceInterface> mappednterfacesToStore = interfacesMapping.getServiceInterfaces();
    				  for (int i =0; i < mappednterfacesToStore.size(); i++){
    					  ServiceInterface tmpInterface = mappednterfacesToStore.get(i);
    					  storeAbstractions.storeMappedInterface(dbConnection, tmpInterface, interfaceMappingsCounter, representativeInterfaceCounter);
    				  }
    				  
    				  //parse and store OperationsMappings of the InterfacesMapping
    				  List<OperationsMapping> operationsMappings = interfacesMapping.getOperationsMappings();
    				  //store representativeOperations of the representative interface
    				  ArrayList<Operation> representativeOperations = (ArrayList<Operation>) tmpFunctionalAbstraction.getInterface().getOperations();
    				  //parse and store RepresentativeOperations
					  for( int e7 = 0; e7 < representativeOperations.size(); e7++ ){
						  Operation tmpRepresentativeOperation = representativeOperations.get(e7);
							
						  Message representativeInputMessage = tmpRepresentativeOperation.getInMessage();
						  Message representativeOutputMessage = tmpRepresentativeOperation.getOutMessage();
							
						  reprOperationsCounter = storeAbstractions.storeRepresentativeOperation(dbConnection, tmpRepresentativeOperation, representativeInterfaceCounter);
						  tmpRepresentativeOperation.setDBKey(representativeInterfaceCounter);
							
						  OperationsMapping tmpOperationMapping = operationsMappings.get(e7);
						  operationsMappingsCounter = storeAbstractions.storeOperationsMapping(dbConnection, tmpOperationMapping, interfaceMappingsCounter);
	    				  
						  //parse and store MappedOperations of the OperationsMapping
						  ArrayList<Operation> mappedOperations = (ArrayList<Operation>) tmpOperationMapping.getOperations();
						  for (int i2 =0; i2 < mappedOperations.size(); i2++){
							  Operation tmpMappedOperation = mappedOperations.get(i2);
							  storeAbstractions.storeMappedOperation(dbConnection, tmpMappedOperation, operationsMappingsCounter, reprOperationsCounter);
						  }
							
							
							
						  if (representativeInputMessage != null) {
							  //store RepresentativeInMessage of the RepresentativeOperation
							  reprMessagesCounter = storeAbstractions.storeRepresentativeMessage(representativeInputMessage, dbConnection, reprOperationsCounter, "IN");
							  representativeInputMessage.setDBKey(reprOperationsCounter);
								
							  MessagesMapping inputMessageMapping = tmpOperationMapping.getInputMappings();
							  messageMappingsCounter = storeAbstractions.storeMessagesMapping(dbConnection, inputMessageMapping, operationsMappingsCounter, "IN");
							  List<Message> mappedMessages = inputMessageMapping.getMessages();
							  //parse and store MappedMessages of the inMessageMapping
							  for (int i3 =0; i3 < mappedMessages.size(); i3++){
								  Message tmpMappedMessage = mappedMessages.get(i3);
								  storeAbstractions.storeMappedMessage(dbConnection, tmpMappedMessage, messageMappingsCounter, reprMessagesCounter);
							  }
								
							  ArrayList<MessageType> representativeInputMessageTypes = (ArrayList<MessageType>) representativeInputMessage.getMessageTypes();
							  if (representativeInputMessageTypes != null){
								  List<MessageTypesMapping> inputMessageTypesMappings = inputMessageMapping.getMessageTypesMapping();
								  //parse and store RepresentativeMessageTypes
								  for( int e8 = 0; e8 < representativeInputMessageTypes.size(); e8++ ){
									  MessageType tmpRepresentativeMessageType = representativeInputMessageTypes.get(e8);
									  reprMessageTypesCounter = storeAbstractions.storeRepresentativeMessageType(tmpRepresentativeMessageType, dbConnection, reprMessagesCounter);
									  tmpRepresentativeMessageType.setDBKey(reprMessagesCounter);
									  MessageTypesMapping tmpMessageTypesMapping = inputMessageTypesMappings.get(e8);
									  messageTypesMappingsCounter = storeAbstractions.storeMessageTypesMapping(dbConnection, tmpMessageTypesMapping, messageMappingsCounter);
									  
									  List<MessageType> mappedMessageTypes = tmpMessageTypesMapping.getMessageTypes();
									  for (int i2 = 0; i2 < mappedMessageTypes.size(); i2++){
										  MessageType tmpMappedMessageType = mappedMessageTypes.get(i2);
										  storeAbstractions.storeMappedMessageType(dbConnection, tmpMappedMessageType, messageTypesMappingsCounter, reprMessageTypesCounter);
									  }
									  
									  
									  
									  
									  List<Object> representativeComponentsAndStructuresToStore = new ArrayList<Object>();
									  representativeComponentsAndStructuresToStore = tmpRepresentativeMessageType.getReferencedComponent().getLeafComponentsAndStructures();
									  ReferencedComponent referencedComponent = tmpRepresentativeMessageType.getReferencedComponent();
									  if (referencedComponent != null){
										  reprComponentsCounter = storeAbstractions.storeRepresentativeReferencedComponent(referencedComponent, dbConnection, reprMessageTypesCounter);
									  }
									  
									  Component materializedComponent = tmpRepresentativeMessageType.getReferencedComponent().getMaterializedComponent();
									  if (materializedComponent != null){
										  reprComponentsCounter = storeAbstractions.storeRepresentativeLeafComponent(materializedComponent, dbConnection, reprMessageTypesCounter);
										  materializedComponent.setDBKey(reprMessageTypesCounter);
									  }
									  
									  if (representativeComponentsAndStructuresToStore != null){
										  ComponentsAndStructuresMapping componentsAndStructuresMapping = (ComponentsAndStructuresMapping) tmpMessageTypesMapping.getComponentsAndStructuresMapping();
										  List<Object> representedRootComponentsAndStructuresToStore = new ArrayList<Object>();
										  if (componentsAndStructuresMapping != null){
											  
											  representedRootComponentsAndStructuresToStore = componentsAndStructuresMapping.getComponentsAndStructureMapping();
											  componentsAndStructuresMappingsCounter = storeAbstractions.storeComponentsAndStructuresMapping(dbConnection, componentsAndStructuresMapping, messageTypesMappingsCounter, -1);
											  for( int e9 = 0; e9 < representedRootComponentsAndStructuresToStore.size(); e9++ ){
												  if (representedRootComponentsAndStructuresToStore.get(e9).getClass().toString().indexOf("component") != -1){
													  Component tmpMappedComponent = (Component) representedRootComponentsAndStructuresToStore.get(e9);
													  storeAbstractions.storeMappedComponent(dbConnection, tmpMappedComponent, componentsAndStructuresMappingsCounter, reprComponentsCounter);
													  
												  }
												  else{
													  Structure tmpMappedStructure = (Structure) representedRootComponentsAndStructuresToStore.get(e9);
													  storeAbstractions.storeMappedStructure(dbConnection, tmpMappedStructure, componentsAndStructuresMappingsCounter, reprComponentsCounter);
												  }
											  }
										  }
										  
										  ArrayList<ComponentsAndStructuresMapping> componentsAndStructuresMappings = (ArrayList<ComponentsAndStructuresMapping>) componentsAndStructuresMapping.getComponentsAndStructuresMappings();
										  // store representative ComponentsAndStructures
										  for( int e9 = 0; e9 < representativeComponentsAndStructuresToStore.size(); e9++ ){
											  
											  if (representativeComponentsAndStructuresToStore.get(e9).getClass().toString().indexOf("component") != -1){
												  Component tmpComponent = (Component) representativeComponentsAndStructuresToStore.get(e9);
												  reprComponentsCounter = storeAbstractions.storeRepresentativeLeafComponent(tmpComponent, dbConnection, reprMessageTypesCounter);
												  tmpComponent.setDBKey(reprMessageTypesCounter);
											  }
											  else{
												  Structure tmpStructure = (Structure) representativeComponentsAndStructuresToStore.get(e9);
												  reprComponentsCounter = storeAbstractions.storeRepresentativeLeafStructure(tmpStructure, dbConnection, reprMessageTypesCounter);
												  tmpStructure.setDBKey(reprMessageTypesCounter);
											  }
											  
											  ComponentsAndStructuresMapping tmpComponentsAndStructuresMapping = componentsAndStructuresMappings.get(e9);
											  List<Object> representedComponentsAndStructuresToStore = new ArrayList<Object>();
											  if (tmpComponentsAndStructuresMapping != null){
												  
												  representedComponentsAndStructuresToStore = tmpComponentsAndStructuresMapping.getComponentsAndStructureMapping();
												  componentsAndStructuresMappingsCounter = storeAbstractions.storeComponentsAndStructuresMapping(dbConnection, tmpComponentsAndStructuresMapping, -1, componentsAndStructuresMappingsCounter);
												  for( int e10 = 0; e10 < representedComponentsAndStructuresToStore.size(); e10++ ){
													  if (representedComponentsAndStructuresToStore.get(e10).getClass().toString().indexOf("component") != -1){
														  Component tmpMappedComponent = (Component) representedComponentsAndStructuresToStore.get(e10);
														  storeAbstractions.storeMappedComponent(dbConnection, tmpMappedComponent, componentsAndStructuresMappingsCounter, reprComponentsCounter);
														  
													  }
													  else{
														  Structure tmpMappedStructure = (Structure) representedComponentsAndStructuresToStore.get(e10);
														  storeAbstractions.storeMappedStructure(dbConnection, tmpMappedStructure, componentsAndStructuresMappingsCounter, reprComponentsCounter);
													  }
												  }
											  }
										  }
									  }//end of if componentsAndStructures
									  
								  }
							  }
						  }
						  
						  if (representativeOutputMessage != null) {
							  //store RepresentativeOutMessage of the RepresentativeOperation
							  reprMessagesCounter = storeAbstractions.storeRepresentativeMessage(representativeOutputMessage, dbConnection, reprOperationsCounter, "OUT");
							  representativeOutputMessage.setDBKey(reprOperationsCounter);
								
							  MessagesMapping outputMessageMapping = tmpOperationMapping.getOutputMappings();
							  messageMappingsCounter = storeAbstractions.storeMessagesMapping(dbConnection, outputMessageMapping, operationsMappingsCounter, "OUT");
							  List<Message> mappedMessages = outputMessageMapping.getMessages();
							  //parse and store MappedMessages of the outMessageMapping
							  for (int i3 =0; i3 < mappedMessages.size(); i3++){
								  Message tmpMappedMessage = mappedMessages.get(i3);
								  storeAbstractions.storeMappedMessage(dbConnection, tmpMappedMessage, messageMappingsCounter, reprMessagesCounter);
							  }
		    				  	  
							  ArrayList<MessageType> representativeOutputMessageTypes = (ArrayList<MessageType>) representativeOutputMessage.getMessageTypes();
							  if (representativeOutputMessageTypes != null){
								  List<MessageTypesMapping> outputMessageTypesMappings = outputMessageMapping.getMessageTypesMapping();
								  
								  //parse and store RepresentativeMessageTypes
								  for( int e10 = 0; e10 < representativeOutputMessageTypes.size(); e10++ ){
									  MessageType tmpRepresentativeMessageType = representativeOutputMessageTypes.get(e10);
									  reprMessageTypesCounter = storeAbstractions.storeRepresentativeMessageType(tmpRepresentativeMessageType, dbConnection, reprMessagesCounter);
									  tmpRepresentativeMessageType.setDBKey(reprMessagesCounter);
									  MessageTypesMapping tmpMessageTypesMapping = outputMessageTypesMappings.get(e10);
									  messageTypesMappingsCounter = storeAbstractions.storeMessageTypesMapping(dbConnection, tmpMessageTypesMapping, messageMappingsCounter);
										
									  List<MessageType> mappedMessageTypes = tmpMessageTypesMapping.getMessageTypes();
									  for (int i2 = 0; i2 < mappedMessageTypes.size(); i2++){
										  MessageType tmpMappedMessageType = mappedMessageTypes.get(i2);
										  storeAbstractions.storeMappedMessageType(dbConnection, tmpMappedMessageType, messageTypesMappingsCounter, reprMessageTypesCounter);
									  }
									  
									  
									  
									  
									  
									  List<Object> representativeComponentsAndStructuresToStore = new ArrayList<Object>();
									  representativeComponentsAndStructuresToStore = tmpRepresentativeMessageType.getReferencedComponent().getLeafComponentsAndStructures();
									  ReferencedComponent referencedComponent = tmpRepresentativeMessageType.getReferencedComponent();
									  if (referencedComponent != null){
										  reprComponentsCounter = storeAbstractions.storeRepresentativeReferencedComponent(referencedComponent, dbConnection, reprMessageTypesCounter);
									  }
									  

									  Component materializedComponent = tmpRepresentativeMessageType.getReferencedComponent().getMaterializedComponent();
									  if (materializedComponent != null){
										  reprComponentsCounter = storeAbstractions.storeRepresentativeLeafComponent(materializedComponent, dbConnection, reprMessageTypesCounter);
										  materializedComponent.setDBKey(reprMessageTypesCounter);
									  }
									  
									  if (representativeComponentsAndStructuresToStore != null){
										  //store componentsAndStructuresMappings
										  ComponentsAndStructuresMapping componentsAndStructuresMapping = tmpMessageTypesMapping.getComponentsAndStructuresMapping();
										  List<Object> representedRootComponentsAndStructuresToStore = new ArrayList<Object>();
										  if (componentsAndStructuresMapping != null){
											  representedRootComponentsAndStructuresToStore = componentsAndStructuresMapping.getComponentsAndStructureMapping();
											  componentsAndStructuresMappingsCounter = storeAbstractions.storeComponentsAndStructuresMapping(dbConnection, componentsAndStructuresMapping, messageTypesMappingsCounter, -1);
											  for( int e9 = 0; e9 < representedRootComponentsAndStructuresToStore.size(); e9++ ){
												  if (representedRootComponentsAndStructuresToStore.get(e9).getClass().toString().indexOf("component") != -1){
													  Component tmpMappedComponent = (Component) representedRootComponentsAndStructuresToStore.get(e9);
													  storeAbstractions.storeMappedComponent(dbConnection, tmpMappedComponent, componentsAndStructuresMappingsCounter, reprComponentsCounter);
													  
												  }
												  else{
													  Structure tmpMappedStructure = (Structure) representativeComponentsAndStructuresToStore.get(e9);
													  storeAbstractions.storeMappedStructure(dbConnection, tmpMappedStructure, componentsAndStructuresMappingsCounter, reprComponentsCounter);
												  }
											  }
										  }
										  
										  ArrayList<ComponentsAndStructuresMapping> componentsAndStructuresMappings = (ArrayList<ComponentsAndStructuresMapping>) componentsAndStructuresMapping.getComponentsAndStructuresMappings();
										  // store representative ComponentsAndStructures
										  for( int e9 = 0; e9 < representativeComponentsAndStructuresToStore.size(); e9++ ){
											  
											  if (representativeComponentsAndStructuresToStore.get(e9).getClass().toString().indexOf("component") != -1){
												  Component tmpComponent = (Component) representativeComponentsAndStructuresToStore.get(e9);
												  reprComponentsCounter = storeAbstractions.storeRepresentativeLeafComponent(tmpComponent, dbConnection, reprMessageTypesCounter);
												  tmpComponent.setDBKey(reprMessageTypesCounter);
											  }
											  else{
												  Structure tmpStructure = (Structure) representativeComponentsAndStructuresToStore.get(e9);
												  reprComponentsCounter = storeAbstractions.storeRepresentativeLeafStructure(tmpStructure, dbConnection, reprMessageTypesCounter);
												  tmpStructure.setDBKey(reprMessageTypesCounter);
											  }
											  
											  ComponentsAndStructuresMapping tmpComponentsAndStructuresMapping = componentsAndStructuresMappings.get(e9);
											  List<Object> representedComponentsAndStructuresToStore = new ArrayList<Object>();
											  if (tmpComponentsAndStructuresMapping != null){
												  
												  representedComponentsAndStructuresToStore = tmpComponentsAndStructuresMapping.getComponentsAndStructureMapping();
												  componentsAndStructuresMappingsCounter = storeAbstractions.storeComponentsAndStructuresMapping(dbConnection, tmpComponentsAndStructuresMapping, -1, componentsAndStructuresMappingsCounter);
												  for( int e11 = 0; e11 < representedComponentsAndStructuresToStore.size(); e11++ ){
													  if (representedComponentsAndStructuresToStore.get(e11).getClass().toString().indexOf("component") != -1){
														  Component tmpMappedComponent = (Component) representedComponentsAndStructuresToStore.get(e11);
														  storeAbstractions.storeMappedComponent(dbConnection, tmpMappedComponent, componentsAndStructuresMappingsCounter, reprComponentsCounter);
														  
													  }
													  else{
														  Structure tmpMappedStructure = (Structure) representedComponentsAndStructuresToStore.get(e11);
														  storeAbstractions.storeMappedStructure(dbConnection, tmpMappedStructure, componentsAndStructuresMappingsCounter, reprComponentsCounter);
													  }
												  }
											  }
										  }
										  
									  }//end of if componentsAndStructures
									  
								  }
							  }
						  }
							
					  }
					 
    			  }
    		  }
    	  
    		  
    		  if( nonFunctionalAbstractionsToStore != null ){
    			  //parse and store FunctionalAbstractions
    			  for( Enumeration<Integer> e1 = nonFunctionalAbstractionsToStore.keys(); e1.hasMoreElements(); ){
    				  NonFunctionalAbstraction tmpNonFunctionalAbstraction = nonFunctionalAbstractionsToStore.get( e1.nextElement() );
    				  abstractionsCounter = storeAbstractions.storeAbstraction(dbConnection, null, tmpNonFunctionalAbstraction, hierachiesCounter);
    				  tmpNonFunctionalAbstraction.setDBKey(abstractionsCounter);
    				  
    				  //parse and store RepresentedInstances of the NonFunctionalAbstraction
    				  Map<Integer, ServiceInstance> representedInstancesToStore = tmpNonFunctionalAbstraction.getRepresentedServiceInstances();
    				  for( Enumeration<Integer> e2 = ( (Hashtable<Integer,ServiceInstance>) representedInstancesToStore).keys(); e2.hasMoreElements(); ){
    					  ServiceInstance tmpInstance = representedInstancesToStore.get( e2.nextElement() );
    					  storeAbstractions.storeRepresentedInstance(dbConnection, tmpInstance, abstractionsCounter);
    				  }
    			  
    				  //parse and store NonFunctionalDescriptions of the NonFunctionalAbstraction
    				  Hashtable<String, String> nonFunctionalDescriptionToStore = tmpNonFunctionalAbstraction.getNonFunctionalDescription();
    				  if( nonFunctionalDescriptionToStore != null ){
    					  Set nfdSet = nonFunctionalDescriptionToStore.entrySet();
    					  Iterator nfdIterator = nfdSet.iterator();
    					  while (nfdIterator.hasNext()) {
    						  Map.Entry entry = (Map.Entry) nfdIterator.next();
    						  storeAbstractions.storeNonFunctionalDescription(entry.getKey().toString(),(String)(entry.getValue()), dbConnection, abstractionsCounter);
    					  }
    				  } 
    			  
    				 //parse and store mappingsToRanges of the NonFunctionalAbstraction
    				  Map<String, Interval> mappingsToRangesToStore = tmpNonFunctionalAbstraction.getMappingsToRanges();
    				  
    				  if (mappingsToRangesToStore != null ){
    					  Set mapToRangesSet = mappingsToRangesToStore.entrySet();
    					  Iterator mapToRangesIterator = mapToRangesSet.iterator();
    					  while (mapToRangesIterator.hasNext()) {
    						  Map.Entry entry = (Map.Entry) mapToRangesIterator.next();
    						  storeAbstractions.storeRange(dbConnection, (Interval)entry.getValue(), entry.getKey().toString(), abstractionsCounter);
    					  }
    				  } 
    			  
    			  }
    		  }
    	  
    		  //store Hierarchy Edges
    		  if( functionalAbstractionsToStore != null ){
    			  for( Enumeration<Integer> e1 = ( (Hashtable<Integer,FunctionalAbstraction>) functionalAbstractionsToStore).keys(); e1.hasMoreElements(); ){
    				  FunctionalAbstraction tmpFunctionalAbstraction = functionalAbstractionsToStore.get( e1.nextElement() );
    				  
    				  storeAbstractions.storeDescentants(dbConnection, tmpFunctionalAbstraction, null);
    			  }
    		  }
    	  
    		  if( nonFunctionalAbstractionsToStore != null ){
    			  for( Enumeration<Integer> e1 = nonFunctionalAbstractionsToStore.keys(); e1.hasMoreElements(); ){
    				  NonFunctionalAbstraction tmpNonFunctionalAbstraction = nonFunctionalAbstractionsToStore.get( e1.nextElement() );
    				  
    				  storeAbstractions.storeDescentants(dbConnection, null, tmpNonFunctionalAbstraction);
    			  }
    		  }
    		  
    	  }//end of for hierarchies
    	 
    	  //retrieveAbstractions(dbConnection);
    	      	  
        }catch(Exception e) {
              System.err.println("Exception storeAbstractions: " + e.getMessage());
            } finally {
              try {
                if(dbConnection != null)
                	dbConnection.close();
              } catch(SQLException e) {}
            }
    	
    }
	
	
	//constructs an ArrayList to store the childs' IDs of each abstraction - for each abstraction a list of its childs IDs is stored
	private static ArrayList<ArrayList<Integer>> constructChildsList(ArrayList<FunctionalAbstraction> functionalAbstractionsToStoreList){
		ArrayList<ArrayList<Integer>> childsOfAbstractions = new ArrayList<ArrayList<Integer>>(functionalAbstractionsToStoreList.size());
			
		for(int j = 0; j < functionalAbstractionsToStoreList.size(); j++){
			ArrayList<Integer> tmp = new ArrayList<Integer>();
			FunctionalAbstraction tmpFunctionalAbstraction = functionalAbstractionsToStoreList.get( j );
				
			List<FunctionalAbstraction> tmpDescendants = null;
			tmpDescendants = tmpFunctionalAbstraction.getDescendants();
			if (tmpDescendants != null){
				for (int i = 0; i < tmpDescendants.size(); i++){
					tmp.add(tmpDescendants.get(i).getID());
				}
			}
				
			else{
				tmpDescendants = new ArrayList<FunctionalAbstraction>();
			}
			childsOfAbstractions.add(tmp);
		}
			
		return childsOfAbstractions;
	}
	

	//decides which abstraction is the next to be stored - this abstraction must have no childs or its childs should be already stored
	private static int chooseAbstractionToStore(ArrayList<ArrayList<Integer>> childsOfAbstractions, ArrayList<FunctionalAbstraction> functionalAbstractionsToStoreList, ArrayList<Integer> insertedAbstractions){
		Integer id = new Integer(-1);
		
		for(int i = 0; i < childsOfAbstractions.size(); i++){
			if (childsOfAbstractions.get(i).isEmpty() && insertedAbstractions.indexOf(i) == -1){
				id = i;
				insertedAbstractions.add(id);
				break;
			}
		}
		
		FunctionalAbstraction FAToStore = null;
		FAToStore = functionalAbstractionsToStoreList.get(id);
		
		if (FAToStore == null){
			System.out.println("No FA to store!!!!!!!!");
		}
		int ancestorPosition = 0;
		for( int i = 0; i < functionalAbstractionsToStoreList.size(); i++ ){
			if (functionalAbstractionsToStoreList.get( i ).equals(FAToStore.getAncestor())){
				int listIndexToRemove = childsOfAbstractions.get(ancestorPosition).indexOf(functionalAbstractionsToStoreList.get(id).getID());
				childsOfAbstractions.get(ancestorPosition).remove(listIndexToRemove);
				break;
			}
			ancestorPosition++;
		}
		
		return id;
	}

	
}//end of class