package org.ow2.choreos.serviceBase;

//imports to connect to the DBMS
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.serviceRegistration.ServiceCollection;
import org.ow2.choreos.serviceRegistration.ServiceSource;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.serviceRepresentation.functional.ServiceType;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;

/**
 * 
 * @author dathanas
 *
 * This class contains methods to retrieve the objects of the Service model from the DBMS
 */
public class ServiceRepresentationRetriever {
	private Statement selectStatement;	// empty SQL statement
	
	//Constructor
	ServiceRepresentationRetriever(){
		//do nothing at the moment
	}
	
	//reconstructs and returns the 'ServiceCollection' object specified by the 'whereClause' argument
	public ServiceCollection retrieveCollection(Connection dbConnection, String whereClause){
		ServiceCollection collectionRetrieved = null;
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM servicecollections " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			collectionRetrieved = new ServiceCollection(id);
  	  			
  	  			//to reconstruct the 'ServiceCollection' we must retrieve its 'ServiceSources' too
  	  			String myWhereClause = new String("where SS_SC_ID = " + id);
  	  			ArrayList<ServiceSource> myServiceSources = retrieveSources(dbConnection, myWhereClause);
  	  			for (int i = 0; i < myServiceSources.size(); i++) {
  	  				collectionRetrieved.addSource(myServiceSources.get(i).getRef2Source(), myServiceSources.get(i).getDocsOfFuncProperties());
  	  			}
  	  		}
  	  		return collectionRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveCollection Exception: " + e.getMessage());
	    }	
		return null;
	}
	
	//reconstructs and returns the 'ServiceSource' objects specified by the 'whereClause' argument
	public ArrayList<ServiceSource> retrieveSources(Connection dbConnection, String whereClause){
		ArrayList<ServiceSource> sourcesRetrieved = new ArrayList<ServiceSource>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM servicesources " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String ref2source = rs.getString(2);
  	  			
  	  			//to reconstruct the 'ServiceSource' we must retrieve its 'DocOfFuncProperties' too
  	  			String myWhereClause = new String("where DOCFUNCPROPS_SS_ID = " + id);
  	  			ArrayList<DocOfFuncProperties> myDocOfFuncProperties = retrieveDocOfFuncProperties(dbConnection, myWhereClause);
  	  			Hashtable<String, DocOfFuncProperties> tmpDocs = new Hashtable<String, DocOfFuncProperties>();
  	  			for (int i = 0; i < myDocOfFuncProperties.size(); i++) {
  	  				tmpDocs.put(myDocOfFuncProperties.get(i).getRef2DocOfFuncProperties(), myDocOfFuncProperties.get(i));
  	  			}
  	  		
  	  			ServiceSource currSource = new ServiceSource(ref2source, tmpDocs);
  	  			sourcesRetrieved.add(currSource);
  	  		}

  	  		return sourcesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveSources Exception: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the 'DocOfFuncProperties' objects specified by the 'whereClause' argument
	public ArrayList<DocOfFuncProperties> retrieveDocOfFuncProperties(Connection dbConnection, String whereClause){
		ArrayList<DocOfFuncProperties> docOfFuncPropsRetrieved = new ArrayList<DocOfFuncProperties>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM docoffuncproperties " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String ref2DocOfFuncProperties = rs.getString(2);
  	  			DocOfFuncProperties currDoc = new DocOfFuncProperties (ref2DocOfFuncProperties);
  	  			
  	  			//to reconstruct the 'DocOfFuncProperties' we must retrieve its 'ServiceTypes' too
  	  			String myWhereClause = new String("where ST_DOCFUNCPROPS_ID = " + id);
  	  			ArrayList<ServiceType> myServiceTypes = retrieveServiceTypes(dbConnection, myWhereClause);
  	  			for (int i = 0 ; i < myServiceTypes.size(); i++) {
  	  				currDoc.add(myServiceTypes.get(i));
  	  			}
  	  			
  	  			docOfFuncPropsRetrieved.add(currDoc);
  	  		}
  	  		return docOfFuncPropsRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveDocOfFuncProperties Exception: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the 'ServiceType' objects specified by the 'whereClause' argument
	public ArrayList<ServiceType> retrieveServiceTypes(Connection dbConnection, String whereClause){
		ArrayList<ServiceType> serviceTypesRetrieved = new ArrayList<ServiceType>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM servicetypes " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String name = rs.getString(2);
  	  			String profile = rs.getString(3);
  	  			String style = rs.getString(4);
  	  			String specification = rs.getString(5);
  	  			
  	  			ServiceType currType = new ServiceType (name, style, specification);
  	  			currType.setDBKey(rs.getInt(6));
  	  			currType.setProfile(profile);
  	  			
  	  			//to reconstruct the 'ServiceType' we must retrieve its 'ServiceInterfaces' too
  	  			String myWhereClause = new String("where SI_ST_ID = " + id);
  	  			ArrayList<ServiceInterface> myServiceInterfaces = retrieveInterfaces(dbConnection, myWhereClause);
  	  			for (int i = 0; i < myServiceInterfaces.size(); i++) {
  	  				currType.add(myServiceInterfaces.get(i));
  	  			}
  	  			
  	  			serviceTypesRetrieved.add(currType);
  	  		}
  	  		return serviceTypesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveServiceTypes Exception: " + e.getMessage());
	    }
		return null;
	}

	//reconstructs and returns the 'ServiceInterface' objects specified by the 'whereClause' argument
	public ArrayList<ServiceInterface> retrieveInterfaces(Connection dbConnection, String whereClause){
		ArrayList<ServiceInterface> interfacesRetrieved = new ArrayList<ServiceInterface>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM serviceinterfaces " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String name = rs.getString(2);
  	  			String b = rs.getString(4);
  	  			
  	  			ServiceInterface currInterface = new ServiceInterface(name);
  	  			currInterface.setDBKey(rs.getInt(6));
  	  			currInterface.seteBehavioralSpecification(b);
  	  			
  	  			//to reconstruct the 'ServiceInterface' we must retrieve its 'ServiceInstances' too
  	  			String myWhereClause = new String("where SIS_SI_ID = " + id);
  	  			ArrayList<ServiceInstance> myServiceInstances = retrieveInstances(dbConnection, myWhereClause);
  	  			for (int i = 0; i < myServiceInstances.size(); i++){
  	  				currInterface.addServiceInstance(myServiceInstances.get(i));
  	  			}
  	  			
  	  			//to reconstruct the 'ServiceInterface' we must retrieve its 'Operations' too
  	  			myWhereClause = "where OP_SI_ID = " + id;
	  			ArrayList<Operation> myOperations = retrieveOperations(dbConnection, myWhereClause);
	  			for (int i = 0; i < myOperations.size(); i++){
	  				currInterface.add(myOperations.get(i));
	  			}
  	  			  	  			
  	  			interfacesRetrieved.add(currInterface);
  	  		}
  
  	  		return interfacesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveInterfaces Exception: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the 'ServiceInstance' objects specified by the 'whereClause' argument
	public ArrayList<ServiceInstance> retrieveInstances(Connection dbConnection, String whereClause){
		ArrayList<ServiceInstance> instancesRetrieved = new ArrayList<ServiceInstance>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM serviceinstances " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String name = rs.getString(2);
  	  			String protocol = rs.getString(3);
  	  			String uri = rs.getString(4);
  	  			
  	  			ServiceInstance currInstance = new ServiceInstance(name, protocol, uri);
  	  			currInstance.setDBKey(rs.getInt(7));
  	  			
  	  			//to reconstruct the 'ServiceInstance' we must retrieve its 'QualityProperties' and 'NonFuncDescriptions' too
  	  			String myWhereClause = new String("where QP_SIS_ID = " + id);
  	  			List<QualityProperties> myQualityProperties = retrieveQualityProperties(dbConnection, myWhereClause);
  	  			myWhereClause = "where NF_SIS_ID = " + id;
  	  			ArrayList<NonFuncDescription> myNonFuncDescriptions = retrieveNonFuncDescriptions(dbConnection, myWhereClause);
  	  			
  	  			for (int i = 0; i <  myNonFuncDescriptions.size(); i++){
  	  				currInstance.set(myNonFuncDescriptions.get(i), myQualityProperties.get(i));
  	  			}
  	  			
  	  			instancesRetrieved.add(currInstance);
   	  		}
  	 
  	  		return instancesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveInstances Exception: " + e.getMessage());
	    }
		return null;
	}
	
	//reconstructs and returns the 'Operation' objects specified by the 'whereClause' argument
	public ArrayList <Operation> retrieveOperations(Connection dbConnection, String whereClause){
		ArrayList <Operation> operationsRetrieved = new ArrayList <Operation>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM operations " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			String name = rs.getString(2);
  	  			int id = rs.getInt(1);
  	  			Message inMessage = null;
  	  			Message outMessage = null;
  	  			
  	  			//to reconstruct the 'Operation' we must retrieve its 'Messages' (IN and OUT) too
  	  			String myWhereClause = new String("where MS_OP_ID = " + id + " AND MS_TYPE = 'IN'");
  	  			ArrayList<Message> messagesRetrieved = retrieveMessages(dbConnection, myWhereClause);
  	  			if (messagesRetrieved != null){
  	  				inMessage = messagesRetrieved.get(0);
  	  			}
  	  			messagesRetrieved.clear();
  	  			
  	  			myWhereClause = new String("where MS_OP_ID = " + id + " AND MS_TYPE = 'OUT'");
	  			messagesRetrieved = retrieveMessages(dbConnection, myWhereClause);
	  			if (messagesRetrieved != null){
	  				outMessage = messagesRetrieved.get(0);
	  			}
  	  			  	  			
  	  			Operation currOperation = new Operation(name, inMessage, outMessage);
  	  			currOperation.setDBKey(rs.getInt(6));
  	  			operationsRetrieved.add(currOperation);	
  	  		}
  	  		
  	  		return operationsRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveOperations Exception: " + e.getMessage());
	    }	
		return null;
	}
	
	//reconstructs and returns the 'Message' objects specified by the 'whereClause' argument
	public ArrayList<Message> retrieveMessages(Connection dbConnection, String whereClause){
		ArrayList<Message> messagesRetrieved = new ArrayList<Message>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM messages " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String name = rs.getString(2);
  	  			String type = rs.getString(4);
  	  			
  	  			Message currMessage = new Message(name);
  	  			currMessage.setDBKey(rs.getInt(5));
  	  			currMessage.setInOrOut(type);
  	  			
  	  			//to reconstruct the 'Message' we must retrieve its 'MessageTypes' too  			
  	  			String myWhereClause = new String("where MT_MS_ID = " + id);
  	  			ArrayList<MessageType> myMessageTypes = retrieveMessageTypes(dbConnection, myWhereClause);
  	  			for (int i = 0 ; i < myMessageTypes.size() ; i++){
  	  				currMessage.add(myMessageTypes.get(i));
  	  			}
  	  			
  	  			messagesRetrieved.add(currMessage);
  	  		}
 
  	  		return messagesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveMessages Exception: " + e.getMessage());
	    }
		
		return null;
	}
	
	//reconstructs and returns the 'NonFuncDescription' objects specified by the 'whereClause' argument
	@SuppressWarnings("rawtypes")
	public ArrayList<NonFuncDescription> retrieveNonFuncDescriptions(Connection dbConnection, String whereClause){
		ArrayList<NonFuncDescription> nonFuncDescriptionsRetrieved = new ArrayList<NonFuncDescription>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM nonfuncdescriptions " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			int id = rs.getInt(1);
  	  			String ref2Source = rs.getString(2);
  	  			String ref2DocOfFuncProperties = rs.getString(3);
  	  			String serviceTypeName = rs.getString(4);
  	  			String serviceInstanceName = rs.getString(5);
  	  			
  	  			NonFuncDescription currNonFuncDescription = new NonFuncDescription(ref2Source, ref2DocOfFuncProperties, serviceTypeName, serviceInstanceName);
  	  			
  	  			//to reconstruct the 'NonFuncDescription' we must retrieve its 'PropertiesValues' too  
  	  			String myWhereClause = new String("where PV_NF_ID = " + id);
  	  			Hashtable<String, Double> myPropertiesValues = retrievePropertiesValues(dbConnection, myWhereClause);
  	  			Set set = myPropertiesValues.entrySet();
				Iterator it = set.iterator();
				while (it.hasNext()) {
					Map.Entry entry = (Map.Entry) it.next();
					currNonFuncDescription.addPropertyAndValue(entry.getKey().toString(), (Double)(entry.getValue()));
				}
 
  	  			nonFuncDescriptionsRetrieved.add(currNonFuncDescription);
  	  		}
  	  		
  	  		return (nonFuncDescriptionsRetrieved);
		}
		
		catch(Exception e) {
	          System.err.println("retrieveNonFuncDescriptions Exception: " + e.getMessage());
	    }
		return null;	
	}
	
	
	//reconstructs and returns the 'MessageType' objects specified by the 'whereClause' argument
	public ArrayList<MessageType> retrieveMessageTypes(Connection dbConnection, String whereClause){
		ArrayList<MessageType> messageTypesRetrieved = new ArrayList<MessageType>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM messagetypes " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		while (rs.next()) {
  	  			String cName = rs.getString(2);
  	  			String cType = rs.getString(5);
  	  			String typeKind = rs.getString(4);
  	  			
  	  			MessageType currMessageType = new MessageType(cName, cType, typeKind);
  	  			currMessageType.setDBKey(rs.getInt(6));
  	  			messageTypesRetrieved.add(currMessageType);
  	  		}
  	  		return messageTypesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveMessageTypes Exception: " + e.getMessage());
	    }
		return null;
	}
	
	
	//TODO: reconstructs and returns the 'LeafComponents' objects specified by the 'whereClause' argument
	public ArrayList<Component> retrieveLeafComponents(Connection dbConnection, String whereClause){
		ArrayList<Component> componentsRetrieved = new ArrayList<Component>();
		
		try {
			selectStatement = dbConnection.createStatement();
	 	  	String upd = new String ("SELECT * FROM componentsandstructures " + whereClause);
	  	  	ResultSet rs = selectStatement.executeQuery(upd);
	  	  	while (rs.next()) {
	  	  		Map<String, String> tagAttributes = new Hashtable<String, String>();
	  	  		String[] schemaPath = null;
	  	  		String[] classPath = null;
	  	  		String concatenatedSchemaPath = rs.getString(7);
	  	  		String concatenatedClassPath = rs.getString(8);
	  	  		boolean cIsGlobal = rs.getBoolean(5);
	  	  		
	  	  		String tagAttributesString = rs.getString(6);
	  	  		if (!tagAttributesString.equals("")){
	  	  			String[] tmp = tagAttributesString.split(",");
	  	  			for (int i = 0; i < tmp.length; i++){
	  	  				String[] tmp2 = tmp[i].split("-");
	  	  				String name = tmp2[0];
	  	  				String attr = tmp2[1];
	  	  				tagAttributes.put(name, attr);
	  	  			}
	  	  		}
	  	  		
	  	  		if (!concatenatedSchemaPath.equals("")){
	  	  			schemaPath = concatenatedSchemaPath.split("-");
	  	  		}
	  	  		
	  	  		if (!concatenatedClassPath.equals("")){
	  	  			classPath = concatenatedClassPath.split("-");
	  	  		}
	  	  		//more things to do to reconstruct component
	  	  	}
	  		return componentsRetrieved;
		}
		catch(Exception e) {
			System.err.println("retrieveLeafComponents Exception: " + e.getMessage());
	    }
		return null;
	}
	
	
	//TODO: reconstructs and returns the 'LeafComponents' objects specified by the 'whereClause' argument
	public ArrayList<Structure> retrieveLeafStructures(Connection dbConnection, String whereClause){
		ArrayList<Structure> structuresRetrieved = new ArrayList<Structure>();
		
		try {
			selectStatement = dbConnection.createStatement();
	 	  	String upd = new String ("SELECT * FROM componentsandstructures " + whereClause);
	  	  	ResultSet rs = selectStatement.executeQuery(upd);
	  	  	while (rs.next()) {
	  	  		Map<String, String> tagAttributes = new Hashtable<String, String>();
	  	  		String[] schemaPath = null;
	  	  		String[] classPath = null;
	  	  		String concatenatedSchemaPath = rs.getString(7);
	  	  		String concatenatedClassPath = rs.getString(8);
	  	  		boolean cIsGlobal = rs.getBoolean(5);
	  	  		
	  	  		String tagAttributesString = rs.getString(6);
	  	  		if (!tagAttributesString.equals("")){
	  	  			String[] tmp = tagAttributesString.split(",");
	  	  			for (int i = 0; i < tmp.length; i++){
	  	  				String[] tmp2 = tmp[i].split("-");
	  	  				String name = tmp2[0];
	  	  				String attr = tmp2[1];
	  	  				tagAttributes.put(name, attr);
	  	  			}
	  	  		}
	  	  		
	  	  		if (!concatenatedSchemaPath.equals("")){
	  	  			schemaPath = concatenatedSchemaPath.split("-");
	  	  		}
	  	  		
	  	  		if (!concatenatedClassPath.equals("")){
	  	  			classPath = concatenatedClassPath.split("-");
	  	  		}
	  	  		
	  	  		for (int i = 0; i < classPath.length; i++){
	  	  			System.out.println(classPath[i]);
	  	  		}
	  	  		//more things to do to reconstruct component
	  	  	}
	 		return structuresRetrieved;
		}
		catch(Exception e) {
			System.err.println("retrieveLeafStructures Exception: " + e.getMessage());
	    }
		return null;
	}
		
		
	//reconstructs and returns the 'PropertiesValues' hashtable specified by the 'whereClause' argument
	public Hashtable<String, Double> retrievePropertiesValues(Connection dbConnection, String whereClause){
		Hashtable<String, Double> propertiesValuesRetrieved = new Hashtable<String, Double>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM propertiesvalues " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			String property = rs.getString(2);
  	  			double value = rs.getDouble(3);
  	  			propertiesValuesRetrieved.put(property, value);
  	  		}
  	  		return (propertiesValuesRetrieved);
		}
		catch(Exception e) {
	          System.err.println("retrievePropertiesValues Exception: " + e.getMessage());
	    }
		
		return null;
	}
	
	
	public ArrayList<QualityProperties> retrieveQualityProperties(Connection dbConnection, String whereClause){
		ArrayList<QualityProperties> qualityPropertiesRetrieved = new ArrayList<QualityProperties>();
		
		try {
			selectStatement = dbConnection.createStatement();
  	  		String upd = new String ("SELECT * FROM qualityproperties " + whereClause);
  	  		ResultSet rs = selectStatement.executeQuery(upd);
  	  		
  	  		while (rs.next()) {
  	  			int qpId = rs.getInt(1);
  	  			String property = rs.getString(2);
  	  			QualityProperties tmpQualityProperties = new QualityProperties();
  	  			ArrayList<String> properties = new ArrayList<String>();
  	  			ArrayList<ArrayList<String>> domains = new ArrayList<ArrayList<String>>();
  	  			
  	  			upd = "SELECT * FROM QPPropertiesNames where QPP_QP_ID = " + qpId + ";";
  	  			Statement selectStatement2 = dbConnection.createStatement();
  	  			ResultSet rs2 = selectStatement2.executeQuery(upd);
  	  			while (rs2.next()) {
  	  				
  	  				int pId = rs2.getInt(1);
  	  				String pName = rs2.getString(2);
  	  				properties.add(pName);
  	  				ArrayList<String> tmpDomains = new ArrayList<String>();
  	  				
  	  				upd = "SELECT * FROM QPRankings where QPD_QPP_ID = " + pId + ";";
  	  				Statement selectStatement3 = dbConnection.createStatement();
  	  				ResultSet rs3 = selectStatement3.executeQuery(upd);
  	  				while (rs3.next()) {
  	  					String dName = rs3.getString(2);
  	  					tmpDomains.add(dName);
  	  				}
  	  				domains.add(tmpDomains);
  	  				tmpQualityProperties.add(property, tmpDomains);
  	  				
  	  			}
  	  			qualityPropertiesRetrieved.add(tmpQualityProperties);
  	  		}
  	  		return qualityPropertiesRetrieved;
		}
		catch(Exception e) {
	          System.err.println("retrieveQualityProperties Exception: " + e.getMessage());
	    }
		
		return null;
	}
	
		
}