package org.ow2.choreos.serviceRegistration;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 */
public class ServiceCollection {


	//Variables.
	private Integer collectionID;

	private Map<String, ServiceSource> sources;
	private boolean hasSources;

	private Map<String, Boolean> hasTransformedSources;//The 'String' is the 'schemaRepresentationmodel'.

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public ServiceCollection( Integer cCollectionID ){

		hasSources = false;

		hasConstraintViolation = false;


		collectionID = cCollectionID;


		PrintMessages.print( PrintMessages.organizationPrint, hasConstraintViolation, this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void addSource( String ref2Source, Map<String, DocOfFuncProperties> docsOfFuncProperties ){

		if( sources == null ) sources = new Hashtable<String, ServiceSource>();

		sources.put( ref2Source, new ServiceSource( ref2Source, docsOfFuncProperties ) );

		hasSources = true;
	}

	void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedSources= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedSources ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasSources && ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedSources ) && ! hasConstraintViolation){

			for( Enumeration<String> e = ( (Hashtable<String, ServiceSource>) sources).keys(); e.hasMoreElements(); ){

				String ref2Source = e.nextElement();

				sources.get( ref2Source ).transform( targetModel );

				if( ! sources.get( ref2Source ).hasTransformedDocsOfFuncProperties( targetModel ) ) return;
			}

			if( hasTransformedSources == null ) hasTransformedSources = new Hashtable<String, Boolean>();

			hasTransformedSources.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedSources= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedSources ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	public void updateSource(){//TODO
		
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return ( getFinalConstraintViolation() != null ) ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		if( hasSources )

			for ( Enumeration<String> e = ( (Hashtable<String, ServiceSource>) sources).keys(); e.hasMoreElements(); ){

				ServiceSource serviceSource = sources.get( e.nextElement() );

				if( serviceSource.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, serviceSource );
			}


		return constraintViolation;
	}

	public int getCollectionID(){

		return collectionID;
	}

	public boolean isEmpty(){

		return sources == null || ( sources != null && sources.isEmpty() ) ? true : false;
	}

	public Map<String, ServiceSource> getSources(){

		return sources;
	}

	public ServiceSource lookUp( String ref2Source ){

		return sources != null ? sources.get(ref2Source) : null;
	}

	DocOfFuncProperties getDocOfFuncProperties( String ref2Source, String ref2DocOfFuncProperties ){

		DocOfFuncProperties docOfFuncProperties = null;

		if( sources == null ) return docOfFuncProperties;

		for( Enumeration<String> e = ((Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ){

			docOfFuncProperties = null;

			String k = e.nextElement();

			if( sources.get( k ).getRef2Source().equals( ref2Source ) ) docOfFuncProperties = sources.get( k ).getDocOfFuncProperties( ref2DocOfFuncProperties );

			if( docOfFuncProperties != null ) break;
		}

		return docOfFuncProperties;
	}

	ServiceInterface getInterface( ServiceInstance serviceInstance ){

		if( sources == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			ServiceInterface i = sources.get(k).getInterface( serviceInstance );

			if( i != null ) return i;
        }

		return null;
	}

	String getRef2SourceOfInterface( ServiceInterface i ){

		if( sources == null ) return null;

		String ref2Source = null;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			ref2Source = sources.get(k).getRef2SourceOfInterface( i );

			if( ref2Source != null ) return ref2Source;
        }

		return null;
	}

	String getRef2SourceOfServiceInstance( ServiceInstance serviceInstance ){

		if( sources == null ) return null;

		String ref2Source = null;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			ref2Source = sources.get(k).getRef2SourceOfServiceInstance( serviceInstance );

			if( ref2Source != null ) return ref2Source;
        }

		return null;
	}

	boolean hasContents(){

		if(sources.size() > 0) return true;

		return false;
	}

	int getNumOfServiceTypes(){

		if( sources == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ){

			String i = e.nextElement();

        	num += sources.get(i).getNumOfServiceTypes();
        }

		return num;
	}

	int getNumOfInterfaces(){

		if( sources == null ) return 0;

		int num = 0;

		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();){

			String i = e.nextElement();

        	num += sources.get(i).getNumOfInterfaces();
        }

		return num;
	}

	int getNumOfInstances(){

		if( sources == null ) return 0;

		int num = 0;

		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();){

			String i = e.nextElement();

        	num += sources.get(i).getNumOfInstances();
        }

		return num;
	}

	int getNumOfInstancesWithNonFunctionalDescription( String ref2Source ){

		if( sources == null ) return 0;

		int num = 0;

		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();){

			String i = e.nextElement();

        	num += sources.get(i).getNumOfInstancesWithNonFunctionalDescription( ref2Source );
        }

		return num;
	}

	//Methods for parsing.
	public boolean isParsed(){

		if(sources == null) return false;

		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();)

			if( ! sources.get(e.nextElement()).isParsed() ) return false;

		return true;
	}

	int getNumOfParsedFuncDocs(){

		if(sources == null) return 0;

		int num = 0;
		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();) num += sources.get(e.nextElement()).getNumOfParsedFuncDocs();

		return num;
	}

	int getNumOfParsableFuncDocs(){

		if( sources == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ) num += sources.get( e.nextElement() ).getNumOfParsableFuncDocs();


		return num;
	}

	List<ServiceInterface> getAllInterfaces(){

		List<ServiceInterface> interfaces = null;


		if( hasSources ){

			for( Enumeration<String> e = ( (Hashtable<String, ServiceSource>) sources).keys(); e.hasMoreElements(); ){

				if( interfaces == null ) interfaces = new ArrayList<ServiceInterface>();

				List<ServiceInterface> tmpInterfaces = sources.get( e.nextElement() ).getAllInterfaces();


				if( tmpInterfaces != null ) interfaces.addAll( tmpInterfaces );
			}
		}


		return interfaces;
	}

	public String getTag(){

		return "service collection '" + collectionID + "'";
	}

	public boolean hasTransformedSources( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedSources );
	}


	//Visualization.
	DefaultMutableTreeNode visualize(){//Visualize the content of each source of the current collection. Note that I do not visualize the content of the documents!

		DefaultMutableTreeNode root;

		if( isEmpty() ) root = new DefaultMutableTreeNode( "You have not retrieved documents from any source yet!" );

		else{

			Integer label = collectionID + 1;

			root = new DefaultMutableTreeNode( "Collection " + label );

			DefaultMutableTreeNode node = new DefaultMutableTreeNode( "Sources" );

			root.add( node );

			if( sources == null ) return node;

			for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); ){

				DefaultMutableTreeNode child1 = sources.get( e.nextElement() ).visualize();

				node.add( child1 );
			}
		}

    	return root;
    }

	DefaultMutableTreeNode visualize( String ref2Source ){

		ServiceSource source = this.lookUp( ref2Source );

        if( source != null )

        	return source.visualize();

    	return null;
    }

	DefaultMutableTreeNode visualizeDetails(){//Visualize the content of each source of the current collection. Note that I vizualize the content of the documents!

		Integer label = collectionID + 1;

		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Collection " + label );

		DefaultMutableTreeNode node = new DefaultMutableTreeNode("Sources");
		root.add( node );

		if(sources == null) return node;

		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();){
			String i = e.nextElement();

        	DefaultMutableTreeNode child1 = sources.get(i).visualizeDetails();
        	node.add(child1);
        }

    	return root;
    }

	DefaultMutableTreeNode visualize( IntHolder labelOfInstances, ServiceInterface i, DefaultMutableTreeNode node ){
		if( sources == null ) return node;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); )

			node = sources.get( e.nextElement() ).visualize( labelOfInstances, i, node );

    	return node;
    }

	DefaultMutableTreeNode visualize( Integer labelOfInstances, ServiceInstance serviceInstance, DefaultMutableTreeNode node ){//This function is dedicated for vizualize non-functional hierarchy!
		if( sources == null ) return node;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements(); )

			node = sources.get( e.nextElement() ).visualize( labelOfInstances, serviceInstance, node );

    	return node;
    }

	DefaultMutableTreeNode visualize( DefaultMutableTreeNode root ){

		int label = collectionID + 1;

		DefaultMutableTreeNode node1 = new DefaultMutableTreeNode("Collection " + label);
		root.add(node1);

		DefaultMutableTreeNode node2 = new DefaultMutableTreeNode("Sources");
		node1.add(node2);

		if(sources == null) return root;

		for(Enumeration<String> e = ( (Hashtable<String,ServiceSource>) sources).keys(); e.hasMoreElements();){
			String i = e.nextElement();

        	DefaultMutableTreeNode child1 = sources.get(i).visualize();
        	node2.add(child1);
        }

    	return root;
    }
}
