package org.ow2.choreos.serviceRegistration;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.pluginManager.Objects;
import org.ow2.choreos.pluginManager.PluginManager;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 */
public class ServiceRegistration {


	//Variables.
	private Map<Integer, ServiceCollection> collections;
	private boolean hasCollections;

	private Integer currentCollectionID = 0;

	private Map<String, Boolean> hasTransformedCollections;//The 'String' is the 'schemaRepresentationmodel'.

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public ServiceRegistration(){

		hasCollections = true;

		hasConstraintViolation = false;

		
		//pilios - start
		//dathanas: The commented code is activated only locally (not in the svn server copy) because database server is not installed by all developers and the code crashes.
		//Otherwise, add a check for the existence of a database server in order to omit the execution of this code.
				
		/*try {
			ResultSet rs = MySQLTransaction.executeQuery("", "SELECT COUNT(*) COUNT FROM ServiceCollections");

			while(rs.next())
				currentCollectionID = rs.getInt("COUNT");
		} catch(QueryEngineFacadeException e){
			System.out.println(e.getMessage());
		} catch(SQLException e){
			System.out.println(e.getMessage());
		}*/
				
		//pilios - end
		


		collections = new Hashtable<Integer, ServiceCollection>();

		ServiceCollection currentCollection = new ServiceCollection( currentCollectionID );

		collections.put( currentCollectionID, currentCollection );


		PrintMessages.print( PrintMessages.registrationPrint, hasConstraintViolation(), this );
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		//It did not find any checkConstraint method.


		if( hasConstraintViolation ) return constraintViolation;


		if( hasCollections )

			for ( Enumeration<Integer> e = ( (Hashtable<Integer, ServiceCollection>) collections).keys(); e.hasMoreElements(); ){

				ServiceCollection collection = collections.get( e.nextElement() );

				if( collection.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, collection );
			}


		return constraintViolation;
	}

	public Map<Integer, ServiceCollection> getCollections(){

		return collections;
	}

	public Integer getCurrentCollectionID(){

		return currentCollectionID;
	}

	public ServiceCollection getCurrentCollection(){

		return lookUp( currentCollectionID );
	}

	public DocOfFuncProperties getDocOfFuncProperties( String ref2Source, String ref2DocOfFuncProperties ){

		DocOfFuncProperties docOfFuncProperties = null;


		if( collections == null ) return docOfFuncProperties;

		for( Enumeration<Integer> e = ( (Hashtable<Integer,ServiceCollection>) collections).keys(); e.hasMoreElements(); ){

			docOfFuncProperties = collections.get( e.nextElement() ).getDocOfFuncProperties( ref2Source, ref2DocOfFuncProperties );

			if( docOfFuncProperties != null ) break;
		}


		return docOfFuncProperties;
	}

	public boolean AreAllCollectionsEmpty(){

		for( Enumeration<Integer> e = ( (Hashtable<Integer,ServiceCollection>) collections).keys(); e.hasMoreElements(); )

			if( !collections.get( e.nextElement() ).isEmpty() ) return false;

		return true;
	}

	public boolean isCurrentCollectionEmpty(){

		if( getCurrentCollection().isEmpty() ) return true;

		return false;
	}

	public boolean isParsed(){

		if(collections.isEmpty()) return false;

		for(Enumeration<Integer> e = ( (Hashtable<Integer,ServiceCollection>) collections).keys(); e.hasMoreElements();)

			if(!collections.get(e.nextElement()).isParsed()) return false;

		return true;
	}

	public String getRef2SourceOfInterface( Integer collectionID, ServiceInterface i ){

		ServiceCollection collection = lookUp( collectionID );

		if( collection == null ) return null;

		return collection.getRef2SourceOfInterface( i );
	}

	public String getRef2SourceOfServiceInstance( Integer collectionID, ServiceInstance serviceInstance ){

		ServiceCollection collection = lookUp( collectionID );

		if( collection == null ) return null;

		return collection.getRef2SourceOfServiceInstance( serviceInstance );
	}

	public ServiceInterface getInterface( Integer collectionID, ServiceInstance serviceIstance ){

		ServiceCollection serviceCollection = lookUp(collectionID);

		if( serviceCollection == null ) return null;

		return serviceCollection.getInterface( serviceIstance );
	}

	public List<ServiceInterface> getAllInterfaces( Integer collectionID ){

		ServiceCollection serviceCollection = lookUp( collectionID );

		if( serviceCollection == null ) return null;


		return serviceCollection.getAllInterfaces();
	}

	public Boolean existNonFuncDescriptions( Integer collectionID ){

		ServiceCollection serviceCollection = lookUp( collectionID );

		if( serviceCollection == null ) return null;

		for(Enumeration<String> e1 = ( (Hashtable<String, ServiceSource>) serviceCollection.getSources()).keys(); e1.hasMoreElements();){

			String i = e1.nextElement();

			if(serviceCollection.getSources().get(i).getDocsOfFuncProperties() != null){

				for(Enumeration<String> e2 = ( (Hashtable<String,DocOfFuncProperties>) serviceCollection.getSources().get(i).getDocsOfFuncProperties()).keys(); e2.hasMoreElements();){

					String j = e2.nextElement();

					if( serviceCollection.getSources().get(i).getDocsOfFuncProperties().get(j).existNonFuncDescriptions() ) return true;
				}
			}
		}

		return false;
	}

	public int getCurrentCollectionNumOfParsedFuncDocs(){

		return getCurrentCollection().getNumOfParsedFuncDocs();
	}

	public int getCurrentCollectionNumOfParsableFuncDocs(){

		return getCurrentCollection().getNumOfParsableFuncDocs();
	}

	public int getCurrentCollectionNumOfServiceTypes(){

		return getCurrentCollection().getNumOfServiceTypes();
	}

	public int getCurrentCollectionNumOfInterfaces(){

		return getCurrentCollection().getNumOfInterfaces();
	}

	public int getNumOfInterfaces( int collectionID ){

		ServiceCollection serviceCollection = lookUp( collectionID );

		return serviceCollection.getNumOfInterfaces();
	}

	public int getNumOfInstances( int collectionID ){

		ServiceCollection serviceCollection = lookUp( collectionID );

		return serviceCollection.getNumOfInstances();
	}

	public int getCurrentCollectionNumOfInstances(){

		return getCurrentCollection().getNumOfInstances();
	}

	public int getNumOfInstancesWithNonFunctionalDescription( String ref2Source ){

		return getCurrentCollection().getNumOfInstancesWithNonFunctionalDescription( ref2Source );
	}

	public String getTag(){

		return "service registration";
	}

	public boolean hasTransformedCollections( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedCollections );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedCollections= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedCollections ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasCollections && ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedCollections ) && ! hasConstraintViolation ){

			for( Enumeration<Integer> e = ( (Hashtable<Integer, ServiceCollection>) collections).keys(); e.hasMoreElements(); ){

				Integer collectionID = e.nextElement();


				collections.get( collectionID ).transform( targetModel );


				if( ! collections.get( collectionID ).hasTransformedSources( targetModel ) ) return;
			}

			if( hasTransformedCollections == null ) hasTransformedCollections = new Hashtable<String, Boolean>();

			hasTransformedCollections.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedCollections= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedCollections ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	public Integer createNewCollection(){
		
		//currentCollectionID++;

		//pilios - start
		//dathanas: The commented code is activated only locally (not in the svn server copy) because database server is not installed by all developers and the code crashes.
		//Otherwise, add a check for the existence of a database server in order to omit the execution of this code.

		/*try {
			ResultSet rs = MySQLTransaction.executeQuery("", "SELECT COUNT(*) COUNT FROM ServiceCollections");

			while(rs.next())
				currentCollectionID = rs.getInt("COUNT");
		} catch(QueryEngineFacadeException e){
			System.out.println(e.getMessage());
		} catch(SQLException e){
			System.out.println(e.getMessage());
		}*/

		//pilios - end

		collections.put( currentCollectionID, new ServiceCollection( currentCollectionID ) );


		return currentCollectionID;
	}

	public int getNumOfDocsOfFuncProperties( String type, String ref2Source ){

		return PluginManager.getNumOfDocuments( type, ref2Source );
	}

	public int register( String type, String ref2Source ){

		int numOfDocsOfFuncProperties = 0;


		Objects objects = PluginManager.retrieve( type, ref2Source );

		if( objects != null ){

			ServiceCollection currentCollection = getCurrentCollection();

			currentCollection.addSource( ref2Source, objects.getDocsOfFuncProperties() );

			if( objects.getDocsOfFuncProperties() != null ) numOfDocsOfFuncProperties = objects.getDocsOfFuncProperties().size();
		}


		return numOfDocsOfFuncProperties;
	}

	public ServiceCollection lookUp( Integer collectionID ){

		return collections.get( collectionID );
	}

	public boolean hasContents( Integer collectionID ){

		return lookUp( collectionID ).hasContents();
	}

	//Methods for Update a collection.
	public void updateCollection(){//TODO
		
	}

	//Visualize the content of each source of the current collection. Note that I do not vizualize the content of the documents!
	public DefaultMutableTreeNode visualizeCurrentCollection(){

		ServiceCollection currentCollection = getCurrentCollection();

		if( currentCollection == null ) return null;

		return currentCollection.visualize();
	}

	//Visualize the content of each source of the input collection. Note that I do not vizualize the content of the documents!
	public DefaultMutableTreeNode visualize( Integer collectionID ){

		ServiceCollection currentCollection = lookUp(collectionID);

		if( currentCollection == null ) return null;

		return currentCollection.visualize();
	}

	//Visualize the content of each source of the current collection. Note that I vizualize the content of the documents!
	public DefaultMutableTreeNode visualizeDetails( Integer collectionID ){

		ServiceCollection currentCollection = lookUp( collectionID );

		if( currentCollection == null ) return null;

		return currentCollection.visualizeDetails();
	}

	//Visualize all the collections.
	public DefaultMutableTreeNode visualize(){

		DefaultMutableTreeNode root;

		if( AreAllCollectionsEmpty() ) root = new DefaultMutableTreeNode("You have not stored any collection yet or the created collections are empty !");

		else{

			root = new DefaultMutableTreeNode("Service Collections");

			for(Enumeration<Integer> e = ((Hashtable<Integer,ServiceCollection>) collections).keys(); e.hasMoreElements();) root = collections.get(e.nextElement()).visualize( root );
		}

		return root;
	}

	public DefaultMutableTreeNode visualize( IntHolder labelOfInstances, Integer collectionID, ServiceInterface i, DefaultMutableTreeNode node ){

		ServiceCollection collection = lookUp( collectionID );

		if( collection == null ) return node;

		return collection.visualize( labelOfInstances, i, node );
	}

	public DefaultMutableTreeNode visualize( Integer collectionID, Integer labelOfInstances, ServiceInstance serviceInstance, DefaultMutableTreeNode node ){//This function is dedicated for vizualize non-functional hierarchy!

		ServiceCollection collection = lookUp( collectionID );

		if( collection == null ) return node;

		return collection.visualize( labelOfInstances, serviceInstance, node );
	}
}
