package org.ow2.choreos.serviceRegistration;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.FunctionalServiceRepresentationController;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 */
public class ServiceSource {


	//Variables.
	private String ref2Source;

	private Map<String, DocOfFuncProperties> docsOfFuncProperties;
	private boolean hasDocsOfFuncProperties;

	private List<Object> docsOfSchemas;
	private boolean hasDocsOfSchemas;

	private Map<String, Boolean> hasTransformedDocsOfFuncProperties;//The 'String' is the 'schema representation model'.

	private int parsedFuncDocs;
	private int parsableFuncDocs;

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public ServiceSource( String cRef2Source, Map<String, DocOfFuncProperties> cDocsOfFuncProperties ){

		ref2Source = cRef2Source;

		parsedFuncDocs = 0;
		parsableFuncDocs = 0;

		hasDocsOfSchemas = false;

		hasConstraintViolation = false;


		if( cDocsOfFuncProperties != null ){

			docsOfFuncProperties = cDocsOfFuncProperties;

			hasDocsOfFuncProperties = true;
	
			for( Enumeration<String> e = ( (Hashtable<String, DocOfFuncProperties>) cDocsOfFuncProperties).keys(); e.hasMoreElements(); ){

				++parsedFuncDocs;

				if( ! cDocsOfFuncProperties.get( e.nextElement() ).hasConstraintViolation() ) ++parsableFuncDocs;
			}
		}

		else hasDocsOfFuncProperties = false;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}


		return constraintViolation;
	}

	protected void addDocOfSchema( Object docOfSchema ){

		if( ObjectConstraintsManipulation.setContainmentError( this, docOfSchema ) == null ){

			if( ! hasDocsOfSchemas ){

				docsOfSchemas = new ArrayList<Object>();

				hasDocsOfSchemas = true;
			}

			docsOfSchemas.add( docOfSchema );
		}
	}

	public void addDocOfFuncProperties( DocOfFuncProperties docOfFuncProperties ){//It is achieved though the constructor.

	}

	public void updateDocOfFuncProperties( DocOfFuncProperties docOfFuncProperties ){//TODO

	}

	public Map<String, DocOfFuncProperties> getDocsOfFuncProperties(){

		return docsOfFuncProperties;
	}

	public DocOfFuncProperties lookUpDocOfFuncProperties( String ref2DocOfFuncProperties ){

		return docsOfFuncProperties != null ? docsOfFuncProperties.get( ref2DocOfFuncProperties ) : null;
	}

	void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedDocsOfFuncProperties= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasDocsOfFuncProperties && ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties ) && ! hasConstraintViolation ){

			for( Enumeration<String> e = ( (Hashtable<String, DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

				String ref2DocOfFuncProperties = e.nextElement();

				docsOfFuncProperties.get( ref2DocOfFuncProperties ).transform( targetModel );

				if( ! docsOfFuncProperties.get( ref2DocOfFuncProperties ).hasTransformedServiceTypes( targetModel ) ) return;
			}

			if( hasTransformedDocsOfFuncProperties == null ) hasTransformedDocsOfFuncProperties = new Hashtable<String, Boolean>();

			hasTransformedDocsOfFuncProperties.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedDocsOfFuncProperties= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties ) + " hasConstraintViolation= " + hasConstraintViolation );
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		//It did not find any checkConstraint method.


		if( hasConstraintViolation ) return constraintViolation;


		if( hasDocsOfFuncProperties )

			for ( Enumeration<String> e = ( (Hashtable<String, DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

				DocOfFuncProperties docOfFuncProperties = docsOfFuncProperties.get( e.nextElement() );

				if( docOfFuncProperties.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, docOfFuncProperties );
			}


		return constraintViolation;
	}

	public List<ServiceInterface> getAllInterfaces(){

		List<ServiceInterface> interfaces = null;


		if( hasDocsOfFuncProperties ){

			for( Enumeration<String> e = ( (Hashtable<String, DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

				if( interfaces == null ) interfaces = new ArrayList<ServiceInterface>();

				List<ServiceInterface> tmpInterfaces = docsOfFuncProperties.get( e.nextElement() ).getAllInterfaces();


				if( tmpInterfaces != null ) interfaces.addAll( tmpInterfaces );
			}
		}


		return interfaces;
	}

	public String getTag(){

		return "source '" + getName() + "'";
	}

	public String getName(){

		String name = null;

		int pos = ref2Source.lastIndexOf("\\");

		if( pos > 0 ) name = ref2Source.substring( pos + 1 );


		return name;
	}

	public boolean hasTransformedDocsOfFuncProperties( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties );
	}

	public String getRef2Source(){

		return ref2Source;
	}

	boolean isParsed(){

		return docsOfFuncProperties != null && docsOfFuncProperties.size() == parsedFuncDocs ? true : false;
	}

	DocOfFuncProperties getDocOfFuncProperties( String ref2DocOfFuncProperties ){

		DocOfFuncProperties docOfFuncProperties = null;

		if( docsOfFuncProperties == null ) return docOfFuncProperties;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			if( docsOfFuncProperties.get( k ).getRef2DocOfFuncProperties().equals( ref2DocOfFuncProperties ) ) return docsOfFuncProperties.get( k );
		}


		return docOfFuncProperties;
	}

	public int getNumOfParsedFuncDocs(){

		return parsedFuncDocs;
	}

	public int getNumOfParsableFuncDocs(){

		return parsableFuncDocs;
	}

	int getNumOfServiceTypes(){

		if( docsOfFuncProperties == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); )

			num += docsOfFuncProperties.get( e.nextElement() ).getNumOfServiceTypes();

		return num;
	}

	int getNumOfInterfaces(){

		if( docsOfFuncProperties == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); )

			num += docsOfFuncProperties.get( e.nextElement() ).getNumOfInterfaces();

		return num;
	}

	int getNumOfInstances(){

		if( docsOfFuncProperties == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); )

			num += docsOfFuncProperties.get( e.nextElement() ).getNumOfInstances();

		return num;
	}

	int getNumOfInstancesWithNonFunctionalDescription( String ref2Source ){

		if( docsOfFuncProperties == null ) return 0;

		int num = 0;

		if( ! this.ref2Source.equals( ref2Source ) )

			return num;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); )

			num += docsOfFuncProperties.get( e.nextElement() ).getNumOfInstancesWithNonFunctionalDescription( ref2Source );

		return num;
	}

	String getRef2SourceOfInterface( ServiceInterface i ){

		if( docsOfFuncProperties == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			if( docsOfFuncProperties.get(k).getRef2SourceOfInterface( ref2Source, i ) != null ) return ref2Source;
        }

		return null;
	}

	ServiceInterface getInterface( ServiceInstance serviceInstance ){

		if( docsOfFuncProperties == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			ServiceInterface i = docsOfFuncProperties.get(k).getInterface( serviceInstance );

			if( i != null ) return i;
        }

		return null;
	}

	String getRef2SourceOfServiceInstance( ServiceInstance serviceInstance ){

		if( docsOfFuncProperties == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			if( docsOfFuncProperties.get( k ).getRef2SourceOfServiceInstance( ref2Source, serviceInstance ) != null ) return ref2Source;
        }

		return null;
	}


	//Visualization.
	DefaultMutableTreeNode visualize(){//Visualize the content of each source of the current collection. Note that I do not vizualize the content of the documents!

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( ref2Source );


        if( docsOfFuncProperties != null )

        	for( Enumeration<String> e = ( (Hashtable<String, DocOfFuncProperties> ) docsOfFuncProperties ).keys(); e.hasMoreElements(); )

               	node.add( new DefaultMutableTreeNode( FunctionalServiceRepresentationController.getRef2DocOfFunProperties( docsOfFuncProperties.get( e.nextElement() ) ) ) );


        for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ) node.add( new DefaultMutableTreeNode( SchemaRepresentationController.getRef2DocOfSchemas( docsOfSchemas.get( i ) ) ) );


    	return node;
    }

	DefaultMutableTreeNode visualizeDetails(){//Visualize the content of each source of the current collection. Note that I vizualize the content of the documents!

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( ref2Source );


        if( docsOfFuncProperties != null ){

        	for (Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements();){

        		DefaultMutableTreeNode child = docsOfFuncProperties.get( e.nextElement() ).visualize( ref2Source );

               	node.add(child);
        	}
        }


    	return node;
    }

	DefaultMutableTreeNode visualize( IntHolder labelOfInstances, ServiceInterface i, DefaultMutableTreeNode node ){

		if( docsOfFuncProperties == null ) return node;


        for (Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements();) node = docsOfFuncProperties.get( e.nextElement() ).visualize( labelOfInstances, ref2Source, i, node );


    	return node;
    }

	DefaultMutableTreeNode visualize( Integer labelOfInstances, ServiceInstance serviceInstance, DefaultMutableTreeNode node ){//This function is dedicated for vizualize non-functional hierarchy!

		if( docsOfFuncProperties == null ) return node;


        for( Enumeration<String> e = ( (Hashtable<String,DocOfFuncProperties>) docsOfFuncProperties).keys(); e.hasMoreElements();) node = docsOfFuncProperties.get( e.nextElement() ).visualize( labelOfInstances, ref2Source, serviceInstance, node );


    	return node;
    }
}
