package org.ow2.choreos.serviceRepresentation.functional;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.Schema;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;


/**
 * 
 * @author dathanas
 *
 */
public class DocOfFuncProperties {


	//Variables.
	private String ref2DocOfFuncProperties;

	private String primarySchemaRepresentationModel;

	private String docTargetNamespace;
	private boolean hasDocTargetNamespace;

	private String schemaDefinitionNamespace;

	private Map<String, ServiceType> serviceTypes;
	private boolean hasServiceTypes;

	private Map<String, Boolean> hasTransformedServiceTypes;//The 'String' is the 'schemaRepresentationmodel'.

	private boolean hasUnconnectedServiceTypes;

	private List<Schema> schemas;
	private boolean hasSchemas;

	private boolean hasUnmaterializedComponents;//Components in each one of the schemas that are not connected with their references types.

	private Map<String, String> qualifierNamespaces;
	private boolean hasQualifierNamespaces;

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public DocOfFuncProperties( String cRef2DocOfFuncProperties ){

		ref2DocOfFuncProperties = cRef2DocOfFuncProperties;

		hasSchemas = false;
		hasServiceTypes = false;
		hasUnconnectedServiceTypes = false;
		hasUnmaterializedComponents = false;
		hasConstraintViolation = false;
		hasQualifierNamespaces = false;
		hasDocTargetNamespace = false;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void set( Map<String, String> namespaceQualifiers ){

		if( ! hasQualifierNamespaces ){//System.out.println( "Doc" );

			if( namespaceQualifiers != null && ! namespaceQualifiers.isEmpty() ){

				qualifierNamespaces = new Hashtable<String, String>();

				for ( Enumeration<String> e = ( (Hashtable<String, String>) namespaceQualifiers ).keys(); e.hasMoreElements(); ){
					
					String namespace = e.nextElement();

					String qualifier = namespaceQualifiers.get( namespace ); //System.out.println( "global qualifier= " + qualifier + " - namespace= " + namespace );

					qualifierNamespaces.put( qualifier, namespace );
				}

				hasQualifierNamespaces = true;
			}
		}

		else ObjectConstraintsManipulation.setErrorAttribute( this, "namespace qualifiers", "Its namespace qualifiers have already been set" );
	}

	public void set( String cDocTargetNamespace ){

		if( ! hasDocTargetNamespace ){

			if( cDocTargetNamespace != null && ! cDocTargetNamespace.isEmpty() ){

				docTargetNamespace = cDocTargetNamespace;//System.out.println( "docTargetNamespace= " + docTargetNamespace );

				hasDocTargetNamespace = true;
			}
		}

		else ObjectConstraintsManipulation.setErrorAttribute( this, "targetNamespace", "It has already been set" );
	}

	protected void add( Schema schema ){

		if( ObjectConstraintsManipulation.setContainmentError( this, schema ) == null ){

			if( ! hasSchemas ){

				schemas = new ArrayList<Schema>();

				hasSchemas = true;

				hasUnmaterializedComponents = true;

				primarySchemaRepresentationModel = schema.getSchemaRepresentationModel();

				schemaDefinitionNamespace = schema.getSchemaDefinitionNamespace();

				schemas.add( schema );
			}

			else if( hasSchemas && schema.getSchemaRepresentationModel().equals( primarySchemaRepresentationModel ) ){

				hasUnmaterializedComponents = true;

				schemas.add( schema );
			}

			else if( hasSchemas && ! schema.getSchemaRepresentationModel().equals( primarySchemaRepresentationModel ) ) ObjectConstraintsManipulation.setConnectionError ( this, schema, "Its schema does NOT have the SAME schema representation model" );
		}
	}

	public void add( ServiceType serviceType ){

		if( ObjectConstraintsManipulation.setContainmentError( this, serviceType ) == null ){

			if( ! hasServiceTypes ){

				serviceTypes = new Hashtable<String, ServiceType>();

				hasServiceTypes = true;
			}

			serviceTypes.put( serviceType.getName(), serviceType );

			hasUnconnectedServiceTypes = true;
		}
	}

	public void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedServiceTypes= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedServiceTypes ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasServiceTypes && ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedServiceTypes ) && ! hasConstraintViolation  ){

			for ( Enumeration<String> i = ( (Hashtable<String, ServiceType>) serviceTypes ).keys(); i.hasMoreElements(); ){

				String serviceType = i.nextElement();


				serviceTypes.get( serviceType ).transform( targetModel );


				if( ! serviceTypes.get( serviceType ).hasTransformedInterfaces( targetModel ) ) return;
			}

			if( hasTransformedServiceTypes == null ) hasTransformedServiceTypes = new Hashtable<String, Boolean>();

			hasTransformedServiceTypes.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedServiceTypes= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedServiceTypes ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	public ServiceType lookUp( String serviceTypeName ){

		return hasServiceTypes ? serviceTypes.get( serviceTypeName ) : null;
	}

	public void updateXMLSchema( Schema xmlSchema ){//TODO

	}

	public void updateServiceType( ServiceType serviceType ){//TODO

	}

	private Map<String, String> makeIndexQualifierNamespaces( Map<String, String> index ){

		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) index = schemas.get( i ).makeIndexQualifierNamespaces( index );


		if( hasQualifierNamespaces )

			for ( Enumeration<String> e = ( (Hashtable<String, String>) qualifierNamespaces ).keys(); e.hasMoreElements(); ){
				
				String qualifier = e.nextElement();

				String namespace = qualifierNamespaces.get( qualifier );

				index = add( index, qualifier, namespace );
			}


		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		MapPrinting.stdPrintStringIndex( printKinds, index, "makeIndexQualifierNamespaces" );


		return index;
	}

	private Map<String, String> makeIndexNamespaceQualifiers( Map<String, String> index ){

		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) index = schemas.get( i ).makeIndexNamespaceQualifiers( index );


		if( hasQualifierNamespaces )

			for ( Enumeration<String> e = ( (Hashtable<String, String>) qualifierNamespaces ).keys(); e.hasMoreElements(); ){
				
				String qualifier = e.nextElement();

				String namespace = qualifierNamespaces.get( qualifier );

				index = add( index, namespace, qualifier );
			}


		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		MapPrinting.stdPrintStringIndex( printKinds, index, "makeIndexNamespaceQualifiers" );


		return index;
	}

	private Map<String, String> add( Map<String, String> index, String key, String value ){

		if( index == null ) index = new Hashtable<String, String>();

		if( key != null && ! index.containsKey( key ) ) index.put( key, value );


		return index;
	}

	private Map<String, Object> makeIndexGlobalComponents( Map<String, Object> indexComponents, String indexKind ){

		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) indexComponents = schemas.get( i ).makeIndexGlobalComponents( indexComponents, indexKind );


		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		MapPrinting.stdPrintObjectsIndex( printKinds, indexComponents, indexKind );


		return indexComponents;
	}

	public void set( NonFuncDescription nonFuncDescription, QualityProperties qualityProperties ){

		if( nonFuncDescription.getServiceTypeName() == null ) addInAllServiceTypes( nonFuncDescription, qualityProperties );

		else{

			ServiceType serviceType = lookUp( nonFuncDescription.getServiceTypeName() );

			if( serviceType != null ) serviceType.set( nonFuncDescription, qualityProperties ); 
		}
	}

	private void addInAllServiceTypes( NonFuncDescription nonFuncDescription, QualityProperties qualityProperties ){

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes ).keys(); e.hasMoreElements();) serviceTypes.get( e.nextElement() ).set( nonFuncDescription, qualityProperties ); 
	}

	public void set( String behavioralSpecification, String ref2DocOfFuncProperties ){//Behavioral specification.

		if( behavioralSpecification != null && ref2DocOfFuncProperties != null && ref2DocOfFuncProperties.equals( getName() ) ) for( Enumeration<String> e = ( (Hashtable<String, ServiceType>) serviceTypes).keys(); e.hasMoreElements();) serviceTypes.get( e.nextElement() ).set( behavioralSpecification );
	}

	public void connect(){

		Map<String, String> qualifierNamespaces = null, namespaceQualifiers = null;

		Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces = null;

		Map<String, Object> indexGlobalComponentsWithTargetNamespaces = null;


		PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "START - connectXMLWithXML: " + getTag() + " - hasSchemas= " + hasSchemas + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents );

		if( hasSchemas && hasUnmaterializedComponents ) {

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** START - Make Indices **" );

			if( qualifierNamespaces == null || ( qualifierNamespaces != null && qualifierNamespaces.isEmpty() ) ) qualifierNamespaces = makeIndexQualifierNamespaces( qualifierNamespaces );

			if( indexGlobalComponentsWithoutTargetNamespaces == null || ( indexGlobalComponentsWithoutTargetNamespaces != null && indexGlobalComponentsWithoutTargetNamespaces.isEmpty() ) ) indexGlobalComponentsWithoutTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithoutTargetNamespaces, "makeIndexGlobalComponentsWithoutTargetNamespaces" );

			if( indexGlobalComponentsWithTargetNamespaces == null || ( indexGlobalComponentsWithTargetNamespaces != null && indexGlobalComponentsWithTargetNamespaces.isEmpty() ) ) indexGlobalComponentsWithTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithTargetNamespaces, "makeIndexGlobalComponentsWithTargetNamespaces" );

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** END - Make Indices **" );


			int countSize = 0;

			for( int i = 0; i < schemas.size(); ++i ){
	
				Schema schema = schemas.get( i );
	
	
				PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** START connection XML with XML in schema= " + schema.getTag() + "**" );
	
				schema.connect( qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces );
	
	
				if( ! schema.hasUnmaterializedComponents() ) countSize++;
	
				PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** END connection XML with XML in schema= " + schema.getTag() + "**" );
	    	}

			if( countSize == schemas.size() ) hasUnmaterializedComponents = false;
		}

		PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "END - connectXMLWithXML: " + getTag() + " - hasSchemas= " + hasSchemas + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents );


		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "START - connectWSDLWithXML: " + getTag() + " - hasSchemas= " + hasSchemas + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents );

		if( hasServiceTypes && hasUnconnectedServiceTypes ) {

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** START - Make Indices **" );

			if( qualifierNamespaces == null || ( qualifierNamespaces != null && qualifierNamespaces.isEmpty() ) ) qualifierNamespaces = makeIndexQualifierNamespaces( qualifierNamespaces );

			if( namespaceQualifiers == null || ( namespaceQualifiers != null && namespaceQualifiers.isEmpty() ) ) namespaceQualifiers = makeIndexNamespaceQualifiers( namespaceQualifiers );

			if( indexGlobalComponentsWithoutTargetNamespaces == null || ( indexGlobalComponentsWithoutTargetNamespaces != null && indexGlobalComponentsWithoutTargetNamespaces.isEmpty() ) ) indexGlobalComponentsWithoutTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithoutTargetNamespaces, "makeIndexGlobalComponentsWithoutTargetNamespaces" );

			if( indexGlobalComponentsWithTargetNamespaces == null || ( indexGlobalComponentsWithTargetNamespaces != null && indexGlobalComponentsWithTargetNamespaces.isEmpty() ) ) indexGlobalComponentsWithTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithTargetNamespaces, "makeIndexGlobalComponentsWithTargetNamespaces" );

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** END - Make Indices **" );


			int countSize = 0;

    		for ( Enumeration<String> e = ( (Hashtable<String, ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ){

    			ServiceType serviceType = serviceTypes.get( e.nextElement() );


    			PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "** START Connection in serviceType= " + serviceType.getTag() + "**" );

    			serviceType.connect( namespaceQualifiers, docTargetNamespace, qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, primarySchemaRepresentationModel, schemaDefinitionNamespace );


    			if( ! serviceType.hasUnconnectedInterfaces() ) countSize++;

    			PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "** END Connection in serviceType= " + serviceType.getTag() + "**" );


    			transform( XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation() );
    		}
  
    		if( countSize == serviceTypes.size() ) hasUnconnectedServiceTypes = false;
		}

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "END - connectWSDLWithXML: " + getTag() + " - hasSchemas= " + hasSchemas + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents );
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) if( schemas.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, schemas.get( i ) );


		if( hasServiceTypes )

			for ( Enumeration<String> e = ( (Hashtable<String, ServiceType>) serviceTypes ).keys(); e.hasMoreElements(); ){

    			ServiceType serviceType = serviceTypes.get( e.nextElement() );

    			if( serviceType.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, serviceType );
    		}


		return constraintViolation;
	}

	public boolean hasTransformedServiceTypes( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedServiceTypes );
	}

	public List<ServiceInterface> getAllInterfaces(){

		List<ServiceInterface> interfaces = null;


		if( hasServiceTypes ){

			for( Enumeration<String> e = ( (Hashtable<String, ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ){

				if( interfaces == null ) interfaces = new ArrayList<ServiceInterface>();

				List<ServiceInterface> tmpInterfaces = serviceTypes.get( e.nextElement() ).getInterfaces();


				if( tmpInterfaces != null ) interfaces.addAll( tmpInterfaces );
			}
		}


		return interfaces;
	}

	public boolean hasSchemas(){

		return hasSchemas;
	}

	public List<Schema> getSchemas(){

		return schemas;
	}

	public String getRef2DocOfFuncProperties(){

		return ref2DocOfFuncProperties;
	}

	public String getName(){

		String name = new String( ref2DocOfFuncProperties );


		int pos = ref2DocOfFuncProperties.lastIndexOf("\\");

		if( pos > 0 ) name = ref2DocOfFuncProperties.substring( pos + 1 );


		pos = name.lastIndexOf( "." );

		if( pos > 0 ) name = name.substring( 0, pos );


		return name;
	}

	public int getNumOfServiceTypes(){

		if( serviceTypes == null ) return 0;

		return serviceTypes.size();
	}

	public int getNumOfInterfaces(){

		if( serviceTypes == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); )

			num += serviceTypes.get( e.nextElement() ).getNumOfInterfaces();

		return num;
	}

	public int getNumOfInstances(){

		if( serviceTypes == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); )

			num += serviceTypes.get( e.nextElement() ).getNumOfInstances();

		return num;
	}

	public int getNumOfInstancesWithNonFunctionalDescription( String ref2Source ){

		if( serviceTypes == null ) return 0;

		int num = 0;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); )

			num += serviceTypes.get( e.nextElement() ).getNumOfInstancesWithNonFunctionalDescription( ref2Source );

		return num;
	}

	public String getRef2SourceOfInterface( String ref2Source, ServiceInterface i ){

		if( serviceTypes == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			if( serviceTypes.get(k).getRef2SourceOfInterface( ref2Source, i ) != null ) return ref2Source;
        }

		return null;
	}

	public ServiceInterface getInterface( ServiceInstance serviceInstance ){

		if( serviceTypes == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			ServiceInterface i = serviceTypes.get(k).getInterface( serviceInstance );

			if( i != null ) return i;
        }

		return null;
	}

	public String getRef2SourceOfServiceInstance( String ref2Source, ServiceInstance serviceInstance ){

		if( serviceTypes == null ) return null;

		for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ){

			String k = e.nextElement();

			if( serviceTypes.get(k).getRef2SourceOfServiceInstance( ref2Source, serviceInstance ) != null ) return ref2Source;
        }

		return null;
	}

	public Map<String, ServiceType> getServiceTypes(){

		return serviceTypes;
	}

	public String getDocTargetNamespace(){

		return docTargetNamespace;
	}

	public boolean hasDocTargetNamespace(){

		return hasDocTargetNamespace;
	}

	public String getTag(){

		return "service document name= '" + getName() + "'";
	}


	//Visualization.
	public DefaultMutableTreeNode visualize( String ref2Source, String endOfPath ){//This function is used by the 'Visualizer' class.

        DefaultMutableTreeNode docOfFuncPropertiesNode = new DefaultMutableTreeNode( ref2DocOfFuncProperties );

		if( endOfPath != null && endOfPath.equals( "Schema" ) ) for( int i = 0; hasSchemas && i < schemas.size(); ++i ) docOfFuncPropertiesNode.add( schemas.get( i ).visualize() );


		else if( endOfPath != null && ( endOfPath.equals( "Service Types" ) || endOfPath.equals( "Service Instances" ) || endOfPath.equals( "Service Interfaces" ) || endOfPath.equals( "Service Operations" ) || endOfPath.equals( "Service Messages" ) ) ){

			if( hasServiceTypes ){

    			int serviceTypeCounter = 0;

    			for ( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); )

    				docOfFuncPropertiesNode.add( serviceTypes.get( e.nextElement() ).visualize( ++serviceTypeCounter, ref2Source, endOfPath ) );
    		}
		}


		return docOfFuncPropertiesNode;
    }

	public DefaultMutableTreeNode visualize( String ref2Source ){//This function is used by the 'Visualizer' class.

		if( hasConstraintViolation() ) return null;


        DefaultMutableTreeNode node = new DefaultMutableTreeNode( ref2DocOfFuncProperties );

        for( int i = 0; hasSchemas && i < schemas.size(); ++i ) node.add( schemas.get( i ).visualize() );


		if( hasServiceTypes ){

    		int serviceTypeCounter = 0;

    		for ( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ) node.add( serviceTypes.get( e.nextElement() ).visualize( ++serviceTypeCounter, ref2Source ) );
		}

		return node;
    }

	public DefaultMutableTreeNode visualize(String endOfPath, String ref2Source, Boolean structured){

        DefaultMutableTreeNode node = new DefaultMutableTreeNode(ref2DocOfFuncProperties);

    	if( serviceTypes != null ){

    		int label = 0;

			for( Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements();){

				label++;

				node.add(serviceTypes.get(e.nextElement()).visualize(endOfPath, label, ref2Source, structured));
			}
		}

		return node;
    }

	public DefaultMutableTreeNode visualize( IntHolder labelOfInstances, String ref2Source, ServiceInterface i, DefaultMutableTreeNode node ){

        if( serviceTypes == null ) return node;

		for (Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements();)

			node = serviceTypes.get( e.nextElement()).visualize( labelOfInstances, ref2Source, i, node );

		return node;
    }

	public DefaultMutableTreeNode visualize( Integer labelOfInstances, String ref2Source, ServiceInstance serviceInstance, DefaultMutableTreeNode node ){//This function is dedicated for vizualize non-functional hierarchy!

        if( serviceTypes == null ) return node;

		for (Enumeration<String> e = ( (Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements();)

			node = serviceTypes.get( e.nextElement()).visualize( labelOfInstances, ref2Source, serviceInstance, node );

		return node;
    }

	public Boolean existNonFuncDescriptions(){

		if( serviceTypes != null ){

			for( Enumeration<String> e = ((Hashtable<String,ServiceType>) serviceTypes).keys(); e.hasMoreElements(); ) if( serviceTypes.get(e.nextElement()).existNonFuncDescriptions() ) return true;
		}

		return false;
	}
}
