package org.ow2.choreos.serviceRepresentation.functional;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;


/**
 * 
 * @author dathanas
 *
 */
public class Message implements java.io.Serializable {


	//Variables.
	private static final long serialVersionUID = -3471791022210326785L;

	private String name;

	private List<MessageType> messageTypes;
	private boolean hasMessageTypes;

	private boolean hasUnconnectedMessageTypes;

	private Map<String, Boolean> hasTransformedMessageTypes;//The String is the 'schemaRepresentationModel'.

	private String constraintViolation;
	private boolean hasConstraintViolation;

	private int dbKey;

	private int dbAbstractionKey;

	private String inOrOut;

	private boolean concrete; //actual (true) or representative (false)

	
	//Constructor.
	public Message( String cName ){

		hasMessageTypes = false;

		hasUnconnectedMessageTypes = false;

		hasConstraintViolation = false;

		concrete = true;

		if( checkConstraint( cName ) == null ) name = cName;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}
	
	
	//Empty constructor used for xml transformation
	public Message(){
		
	}


	//Constraint checkers.
	private String checkConstraint( String value ){

		if( value == null || ( value != null && value.isEmpty() ) ) return ObjectConstraintsManipulation.setErrorAttribute( this, "name", "It cannot be anonymous" );


		return null;
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}
	
	public void setConcrete( boolean cIsConcrete ){

		concrete = cIsConcrete;
	}

	public void setInOrOut( String cType ){

		inOrOut = cType;
	}
	
	
	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}

	public void add( MessageType messageType ){

		if( ObjectConstraintsManipulation.setContainmentError( this, messageType ) == null ){

			if( ! hasMessageTypes ){

				messageTypes = new ArrayList<MessageType>();

				hasMessageTypes = true;
			}

			messageTypes.add( messageType );

			hasUnconnectedMessageTypes = true;
		}
	}

	void connect( Map<String, String> namespaceQualifiers, String docTargetNamespace, Map<String, String> qualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel, String schemaTargetNamespace ){

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "START - connect: " + getTag() + " - hasUnconnectedMessageTypes= " + hasUnconnectedMessageTypes + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaTargetNamespace );

		if( hasUnconnectedMessageTypes && ! hasConstraintViolation ){

			for( int i = 0; hasMessageTypes && i < messageTypes.size(); ++i ){


				messageTypes.get( i ).connect( namespaceQualifiers, docTargetNamespace, qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel, schemaTargetNamespace );

				if( messageTypes.get( i ).hasUnmaterializedReferencedComponent() ) return;
			}

			hasUnconnectedMessageTypes = false;
		}

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "END - connect: " + getTag() + " - hasUnconnectedMessageTypes= " + hasUnconnectedMessageTypes + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel  + " - schemaDefinitionNamespace= " + schemaTargetNamespace );
	}

	void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedMessageTypes= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessageTypes ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessageTypes ) && ! hasConstraintViolation ){

			for( int i = 0; hasMessageTypes && i < messageTypes.size(); ++i ){


				messageTypes.get( i ).transform( targetModel );


				if( ! messageTypes.get( i ).hasTransformedReferencedComponent( targetModel ) ) return;
			}

			if( hasTransformedMessageTypes == null ) hasTransformedMessageTypes = new Hashtable<String, Boolean>();

			if( targetModel != null ) hasTransformedMessageTypes.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedMessageTypes= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessageTypes ) + " hasConstraintViolation= " + hasConstraintViolation );
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; messageTypes != null && i < messageTypes.size(); ++i ) if( messageTypes.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, messageTypes.get( i ) );


		return constraintViolation;
	}

	boolean hasUnconnectedMessageTypes(){

		return hasUnconnectedMessageTypes;
	}

	boolean hasTransformedMessageTypes( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessageTypes );
	}

	public int getDBKey(){

		return dbKey;
	}
	
	public boolean isConcrete(){

		return concrete;
	}
	
	public int getDBAbstractionKey(){

		return dbAbstractionKey;
	}

	public String getInOrOut(){

		return inOrOut;
	}

	public String getName(){

		return name;
	}

	public boolean hasMessageTypes(){

		return hasMessageTypes;
	}

	public List<MessageType> getMessageTypes(){

		return messageTypes;
	}

	public String getTag(){

		return "message name= '" + name + "'";
	}


	//Visualization.
	DefaultMutableTreeNode visualize( String label ){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( label + " name= '" + name + "'" );


		for ( int i = 0; hasMessageTypes && i < messageTypes.size(); ++i ) node.add( messageTypes.get( i ).visualize() );


		return node;
	}
	
	DefaultMutableTreeNode visualizeForQueryResults2( String label ){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( label + " name= '" + name + "'" );


		for ( int i = 0; hasMessageTypes && i < messageTypes.size(); ++i ) node.add( messageTypes.get( i ).visualizeForQueryResults2() );


		return node;
	}
	
	public DefaultMutableTreeNode visualizeForQueryResults( String label ){

		if( hasConstraintViolation ) return null;


		DefaultMutableTreeNode node = new DefaultMutableTreeNode( label + " name= '" + name + "'" );


		for ( int i = 0; hasMessageTypes && i < messageTypes.size(); ++i ) node.add( messageTypes.get( i ).visualizeForQueryResults() );


		return node;
	}
	
	
	//Setters used for xml transformation
	
	public void setxName( String cName ){
		name = cName;
	}
		
	public void setxMessageTypes( List<MessageType> cMessageTypes ){
		messageTypes = cMessageTypes;
	}
		
	public void setxHasMessageTypes( boolean cHasMessageTypes ){
		hasMessageTypes = cHasMessageTypes;
	}
		
	public void setxHasUnconnectedMessageTypes( boolean cHasUnconnectedMessageTypes ){
		hasUnconnectedMessageTypes = cHasUnconnectedMessageTypes;
	}
		
	public void setxHasTransformedMessageTypes( Map<String, Boolean> cHasTransformedMessageTypes ){
		hasTransformedMessageTypes = cHasTransformedMessageTypes;
	}
		
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
		
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
		
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
		
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
	
	public void setxInOrOut( String cInOrOut ){
		inOrOut = cInOrOut;
	}
		
	public void setxConcrete( boolean cConcrete ){
		concrete = cConcrete;
	}
	
	//Gtters used for xml transformation
	
	public String getxName(){
		return name;
	}
		
	public List<MessageType> getxMessageTypes(){
		return messageTypes;
	}
		
	public boolean getxHasMessageTypes(){
		return hasMessageTypes;
	}
		
	public boolean getxHasUnconnectedMessageTypes(){
		return hasUnconnectedMessageTypes;
	}
		
	public Map<String, Boolean> getxHasTransformedMessageTypes(){
		return hasTransformedMessageTypes;
	}
		
	public String getxConstraintViolation(){
		return constraintViolation;
	}
		
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
		
	public int getxDBKey(){
		return dbKey;
	}
		
	public int getxDBAbstractionKey(){
		return dbAbstractionKey;
	}
	
	public String getxInOrOut(){
		return inOrOut;
	}
		
	public boolean getxConcrete(){
		return concrete;
	}
}
