package org.ow2.choreos.serviceRepresentation.functional;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;


/**
 * 
 * @author dathanas
 *
 */
public class ServiceType {


	//Variables.
	private String name;

	private List<ServiceInterface> interfaces;
	private boolean hasInterfaces;

	private boolean hasUnconnectedInterfaces;

	private Map<String, Boolean> hasTransformedInterfaces;//The String is the representation model.

	private String profile;//Optional: service profile.
	private String style;//Style: WS or Rest.
	private String specification;//Language specification.

	private String constraintViolation;
	private boolean hasConstraintViolation;

	private int dbKey;


	//Constructor.
	public ServiceType( String cName, String cStyle, String cSpecification ){

		name = cName;
		style = cStyle;
		specification = cSpecification;

		hasInterfaces = false;

		hasUnconnectedInterfaces = false;

		hasConstraintViolation = false;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}

	public void setProfile( String cProfile ){

		profile = cProfile;
	}

	public void add( ServiceInterface serviceInterface ){

		if( interfaces == null ){

			interfaces = new ArrayList<ServiceInterface>();

			hasInterfaces = true;
		}

		serviceInterface.setServiceTypeName( name );

		interfaces.add( serviceInterface );

		hasUnconnectedInterfaces = true;
	}

	public ServiceInterface exist( String name ){

		for( int i = 0; hasInterfaces && i < interfaces.size(); ++i ) if( interfaces.get(i).getName().equals(name) ) return interfaces.get(i);

		return null;
	}

	void connect( Map<String, String> namespaceQualifiers, String docTargetNamespace, Map<String, String> qualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel, String schemaDefinitionNamespace ){

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "START - connect: " + getTag() + " - hasUnconnectedInterfaces= " + hasUnconnectedInterfaces + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );

		if( hasUnconnectedInterfaces && ! hasConstraintViolation ){

			for( int i = 0; hasInterfaces && i < interfaces.size(); ++i ){


				interfaces.get( i ).connect( namespaceQualifiers, docTargetNamespace, qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel, schemaDefinitionNamespace );

				if( interfaces.get( i ).hasUnconnectedOperations() ) return;
			}

			hasUnconnectedInterfaces = false;
		}

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "END - connect: " + getTag() + " - hasUnconnectedInterfaces= " + hasUnconnectedInterfaces + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );
	}

	void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedInterfaces= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedInterfaces ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedInterfaces ) && ! hasConstraintViolation ){

			for( int i = 0; hasInterfaces && i < interfaces.size(); ++i ){


				interfaces.get( i ).transform( targetModel );


				if( ! interfaces.get( i ).hasTransformedOperations( targetModel ) ) return;
			}

			if( hasTransformedInterfaces == null ) hasTransformedInterfaces = new Hashtable<String, Boolean>();

			hasTransformedInterfaces.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedInterfaces= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedInterfaces ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	void set( NonFuncDescription nonFuncDescription, QualityProperties qualityProperties ){

		for(int i = 0; interfaces != null && i < interfaces.size(); ++i) interfaces.get(i).set( nonFuncDescription, qualityProperties );
	}

	void set( String behavioralSpecification ){//Behavioral specification.

		for( int i = 0; behavioralSpecification != null && interfaces != null && i < interfaces.size(); ++i ) interfaces.get( i ).seteBehavioralSpecification( behavioralSpecification );
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; hasInterfaces && i < interfaces.size(); ++i ) if( interfaces.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, interfaces.get( i ) );


		return constraintViolation;
	}

	boolean hasUnconnectedInterfaces(){

		return hasUnconnectedInterfaces;
	}

	boolean hasTransformedInterfaces( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedInterfaces );
	}

	public int getDBKey(){

		return dbKey;
	}

	public String getProfile(){

		return profile;
	}

	public List<ServiceInterface> getInterfaces(){

		return interfaces;
	}

	public String getName(){

		return name;
	}

	public String getStyle(){

		return style;
	}

	public String getSpecification(){

		return specification;
	}

	int getNumOfInterfaces(){

		if( interfaces == null ) return 0;

		return interfaces.size();
	}

	int getNumOfInstances(){

		if( interfaces == null ) return 0;

		int num = 0;

		for( int k = 0; interfaces != null && k < interfaces.size(); ++k )

			num += interfaces.get( k ).getNumOfInstances();

		return num;
	}

	int getNumOfInstancesWithNonFunctionalDescription( String ref2Source ){

		if( interfaces == null ) return 0;

		int num = 0;

		for( int k = 0; interfaces != null && k < interfaces.size(); ++k )

			num += interfaces.get( k ).getNumOfInstancesWithNonFunctionalDescription( ref2Source );

		return num;
	}

	String getRef2SourceOfInterface( String ref2Source, ServiceInterface i ){

		for( int k = 0; interfaces != null && k < interfaces.size(); ++k )

			if(interfaces.get(k).equals( i ) ) return ref2Source;

		return null;
	}

	ServiceInterface getInterface( ServiceInstance serviceInstance ){

		for(int k = 0; interfaces != null && k < interfaces.size(); ++k){

			ServiceInterface i = interfaces.get(k).getInterface( serviceInstance );

			if( i != null ) return i;
        }

		return null;
	}

	String getRef2SourceOfServiceInstance( String ref2Source, ServiceInstance serviceInstance ){

		for( int i = 0; interfaces != null && i < interfaces.size(); ++i )

			if( interfaces.get(i).getRef2SourceOfServiceInstance( ref2Source, serviceInstance ) != null ) return ref2Source;

		return null;
	}

	public String getTag(){

		return "service type name= '" + name + "'" + " - Specified in the " + specification + " version.";
	}


	//Visualization.
	DefaultMutableTreeNode visualize( Integer label, String ref2Source, String endOfPath ){//This function is used by the 'Visualizer' class!

		if( hasConstraintViolation() ) return null;


		DefaultMutableTreeNode node = new DefaultMutableTreeNode("Service type (" + label + "): " + name + " - Specified in the " + specification + " version.");

		if( endOfPath != null && ! endOfPath.equals("Service Types") ){

			for( int k = 0; interfaces != null && k < interfaces.size(); k++ ){

				label = k + 1;

				DefaultMutableTreeNode tmp = interfaces.get( k ).visualize( label, ref2Source, endOfPath );

				if( tmp != null ) node.add( tmp );
			}
		}

        return node;
    }

	DefaultMutableTreeNode visualize( Integer label, String ref2Source ){//This function is used by the 'Visualizer' class!

		if( hasConstraintViolation() ) return null;


		DefaultMutableTreeNode node = new DefaultMutableTreeNode( "Service type (" + label + "): " + name + " - Specified in the " + specification + " version." );

		for( int k = 0; interfaces != null && k < interfaces.size(); k++ ){

			label = k + 1;

			DefaultMutableTreeNode tmp = interfaces.get( k ).visualize( label, ref2Source );

			if( tmp != null ) node.add( tmp );
		}


        return node;
    }

	DefaultMutableTreeNode visualize( String endOfPath, Integer label, String ref2Source, Boolean structured ){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode("Service type (" + label + "): " + name + " - Specified in the " + specification + " version.");

		for(int k = 0; interfaces != null && k < interfaces.size(); k++){

			label = k + 1;

			//node = interfaces.get( k ).visualize( label, ref2Source, node );
		}

        return node;
    }

	DefaultMutableTreeNode visualize( IntHolder labelOfInstances, String ref2Source, ServiceInterface i, DefaultMutableTreeNode node ){

		if( interfaces == null ) return node;

		for( int k = 0; k < interfaces.size(); k++ )

			if( interfaces.get(k).equals(i) )

				node = interfaces.get(k).visualize( labelOfInstances, ref2Source, node );

        return node;
    }

	DefaultMutableTreeNode visualize( Integer labelOfInstances, String ref2Source, ServiceInstance serviceInstance, DefaultMutableTreeNode node ){//This function is dedicated for vizualize non-functional hierarchy!

		for( int k = 0; interfaces != null && k < interfaces.size(); k++ )

			node = interfaces.get(k).visualize( labelOfInstances, serviceInstance, ref2Source, node );

        return node;
    }

	Boolean existNonFuncDescriptions(){

		for( int i = 0; interfaces != null && i < interfaces.size(); ++i )

			if( interfaces.get(i).existNonFuncDescriptions() ) return true;

		return false;
	}
}
