/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package org.ow2.choreos.executable_service_discovery.DummyActiveRegistry;

import java.net.Socket;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class Registry {

    private static final Registry INSTANCE = new Registry();
    private long MILLIS;
    private String PATH;
    private int PORT;
    private ConnectionAcceptThread acceptThread;
    private ScheduledThreadPoolExecutor dirScanExecutor = new ScheduledThreadPoolExecutor(1);
    
    private LinkedList<RegistryEntry> currentServices;
    private LinkedList<RegistryEntry> registeredServices;
    private ConcurrentLinkedQueue<Socket> newConnectionQ;
    private LinkedList<Socket> connectionList;
    
    private Registry() {
        /**/
    }
    
    public void start(long millis, String path, int port) {
        MILLIS = millis;
        PATH = path;
        PORT = port;
        
        currentServices = new LinkedList<RegistryEntry>();
        registeredServices = new LinkedList<RegistryEntry>();
        connectionList = new LinkedList<Socket>();
        newConnectionQ = new ConcurrentLinkedQueue<Socket>();
        
        DirectoryScanThread scanThread = new DirectoryScanThread(path, millis);
        dirScanExecutor.scheduleWithFixedDelay(scanThread, 0, millis, TimeUnit.MILLISECONDS);
        
        acceptThread = new ConnectionAcceptThread(port);
        acceptThread.start();
    }
    public void stop() {
        dirScanExecutor.shutdown();
        acceptThread.kill();
    }
    public static Registry getInstance() {
        return INSTANCE;
    }
    public ConnectionAcceptThread getConnectionAcceptThread() {
        return acceptThread;
    }

    public LinkedList<Socket> getConnectionList() {
        return connectionList;
    }

    public LinkedList<RegistryEntry> getCurrentServices() {
        return currentServices;
    }

    public ConcurrentLinkedQueue<Socket> getNewConnectionQ() {
        return newConnectionQ;
    }

    public LinkedList<RegistryEntry> getRegisteredServices() {
        return registeredServices;
    }
    
}
