/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package org.ow2.choreos.executable_service_discovery.plugin_manager;

import java.util.List;

/**
 * Interface for plugin management.
 * Currently, plugins are loaded using reflection but this could change in
 * future versions. Alternatively, we could use the ServiceLoader class (Java6)
 * or even the OSGi Framework.
 * 
 */
public interface ManagePlugins {
    
    /**
     * Reads the plugin.properties configuration file and loads all
     * classes that are marked as 'enabled'. 
     */
    public void addPluginsFromConf();
    
    /**
     * Adds plugin type to the plugin manager. 
     * 
     * @param pluginUID the fully qualified name of the specific plugin type
     * @return true if plugin type was added successfully, false otherwise
     */
    public boolean addPluginType(String pluginUID);
    
    /**
     * @return list of all available plugin types (only those
     * marked as enabled in the plugin.properties file)
     */
    public List<String> getAvailablePluginTypes();
    
    /**
     * @return list of plugin types that are currently loaded
     * by the Plugin Manager.
     */
    public List<String> getLoadedPluginTypes();
    
    
    /**
     * Removes plugin type. Not supported yet.
     */
    public void removePluginType(String pluginUID);
    
    /**
     * Removes all plugin types. Not supported yet.
     */
    public void removeAllPluginTypes();
    
    /**
     * Creates new plugin instance of type 'pluginUID' for a specific
     * registry.
     * 
     * @param pluginUID the fully qualified name of the specific plugin type
     * @param registryURI the registry URI
     * @return true if the instance was successfully created, false otherwise
     */
    public boolean createPluginInstance(String pluginUID, String registryURI);
    
    /**
     * Destroys specified plugin instance.
     * 
     * @param pluginUID the fully qualified name of the specific plugin type
     * @param registryURI the registry URI
     */
    public void destroyPluginInstance(String pluginUID, String registryURI);
    
    /**
     * @return list of registry URI's for created plugin instances of the
     * specific type.
     */
    public List<String> getPluginInstancesOfType(String pluginUID);
}
