package org.ow2.choreos.distributions;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Arrays;

public class CoverageProbability {
	/**
	 * The method is supposed to access Mathematica to which it passes a command
	 * to compute the value of the coverage probability.
	 * 
	 * @param command
	 * @return
	 */

	public static double getValue(String command) {
		double result;
		// File dirFile = new File(Constants.ROOTPATH + "local/mathematica");
		// File dirFile = new File("/Users/shachem/");

		String scriptDirPath = System
				.getProperty("org.ow2.choreos.scriptDirPath");
		if (scriptDirPath == null) {
			scriptDirPath = "./";
			System.err
					.println("No input provided for mathematica script location, using default path ./");
		} else
			System.out.println("using path " + scriptDirPath
					+ " for mathematica script location");

		File dirFile = new File(scriptDirPath);
		String mathematicaAppDirPath = System
				.getProperty("org.ow2.choreos.mathematicaAppDirPath");
		if (mathematicaAppDirPath == null) {
			System.err
					.println("No path provided for mathematica, the application will stop");
			System.exit(0);
		} else
			System.out.println("using path " + mathematicaAppDirPath
					+ " for mathematica application location");

		double response = 0;
		String distToSend;

		ArrayList<String> commandList = new ArrayList<String>();

		commandList.add(mathematicaAppDirPath + "MathKernel");
		// commandList
		// .add("/Applications/Mathematica.app/Contents/MacOS/MathKernel");
		// commandList.add(Constants.ROOTPATH + "local/mathematica/MathKernel");
		//
		commandList.add("-script");
		// this is the file where the script is saved
		commandList.add("integralLists2905.m");
		commandList.addAll(Arrays.asList(command.split(" ")));
		command.split(" ");
		// System.out.println(commandList.get(3));
		String num = commandList.get(3).replace(".0", "");
		int length = Integer.parseInt(num);
		String dist = commandList.get(8 + 3 * length);
		// System.out.println(" the command is " + command);
		try {
			// this checks if the distribution of the initial locations of
			// sensors is already defined in our distributions
			// class
			distToSend = Distribution.checkAndSubDistribution(dist);
			commandList.set(8 + 3 * length, distToSend);

			ProcessBuilder pb = new ProcessBuilder(commandList);

			pb.directory(dirFile);
			try {
				Process shell = pb.start();

				InputStream shellIn = shell.getInputStream();

				String responseString = convertStreamToStr(shellIn);
				if (responseString.contains("^")) {
					String stringToParse = responseString.substring(
							responseString.indexOf("^") + 1,
							responseString.length());
					String stringToKeep = responseString.substring(0,
							responseString.indexOf("^") - 1);
					Double firstPart = Double.parseDouble(stringToKeep);

					Double power = Math.pow(10,
							Double.parseDouble(stringToParse));
					response = firstPart * power;
				} else
					if (responseString.contains("0.99")){
						response = 0.99;
					}
					else
					response = Double.parseDouble(responseString);
				System.out.println(response);
				shellIn.close();

			} catch (IOException e) {
				System.out
						.println("Error occured while executing command. Error Description: "
								+ e.getMessage());
			}
		} catch (NullPointerException e) {
			System.out.println("distribution doesn't exist");
		}

		return response;

	}

	/**
	 * this method converts the stream returned from mathematica to a string
	 * 
	 * @param is
	 *            : the stream to convert
	 * @return a the stream converted into a string
	 * @throws IOException
	 */
	public static String convertStreamToStr(InputStream is) throws IOException {
		if (is != null) {
			Writer writer = new StringWriter();
			char[] buffer = new char[1024];
			try {
				Reader reader = new BufferedReader(new InputStreamReader(is,
						"UTF-8"));
				int n;
				while ((n = reader.read(buffer)) != -1) {
					writer.write(buffer, 0, n);
				}
			} finally {
				is.close();
			}
			return writer.toString();
		} else {
			return "";
		}
	}
}
