package test.java;

import static org.junit.Assert.*;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.chereos.iots.common.BorderPoints;
import org.ow2.chereos.iots.common.Constants;
import org.ow2.chereos.iots.common.Location;
import org.ow2.chereos.iots.common.Point;
import org.ow2.chereos.iots.common.ServiceDescription;
import org.ow2.choreos.registry.RegistryMap;
import org.ow2.choreos.statistics.StatUtil;

public class TestRegistryMap {
	static Set<ServiceDescription> regSet;
	String[] concepts = new String[2];
	ServiceDescription s;
	String relativeWebPath = "./sensorsAndActuators.rdf";

	@Before
	public void setUp() throws Exception {
		RegistryMap.add("www.test.com/service_addressabcd", "abcd",
				"getWindchill", "windchill_service", 1, "windchill_sensor", 1,
				"Paris", "windchill_factor", Constants.DOUBLE, 1, 1, 12345666L,
				"KcalPermPers", "");

		concepts[0] = "temperature";
		concepts[1] = "windchill_factor";
		s = new ServiceDescription("12", "www.test.com/service_address12",
				"getTemperature", "temperature_service", 1, "Paris", 1,
				"thermometer", "temperature", Constants.DOUBLE, 1, 1,
				"DegreeC", "");

	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testAdd() throws SQLException {

		assertTrue("service was not added",
				RegistryMap.add("www.test.com/service_address12", "12",
						"getTemperature", "temperature_service", 1,
						"thermometer", 1, "Paris", "temperature",
						Constants.DOUBLE, 1, 1, 12345678L, "DegreeC", ""));
	}

	@Test
	public void testRemove() {
		assertTrue("service was not deleted",
				RegistryMap.remove("www.test.com/service_address12"));
		assertFalse(
				"service seems to have been deleted although it doesnt exist",
				RegistryMap.remove("www.test.com/service_address134"));
	}

//	@SuppressWarnings("deprecation")
//	@Test
//	public void testGetNumberOfNodes() throws SQLException {
//
//		assertTrue("the number of services in not correct",
//				(double) RegistryMap.getNumberOfNodes("1", "Paris", concepts)
//						.get("windchill_factor") == 1.0);
//		assertTrue("the number of services in not correct",
//				(double) RegistryMap.getNumberOfNodes("1", "Paris", concepts)
//						.get("temperature") == 0.0);
//
//	}
//
//	@Test
//	public void testFindDevice() {
//
//		assertTrue(
//				"did not return device abcd",
//				RegistryMap.findDevice("Paris", concepts).iterator().next()
//						.getAddress().toString()
//						.equals("www.test.com/service_addressabcd"));
//
//	}

	@Test
	public void testUpdateRegisteredService() throws SQLException {
		RegistryMap.add("www.test.com/service_address12", "12",
				"getTemperature", "temperature_service", 1, "thermometer", 1,
				"Paris", "temperature", Constants.DOUBLE, 1, 1, 12345678L,
				"DegreeC", "");

		assertTrue("Could not update the expiry date",
				RegistryMap.updateRegisteredService(
						"www.test.com/service_address12", 12366678L));
	}

	@Test
	public void testRemoveAllServices() throws SQLException {
		RegistryMap.empty();
		assertTrue("Could not delete all services",
				RegistryMap.regSet.isEmpty());
	}

	@Test
	public void testFindSubSetOfDevices() throws SQLException {
		String globalRegion = "Paris";
		String[] concepts = { "noiselevel" };
		String distribution = "normal";
		Long t = (long) 1347626255055.0;
		double dMaxVal = 30;
		int subsetSizeVal = 4;
		Point locOfInterest = new Point(0, 0);
		RegistryMap.add("service1", "1", "getNoise", "noiselevel_service", 1,
				"noiselevel_sensor", 1, "Paris", "noiselevel",
				Constants.DOUBLENOISE, 1, 1, t + 50000, "db", "1,1," + (t - 5)
						+ ";3,3," + (t + 5));

		RegistryMap.add("service2", "2", "getNoise", "noiselevel_service", 1,
				"noiselevel_sensor", 1, "Paris", "noiselevel",
				Constants.DOUBLENOISE, 1, 1, t + 50000, "db", "4,5," + (t - 5)
						+ ";6,7," + (t + 5));
		RegistryMap.add("service3", "3", "getNoise", "noiselevel_service", 1,
				"noiselevel_sensor", 1, "Paris", "noiselevel",
				Constants.DOUBLENOISE, 1, 1, t + 50000, "db", "9,9," + (t - 5)
						+ ";11,11," + (t + 5));
		RegistryMap.add("service4", "4", "getNoise", "noiselevel_service", 1,
				"noiselevel_sensor", 1, "Paris", "noiselevel",
				Constants.DOUBLENOISE, 1, 1, t + 50000, "db", "14,14,"
						+ (t - 5) + ";16,16," + (t + 5));
		Set<ServiceDescription> selectedSubSet = RegistryMap
				.findSubSetOfDevice(globalRegion, concepts, distribution,
						dMaxVal, subsetSizeVal, locOfInterest, t);

		assertTrue("number of selected services should be 4",
				selectedSubSet.size() == 4);

	}

	@Test
	public void testFindUniformSubSetOfDevices() throws SQLException {
		String globalRegion = "Paris";
		String[] concepts = { "temperature" };
		String distribution = "uniform";
		int subsetSizeVal = 25;
		Long t = (long) 1347626255055.0;
		RegistryMap.add("service1", "1", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,1," + (t - 5) + ";1.2,1.2," + (t + 5));
		RegistryMap.add("service2", "2", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,3," + (t - 5) + ";1.2,3.2," + (t + 5));
		RegistryMap.add("service3", "3", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,5," + (t - 5) + ";1.2,5.2," + (t + 5));
		RegistryMap.add("service4", "4", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,7," + (t - 5) + ";1,7.2," + (t + 5));
		RegistryMap.add("service5", "5", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,9," + (t - 5) + ";1,9.2," + (t + 5));
		RegistryMap.add("service6", "6", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,1," + (t - 5) + ";3,1.2," + (t + 5));
		RegistryMap.add("service7", "7", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,3," + (t - 5) + ";3,3.2," + (t + 5));
		RegistryMap.add("service8", "8", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,5," + (t - 5) + ";3,5.2," + (t + 5));
		RegistryMap.add("service9", "9", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,7," + (t - 5) + ";3,7.2," + (t + 5));
		RegistryMap.add("service10", "10", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,9," + (t - 5) + ";3,9.2," + (t + 5));

		RegistryMap.add("service11", "11", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,1," + (t - 5) + ";5,1.2," + (t + 5));
		RegistryMap.add("service12", "12", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,3," + (t - 5) + ";5,3.2," + (t + 5));
		RegistryMap.add("service13", "13", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,5," + (t - 5) + ";5,5.2," + (t + 5));
		RegistryMap.add("service14", "14", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,7," + (t - 5) + ";5,7.2," + (t + 5));
		RegistryMap.add("service15", "15", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,9," + (t - 5) + ";5,9.2," + (t + 5));

		RegistryMap.add("service16", "16", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,1," + (t - 5) + ";7,1.2," + (t + 5));
		RegistryMap.add("service17", "17", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,3," + (t - 5) + ";7,3.2," + (t + 5));
		RegistryMap.add("service18", "18", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,5," + (t - 5) + ";7,5.2," + (t + 5));
		RegistryMap.add("service19", "19", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,7," + (t - 5) + ";7,7.2," + (t + 5));
		RegistryMap.add("service20", "20", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,9," + (t - 5) + ";7,9.2," + (t + 5));

		RegistryMap.add("service21", "21", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,1," + (t - 5) + ";9,1.2," + (t + 5));
		RegistryMap.add("service22", "22", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,3," + (t - 5) + ";9,3.2," + (t + 5));
		RegistryMap.add("service23", "23", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,5," + (t - 5) + ";9,5.2," + (t + 5));
		RegistryMap.add("service24", "24", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,7," + (t - 5) + ";9,7.2," + (t + 5));
		RegistryMap.add("service25", "25", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,9," + (t - 5) + ";9,9.2," + (t + 5));

		RegistryMap.add("service26", "26", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,1.2," + (t - 5) + ";1,1.3," + (t + 5));
		RegistryMap.add("service27", "27", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,3.2," + (t - 5) + ";1,3.4," + (t + 5));
		RegistryMap.add("service28", "28", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,5.2," + (t - 5) + ";1,5.3," + (t + 5));
		RegistryMap.add("service29", "29", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,7.2," + (t - 5) + ";1,7.3," + (t + 5));
		RegistryMap.add("service30", "30", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "1,9.2," + (t - 5) + ";1,9.3," + (t + 5));
		RegistryMap.add("service31", "31", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,1.2," + (t - 5) + ";3,1.4," + (t + 5));
		RegistryMap.add("service32", "32", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,3.2," + (t - 5) + ";3,3.4," + (t + 5));
		RegistryMap.add("service33", "33", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,5.2," + (t - 5) + ";3,5.4," + (t + 5));
		RegistryMap.add("service34", "34", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,7.2," + (t - 5) + ";3.8,7.4," + (t + 5));
		RegistryMap.add("service35", "35", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "3,9.2," + (t - 5) + ";3,9.4," + (t + 5));

		RegistryMap.add("service36", "36", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,1.2," + (t - 5) + ";5,1.4," + (t + 5));
		RegistryMap.add("service37", "37", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,3.2," + (t - 5) + ";5,3.4," + (t + 5));
		RegistryMap.add("service38", "38", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,5.2," + (t - 5) + ";5,5.4," + (t + 5));
		RegistryMap.add("service39", "39", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,7.2," + (t - 5) + ";5,7.4," + (t + 5));
		RegistryMap.add("service40", "40", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "5,9.2," + (t - 5) + ";5,9.4," + (t + 5));

		RegistryMap.add("service41", "41", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,1.2," + (t - 5) + ";7,1.3," + (t + 5));
		RegistryMap.add("service42", "42", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,3.2," + (t - 5) + ";7,3.3," + (t + 5));
		RegistryMap.add("service43", "43", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,5.2," + (t - 5) + ";7,5.4," + (t + 5));
		RegistryMap.add("service44", "44", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,7.2," + (t - 5) + ";7,7.4," + (t + 5));
		RegistryMap.add("service45", "45", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "7,9.2," + (t - 5) + ";7,9.4," + (t + 5));

		RegistryMap.add("service46", "46", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,1.2," + (t - 5) + ";9,1.4," + (t + 5));
		RegistryMap.add("service47", "47", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,3.2," + (t - 5) + ";9,3.4," + (t + 5));
		RegistryMap.add("service48", "48", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,5.2," + (t - 5) + ";9,5.4," + (t + 5));
		RegistryMap.add("service49", "49", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,7.2," + (t - 5) + ";9,7.3," + (t + 5));
		RegistryMap.add("service50", "50", "getTemperature",
				"temperature_service", 1, "thermometer", 1, "Paris",
				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, t + 50000,
				"degreeC", "9,9.2," + (t - 5) + ";9,9.3," + (t + 5));

		BorderPoints AOI = new BorderPoints(new Point(0, 0), new Point(10, 0),
				new Point(0, 10), new Point(10, 10));
		Set<ServiceDescription> selectedServices = new HashSet<ServiceDescription>();
		selectedServices = RegistryMap.findUniformSubSetOfDevice(globalRegion,
				concepts, distribution, subsetSizeVal, AOI, t);
		System.out.println(selectedServices);
		System.out.println(selectedServices.size());
		assertTrue("the size of the selected services set should be 25",
				selectedServices.size() == 25);
		boolean selected = false;

		for (ServiceDescription s : selectedServices) {

			if (s.getAddress().equals("service1")
					|| s.getAddress().equals("service26")) {
				selected = true;

			}

		}

		assertTrue("service 1 or 25 was never selected", selected == true);
	}
//
//	@Test
//	public void testDecide1() throws SQLException {
//		List<Location> mainPathPoints = new ArrayList<Location>();
//		ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
//		double range = 3;
//
//		ArrayList<Location> resultLocations = StatUtil.intersects(
//				mainPathPoints, estimatedPathPoints, range);
//		 RegistryMap.empty();
//		RegistryMap.add("service51", "51", "getTemperature",
//				"temperature_service", 1, "thermometer", 3, "Paris",
//				"temperature", Constants.DOUBLETEMPERATUREC, 1, 1, 3,
//				"degreeC", "2,2,1;4,4,2;5,5,3");
//		String estimatedPath = "1,1,1;5,5,2;6,6,3";
//		Boolean decision = null;
//		HashMap<Boolean, ArrayList<Double>> decisionMap = RegistryMap.decide(
//				relativeWebPath, "thermometer", estimatedPath,
//				"temperature", 0.8, 3);
//		for (Entry<Boolean, ArrayList<Double>> entry : decisionMap.entrySet()) {
//			decision = entry.getKey();
//		}
//		assertTrue(" decision should be not to register, i.e., false",
//				decision == false);
//
//	}
//
//	@Test
//	public void testDecide2() throws SQLException {
//		List<Location> mainPathPoints = new ArrayList<Location>();
//		ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
//		double range = 2;
//
//		ArrayList<Location> resultLocations = StatUtil.intersects(
//				mainPathPoints, estimatedPathPoints, range);
//		RegistryMap.empty();
//		RegistryMap.add("service51", "51", "getTemperature",
//				"temperature_service", 1, "thermometer", 0.5, "Paris",
//				"temperature", Constants.DOUBLETEMPERATUREC, 5753.792023927673,
//				4446.353963748428, 3, "degreeC",
//				"5753.792023927673,4446.353963748428,1;"
//						+ "5753.7921337572325,4446.356436025158,2;"
//						+ "5753.792243586792,4446.358908301887,3;"
//						+ "5753.792353416352,4446.361380578616,4;"
//						+ "5753.790891449999,4446.358465405595,5");
//		String estimatedPath = "5749.331906251922,4435.504946350286,1;"
//				+ "5749.3319167422505,4435.504972587905,2;"
//				+ "5749.33192723258,4435.504998825523,3;"
//				+ "5749.331937722907,4435.505025063143,4;"
//				+ "5749.3319482132365,4435.505051300761,5";
//		Boolean decision = null;
//		HashMap<Boolean, ArrayList<Double>> decisionMap = RegistryMap.decide(
//				relativeWebPath, "noiselevel_sensor", estimatedPath,
//				"noiselevel", 1, 0.5);
//		for (Entry<Boolean, ArrayList<Double>> entry : decisionMap.entrySet()) {
//			decision = entry.getKey();
//		}
//		assertTrue(" decision should be to register, i.e., true",
//				decision == true);
//
//	}
//
//	@Test
//	public void testDecide3() throws SQLException {
//		List<Location> mainPathPoints = new ArrayList<Location>();
//		ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
//		double range = 2;
//
//		ArrayList<Location> resultLocations = StatUtil.intersects(
//				mainPathPoints, estimatedPathPoints, range);
//		RegistryMap.empty();
//		RegistryMap.add("service52", "52", "getNoiselevel",
//				"noiselevel_service", 1, "noiselevel_sensor", 0.5, "Paris",
//				"noiselevel", Constants.DOUBLENOISE, 5753.792023927673,
//				4446.353963748428, 3, "db",
//				"5753.792023927673,4446.353963748428,1;"
//						+ "5753.7921337572325,4446.356436025158,2;"
//						+ "5753.792243586792,4446.358908301887,3;"
//						+ "5753.792353416352,4446.361380578616,4;"
//						+ "5753.790891449999,4446.358465405595,5");
//		String estimatedPath = "5749.331906251922,4435.504946350286,1;"
//				+ "5749.3319167422505,4435.504972587905,2;"
//				+ "5749.33192723258,4435.504998825523,3;"
//				+ "5749.331937722907,4435.505025063143,4;"
//				+ "5749.3319482132365,4435.505051300761,5";
//		Boolean decision = null;
//		HashMap<Boolean, ArrayList<Double>> decisionMap = RegistryMap.decide(
//				relativeWebPath, "noiselevel_sensor", estimatedPath,
//				"noiselevel", 1, 0.5);
//		for (Entry<Boolean, ArrayList<Double>> entry : decisionMap.entrySet()) {
//			decision = entry.getKey();
//		}
//
//		assertTrue(" decision should be to register, i.e., true",
//				decision == true);
//
//	}
//
//	@Test
//	public void testDecide4() throws SQLException {
//		List<Location> mainPathPoints = new ArrayList<Location>();
//		ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
//		double range = 3;
//
//		ArrayList<Location> resultLocations = StatUtil.intersects(
//				mainPathPoints, estimatedPathPoints, range);
//		// RegistryMap.empty();
//		RegistryMap.add("service53", "53", "getsound", "sound_service", 1,
//				"sound_sensor", 3, "Paris", "sound",
//				Constants.DOUBLENOISE, 1, 1, 3, "db",
//				"2,2,1;4,4,2;5,5,3");
//		String estimatedPath = "1,1,1;5,5,2;6,6,3";
//		Boolean decision = null;
//		HashMap<Boolean, ArrayList<Double>> decisionMap = RegistryMap.decide(
//				relativeWebPath, "noiselevel_sensor", estimatedPath,
//				"noiselevel", 1, 2);
//		for (Entry<Boolean, ArrayList<Double>> entry : decisionMap.entrySet()) {
//			decision = entry.getKey();
//		}
//		assertTrue(" decision should be not to register, i.e., false",
//				decision == false);
//
//	}
}
